unit Dialog97;
{******************************************************************************
Freeware use and abuse
Use at your own risk, no liablity for damages of any sort will be accept by the authors
or anyone connected with them in anyway whatsoever

TDialog97 a 97 style dialog for use with Delphi 1 & Delphi 3 (Different files for each)
It combines the 3 ways of showing dialogs into one

viz Execute,Showmessage,Messagedlg
    & also operates as an inputquery dialog

Note: There is no wrap around for long , so if you are using long sentences then
      put in your own Carriage Returns.

 Authors
 Edward de la Rey & Paul Cook
 email:edwardr@mailbox.ru.ac.za

Files: Dialog97.Pas - Install this one to install the component
       dialog97.dcr - Resource File for the component
       frmdlg97.dfm - Form file    ie. Actual Dialog form
       frmdlg97.pas - Pas file for the form file

Extra files: You will need the TExplorer freeware transparent button component
             installed for the Delphi 1 Version
PROPERTIES:
     DialogType:TMsgDlgtype;
     DialogButtons: TMsgDlgButtons;
     HelpContextID: Integer;
     Msg:String;
METHODS:
      InputQuery(const ACaption, APrompt: string; var Value: string): Integer;
                Shows as an input box, and returns the text entered, also returns mrResult value
                to say wether user pressed ok or cancel
                if no msgpassed then the msg property value is used
                if msg porperty is blank then  no message displayed

      Execute: Shows the dialog form and returns result as ButtonPressed ie MROk,MRCancel etc or
               returns MRCANCEL if User Closes form without choosing
               uses the above properties ie Dialogtype etc
      ShowMessage(const Msg: string): Shows the dialog as an information dialog, using the message passed to it
                and an OK button,
                The name of the application's title file appears as the caption of the message box.
                Useful for debugging
      MessageDlg: Shows the dialog using the parameters passed across, returns button pressed
                  or MRCANCEL if appropriate this is the same as normal messagedlg function
EVENTS: None

Note: When using the Showmessage feature, or if choosing the mtcustom message type, the
      caption displayed on the dialog is that of the application objects title
}

interface
uses classes,controls,winprocs,wintypes,graphics,frmdlg97,dialogs,sysutils,explbtn,forms;
type
  TDialog97 = class(Tcomponent)
  private
  GetMsg:String; {Message to be displayed}
  GetDialogType:TmsgdlgType; {The Current Messagedlg type}
  GetMessageDialogButtons :TMsgDlgButtons;
  GetHelpContextId:LongInt;
  procedure SetdialogAttributes; {Setup the dialog using UserAttributes}
  procedure SetHelpContextId (Value:LongInt);
  procedure SetMessageDialogButtons (dlgBtns:TMsgDlgButtons);
  procedure SetDialogType (dlgType:TmsgdlgType);
  procedure SetMsg (Value:String);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute:Word; {Method Used to Execute the dialog}
    function InputQuery(const ACaption, APrompt: string; var Value: string): Integer;
    procedure ShowMessage (MyMsg:String);
    function MessageDlg (MyMsg:String;MsgType:TMsgDlgtype;DlgBtns:TmsgDlgButtons;HlpCnt:LongInt):Word;
    published {execute is a method}
    property DialogType:TMsgDlgtype Read GetDialogType Write SetDialogType Default Mtinformation;
    Property DialogButtons: TMsgDlgButtons Read GetMessageDialogButtons Write SetMessageDialogButtons Default [MbOk];
    Property HelpContextID: LongInt Read GetHelpContextId Write SetHelpContextId Default 0;
    Property Msg:String Read GetMsg Write SetMsg;

  end;

 procedure Register;

implementation
 var
 TheDialog:TfrmDialog97; {Instance of the actual dialog form}
 i:TMsgDlgBtn;
 btnCount,btnNum: integer;
 dlgWidth,btnSpace: longint;
 {-------------------------------------------------------------}
 {---------------------- TDialog97 --------------------------}
 {-------------------------------------------------------------}

Function TDialog97.InputQuery(const ACaption, APrompt: string; var Value: string): Integer;
{Returns the users input in msg and btnPressed as return value}
Var
  RemGetMsg:String; {Message to be displayed}
  RemGetDialogType:TmsgdlgType; {The Current Messagedlg type}
  RemGetMessageDialogButtons :TMsgDlgButtons;
  begin
{remember the component settings}
RemGetMsg:=GetMsg;
remGetDialogType:=GetDialogType; {The Current Messagedlg type}
RemGetMessageDialogButtons :=GetMessageDialogButtons;

{Set the new values}
  if APrompt <> '' then GetMsg:=APrompt; {If not use the message property value}
  GetDialogType:=mtCustom; {Custom so no bitmap}
  GetMessageDialogButtons:=[MBOk,MBCancel]; {Dialog buttons}

TheDialog := TfrmDialog97.Create(Application);
 With TheDialog do begin
 dlglabel.align :=alTop;
 dlglabel.alignment :=taCenter;
 SetDialogAttributes; {Configure the dialog to show correctly}
 if Acaption <> '' then Thedialog.caption := ACaption;
 txtEdit.top := dlgLabel.height + 10;
 txtEdit.width := Thedialog.width - 50;
 txtEdit.left := (Thedialog.width - txtEdit.width) div 2;
 if value <> '' then
    txtEdit.text := Value
 else
    txtEdit.Text :='';
 txtEdit.visible := true;


 If TheDialog.showmodal = MrOk Then
   {Only put text in if user enters valid text}
    if txtEdit.text <> '' then Value := txtEdit.text;

     InputQuery := frmDlg97.ButtonPressed;

 { Retore the component settings}
 dlgLabel.wordwrap:= false;
 Thedialog.dlglabel.align :=alNone;
 Thedialog.dlglabel.alignment :=taLeftjustify;
 GetMsg:=RemGetMsg;
 GetDialogType:=RemGetDialogType;
 GetMessageDialogButtons:=RemGetMessageDialogButtons;
 TheDialog.free;
 end;{with}
end;

function TDialog97.MessageDlg (MyMsg:String;MsgType:TMsgDlgtype;DlgBtns:TmsgDlgButtons;HlpCnt:LongInt):word;
Var
  RemGetMsg:String; {Message to be displayed}
  RemGetDialogType:TmsgdlgType; {The Current Messagedlg type}
  RemGetMessageDialogButtons :TMsgDlgButtons;
  RemGetHelpContextId:LongInt;
begin
  {remember the component settings}
  RemGetMsg:=GetMsg;
  remGetDialogType:=GetDialogType; {The Current Messagedlg type}
  RemGetMessageDialogButtons:=GetMessageDialogButtons;
  RemGetHelpContextId:= GetHelpContextId;

  {Set the new values}
  GetMsg:=MyMsg;
  GetDialogType:=MsgType;
  GetMessageDialogButtons:=DlgBtns;
  GetHelpContextId:=HlpCnt;

   {Create and run the dialog form}
 TheDialog := TfrmDialog97.Create(Application);
 SetDialogAttributes; {Configure the dialog to show correctly}

 If TheDialog.showmodal = MrOk Then
     messagedlg := frmDlg97.ButtonPressed
 else
     messagedlg := mrCancel;

 TheDialog.Free ;

{ Retore the component settings}
 GetMsg:=RemGetMsg;
 GetDialogType:=RemGetDialogType;
 GetMessageDialogButtons:=RemGetMessageDialogButtons;
 GetHelpContextId:= RemGetHelpContextId;
 end;

Procedure Tdialog97.SetDialogAttributes;
procedure ApplyFunction (WhichButton:Texplorerbutton);
begin
   WhichButton.left:=(btnSpace*btnNum)-45;
   WhichButton.top:=theDialog.height- 60;
   btnNum:=btnNum+1;
   WhichButton.visible:=true;
end;
begin
  btnCount:= 0;

  with TheDialog do begin
  dlgLabel.left:=60;
  dlgLabel.top := 8;
  dlgLabel.autosize:= true;
  dlglabel.Wordwrap:= false;
  helpcontext:=HelpContextID;
  dlgLabel.caption := getmsg;
 {Set up the dialog to show the way the user wants}
  Height := dlgLabel.Height + 100;
  Width := dlgLabel.Width + 100;

 case dialogtype of
  mtinformation : begin expinfo.visible := true;caption:='Information';end;
  mterror: begin experror.visible := true;caption:='Error';end;
  mtwarning: begin expwarn.visible :=true;caption:='Warning';end;
  mtconfirmation :begin expconf.visible:= true;caption:='Confirmation';end;
  mtcustom:caption:=application.title;
 end; {Case}

 for i:= mbYes to mbHelp do
  begin  {Count the number of buttons}
  if i in dialogbuttons then
  btnCount:=btnCount+1;
end;{For}

if (width < (btnCount * 100)) then
   width:=(btnCount*100)+8;

dlgWidth:= width;

btnNum:=1;
btnSpace:= round(((dlgWidth)/(btncount+1)));

for i:= mbYes to mbHelp do
  begin
  if i in dialogbuttons then begin
  Case i of
  mbHelp:ApplyFunction(cmdHelp);
  mball:ApplyFunction(cmdAll);
  mbIgnore:ApplyFunction(cmdIgnore);
  mbRetry:ApplyFunction(cmdRetry);
  mbAbort:ApplyFunction(cmdAbort);
  mbCancel:ApplyFunction(cmdCancel);
  mbOk: ApplyFunction(cmdOK);
  mbNo:ApplyFunction(cmdNo);
  mbYes:ApplyFunction(cmdYes);
 end;{case}
 end;{if}
end;{For}
 
end;{with}

end;

procedure TDialog97.ShowMessage (MyMsg:String);
begin
{Create and run the dialog form}
 TheDialog := TfrmDialog97.Create(Application);
  With thedialog do begin
   dlgLabel.left :=60;
   dlgLabel.wordwrap:=false;
   dlgLabel.AutoSize:=true;
   dlgLabel.caption := MyMsg;
   TheDialog.caption := Application.Title;
   expInfo.visible := true;
   Height := dlgLabel.Height + 100;
   Width := dlgLabel.Width + 100;
   cmdOk.left:=(width div 2) - (cmdOk.width div 2);
   cmdOK.top:=height-60;
   cmdOK.visible:=true;
   showmodal;
  end;
 TheDialog.free;
end;

 procedure Tdialog97.SetMsg (Value:String);
 begin
   GetMsg := Value;
 end;

 procedure Tdialog97.SetHelpContextId (Value:LongInt);
 begin
   GetHelpContextId:=Value;
 end;

 procedure TDialog97.SetDialogtype (dlgtype:TMsgDlgtype);
 begin
  GetDialogType:=dlgType;
 end;

 procedure TDialog97.SetMessageDialogButtons (dlgBtns:TMsgDlgButtons);
 begin
  GetMessageDialogButtons:=dlgBtns;
 end;

 function TDialog97.Execute:Word;
 begin
 {Create and run the dialog form}
 TheDialog := TfrmDialog97.Create(Application);
 SetDialogAttributes; {Configure the dialog to show correctly}

 If TheDialog.showmodal = MrOk Then
    Execute := frmDlg97.ButtonPressed
 else
    Execute := mrCancel;

 TheDialog.Free ;
 end;

 constructor TDialog97.Create(AOwner: TComponent);
begin
   INHERITED CREATE(Aowner);

     GetDialogType:=Mtinformation;
     GetMessageDialogButtons:= [mbok];
     GetHelpContextId:=0;
     GetMsg := 'Message';
   
End;

destructor TDialog97.Destroy;
begin
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Freeware', [TDIALOG97]);
end;

end.
