unit Barsize;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Cus_Bas;

const
  HAUTEUR_BARRE_RESIZE    = 5 ;

type
  TSL_Mouvement = procedure(Sender: TObject) of Object ;
  TSL_DuringSlide = procedure(Sender: TObject;X,Y:Integer) of Object ;

  TBarreResize = class(TCustomControl_Base)
  private
    FThickness       : integer;
    FIsHorizontale   : Boolean ;
    IsBuildMode      : Boolean ;
    ValeurDecallage  : Integer ;
    FOnSl_mouvement  : TSL_Mouvement ;
    FOnDuringSlide   : TSL_DuringSlide ;
    FOnBeforeMouseUp : TMouseEvent;
    FOnBeforeMouseDown : TMouseEvent;
    FMax             : integer;
    FMin             : integer;
    FIsLoaded        : boolean;  
    procedure SetIsHorizontale(Value: Boolean);
    procedure Dessiner ;
  protected
    procedure Loaded; override;
    procedure Paint; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure SetThickness (value : integer);
    property IsLoaded : boolean read FIsLoaded;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Thickness : integer read FThickness write SetThickness default HAUTEUR_BARRE_RESIZE;
    property Max : integer read FMax write FMax default -1;
    property Min : integer read FMin write FMin default -1;
    property Align;
    property IsHorizontal : Boolean read FIsHorizontale write SetIsHorizontale default true;
    property OnNewPosition : TSL_Mouvement read FOnSl_mouvement write FOnSl_mouvement ;
    property OnSliding : TSL_DuringSlide read FOnDuringSlide write FOnDuringSlide ;
    property OnMouseDown;
    property OnMouseUp;
    property OnBeforeMouseUp : TMouseEvent read FOnBeforeMouseUp write FOnBeforeMouseUp;
    property OnBeforeMouseDown : TMouseEvent read FOnBeforeMouseDown write FOnBeforeMouseDown;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('NewComps', [TBarreResize]);
end;

{---------------------------------------------------------------------------}

constructor TBarreResize.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width:=100 ;
  Height:=HAUTEUR_BARRE_RESIZE ;
  FThickness := HAUTEUR_BARRE_RESIZE;
  FIsHorizontale:=TRUE ;
  FMax := -1;
  FMin := -1;
end;

procedure TBarreResize.Loaded;
begin
  inherited Loaded;
  FIsLoaded := true;
end;

procedure TBarreResize.SetThickness;
begin
  if FThickness <> value
    then
    begin
      FThickness := value;
      if (csDesigning in ComponentState) and FIsLoaded
        then if FIsHorizontale
          then Height := FThickness
          else Width := FThickness;
      Refresh;
    end;
end;

procedure TBarreResize.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  IsBuildMode:=csDesigning in ComponentState ;
  if FISHorizontale
    then Cursor:=crVSplit
    else Cursor:=crHSplit;
end;

procedure TBarreResize.Paint;
begin
  with Canvas do
  begin
    Pen.Color:=RGB(0,0,0) ;
    Brush.Color:=RGB(192,192,192) ; ;
    if FIsHorizontale then
    begin
      Rectangle(0,0,Width,FThickness) ;
      Pen.Color:=RGB(128,128,128) ;
      MoveTo(1,1) ;
      LineTo(Width-1,1) ;
      Pen.Color:=RGB(255,255,255) ;
      MoveTo(1,3) ;
      LineTo(Width-1,3) ;
    end
    else
    begin
      Rectangle(0,0, FThickness,Height) ;
      Pen.Color:=RGB(128,128,128) ;
      MoveTo(1,1) ;
      LineTo(1,Height-1) ;
      Pen.Color:=RGB(255,255,255) ;
      MoveTo(3,1) ;
      LineTo(3,Height-1) ;
    end ;
  end ;
end ;

procedure TBarreResize.SetIsHorizontale(Value: Boolean);
var
  Temp : integer;
begin
  if (IsBuildMode) and (FIsHorizontale <> Value) and IsLoaded then
    begin
      Temp  := Width;
      Width := Height ;
      Height:= Temp;
    end ;
  FIsHorizontale:=Value ;
  Redraw ;
end ;

procedure TBarreResize.MouseDown(Button: TMouseButton; Shift: TShiftState;X, Y: Integer);
var
  Mypoint       : TPoint ;
  Last : integer;
begin
  if assigned (FOnBeforeMouseDown)
    then FOnBeforeMouseDown (self, Button, Shift, X, Y);
  if Button = mbLeft then
  begin
    MyPoint.X:=X;
    MyPoint.Y:=Y ;
    MyPoint:=ClientToScreen(MyPoint) ;
    MyPoint:=TForm(Parent).ScreenToClient(MyPoint) ;
    Last := ValeurDecallage;
    if FIsHorizontale then
      ValeurDecallage:=MyPoint.Y
    else
      ValeurDecallage:=MyPoint.X ;
    if ((ValeurDecallage > FMax) and (FMax >= 0)) or ((ValeurDecallage < FMin) and (FMin >= 0))
      then ValeurDecallage := Last;
    Dessiner ;
  end ;
  inherited MouseDown(Button,Shift, X, Y);
end ;

procedure TBarreResize.MouseUp(Button: TMouseButton; Shift: TShiftState;X, Y: Integer);
var
  IsMouvement       : Boolean ;
begin
  if assigned (FOnBeforeMouseUp)
    then FOnBeforeMouseUp (self, Button, Shift, X, Y);
  if Button = mbLeft then
  begin
    Dessiner ;
    IsMouvement:=FALSE ;
    if FIsHorizontale then
    begin
      if (ValeurDecallage<(TForm(Parent).ClientHeight-FThickness-2)) and
         (ValeurDecallage>2) then
      begin
        IsMouvement:=(Top<>ValeurDecallage) ;
        Top:=ValeurDecallage ;
      end ;
    end
    else
    begin
      if (ValeurDecallage<(TForm(Parent).ClientWidth-FThickness-2)) and
         (ValeurDecallage>2) then
      begin
        IsMouvement:=(Left<>ValeurDecallage) ;
        Left:=ValeurDecallage ;
      end ;
    end ;
    if (IsMouvement) and (Assigned(FOnSL_Mouvement)) then
    begin
      OnNewPosition(Self) ;
    end ;
  end ;
  inherited MouseUp(Button,Shift, X, Y);
end ;

procedure TBarreResize.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  MyPoint       : TPoint ;
  Temp          : Integer ;
  Temp2         : Integer ;
  NoChange : boolean;
begin
  NoChange := false;
  if ssLeft in Shift then
  begin
    MyPoint.X:=X;
    MyPoint.Y:=Y ;
    MyPoint:=ClientToScreen(MyPoint) ;
    MyPoint:=TForm(Parent).ScreenToClient(MyPoint) ;
    if FIsHorizontale then
    begin
      Temp:=MyPoint.Y ;
      Temp2:=TForm(Parent).ClientHeight ;
    end
    else
    begin
      Temp:=MyPoint.X ;
      Temp2:=TForm(Parent).ClientWidth ;
    end;
    if ((Temp > FMax) and (FMax >= 0)) or ((Temp < FMin) and (FMin >= 0))
      then NoChange := true;
    if not NoChange
      then
      begin
        if (Temp<=0) then
          Temp:=1
        else if (Temp>=Temp2-FThickness-1) then
          Temp:=Temp2-FThickness-2 ;
        Dessiner ;
        if (Assigned(FOnDuringSlide)) then
        begin
          if FIsHorizontale then
            OnSliding(Self,Left,ValeurDecallage)
          else
            OnSliding(Self,ValeurDecallage,Top) ;
        end ;
        ValeurDecallage:=Temp ;
        Dessiner ;
      end;
  end ;
  inherited MouseMove(Shift, X, Y);
end ;

procedure TBarreResize.Dessiner ;
var
  MyDC         : HDC ;
  MyPoint      : TPoint ;
begin
  MyDC:=GetDC(0) ;
  if FIsHorizontale then
  begin
    MyPoint.X:=Left ;
    MyPoint.Y:=ValeurDecallage ;
    MyPoint:=TForm(Parent).ClientToScreen(MyPoint) ;
    BitBlt(MyDC,MyPoint.X,MyPoint.Y,Width,Height,0,0,0,DSTINVERT) ;
  end
  else
  begin
    MyPoint.X:=ValeurDecallage ;
    MyPoint.Y:=Top ;
    MyPoint:=TForm(Parent).ClientToScreen(MyPoint) ;
    BitBlt(MyDC,MyPoint.X,MyPoint.Y,Width,Height,0,0,0,DSTINVERT) ;
  end ;
  ReleaseDC(0,MyDC) ;
end ;

end.
