//---------------------------------------------------------------------------
#ifndef chessH
#define chessH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
//---------------------------------------------------------------------------

#define NoPiece   -1
#define Black      0
#define White      1


enum pieces
{BP=0,BN,BB,BR,BK,BQ,WP=8,WN,WB,WR,WQ,WK};

enum Square
{  None=0,
   A8,B8,C8,D8,E8,F8,G8,H8,
   A7,B7,C7,D7,E7,F7,G7,H7,
   A6,B6,C6,D6,E6,F6,G6,H6,
   A5,B5,C5,D5,E5,F5,G5,H5,
   A4,B4,C4,D4,E4,F4,G4,H4,
   A3,B3,C3,D3,E3,F3,G3,H3,
   A2,B2,C2,D2,E2,F2,G2,H2,
   A1,B1,C1,D1,E1,F1,G1,H1};

enum StandardSet
{ Size32, Size40, Size64, Size80};

enum DisplayCoords { West, North, East, South };
enum CanStillCastle { WhiteKingSide,WhiteQueenSide, BlackKingSide, BlackQueenSide };

typedef Set<CanStillCastle, WhiteKingSide, BlackQueenSide>  CastleSet;
typedef Set<DisplayCoords,  West, South>  CoordSet;

typedef void __fastcall (__closure *TMoveEvent) (System::TObject *Sender,  int oldSq, int newSq);
typedef void __fastcall (__closure *TCaptureEvent) (System::TObject *Sender,  int oldSq, int newSq, char CapturedPiece);
typedef void __fastcall (__closure *TOneSquareEvent) (System::TObject *Sender,  int square);
typedef void __fastcall (__closure *TPromotionEvent) (System::TObject *Sender,  int oldSq, int newSq, char &NewPiece);

class MoveInfo
{
    public:
    __fastcall MoveInfo(void);

    char position[66];
    CastleSet Castling;
    Square OldSquare, NewSquare;
    Square EnPassant;
};

class TChessBoard : public TGraphicControl
{
private:
    Square enPassant;
    MoveInfo MoveList[256][2];

    int minsize,maxsize;

    bool ResizeState, resizable;
    char PromoteTo;
    int PieceIndex[2][6];

    int Boardx,Boardy,BoardSize;
    int PieceSize;
    int SquareSize;
    int currentMove;

    StandardSet sSet;

    TMoveEvent legalmove,check,mate,stalemate,castle;
    TNotifyEvent onlytwokingsleft;
    TCaptureEvent capture;
    TOneSquareEvent illegalmove;
    TPromotionEvent promotion;

    char position[66];
    String posstr;

    TImageList *list;
    Graphics::TBitmap *bmpwhite,*bmpblack, *bmpborder;
    Graphics::TBitmap *Custom,  *Default;
    TPen *lines;
    TFont *coordFont;

    CastleSet Castling;
    CoordSet ShowCoords;

    int Border;
    int SquareClick1,SquareClick2;

    bool WhiteAbove,WhiteMove,ShowLines;
    TColor scwhite, scblack, bordercol;

    bool __fastcall MoveIsLegal (int oldsq, int newsq);
    bool __fastcall CheckLegalBishopMove(int oldsq,int newsq);
    bool __fastcall CheckLegalKingMove(int oldsq, int newsq);
    bool __fastcall CheckLegalKnightMove(int oldsq,int newsq);
    bool __fastcall CheckLegalPawnMove(int oldsq,int newsq);
    bool __fastcall CheckLegalQueenMove(int oldsq,int newsq);
    bool __fastcall CheckLegalRookMove(int oldsq,int newsq);
    bool __fastcall ExistBmpBlack(void);
    bool __fastcall ExistBmpBorder(void);
    bool __fastcall ExistBmpWhite(void);
    bool __fastcall ExistCustomPieceSet(void);
    bool __fastcall ExistDefaultPieceSet(void);

    int  __fastcall PieceToInt (char piece);
    void __fastcall DoPromotion (int square);
    void __fastcall DrawBoard (void);
    void __fastcall DrawBorder (void);
    void __fastcall DrawPiece (int sq, char piece);
    void __fastcall DrawPieces (void);
    void __fastcall Fillint (int sqn);
    void __fastcall InitializeBitmaps (void);
    void __fastcall OrganizeBitmaps(void);

    //boring write methods
    void __fastcall SetMinSize (int size);
    void __fastcall SetMaxSize (int size);
    void __fastcall SetCurrentMove (int moveno);
    void __fastcall SetEnPassant (Square sqno);
    void __fastcall SetPosStr(String pos);
    void __fastcall SetWidth(int wdt);
    void __fastcall SetHeight(int hgt);
    void __fastcall SetSizeOfSquare(int size);
    void __fastcall SetBorderBitmap(Graphics::TBitmap *bmp);
    void __fastcall SetBorderColor(TColor c);
    void __fastcall SetBorderWidth (int border);
    void __fastcall SetCustomPieceSet(Graphics::TBitmap *bmp);
    void __fastcall SetDarkSquare(Graphics::TBitmap *bmp);
    void __fastcall SetFont(TFont *font);
    void __fastcall SetLightSquare(Graphics::TBitmap *bmp);
    void __fastcall SetLineStyle(TPen *pen);
    void __fastcall SetShowCoords(CoordSet set);
    void __fastcall SetShowLines(bool show);
    void __fastcall SetSquareColorDark(TColor c);
    void __fastcall SetSquareColorLight(TColor c);
    void __fastcall SetStandardSize(StandardSet set);
    void __fastcall SetWhiteAbove(bool wabove);
    void __fastcall SetWhiteMove(bool wmove);
    void __fastcall SetWhiteSquareBitmap(Graphics::TBitmap *bmp);

protected:
    //Event handlers
    virtual void __fastcall Capture (System::TObject *Sender,  int oldSq, int newSq, char CapturedPiece);
    virtual void __fastcall Castle (System::TObject *Sender, int oldSq, int newSq);
    virtual void __fastcall Check (System::TObject *Sender,  int oldSq, int newSq);
    virtual void __fastcall Click(void);
    virtual void __fastcall DragCanceled(void);
    virtual void __fastcall DragDrop(System::TObject *Source,int X, int Y);
    virtual void __fastcall DragOver(System::TObject* Source, int X, int Y,TDragState State, bool &Accept );
    virtual void __fastcall EndDrag(bool drop);
    virtual void __fastcall IllegalMove (System::TObject *Sender,  int square);
    virtual void __fastcall LegalMove (System::TObject *Sender,  int oldSq, int newSq);
    virtual void __fastcall Mate (System::TObject *Sender,  int oldSq, int newSq);
    virtual void __fastcall MouseDown(TMouseButton Button, Classes::TShiftState Shift, int X, int Y);
    virtual void __fastcall MouseMove(Classes::TShiftState Shift, int X, int Y);
    virtual void __fastcall MouseUp(TMouseButton Button, Classes::TShiftState Shift, int X, int Y);
    virtual void __fastcall OnlyTwoKingsLeft (System::TObject *Sender);
    virtual void __fastcall Paint(void);
    virtual void __fastcall Promotion (System::TObject *Sender,  int oldSq, int newSq, char &NewPiece);
    virtual void __fastcall StaleMate (System::TObject *Sender,  int oldSq, int newSq);

public:

    int FirstMove, LastMove;
    bool FirstTurn, LastTurn;

    //Constructor, destructor
    __fastcall TChessBoard(TComponent* Owner);
    virtual __fastcall ~TChessBoard(void);

    //Public interface
    bool __fastcall BlackInCheckAfter(int oldsq, int newsq);
    bool __fastcall ClearSquare(int sqn);
    static int inline __fastcall ColorOfPiece (char piece);
    int  __fastcall ColorOfPiece (int sqno);
    int  __fastcall ColorOfSquare (int sqno);
    MoveInfo& __fastcall GetMove (int moveno, bool WhiteMoves);
    TStringList* __fastcall GetMoveList (void);
    bool __fastcall GotoMove (int moveno, bool WhiteMoves);
    bool __fastcall LegalMoveAvailable (void);
    static int inline __fastcall MouseToSquare (int x, int y);
    bool __fastcall Move (int oldsq, int newsq);
    bool __fastcall MoveBackward (void);
    bool __fastcall MoveForward (void);
    void __fastcall NewGame(void);
    bool __fastcall SetUpPosition (MoveInfo &pos, int moveno, bool whitemoves);
    void __fastcall UpdateChessBoard (char *oldpos);
    bool __fastcall WhiteInCheckAfter(int oldsq, int newsq);
    int  __fastcall WindowToSquare (int x, int y);
    static int inline __fastcall XPos (int sqno);
    static int inline __fastcall YPos (int sqno);

__published:
    __property bool BoardLines={read=ShowLines,write=SetShowLines};
    __property Graphics::TBitmap *BorderBitmap={read=bmpborder, write=SetBorderBitmap};
    __property TColor BorderColor={read=bordercol, write=SetBorderColor};
    __property CastleSet CastlingAllowed={read=Castling,write=Castling};
    __property TFont *CoordFont={read=coordFont, write=SetFont};
    __property int CurrentMove={read=currentMove, write=SetCurrentMove};
    __property Graphics::TBitmap *CustomPieceSet={read=Custom, write=SetCustomPieceSet};
    __property CoordSet DisplayCoords={read=ShowCoords,write=SetShowCoords};
    __property Square EnPassant={read=enPassant, write=SetEnPassant};
    __property TPen *LineStyle={read=lines, write=SetLineStyle};
    __property String Position={read=posstr, write=SetPosStr};
    __property bool Resizable={read=resizable, write=resizable};
    __property int  ResizeMinSize={read=minsize, write=SetMinSize};
    __property int  ResizeMaxSize={read=maxsize, write=SetMaxSize};
    __property int SizeOfBorder={read=Border, write=SetBorderWidth};
    __property int SizeOfSquare={read=SquareSize, write=SetSizeOfSquare};
    __property TColor SquareColorDark={read=scblack, write=SetSquareColorDark};
    __property TColor SquareColorLight={read=scwhite, write=SetSquareColorLight};
    __property Graphics::TBitmap *SquareDark={read=bmpblack, write=SetDarkSquare};
    __property Graphics::TBitmap *SquareLight={read=bmpwhite, write=SetLightSquare};
    __property StandardSet StandardSize={read=sSet, write=SetStandardSize, default=Size40};
    __property bool WhiteOnTop={read=WhiteAbove,write=SetWhiteAbove};
    __property bool WhiteToMove={read=WhiteMove,write=SetWhiteMove,default=TRUE};

    __property TMoveEvent OnCastle={read=castle, write=castle};
    __property TMoveEvent OnCheck={read=check, write=check};
    __property TMoveEvent OnLegalMove={read=legalmove, write=legalmove};
    __property TMoveEvent OnMate={read=mate, write=mate};
    __property TMoveEvent OnStaleMate={read=stalemate, write=stalemate};
    __property TNotifyEvent OnOnlyTwoKingsLeft={read=onlytwokingsleft, write=onlytwokingsleft};
    __property TOneSquareEvent OnIllegalMove={read=illegalmove, write=illegalmove};
    __property TPromotionEvent OnPromotion={read=promotion, write=promotion};
    __property TCaptureEvent OnCapture={read=capture, write=capture};

    __property Visible;
    __property Align;

    __property DragCursor;
    __property DragMode;
    __property OnClick;
    __property OnDblClick ;
    __property OnDragDrop ;
    __property OnDragOver ;
    __property OnEndDrag ;
    __property OnMouseDown ;
    __property OnMouseMove ;
    __property OnMouseUp;
    __property OnStartDrag;
};
//---------------------------------------------------------------------------
#endif
