unit DbNav97;

interface

uses SysUtils, WinTypes, WinProcs, Messages, Classes, Controls, Forms,
  Graphics, Menus, StdCtrls, ExtCtrls, DB, DBTables, Mask, Buttons,
  ExplBtn;

const
  InitRepeatPause = 400;  { pause before repeat timer (ms) }
  RepeatPause     = 100;  { pause before hint window displays (ms)}
  SpaceSize       =  5;   { size of space between special buttons }

type
  tNavBtn97 = class;
  tNavDatalink = class;

  tNavGlyph = (ngEnabled, ngDisabled);
  tNavBackType = (bkNormal, bkStretch);
  tNavBtnType = (btBitmap, btCaption, btBoth);
  tNavDirection = (dirAuto, dirHorizontal, dirVertical);
  tNavigateBtn = (nbFirst, nbPrior, nbNext, nbLast,
                  nbInsert, nbDelete, nbEdit, nbPost, nbCancel, nbRefresh);
  TVertButtonSet = set of tNavigateBtn;
  tNavBtn97Style = set of (nsAllowTimer, nsFocusRect);

  EVertNavClick = procedure (Sender: TObject; Button: tNavigateBtn) of object;

{ tDbNav97 }

  tDbNav97 = class (TCustomPanel)
  private
    FDataLink: tNavDatalink;
    FVisibleButtons: TVertButtonSet;
    FHints: TStrings;
    FCaptions: TStrings;
    FCaptionStyle: TExplorerButtonLayout;
    FBackground : tBitmap;
    FBackType : tNavBackType;
    FBtnType : tNavBtnType;
    FDirection : tNavDirection;
    ButtonWidth: Integer;
    ButtonHeight: Integer;
    MinBtnSize: TPoint;
    FOnNavClick: EVertNavClick;
    FocusedButton: tNavigateBtn;
    FConfirmDelete: Boolean;
    function GetDataSource: TDataSource;
    procedure SetDataSource(Value: TDataSource);
    procedure LoadBitmaps (Btn : tNavBtn97; BtnType : pChar);
    procedure UnLoadbitmaps (Btn : tNavBtn97);
    procedure InitButtons;
    procedure InitHints;
    procedure InitCaptions;
    procedure Click(Sender: TObject);
    procedure BtnMouseDown (Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SetVisible(Value: TVertButtonSet);
    procedure AdjustSize (var W: Integer; var H: Integer);
    procedure SetHints(Value: TStrings);
    procedure SetCaptions(Value: TStrings);
    procedure FSetBackground(value: TBitmap);
    procedure FEnableCaptions(Value: Boolean);
    procedure FEnableBitmaps(Value: Boolean);
    procedure SetCaptionStyle(Value : TExplorerButtonLayout);
    procedure FSetBtnType(Value : tNavBtnType);
    procedure FSetBackType(Value : tNavBackType);
    procedure FSetDirection(Value : tNavDirection);
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  protected
    Buttons: array[tNavigateBtn] of tNavBtn97;
    procedure DataChanged;
    procedure EditingChanged;
    procedure ActiveChanged;
    procedure Loaded; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure BtnClick(Index: tNavigateBtn);
  published
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property VisibleButtons: TVertButtonSet read FVisibleButtons write SetVisible
      default [nbFirst, nbPrior, nbNext, nbLast, nbInsert, nbDelete,
        nbEdit, nbPost, nbCancel, nbRefresh];
    property Align;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Ctl3D;
    property Hints: TStrings read FHints write SetHints;
    property Captions: TStrings read FCaptions write SetCaptions;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ConfirmDelete: Boolean read FConfirmDelete write FConfirmDelete default True;
    property ShowHint;
    property CaptionStyle : TExplorerButtonLayout read FCaptionStyle write SetCaptionStyle;
    property Style : tNavBtnType read FBtnType write FSetBtnType default btBitmap;
    property Direction : tNavDirection read FDirection write FSetDirection default dirAuto;
    property Background : TBitmap read FBackground write FSetBackground stored true;
    property BackgroundStyle : tNavBackType read FBackType write FSetBackType default bkNormal;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick: EVertNavClick read FOnNavClick write FOnNavClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnResize;
  end;

{ tNavBtn97 }

  tNavBtn97 = class(TExplorerButton)
  private
    FIndex: tNavigateBtn;
    FNavStyle: tNavBtn97Style;
    FRepeatTimer: TTimer;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    destructor Destroy; override;
    property NavStyle: tNavBtn97Style read FNavStyle write FNavStyle;
    property Index : tNavigateBtn read FIndex write FIndex;
  end;

{ tNavDatalink }

  tNavDatalink = class(TDataLink)
  private
    FNavigator: tDbNav97;
  protected
    procedure EditingChanged; override;
    procedure DataSetChanged; override;
    procedure ActiveChanged; override;
  public
    constructor Create(ANav: tDbNav97);
    destructor Destroy; override;
  end;

procedure Register;

implementation

{$R *.RES}

uses DBIErrs, DBITypes, Clipbrd, DBConsts, Dialogs;

procedure Register;
begin
  RegisterComponents('Freeware',[tDbNav97]);
end;


{ Drawing of a bitmap on a canvas, with transparency
  (adapted from code found in the Microsoft Knowledge Base)}

procedure DrawTransparentBitmap (canvas: TCanvas; bmp: TBitmap;
                                 xStart, yStart: Integer;
                                 xStop, yStop: Integer;
                                 cTransparentColor: LongInt;
                                 Stretched : Boolean);
var bm: WinTypes.TBitmap;
	cColor: TColorRef;
	bmAndBack, bmAndObject, bmAndMem, bmSave, oldBmp: HBITMAP;
	bmBackOld, bmObjectOld, bmMemOld, bmSaveOld, hBmp: HBITMAP;
	hdcMem, hdcBack, hdcObject, hdcTemp, hdcSave, dc: HDC;
	ptSize: WinTypes.TPoint;
	temp_bitmap: TBitmap;
begin
	temp_bitmap := TBitmap.Create;
	temp_bitmap.Assign(bmp);
	try
		dc := canvas.Handle;
		hBmp := temp_bitmap.Handle;
		hdcTemp := CreateCompatibleDC(dc);
		oldBmp := SelectObject(hdcTemp, hBmp);

		GetObject(hBmp, SizeOf(bm), @bm);
		ptSize.x := bm.bmWidth;
		ptSize.y := bm.bmHeight;

		hdcBack   := CreateCompatibleDC(dc);
		hdcObject := CreateCompatibleDC(dc);
		hdcMem    := CreateCompatibleDC(dc);
		hdcSave   := CreateCompatibleDC(dc);

		bmAndBack   := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);

		bmAndObject := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);

		bmAndMem    := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);
		bmSave      := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);

		bmBackOld   := SelectObject(hdcBack, bmAndBack);
		bmObjectOld := SelectObject(hdcObject, bmAndObject);
		bmMemOld    := SelectObject(hdcMem, bmAndMem);
		bmSaveOld   := SelectObject(hdcSave, bmSave);

		SetMapMode(hdcTemp, GetMapMode(dc));

		BitBlt(hdcSave, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

		cColor := SetBkColor(hdcTemp, cTransparentColor);

		BitBlt(hdcObject, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

		SetBkColor(hdcTemp, cColor);

		BitBlt(hdcBack, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, NOTSRCCOPY);
    If not Stretched
      then BitBlt (hdcMem, 0, 0, ptSize.x, ptSize.y,
                   dc, xStart, yStart, SRCCOPY)
      else StretchBlt (hdcMem, 0, 0, ptSize.x, ptSize.y,
                       dc, xStart, yStart, xStop-xStart, yStop-yStart,SRCCOPY);
		BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, SRCAND);
		BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcBack, 0, 0, SRCAND);
		BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCPAINT);
    If not Stretched
      then BitBlt (dc, xStart, yStart, ptSize.x, ptSize.y,
                   hdcMem, 0, 0, SRCCOPY)
      else StretchBlt (dc, xStart, yStart, xStop-xStart, yStop-yStart,
                       hdcMem, 0, 0, ptSize.x, ptSize.y, SRCCOPY);
		BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcSave, 0, 0, SRCCOPY);

		DeleteObject(SelectObject(hdcBack, bmBackOld));
		DeleteObject(SelectObject(hdcObject, bmObjectOld));
		DeleteObject(SelectObject(hdcMem, bmMemOld));
		DeleteObject(SelectObject(hdcSave, bmSaveOld));

		SelectObject(hdcTemp, oldBmp);

		DeleteDC(hdcMem);
		DeleteDC(hdcBack);
		DeleteDC(hdcObject);
		DeleteDC(hdcSave);
		DeleteDC(hdcTemp);
		finally
			temp_bitmap.Free;
	end;
end;

{ tDbNav97 }

const
  BtnStateName: array[tNavGlyph] of PChar = ('EN', 'DI');
  BtnTypeName: array[tNavigateBtn] of PChar = ('FIRST', 'PRIOR', 'NEXT',
    'LAST', 'INSERT', 'DELETE', 'EDIT', 'POST', 'CANCEL', 'REFRESH');
  BtnHintId: array[tNavigateBtn] of Word = (SFirstRecord, SPriorRecord,
    SNextRecord, SLastRecord, SInsertRecord, SDeleteRecord, SEditRecord,
    SPostEdit, SCancelEdit, SRefreshRecord);

constructor tDbNav97.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle - [csFramed, csAcceptsControls, csSetCaption] +
    [csOpaque];
  FDataLink := tNavDatalink.Create(Self);
  FVisibleButtons := [nbFirst, nbPrior, nbNext, nbLast, nbInsert,
    nbDelete, nbEdit, nbPost, nbCancel, nbRefresh];
  FHints := TStringList.Create;
  FCaptions := TStringList.Create;
  FBackground := TBitmap.Create;
  InitButtons;
  BevelOuter := bvNone;
  BevelInner := bvNone;
  Width := 25;
  Height := 241;
  ButtonWidth := 0;
  ButtonHeight := 0;
  FocusedButton := nbFirst;
  FConfirmDelete := True;
end;

destructor tDbNav97.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  FBackground.Free;
  inherited Destroy;
end;

procedure tDbNav97.Loadbitmaps (Btn : tNavBtn97; BtnType : pChar);
var
  ResName: array[0..40] of Char;
begin
  Btn.Bitmap.Handle := LoadBitmap(HInstance,
      StrFmt(ResName, 'DBVN_%s_FCSD', [BtnType]));
  Btn.NoFocusBitmap.Handle := LoadBitmap(HInstance,
      StrFmt(ResName, 'DBVN_%s_NOFC', [BtnType]));
  Btn.DisabledBitmap.Handle := LoadBitmap(HInstance,
      StrFmt(ResName, 'DBVN_%s_DSBL', [BtnType]));
end;

procedure tDbNav97.UnLoadbitmaps (Btn : tNavBtn97);
begin
  Btn.Bitmap.ReleaseHandle;
  Btn.NoFocusBitmap.ReleaseHandle;
  Btn.DisabledBitmap.ReleaseHandle;
  Btn.Invalidate;
end;

procedure tDbNav97.InitButtons;
var
  I: tNavigateBtn;
  Btn: tNavBtn97;
  X: Integer;
begin
  MinBtnSize := Point(20, 18);
  X := 0;
  for I := Low(Buttons) to High(Buttons) do
  begin
    Btn := tNavBtn97.Create (Self);
    Btn.Index := I;
    Btn.Visible := I in FVisibleButtons;
    Btn.Enabled := True;
    Btn.SetBounds (X, 0, MinBtnSize.X, MinBtnSize.Y);
    LoadBitmaps (Btn, BtnTypeName[I]);
    Btn.Caption := '';
    Btn.Options := [boMonoDisplay, boIconOnly, boPopupMark, boShowBevel];
    Btn.OnClick := Click;
    Btn.OnMouseDown := BtnMouseDown;
    Btn.Parent := Self;
    Buttons[I] := Btn;
    X := X + MinBtnSize.X;
  end;
  InitHints;
  InitCaptions;
  Buttons[nbPrior].NavStyle := Buttons[nbPrior].NavStyle + [nsAllowTimer];
  Buttons[nbNext].NavStyle  := Buttons[nbNext].NavStyle + [nsAllowTimer];
end;

procedure tDbNav97.InitHints;
var
  I: Integer;
  J: tNavigateBtn;
begin
  for J := Low(Buttons) to High(Buttons) do
    Buttons[J].Hint := LoadStr (BtnHintId[J]);
  J := Low(Buttons);
  for I := 0 to (FHints.Count - 1) do
  begin
    if FHints.Strings[I] <> ''
      then Buttons[J].Hint := FHints.Strings[I];
    if J = High(Buttons)
      then Exit;
    Inc(J);
  end;
end;

procedure tDbNav97.InitCaptions;
var
  I: Integer;
  J: tNavigateBtn;
begin
  {default caption is button's hint. Why not?}
  for J := Low(Buttons) to High(Buttons) do
    Buttons[J].Caption := LoadStr (BtnHintId[J]);
  J := Low(Buttons);
  for I := 0 to (FCaptions.Count - 1) do
  begin
    if FCaptions.Strings[I] <> ''
      then Buttons[J].Caption := FCaptions.Strings[I];
    if J = High(Buttons)
      then Exit;
    Inc(J);
  end;
end;

procedure tDbNav97.SetHints(Value: TStrings);
begin
  FHints.Assign(Value);
  InitHints;
end;

procedure tDbNav97.SetCaptions(Value: TStrings);
begin
  FCaptions.Assign(Value);
  InitCaptions;
end;

procedure tDbNav97.FEnableCaptions(Value: Boolean);
var
  I: tNavigateBtn;
begin
  if Value then
    for I := Low(Buttons) to High(Buttons) do
      Buttons[I].Options := Buttons[I].Options - [boIconOnly]
  else
    for I := Low(Buttons) to High(Buttons) do
      Buttons[I].Options := Buttons[I].Options + [boIconOnly];
end;

procedure tDbNav97.FEnableBitmaps(Value: Boolean);
var
  I: tNavigateBtn;
begin
  if Value then
    for I := Low(Buttons) to High(Buttons) do
      LoadBitmaps (Buttons[I], BtnTypeName[I])
  else
    for I := Low(Buttons) to High(Buttons) do
      UnLoadBitmaps (Buttons[I]);
end;

procedure tDbNav97.SetCaptionStyle(Value : TExplorerButtonLayout);
var
  I: tNavigateBtn;
begin
  for I := Low(Buttons) to High(Buttons) do
    Buttons[I].Layout := Value;
  FCaptionStyle := Value;
end;

procedure tDbNav97.FSetBtnType(Value : tNavBtnType);
begin
  Case Value of
    btCaption : begin
                  FEnableBitmaps(false);
                  if FBtnType=btBitmap {caption inactive}
                    then FEnableCaptions(true);
                end;
    btBitmap  : begin
                  FEnableCaptions(false);
                  if FBtnType=btCaption {bitmap inactive}
                    then FEnableBitmaps(true);
                end;
    btBoth    : begin
                  if FBtnType=btCaption {bitmap inactive}
                    then FEnableBitmaps(true);
                  if FBtnType=btBitmap {caption inactive}
                    then FEnableCaptions(true);
                end;
  End;
  FBtnType := Value;
end;

procedure tDbNav97.FSetBackType(Value : tNavBackType);
begin
  FBackType := Value;
  Invalidate;
end;

procedure tDbNav97.FSetBackground(value: TBitmap);
begin
  FBackground.Assign(value);
  if not FBackground.Empty
    then FBackground.Dormant;
end;

procedure tDbNav97.FSetDirection(Value : tNavDirection);
var
  W, H: Integer;
begin
  W := Width;
  H := Height;
  FDirection := value;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height)
    then inherited SetBounds (Left, Top, W, H);
  Invalidate;
end;

procedure tDbNav97.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
     (AComponent = DataSource)
    then DataSource := nil;
end;

procedure tDbNav97.Paint;
begin
  inherited Paint;
  DrawTransparentBitmap (Canvas, FBackground,
                         Canvas.ClipRect.Left, Canvas.ClipRect.Top,
                         Canvas.ClipRect.Right, Canvas.ClipRect.Bottom,
                         ColorToRGB(FBackground.Canvas.Pixels[0, 0]),
                         FBackType=bkStretch);
end;

procedure tDbNav97.SetVisible(Value: TVertButtonSet);
var
  I: tNavigateBtn;
  W, H: Integer;
begin
  W := Width;
  H := Height;
  FVisibleButtons := Value;
  for I := Low(Buttons) to High(Buttons) do
    Buttons[I].Visible := I in FVisibleButtons;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height)
    then inherited SetBounds (Left, Top, W, H);
  Invalidate;
end;

procedure tDbNav97.AdjustSize (var W: Integer; var H: Integer);
var
  Count: Integer;
  MinW: Integer;
  MinH: Integer;
  I: tNavigateBtn;
  LastBtn: tNavigateBtn;
  Space, Temp, Remain: Integer;
  X: Integer;
  Y: Integer;
begin
  if (csLoading in ComponentState)
    then Exit;
  if Buttons[nbFirst] = nil
    then Exit;

  Count := 0;
  LastBtn := High(Buttons);
  for I := Low(Buttons) to High(Buttons) do
  begin
    if Buttons[I].Visible
    then begin
      Inc(Count);
      LastBtn := I;
    end;
  end;
  if Count = 0
    then Inc(Count);

  {horizontal}
  if ((FDirection=dirAuto) and (W >= H)) or (FDirection=dirHorizontal)
  then begin
    MinW := Count * (MinBtnSize.X);
    if W < MinW
      then W := MinW;
    if H < MinBtnSize.Y
      then H := MinBtnSize.Y;
    ButtonWidth := ((W) div Count);
    Temp := Count * (ButtonWidth);
    if Align = alNone
      then W := Temp;
    X := 0;
    Remain := W - Temp;
    Temp := Count div 2;
    for I := Low(Buttons) to High(Buttons) do
    begin
      if Buttons[I].Visible
      then begin
        Space := 0;
        if Remain <> 0
        then begin
          Dec (Temp, Remain);
          if Temp < 0
          then begin
            Inc (Temp, Count);
            Space := 1;
          end;
        end;
        Buttons[I].SetBounds (X, 0, ButtonWidth + Space, Height);
        Inc (X, ButtonWidth + Space);
        LastBtn := I;
      end
      else
        Buttons[I].SetBounds (Width, 0, ButtonWidth, Height);
    end;
  end {vertical: ((FDirection=dirAuto) and (W < H)) or (FDirection=dirVertical)}
  else begin
    MinH := Count * (MinBtnSize.Y);
    if H < MinH
      then H := MinH;
    if W < MinBtnSize.X
      then W := MinBtnSize.X;
    ButtonHeight := (H div Count);
    Temp := Count * (ButtonHeight);
    if Align = alNone
      then H := Temp;
    Y := 0;
    Remain := H - Temp;
    Temp := Count div 2;
    for I := Low(Buttons) to High(Buttons) do
    begin
      if Buttons[I].Visible
      then begin
        Space := 0;
        if Remain <> 0
        then begin
          Dec (Temp, Remain);
          if Temp < 0
          then begin
            Inc (Temp, Count);
            Space := 1;
          end;
        end;
        Buttons[I].SetBounds (0, Y, Width, ButtonHeight + Space);
        Inc (Y, ButtonHeight + Space);
        LastBtn := I;
      end
      else
        Buttons[I].SetBounds (0, Height, ButtonHeight, Width);
      end;
    end;
end;

procedure tDbNav97.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  W, H: Integer;
begin
  W := AWidth;
  H := AHeight;
  AdjustSize (W, H);
  inherited SetBounds (ALeft, ATop, W, H);
end;

procedure tDbNav97.WMSize(var Message: TWMSize);
var
  W, H: Integer;
begin
  inherited;
  { check for minimum size }
  W := Width;
  H := Height;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height)
    then inherited SetBounds(Left, Top, W, H);
  Message.Result := 0;
end;

procedure tDbNav97.Click(Sender: TObject);
begin
  BtnClick (tNavBtn97 (Sender).Index);
end;

procedure tDbNav97.BtnMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  OldFocus: tNavigateBtn;
  Form: TForm;
begin
  OldFocus := FocusedButton;
  FocusedButton := tNavBtn97 (Sender).Index;
  if TabStop and (GetFocus <> Handle) and CanFocus
  then begin
    SetFocus;
    if (GetFocus <> Handle)
      then Exit;
  end
  else if TabStop and (GetFocus = Handle) and (OldFocus <> FocusedButton)
  then begin
    Buttons[OldFocus].Invalidate;
    Buttons[FocusedButton].Invalidate;
  end;
end;

procedure tDbNav97.BtnClick(Index: tNavigateBtn);
begin
  if (DataSource <> nil) and (DataSource.State <> dsInactive)
  then begin
    with DataSource.DataSet do
    begin
      case Index of
        nbPrior   : Prior;
        nbNext    : Next;
        nbFirst   : First;
        nbLast    : Last;
        nbInsert  : Insert;
        nbEdit    : Edit;
        nbCancel  : Cancel;
        nbPost    : Post;
        nbRefresh : Refresh;
        nbDelete  : begin
                      if not FConfirmDelete
                         or (MessageDlg (LoadStr(SDeleteRecordQuestion),
                             mtConfirmation, mbOKCancel, 0) <> idCancel)
                        then Delete;
                    end;
      end;
    end;
  end;
  if not (csDesigning in ComponentState) and Assigned(FOnNavClick)
    then FOnNavClick(Self, Index);
end;

procedure tDbNav97.WMSetFocus(var Message: TWMSetFocus);
begin
  Buttons[FocusedButton].Invalidate;
end;

procedure tDbNav97.WMKillFocus(var Message: TWMKillFocus);
begin
  Buttons[FocusedButton].Invalidate;
end;

procedure tDbNav97.KeyDown(var Key: Word; Shift: TShiftState);
var
  NewFocus: tNavigateBtn;
  OldFocus: tNavigateBtn;
begin
  OldFocus := FocusedButton;
  case Key of
    VK_RIGHT: begin
                NewFocus := FocusedButton;
                repeat
                  if NewFocus < High(Buttons)
                    then NewFocus := Succ(NewFocus);
                until (NewFocus = High(Buttons)) or (Buttons[NewFocus].Visible);
                if NewFocus <> FocusedButton
                then begin
                  FocusedButton := NewFocus;
                  Buttons[OldFocus].Invalidate;
                  Buttons[FocusedButton].Invalidate;
                end;
              end;
    VK_LEFT : begin
                NewFocus := FocusedButton;
                repeat
                  if NewFocus > Low(Buttons)
                    then NewFocus := Pred(NewFocus);
                until (NewFocus = Low(Buttons)) or (Buttons[NewFocus].Visible);
                if NewFocus <> FocusedButton
                then begin
                  FocusedButton := NewFocus;
                  Buttons[OldFocus].Invalidate;
                  Buttons[FocusedButton].Invalidate;
                end;
              end;
    VK_SPACE: begin
                if Buttons[FocusedButton].Enabled
                  then Buttons[FocusedButton].Click;
              end;
  end;
end;

procedure tDbNav97.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure tDbNav97.DataChanged;
var
  UpEnable, DnEnable: Boolean;
begin
  UpEnable := Enabled and FDataLink.Active and not FDataLink.DataSet.BOF;
  DnEnable := Enabled and FDataLink.Active and not FDataLink.DataSet.EOF;
  Buttons[nbFirst].Enabled := UpEnable;
  Buttons[nbPrior].Enabled := UpEnable;
  Buttons[nbNext].Enabled := DnEnable;
  Buttons[nbLast].Enabled := DnEnable;
  Buttons[nbDelete].Enabled := Enabled and FDataLink.Active
    and FDataLink.DataSet.CanModify
    and not (FDataLink.DataSet.BOF and FDataLink.DataSet.EOF);
end;

procedure tDbNav97.EditingChanged;
var
  CanModify: Boolean;
begin
  CanModify := Enabled and FDataLink.Active and FDataLink.DataSet.CanModify;
  Buttons[nbInsert].Enabled := CanModify;
  Buttons[nbEdit].Enabled := CanModify and not FDataLink.Editing;
  Buttons[nbPost].Enabled := CanModify and FDataLink.Editing;
  Buttons[nbCancel].Enabled := CanModify and FDataLink.Editing;
  Buttons[nbRefresh].Enabled := not (FDataLink.DataSet is TQuery);
end;

procedure tDbNav97.ActiveChanged;
var
  I: tNavigateBtn;
begin
  if not (Enabled and FDataLink.Active)
  then for I := Low(Buttons) to High(Buttons) do
         Buttons[I].Enabled := False
  else begin
    DataChanged;
    EditingChanged;
  end;
end;

procedure tDbNav97.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  if not (csLoading in ComponentState)
    then ActiveChanged;
end;

procedure tDbNav97.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if not (csLoading in ComponentState)
    then ActiveChanged;
end;

function tDbNav97.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure tDbNav97.Loaded;
var
  W, H: Integer;
begin
  inherited Loaded;
  W := Width;
  H := Height;
  AdjustSize (W, H);
  if (W <> Width) or (H <> Height)
    then inherited SetBounds (Left, Top, W, H);
  InitHints;
  InitCaptions;
  ActiveChanged;
end;

{tNavBtn97}

destructor tNavBtn97.Destroy;
begin
  if FRepeatTimer <> nil
    then FRepeatTimer.Free;
  inherited Destroy;
end;

procedure tNavBtn97.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown (Button, Shift, X, Y);
  if nsAllowTimer in FNavStyle
  then begin
    if FRepeatTimer = nil
      then FRepeatTimer := TTimer.Create(Self);
    FRepeatTimer.OnTimer := TimerExpired;
    FRepeatTimer.Interval := InitRepeatPause;
    FRepeatTimer.Enabled  := True;
  end;
end;

procedure tNavBtn97.MouseUp(Button: TMouseButton; Shift: TShiftState;
                                 X, Y: Integer);
begin
  inherited MouseUp (Button, Shift, X, Y);
  if FRepeatTimer <> nil
    then FRepeatTimer.Enabled  := False;
end;

procedure tNavBtn97.TimerExpired(Sender: TObject);
begin
  FRepeatTimer.Interval := RepeatPause;
  if (Down) and MouseCapture
  then begin
    try
      Click;
    except
      FRepeatTimer.Enabled := False;
      raise;
    end;
  end;
end;

procedure tNavBtn97.Paint;
var
  R: TRect;
begin
  inherited Paint;
  if (GetFocus = Parent.Handle) and
     (FIndex = tDbNav97 (Parent).FocusedButton)
  then begin
    R := Bounds(0, 0, Width, Height);
    InflateRect(R, -3, -3);
    if Down
      then OffsetRect(R, 1, 1);
    DrawFocusRect(Canvas.Handle, R);
  end;
end;

{ tNavDatalink }

constructor tNavDatalink.Create(ANav: tDbNav97);
begin
  inherited Create;
  FNavigator := ANav;
end;

destructor tNavDatalink.Destroy;
begin
  FNavigator := nil;
  inherited Destroy;
end;

procedure tNavDatalink.EditingChanged;
begin
  if FNavigator <> nil
    then FNavigator.EditingChanged;
end;

procedure tNavDatalink.DataSetChanged;
begin
  if FNavigator <> nil
    then FNavigator.DataChanged;
end;

procedure tNavDatalink.ActiveChanged;
begin
  if FNavigator <> nil
    then FNavigator.ActiveChanged;
end;

end.
