unit DelphStb;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, ExptIntf, StdCtrls, DelphPxy
{$IFDEF VER100}
  , LibIntf, DsgnIntf, Windows
{$ELSE}
  , LibMain
{$ENDIF};

const
  WM_SHOWCLASSHELP = WM_USER + 0;

type
  TDelphiStub = class(TForm)
    UnitsLbx: TListBox;
    procedure FormCreate(Sender: TObject);
  protected
    procedure BringToForeGround;
    procedure SetStubName( var Msg: TWMSetText);
    procedure WMShowClassHelp( var Msg: TMessage); message WM_SHOWCLASSHELP;
    procedure WMSetText( var Msg: TWMSetText); message WM_SETTEXT;
  public
    { Public declarations }
    procedure ReloadFile( Name: PChar);
    procedure CheckSyntax;
  end;

var
  DelphiStub: TDelphiStub;

procedure Register;

implementation

{$IFNDEF WIN32}
uses
  MMIntW32;
{$ENDIF}

{$R *.DFM}

var HelpTopic: string;

procedure Register;
begin
{$IFDEF VER100}
  DelphiStub := TDelphiStub.Create( Application);
{$ELSE}
  DelphiStub := TDelphiStub.Create(nil);
{$ENDIF}
end;

procedure _v80finalization; far;
begin
  DelphiStub.Free;
end;

procedure TDelphiStub.BringToForeGround;
var
  HW: HWnd;
begin
  if ToolServices <> nil then
  begin
    HW := ToolServices.GetParentHandle;
    {$IFDEF WIN32}
    SetForeGroundWindow(HW);
    {$ELSE}
    if IsWindowsNT then SetForegroundWindow( Application.Handle);
    {$ENDIF}
  end;
end;

procedure TDelphiStub.SetStubName( var Msg: TWMSetText);
var
  M: TWMSetText;
begin
  M.Msg := Msg.Msg;
  M.Unused :=  Msg.Unused;
  M.Text := Msg.Text+2;
  M.Result := Msg.Result;
  DefaultHandler(M);
  Msg.Result := M.Result;
end;

procedure TDelphiStub.CheckSyntax;
var
  HW: HWnd;
begin
  BringToForeGround;
  HW := FindWindow('TAppBuilder', nil);
  {$IFDEF VER100}
  if HW <> 0 then PostMessage(HW, WM_COMMAND, $57, 0);
  {$ENDIF}
  {$IFDEF VER90}
  if HW <> 0 then PostMessage(HW, WM_COMMAND, 84, 0);
  {$ENDIF}
  {$IFDEF VER80}
  if HW <> 0 then PostMessage(HW, WM_COMMAND, 78, 0);
  {$ENDIF}
end;

procedure TDelphiStub.WMShowClassHelp( var Msg: TMessage);
begin
  BringToForeGround;
  {$IFDEF VER100}
  if DelphiIDE <> nil then DelphiIDE.ShowClassHelp(HelpTopic);
  {$ELSE}
  if AppBuilder <> nil then AppBuilder.ShowClassHelp(HelpTopic);
  {$ENDIF}
end;

{
TWMSetText = record
    Msg: Cardinal;
    Unused: Integer;
    Text: PChar;
    Result: Longint;
  end;
}
procedure TDelphiStub.WMSetText( var Msg: TWMSetText);
begin
  if (StrLen(Msg.Text) >= 2) and (Msg.Text[1] = '@') then
  begin
    case Msg.Text[0] of
      STUB_NAME       : SetStubName(Msg);
      RELOAD_FILE     : ReloadFile(@Msg.Text[2]);
      SYNTAX_CHECK    : CheckSyntax;
      SHOW_CLASS_HELP : begin
                          { Decouple using a PostMessage here }
                          HelpTopic := StrPas(@Msg.Text[2]);
                          PostMessage(Handle, WM_SHOWCLASSHELP, 0, 0);
                        end;
    end;
  {$IFDEF DEBUG}
    { log received commands }
    UnitsLbx.Items.Add(StrPas(CmdBuffer));
  {$ENDIF}
  end;
end;

procedure TDelphiStub.FormCreate(Sender: TObject);
begin
  {$IFDEF VER100}
  SendCommand( '', STUB_NAME, DELPHI_30);
  {$ENDIF}
  {$IFDEF VER90}
  SendCommand( '', STUB_NAME, DELPHI_20);
  {$ENDIF}
  {$IFDEF VER80}
  SendCommand( '', STUB_NAME, DELPHI_10);
  {$ENDIF}
end;

procedure TDelphiStub.ReloadFile( Name: PChar);
  function LongFileName(FileName: PChar): string;
  begin
    Result := StrPas( FileName);
  end;
begin
  if ToolServices <> nil then
    ToolServices.ReloadFile(LongFileName(Name));
end;

{$IFDEF VER80}
initialization
  AddExitProc(_v80finalization);
{$ENDIF}
{$IFDEF VER90}
initialization
finalization
  DelphiStub.Free;
{$ENDIF}
{ for ver100 application is the owner of the stub }
end.
