{*********************************************************}
{                                                         }
{     TExcel Component Demo 2.3 for Borland Delphi        }
{                                                         }
{     Copyright (c) 1996, 1997 by                         }
{                                                         }
{       Stefan Hoffmeister           (Advanced version)   }
{       Stefan.Hoffmeister@Uni-Passau.de                  }
{   and                                                   }
{       Tibor F. Liska               (Basic version)      }
{       Tel/Fax:    00-36-1-165-2019                      }
{       Office:     00-36-1-269-8284                      }
{       E-mail: liska@sztaki.hu                           }
{                                                         }
{*********************************************************}
unit DemoXL1;

{.$DEFINE UltraSpeed}

interface

uses
  {$IFDEF Win32} ComCtrls, {$ENDIF}

  WinTypes, WinProcs,

  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Spin, Buttons, TabNotBk, DdeMan, Excels;

type
  TForm1 = class(TForm)
    cmClose: TBitBtn;
    Timer1: TTimer;
    TabbedNotebook1: TTabbedNotebook;
    cmTable: TBitBtn;
    Label1: TLabel;
    SpinEdit1: TSpinEdit;
    SpinEdit2: TSpinEdit;
    Label2: TLabel;
    Label3: TLabel;
    Edit1: TEdit;
    SpinEdit5: TSpinEdit;
    SpinEdit4: TSpinEdit;
    SpinEdit3: TSpinEdit;
    cmRequest: TBitBtn;
    cmRun: TBitBtn;
    ComboBox3: TComboBox;
    Label4: TLabel;
    cmCommand: TBitBtn;
    ComboBox1: TComboBox;
    Label6: TLabel;
    RequestMemo: TMemo;
    ComboBox4: TComboBox;
    Label7: TLabel;
    SpecialRequestEdit: TEdit;
    cmRequestSelection: TBitBtn;
    SelectionLabel: TLabel;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    cmPutData: TBitBtn;
    RadioGroup1: TRadioGroup;
    Edit2: TEdit;
    cmRequestData: TBitBtn;
    reqDataLabel: TLabel;
    Label8: TLabel;
    cmListSelection: TBitBtn;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    cmHighSet: TButton;
    cmHighGet: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Timer1Timer(Sender: TObject);
    procedure ExcelOpen(Sender: TObject);
    procedure ExcelClose(Sender: TObject);
    procedure cmCloseClick(Sender: TObject);
    procedure cmTableClick(Sender: TObject);
    procedure cmCommandClick(Sender: TObject);
    procedure cmRequestClick(Sender: TObject);
    procedure cmRunClick(Sender: TObject);
    procedure cmRequestSelectionClick(Sender: TObject);
    procedure ComboBox4Change(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure cmPutDataClick(Sender: TObject);
    procedure cmRequestDataClick(Sender: TObject);
    procedure cmListSelectionClick(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
    procedure CheckBox4Click(Sender: TObject);
    procedure cmHighSetClick(Sender: TObject);
    procedure cmHighGetClick(Sender: TObject);
  private
    MacroFile : TFileName;
{.$DEFINE INSTALLED}
    { remove this define if you have installed TExcel in your
      component library }

{$IFNDEF INSTALLED}
    Excel: TAdvExcel;
{$ENDIF}
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  DemoXL2; { a form used to display rectangular selections
              retrieved from Excel }

procedure TForm1.FormCreate(Sender: TObject);
begin
  TabbedNotebook1.ActivePage := 'Create+Store';

{$IFNDEF INSTALLED}
  Excel := TAdvExcel.Create(Self);
{$ENDIF}

  ComboBox1.ItemIndex := -1;
  ComboBox3.ItemIndex := -1;
  ComboBox4.ItemIndex := -1;

  RadioGroup1.ItemIndex := ord(Excel.MoveActiveCell);

  MacroFile := ExtractFilePath(ParamStr(0))+'DemoXL.xls';
  if not FileExists(Macrofile) then
    MacroFile := '';

  Timer1.Enabled := True;          { Delayed Connect }
  Screen.Cursor := crHourGlass;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Excel.CloseMacroFile;

  { don't want to have any notifications any longer }
  Excel.OnClose := nil;
  Excel.Disconnect;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  ExcelPath : TFileName;
begin
  Timer1.Enabled := False;
  try
    Excel.OnOpen  := ExcelOpen;
    Excel.OnClose := ExcelClose;
    try
      Excel.Connect;   { Same as Excel.Connected := True; }
    except
      ExcelPath := '';
      if InputQuery('Excel not found', 'Specify Excel path',
                                        ExcelPath) then
      begin
        Excel.ExeName := ExcelPath;
        Excel.Connect;
      end
      else
        raise;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.ExcelOpen(Sender: TObject);
begin
  cmTable  .Enabled := True;
  cmCommand.Enabled := True;
  cmRun    .Enabled := MacroFile <> '';
  cmRequest.Enabled := True;
  cmRequestSelection.Enabled := true;
  cmListSelection.Enabled := true;
  cmRequestData.Enabled := true;
  cmPutData.Enabled := True;

  cmHighSet.Enabled := true;
  cmHighGet.Enabled := true;
  { ShowMessage('Excel connection opened'); }
end;

procedure TForm1.ExcelClose(Sender: TObject);
begin
  cmTable  .Enabled := False;
  cmCommand.Enabled := False;
  cmRun    .Enabled := False;
  cmRequest.Enabled := False;
  cmRequestSelection.Enabled := False;
  cmListSelection.Enabled := False;
  cmRequestData.Enabled := False;
  cmPutData.Enabled := False;

  cmHighSet.Enabled := false;
  cmHighGet.Enabled := false;
  { ShowMessage('Excel connection closed'); }
end;

procedure TForm1.cmCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.cmTableClick(Sender: TObject);
begin
  Excel.StartTable;        { Create new workbook }
  Excel.PutStrAt(SpinEdit1.Value, SpinEdit2.Value, Edit1.Text);
  Excel.PutIntAt(SpinEdit3.Value, SpinEdit4.Value, SpinEdit5.Value);
  Excel.EndTable;          { Show excel table }
end;

procedure TForm1.cmCommandClick(Sender: TObject);
begin
  Excel.Exec(ComboBox1.Text);
end;

procedure TForm1.cmRequestClick(Sender: TObject);
begin
  RequestMemo.Clear;
  RequestMemo.Lines.Add(Excel.Request(SpecialRequestEdit.Text));
end;

procedure TForm1.cmRunClick(Sender: TObject);
begin
  Excel.OpenMacroFile(MacroFile);
  Excel.RunMacro(ComboBox3.Text);
end;

procedure TForm1.cmRequestSelectionClick(Sender: TObject);
begin
  SelectionLabel.Caption := Excel.Selection;
end;

procedure TForm1.ComboBox4Change(Sender: TObject);
  var
    Result : TStringList;
begin
  RequestMemo.Clear;
  case ComboBox4.ItemIndex of
    0 : Result := Excel.Topics;
    1 : Result := Excel.Protocols;
    2 : Result := Excel.Formats;
    3 : Result := Excel.EditEnvItems;
  else
        Result := nil;
  end;
  if Result <> nil then
    RequestMemo.Lines.Assign(Result);
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  with Excel do
    if CheckBox1.Checked then
      EchoOn
    else
      EchoOff;
end;

procedure TForm1.CheckBox2Click(Sender: TObject);
begin
  with Excel do
    if CheckBox2.Checked then
      EnableInput
    else
      DisableInput;
end;

procedure TForm1.CheckBox3Click(Sender: TObject);
begin
  Excel.FilterReturnedData := CheckBox3.Checked;
end;

procedure TForm1.CheckBox4Click(Sender: TObject);
begin
  Excel.StripCRLF := CheckBox4.Checked;
end;


procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  Excel.MoveActiveCell := TCellDir(RadioGroup1.ItemIndex);
end;

procedure TForm1.cmPutDataClick(Sender: TObject);
begin
  Excel.PutData( [Edit2.Text] );
(*
  { Extended examples of usage: }
  Excel.PutDataAt(0, 0, [1, 2.2, 'Hello!', '', 1], dirRight);
*)
end;

procedure TForm1.cmRequestDataClick(Sender: TObject);
var
  TopLeft,
  BottomRight: TExcelCell;

  i,j: integer;
begin
  reqDataLabel.Caption := Excel.GetData;
  { retrieve single cell at selection }

(*
  { Extended example of usage -
        gets data in call Row 10, Column 20 in the CURRENT sheet }

  reqDataLabel.Caption := Excel.GetDataAt(10, 20);
*)

 with TArrayForm.Create(Self) do
 try

   Label2.Caption := Excel.GetRectSelection(TopLeft, BottomRight);

   if (TopLeft.Col <> BottomRight.Col) or
      (TopLeft.Row <> BottomRight.Row) then
   begin
     { we do have an array selection }

     { adapt grid to selection }
     StringGrid1.ColCount := BottomRight.Col - TopLeft.Col + 1;
     StringGrid1.RowCount := BottomRight.Row - TopLeft.Row + 1;

     { fill grid }
     for i := TopLeft.Col to BottomRight.Col do
       for j := TopLeft.Row to BottomRight.Row do
         StringGrid1.Cells[i-TopLeft.Col, j-TopLeft.Row] :=
           { speed matters up - we tell the component which File+Sheet we
             want - this has been stored in Label2.Caption previously }
           Excel.GetDataAtFileSheet(j, i, Label2.Caption);

     { show form }
     ShowModal;
   end;
 finally
   Release;
 end;
end;

procedure TForm1.cmListSelectionClick(Sender: TObject);
var
  counter: integer;
begin
  Excel.RetrieveSelection;

  RequestMemo.Clear;

  RequestMemo.Lines.Add('Parsed selection:');
  for counter := 0 to Excel.SelectionList.Count-1 do
    with PExcelSelectionArray(Excel.SelectionList.Objects[counter])^ do

      RequestMemo.Lines.Add(  Format('%s: %s%d%s%d:%s%d%s%d',
                              [ Excel.SelectionList[counter],

                                Excel.ColChar, TopLeft.Col,
                                Excel.RowChar, TopLeft.Row,

                                Excel.ColChar, BottomRight.Col,
                                Excel.RowChar, BottomRight.Row
                                 ]) );
end;


procedure TForm1.cmHighSetClick(Sender: TObject);
{$IFNDEF UltraSpeed}
begin
  ShowMessage( 'Please contact Stefan Hoffmeister for the high'#13 +
               'speed DDE code, that will allow you to move 1000 elements'#13+
               'in less than 150 milliseconds.'#13#13+
               'eMail: Stefan.Hoffmeister@Uni-Passau.de');
end;
{$ELSE}
var
  i: integer;
  x : TSetXlTableData;

  t: longint;
begin
  t := GetTickCount;

  Excel.Select(1, 1);
  { create 1000 rows * 1 column table }
  x := TSetXlTableData.Create(1000, 1);
  try

    { ...add some data }
    x.AddBool(true);
    x.AddBlank; { empties a cell }
    x.AddBool(false);
    x.AddInt(123);

    for i := 1 to 498 do
    begin
      x.AddFloat(1.2345678);
      x.AddString(FloatToStr(Random));
    end;

    { ...and write the data into Excel }
    with Excel do
      SetXlTable(x);
  finally
    x.Free;
  end;

  ShowMessage(Format('Operation took %d ms', [GetTickCount-t]));
end;
{$ENDIF}

procedure TForm1.cmHighGetClick(Sender: TObject);
{$IFNDEF UltraSpeed}
begin
  ShowMessage( 'Please contact Stefan Hoffmeister for the high'#13 +
               'speed DDE code, that will allow you to move 1000 elements'#13+
               'in less than 150 milliseconds.'#13#13+
               'eMail: Stefan.Hoffmeister@Uni-Passau.de');
end;
{$ELSE}

  procedure ShowMessage(const s: string);
  begin
    { just a dummy placeholder };
  end;

var
  x : TGetXlTableData;
  i : integer;

  t: longint;
const
  BoolString : array[boolean] of string[5] = ('false', 'true');
begin
  t := GetTickCount;

  { Get a 3*3 selection at row 1, col 1}
  x := Excel.GetXlTableAt(1, 1, 1000, 5);

  { Excel may return less data than requested, for instance if the
    whole table returned is empty (all cells are xlDataBlank) }
  for i := 1 to x.RowCount * x.ColCount do
  begin

    { read through the stream of data and take appropriate action }
    case x.NextType of
      xlDataBlank : begin ShowMessage('- Blank cell -'); x.Skip; end;

      xlDataFloat : ShowMessage(FloatToStr(x.GetFloat));

      xlDataString: ShowMessage(x.GetString);

      xlDataBool  : ShowMessage(BoolString[x.GetBool]);

      xlDataError : begin ShowMessage('An error value'); { Z:= } x.GetError end;

      xlDataInt   : ShowMessage(IntToStr(x.GetInt));

      xlDataNone:   break; { no more data }
    else
      ShowMessage('Error: cannot handle this'); { should not happen }
      x.Skip;
    end;
  end;
  x.Free;

  Dialogs.ShowMessage(Format('Operation took %d ms', [GetTickCount-t]));
end;
{$ENDIF}

end.
