{
  Author:	Fredrik Nordbakke - FNProgramvare  1997
  E-mail:	fredrik.nordbakke@ostfoldnett.no
  WWW:	  http://www.prodat.no/fnp/delphi.html

  Special thanks to T. van Noppen.
}

unit FnpFormEnh;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, MessageGrabber;

type
  TRootKey = (rkCurrentUser, rkLocalMachine);

  TFnpFormEnh = class(TMessageGrabber)
  private
    { Private declarations }
    FAllowMaximized: Boolean;
    FAllowMinimized: Boolean;
    FCheckPos: Boolean;
    FMinSizeX: Integer;
    FMinSizeY: Integer;
    FMaxSizeX: Integer;
    FMaxSizeY: Integer;
    FRootKey: TRootKey;
    FSaveSize: Boolean;
    FSavePosition: Boolean;
    FSubKey: String;
    FVersion: String;
    BkupOwner: TForm;
    Wp: TWINDOWPLACEMENT;
    procedure SetAllowMaximized(Value: Boolean);
    procedure SetAllowMinimized(Value: Boolean);
    procedure SetCheckPos(Value: Boolean);
    procedure SetMinSizeX(Value: Integer);
    procedure SetMinSizeY(Value: Integer);
    procedure SetMaxSizeX(Value: Integer);
    procedure SetMaxSizeY(Value: Integer);
    procedure SetRootKey(Value: TRootKey);
    procedure SetSavePosition(Value: Boolean);
    procedure SetSaveSize(Value: Boolean);
    procedure SetSubKey(Value: String);
    procedure SetVersion(Value: String);
    procedure DoReadPos;
    procedure DoSavePos;
  protected
    { Protected declarations }
		procedure WMGetMinMaxInfo(var Msg: TWMGetMinMaxInfo); message WM_GetMinMaxInfo;
    procedure WMSize(var Msg: TWMSize); message WM_Size;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
  published
    { Published declarations }
    property AllowMaximized: Boolean read FAllowMaximized write SetAllowMaximized default True;
    property AllowMinimized: Boolean read FAllowMinimized write SetAllowMinimized default True;
    property CheckPos: Boolean read FCheckPos write SetCheckPos default False;
    property MinSizeX: Integer read FMinSizeX write SetMinSizeX default 0;
    property MinSizeY: Integer read FMinSizeY write SetMinSizeY default 0;
    property MaxSizeX: Integer read FMaxSizeX write SetMaxSizeX default 0;
    property MaxSizeY: Integer read FMaxSizeY write SetMaxSizeY default 0;
    property RootKey: TRootKey read FRootKey write SetRootKey default rkCurrentUser;
    property SavePosition: Boolean read FSavePosition write SetSavePosition default False;
    property SaveSize: Boolean read FSaveSize write SetSaveSize default False;
    property SubKey: String read FSubKey write SetSubKey;
    property Version: String read FVersion write SetVersion stored False;
  end;

procedure Register;

implementation

uses Registry;

constructor TFnpFormEnh.Create(AOwner: TComponent);
var
  X: Integer;
begin
  if not (AOwner is TForm) then
	  raise Exception.Create('Owner must be a form');
  for X := 0 to AOwner.ComponentCount - 1 do
    if AOwner.Components[X] is TFnpFormEnh then
      raise Exception.Create('This form already contains a TFnpFormEnh component');
  inherited Create(AOwner);
  FAllowMaximized := True;
  FAllowMinimized := True;
  FCheckPos := False;
  FMinSizeX := 0;
  FMinSizeY := 0;
  FMaxSizeX := 0;
  FMaxSizeY := 0;
  FSavePosition := False;
  FSaveSize := False;
  FRootKey := rkCurrentUser;
  FVersion := '1.01.00';
  { Need to store the owner since we need it in DoSavePos called from Destory }
  BkupOwner := (AOwner as TForm);
  WP.Length := SizeOf(WP);
end;

destructor TFnpFormEnh.Destroy;
begin
  DoSavePos;
  inherited Destroy;
end;

procedure TFnpFormEnh.WMGetMinMaxInfo(var Msg: TWMGetMinMaxInfo);
begin
  if (Owner as TForm).BorderStyle = bsSizeable then
  begin
    if FMinSizeX > 0 then
      Msg.MinMaxInfo^.ptMinTrackSize.X := FMinSizeX;
    if FMinSizeY > 0 then
      Msg.MinMaxInfo^.ptMinTrackSize.Y := FMinSizeY;
    if FMaxSizeX > 0 then
      Msg.MinMaxInfo^.ptMaxTrackSize.X := FMaxSizeX;
    if FMaxSizeY > 0 then
      Msg.MinMaxInfo^.ptMaxTrackSize.Y := FMaxSizeY;
  end;
  inherited;
end;

procedure TFnpFormEnh.WMSize(var Msg: TWMSize);
begin
  inherited;
  { We need to store the form's size this way in case WindowState =
    wsMinimized or wsMaximized. We need the values returned by
    GetWindowPlacement to make it possible to restore the form's pos/size
    back to normal next time the form is shown. It is not possible to use
    GetWindowPlacement in TFnpFormEnh.Destory because the form's window
    handle is then already destroyed. }
  GetWindowPlacement((Owner as TForm).Handle, @WP);
end;

procedure TFnpFormEnh.Loaded;
begin
  inherited Loaded;
  DoReadPos;
end;

procedure TFnpFormEnh.SetAllowMaximized(Value: Boolean);
begin
  if FAllowMaximized <> Value then
    FAllowMaximized := Value;
end;

procedure TFnpFormEnh.SetAllowMinimized(Value: Boolean);
begin
  if FAllowMinimized <> Value then
    FAllowMinimized := Value;
end;

procedure TFnpFormEnh.SetCheckPos(Value: Boolean);
begin
  if FCheckPos <> Value then
    FCheckPos := Value;
end;

procedure TFnpFormEnh.SetMinSizeX(Value: Integer);
begin
  if FMinSizeX <> Value then
    if Value < 0 then
      FMinSizeX := 0
    else
      FMinSizeX := Value;
end;

procedure TFnpFormEnh.SetMinSizeY(Value: Integer);
begin
  if FMinSizeY <> Value then
    if Value < 0 then
      FMinSizeY := 0
    else
      FMinSizeY := Value;
end;

procedure TFnpFormEnh.SetMaxSizeX(Value: Integer);
begin
  if FMaxSizeX <> Value then
    if Value < 0 then
      FMaxSizeX := 0
    else
      FMaxSizeX := Value;
end;

procedure TFnpFormEnh.SetMaxSizeY(Value: Integer);
begin
  if FMaxSizeY <> Value then
    if Value < 0 then
      FMaxSizeY := 0
    else
      FMaxSizeY := Value;
end;

procedure TFnpFormEnh.SetRootKey(Value: TRootKey);
begin
  if FRootKey <> Value then
    FRootKey := Value;
end;

procedure TFnpFormEnh.SetSavePosition(Value: Boolean);
begin
  if FSavePosition <> Value then
    FSavePosition := Value;
end;

procedure TFnpFormEnh.SetSaveSize(Value: Boolean);
begin
  if FSaveSize <> Value then
    FSaveSize := Value;
end;

procedure TFnpFormEnh.SetSubKey(Value: String);
begin
  if FSubKey <> Value then
    FSubKey := Value;
end;

procedure TFnpFormEnh.SetVersion(Value: String);
begin
  { This property is read only! }
end;

procedure TFnpFormEnh.DoReadPos;
var
  Reg: TRegIniFile;
  WinState: TWindowState;
  X: Integer;
  CR: TRect;
begin
  if csDesigning in ComponentState then
    exit;
  if (FSavePosition or FSaveSize) and (Length(FSubKey) > 0) then begin
    Reg := TRegIniFile.Create(FSubKey);
    try
      if FRootKey = rkCurrentUser then
        Reg.RootKey := HKEY_CURRENT_USER
      else
        Reg.RootKey := HKEY_LOCAL_MACHINE;
      { Initiate Wp }
      GetWindowPlacement((Owner as TForm).Handle, @WP);
      if FSavePosition then begin
        X := Reg.ReadInteger((Owner as TForm).Name, 'Left', (Owner as TForm).Left);
        WP.rcNormalPosition.Right := WP.rcNormalPosition.Right - ((Owner as TForm).Left - X);
        WP.rcNormalPosition.Left := X;
        X := Reg.ReadInteger((Owner as TForm).Name, 'Top', (Owner as TForm).Top);
        WP.rcNormalPosition.Bottom := WP.rcNormalPosition.Bottom - ((Owner as TForm).Top - X);
        WP.rcNormalPosition.Top := X;
      end;
      if FSaveSize then begin
        WinState := TWindowState(Reg.ReadInteger((Owner as TForm).Name, 'WindowState',
            LongInt(wsNormal)));
        if not FAllowMaximized and (WinState = wsMaximized) then
          WinState := wsNormal
        else if not FAllowMinimized and (WinState = wsMinimized) then
          WinState := wsNormal;
        WP.rcNormalPosition.Right := WP.rcNormalPosition.Left +
            Reg.ReadInteger((Owner as TForm).Name, 'Width', (Owner as TForm).Width);
        WP.rcNormalPosition.Bottom := WP.rcNormalPosition.Top +
            Reg.ReadInteger((Owner as TForm).Name, 'Height', (Owner as TForm).Height);
        case WinState of
          wsMinimized:
            WP.ShowCmd := SW_SHOWMINIMIZED;
          wsMaximized:
            WP.ShowCmd := SW_SHOWMAXIMIZED;
          else
            WP.ShowCmd := SW_SHOWNORMAL;
        end;
      end;
      if (Owner as TForm).Visible = False then    // added by
        WP.ShowCmd := SW_HIDE;                    // T. van Noppen
      SetWindowPlacement((Owner as TForm).Handle, @WP);
      if (FCheckPos) and (WP.ShowCmd = SW_SHOWNORMAL) then begin
        if (Owner as TForm).FormStyle = fsMDIChild then begin
          CR := ((Owner as TForm).Owner as TForm).ClientRect;
          if (Owner as TForm).Left + (Owner as TForm).Width > CR.Right then
            (Owner as TForm).Left := CR.Right - (Owner as TForm).Width;
          if (Owner as TForm).Top + (Owner as TForm).Height > CR.Bottom then
            (Owner as TForm).Top := CR.Bottom - (Owner as TForm).Height;
        end else begin
          if (Owner as TForm).Left + (Owner as TForm).Width > Screen.Width then
            (Owner as TForm).Left := Screen.Width - (Owner as TForm).Width;
          if (Owner as TForm).Top + (Owner as TForm).Height > Screen.Height then
            (Owner as TForm).Top := Screen.Height - (Owner as TForm).Height
        end;
        if (Owner as TForm).Left < 0 then
          (Owner as TForm).Left := 0;
        if (Owner as TForm).Top < 0 then
          (Owner as TForm).Top := 0;
      end;
    finally
      Reg.Free;
    end;
  end;
end;

procedure TFnpFormEnh.DoSavePos;
var
  Reg: TRegIniFile;
  L: Integer;
  T: Integer;
  W: Integer;
  H: Integer;
begin
  if csDesigning in ComponentState then
    exit;
  if (FSavePosition or FSaveSize) and (Length(FSubKey) > 0) then begin
    Reg := TRegIniFile.Create(FSubKey);
    try
      if FRootKey = rkCurrentUser then
        Reg.RootKey := HKEY_CURRENT_USER
      else
        Reg.RootKey := HKEY_LOCAL_MACHINE;
      { Here we need to use BkupOwner because Owner is already Nil! }
      if (BkupOwner as TForm).WindowState = wsNormal then begin
        L := (BkupOwner as TForm).Left;
        T := (BkupOwner as TForm).Top;
        W := (BkupOwner as TForm).Width;
        H := (BkupOwner as TForm).Height;
      end else begin
        { We cannot read the Left/Top/Width/Height properites of the form
          when WindowState = wsMinimized or wsMaximized. We need the values
          returned by GetWindowPlacement to make it possible to restore the
          form's pos/size back to normal next time the form is shown. It is not
          possible to use GetWindowPlacement in TFnpFormEnh.Destory because
          the form's window handle is then already destroyed. }
        L := WP.rcNormalPosition.Left;
        T := WP.rcNormalPosition.Top;
        W := WP.rcNormalPosition.Right - L;
        H := WP.rcNormalPosition.Bottom - T;
      end;
      if FSavePosition then begin
        Reg.WriteInteger((BkupOwner as TForm).Name, 'Left', L);
        Reg.WriteInteger((BkupOwner as TForm).Name, 'Top', T);
      end;
      if FSaveSize then begin
        Reg.WriteInteger((BkupOwner as TForm).Name, 'Width', W);
        Reg.WriteInteger((BkupOwner as TForm).Name, 'Height', H);
        Reg.WriteInteger((BkupOwner as TForm).Name, 'WindowState', Integer((BkupOwner as TForm).WindowState));
      end;
    finally
      Reg.Free;
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TFnpFormEnh]);
end;

end.
