unit IB_BufferGrid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids,

  IB_Header,
  IB_Components;

type
  TIB_GridClickEvent = procedure ( Col, Row: longint ) of object;

  TIB_BufferGrid = class( TCustomGrid )
  private
  { Storage fields }
    FIB_DataLink: TIB_DataLink;
    FMovingRow: boolean;
    FAligningRow: boolean;
    FSizingRow: boolean;
    FEditText: string;
    FAcquireFocus: Boolean;
  { Event storage fields }
    FOnDrawCell: TDrawCellEvent;
    FOnGetEditMask: TGetEditEvent;
    FOnGetEditText: TGetEditEvent;
    FOnSelectCell: TSelectCellEvent;
    FOnSetEditText: TSetEditEvent;
    FOnTopLeftChanged: TNotifyEvent;
    FOnCellClick: TIB_GridClickEvent;
    FOnTitleClick: TIB_GridClickEvent;
  { Datalink event handlers }
    procedure IB_StateChanged( Sender: TIB_DataLink;
                               IB_DataSource: TIB_DataSource );
    procedure IB_DataChanged( Sender: TIB_DataLink;
                              IB_DataSource: TIB_DataSource;
                              IB_Column: TIB_Column );
    procedure IB_UpdateData( Sender: TIB_DataLink;
                             IB_DataSource: TIB_DataSource;
                             IB_Column: TIB_Column );
  { Messages }
    procedure CMEnter( var Message: TCMEnter ); message CM_ENTER;
    procedure CMExit( var Message: TMessage ); message CM_EXIT;
    procedure WMVScroll( var Message: TWMVScroll ); message WM_VSCROLL;
  protected
  { Inherited Methods }
    procedure Loaded; override;
    procedure KeyDown( var Key: Word; Shift: TShiftState ); override;
    procedure KeyPress( var Key: Char ); override;
    procedure MouseDown( Button: TMouseButton;
                         Shift: TShiftState;
                         X, Y: Integer ); override;
    procedure MouseUp( Button: TMouseButton;
                       Shift: TShiftState;
                       X, Y: Integer ); override;
    procedure CreateWnd; override;
    procedure SizeChanged( OldColCount, OldRowCount: Longint ); override;
  { Inherited Methods For Event Dispatch }
    procedure DrawCell( ACol, ARow: Longint;
                        ARect: TRect;
                        AState: TGridDrawState ); override;
    function GetEditMask( ACol, ARow: Longint ): string; override;
    function GetEditText( ACol, ARow: Longint ): string; override;
    function SelectCell( ACol, ARow: Longint ): Boolean; override;
    procedure SetEditText( ACol, ARow: Longint; const Value: string ); override;
    procedure TopLeftChanged; override;
  { Property Access Methods }
    procedure SetIB_DataSource( AValue: TIB_DataSource ); virtual;
    function GetIB_DataSource: TIB_DataSource; virtual;
    function GetCells( ACol, ARow: Integer ): string; virtual;
    procedure SetCells( ACol, ARow: Integer; const Value: string ); virtual;
  { Utility Methods }
    function AcquireFocus: Boolean;
    procedure InitColumnWidths;
    procedure UpdateColumnWidths;
  { System Methods }
    procedure SysStateChanged;
    procedure SysUpdateData;
    procedure SysUpdateRowCount;
    procedure SysUpdateScrollBar;
  { Event dispatch Methods }
    procedure TitleClick( ACol, ARow: integer ); dynamic;
    procedure CellClick( ACol, ARow: integer ); dynamic;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
  { Methods }
    procedure DefaultDrawCell( ACol, ARow: Longint;
                               ARect: TRect;
                               AState: TGridDrawState;
                               CellText: string );
  { Properties }
    property Cells[ ACol, ARow: Integer ]: string read GetCells write SetCells;
  published
  { Properties }
    property DataSource: TIB_DataSource read GetIB_DataSource
                                        write SetIB_DataSource;
  { Inherited properties }
    property Align;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DefaultDrawing;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FixedColor;
    property Font;
    property Options;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
  { Events }
    property OnDrawCell: TDrawCellEvent read FOnDrawCell write FOnDrawCell;
    property OnGetEditMask: TGetEditEvent read FOnGetEditMask
                                          write FOnGetEditMask;
    property OnGetEditText: TGetEditEvent read FOnGetEditText
                                          write FOnGetEditText;
    property OnSelectCell: TSelectCellEvent read FOnSelectCell
                                            write FOnSelectCell;
    property OnSetEditText: TSetEditEvent read FOnSetEditText
                                          write FOnSetEditText;
    property OnTopLeftChanged: TNotifyEvent read FOnTopLeftChanged
                                            write FOnTopLeftChanged;
    property OnCellClick: TIB_GridClickEvent read FOnCellClick
                                             write FOnCellClick;
    property OnTitleClick: TIB_GridClickEvent read FOnTitleClick
                                              write FOnTitleClick;
  { Inherited events }
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

implementation

type
{ This is used to suppress scrollbar flicker while scrolling. }
  THackCustomGrid = class( TCustomControl )
  private
    FAnchor: TGridCoord;
    FBorderStyle: TBorderStyle;
    FCanEditModify: Boolean;
    FColCount: Longint;
    FColWidths: Pointer;
    FTabStops: Pointer;
    FCurrent: TGridCoord;
    FDefaultColWidth: Integer;
    FDefaultRowHeight: Integer;
    FFixedCols: Integer;
    FFixedRows: Integer;
    FFixedColor: TColor;
    FGridLineWidth: Integer;
    FOptions: TGridOptions;
    FRowCount: Longint;
    FRowHeights: Pointer;
    FScrollBars: TScrollStyle;
end;

constructor TIB_BufferGrid.Create( AOwner: TComponent );
begin
  inherited Create( AOwner );
  FAcquireFocus := True;
  FIB_DataLink := TIB_DataLink.Create( Self );
  with FIB_DataLink do begin
    OnStateChanged := IB_StateChanged;
    OnDataChanged := IB_DataChanged;
    OnUpdateData := IB_UpdateData;
  end;
  inherited Options := [ goFixedHorzLine, goFixedVertLine, goHorzLine,
                         goVertLine, goColSizing, goTabs, goEditing ];
end;

destructor TIB_BufferGrid.Destroy;
begin
  DataSource := nil;
  with FIB_DataLink do begin
    OnStateChanged := nil;
    OnDataChanged := nil;
    OnUpdateData := nil;
  end;
  inherited Destroy;
end;

procedure TIB_BufferGrid.Loaded;
begin
  inherited Loaded;
  InitColumnWidths;
  SysStateChanged;
end;

procedure TIB_BufferGrid.CreateWnd;
begin
  inherited CreateWnd;
  SysUpdateRowCount;
  SysUpdateScrollBar;
end;

{------------------------------------------------------------------------------}

function TIB_BufferGrid.GetIB_DataSource: TIB_DataSource;
begin
  if FIB_DataLink <> nil then begin
    Result := FIB_DataLink.DataSource;
  end else begin
    Result := nil;
  end;
end;

procedure TIB_BufferGrid.SetIB_DataSource( AValue: TIB_DataSource );
begin
  if FIB_DataLink <> nil then begin
    if FIB_DataLink.DataSource <> AValue then begin
      FIB_DataLink.DataSource := AValue;
      SysStateChanged;
    end;
  end;
end;

{------------------------------------------------------------------------------}

function TIB_BufferGrid.GetCells( ACol, ARow: integer ): string;
begin
  if FIB_DataLink.Prepared then with FIB_DataLink.IB_DataSet do begin
    if ARow = 0 then begin
      Result := OutputRow[ ACol ].DisplayName;
    end else if ARow = RowNum then begin
      Result := OutputRow[ ACol ].AsString;
    end else if Active then begin
      BufferRowNum := ARow;
      if BufferRowNum = ARow then begin
        Result := BufferRow[ ACol ].AsString;
      end else begin
        Result := '';
      end;
    end else begin
      Result := '';
    end;
  end;
end;

procedure TIB_BufferGrid.SetCells( ACol, ARow: Integer; const Value: string );
begin
  if FIB_DataLink.Active then with FIB_DataLink.IB_DataSet do begin
    if RowNum <> ARow then begin
      RowNum := ARow;
    end;
    if RowNum = ARow then begin
      Self.DataSource.Modify;
      OutputRow[ ACol ].AsString := Value;
    end else begin
      raise EIB_DataSetError.Create( 'Unable to perform update.' );
    end;
    InvalidateCell( ACol, ARow );
    if ( ACol = Col ) and ( ARow = Row ) then InvalidateEditor;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_BufferGrid.IB_StateChanged( Sender: TIB_DataLink;
                                          IB_DataSource: TIB_DataSource );
begin
  SysStateChanged;
end;

procedure TIB_BufferGrid.IB_DataChanged( Sender: TIB_DataLink;
                                         IB_DataSource: TIB_DataSource;
                                         IB_Column: TIB_Column );
var
  NewEditText: string;
  OldEditText: string;
begin
  if FIB_DataLink.Active then begin
    with FIB_DataLink.IB_DataSet do begin
      if Row <> RowNum then begin
        SysUpdateRowCount;
        if not FAligningRow then begin
          try
            FMovingRow := true;
            if BOF then begin
              Row := FixedRows;
            end else if EOF then begin
              Row := RowNum - 1;
            end else begin
              Row := RowNum;
            end;
          finally
            FMovingRow := false;
          end;
        end;
        SysUpdateScrollBar;
      end else begin
        if IB_Column = nil then begin
          InvalidateRow( RowNum );
        end else begin
          InvalidateCell( IB_Column.SQLIndex + FixedCols, RowNum );
        end;
        OldEditText := FEditText;
        NewEditText := GetEditText( Col, Row );
        if NewEditText <> OldEditText then begin
          InvalidateEditor;
          if InplaceEditor <> nil then begin
            InplaceEditor.Deselect;
          end;
        end;
      end;
    end;
  end;
end;

procedure TIB_BufferGrid.IB_UpdateData( Sender: TIB_DataLink;
                                        IB_DataSource: TIB_DataSource;
                                        IB_Column: TIB_Column );
begin
  SysUpdateData;
end;

{------------------------------------------------------------------------------}

procedure TIB_BufferGrid.SysStateChanged;
begin
  if not ( csLoading    in ComponentState ) and
     not ( csDestroying in ComponentState ) then begin
    case FIB_DataLink.DataSetState of
      dsInactive: InitColumnWidths;
      dsPrepared: UpdateColumnWidths;
    end;
  end;
  SysUpdateRowCount;
  SysUpdateScrollBar;
  Invalidate;
end;

procedure TIB_BufferGrid.SysUpdateData;
begin
  if ( InPlaceEditor <> nil ) and InPlaceEditor.Modified then begin
    Cells[ Col, Row ] := FEditText;
    InPlaceEditor.Modified := false;
  end;
end;

{------------------------------------------------------------------------------}

procedure TIB_BufferGrid.SysUpdateRowCount;
var
  ii: integer;
  OldScrollBars: TScrollStyle; 
begin
  if FIB_DataLink.Prepared then begin
    with FIB_DataLink.IB_DataSet do begin
      if Active then begin
        OldScrollBars := ScrollBars;
        try
          if ScrollBars = ssBoth then begin
            THackCustomGrid( Self ).FScrollBars := ssHorizontal;
          end else begin
            THackCustomGrid( Self ).FScrollBars := ssNone;
          end;
          if BufferHasEOF then begin
            if BufferRowCount > 1 then begin
              RowCount := BufferRowCount + FixedRows;
            end else begin
              RowCount := FixedRows + 1;
            end;
          end else begin
            RowCount := BufferRowCount + FixedRows + 1000;
            for ii := 0 to VisibleRowCount - 1 do begin
              if ( RowNum <> TopRow + ii ) or
                 ( DataSetState = dsActive ) then begin
                BufferRowNum := TopRow + ii;
              end;
              if BufferEOF then Break;
            end;
            if BufferRowCount < 1 then begin
              RowCount := FixedRows + 1;
            end else if BufferHasEOF then begin
              RowCount := BufferRowCount + FixedRows;
            end;
          end;
        finally
          THackCustomGrid( Self ).FScrollBars := OldScrollBars;
        end;
      end else begin
        RowCount := FixedRows + 1;
        Row      := FixedRows;
      end;
    end;
  end;
end;

procedure TIB_BufferGrid.SysUpdateScrollBar;
var
  SIOld, SINew: TScrollInfo;
begin
  if FIB_Datalink.Prepared and HandleAllocated then begin
    with FIB_Datalink.IB_DataSet do
    begin
      SIOld.cbSize := sizeof( SIOld );
      SIOld.fMask := SIF_ALL;
      GetScrollInfo( Self.Handle, SB_VERT, SIOld );
      SINew := SIOld;
      SINew.nMin  := 1;
      SINew.nPage := VisibleRowCount;
      SINew.nMax  := VisibleRowCount + BufferRowCount + FixedRows; 
      SINew.nPos  := RowNum;
      if ( SINew.nMin  <> SIOld.nMin  ) or
         ( SINew.nMax  <> SIOld.nMax  ) or
         ( SINew.nPage <> SIOld.nPage ) or
         ( SINew.nPos  <> SIOld.nPos  ) then begin
        SetScrollInfo( Self.Handle, SB_VERT, SINew, True );
      end;
    end;
  end;
end;

{------------------------------------------------------------------------------}
{   Grid handling procedures                                                   }
{------------------------------------------------------------------------------}

procedure TIB_BufferGrid.SizeChanged(OldColCount, OldRowCount: Longint);
begin
  SysUpdateScrollBar;
end;

procedure TIB_BufferGrid.MouseDown( Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer );
var
  Cell: TGridCoord;
  OldCol,OldRow: Integer;
begin
  if not AcquireFocus then Exit;
  if ( ssDouble in Shift ) and ( Button = mbLeft ) then
  begin
    DblClick;
    Exit;
  end;
  if Sizing( X, Y ) then
  begin
    SysUpdateData;
    inherited MouseDown( Button, Shift, X, Y )
  end else begin
    Cell := MouseCoord( X, Y );
    if ( csDesigning in ComponentState ) and
       ( Cell.Y < FixedRows ) then begin
      FIB_DataLink.UpdateData;
      inherited MouseDown( Button, Shift, X, Y );
    end else if FIB_Datalink.Active then begin
      with Cell, FIB_Datalink.IB_Dataset do begin
        HideEditor;
        OldCol := Col;
        OldRow := Row;
        if ( Y >= FixedRows ) and ( X >= FixedCols ) then begin
          MoveColRow( X, Y, true, true );
        end;
        if ( X = OldCol ) and ( Y = OldRow ) then begin
          ShowEditor;
        end else begin
          InvalidateEditor; 
        end;
      end;
    end;
  end;
end;

procedure TIB_BufferGrid.MouseUp( Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer );
var
  Cell: TGridCoord;
  SaveState: TGridState;
begin
  SaveState := FGridState;
  inherited MouseUp( Button, Shift, X, Y );
  if ( SaveState = gsRowSizing ) or
     ( SaveState = gsColSizing ) or
     (( InplaceEditor <> nil ) and
      ( InplaceEditor.Visible ) and
      ( PtInRect( InplaceEditor.BoundsRect, Point( X, Y )))) then begin
    Exit;
  end else begin
    Cell := MouseCoord( X, Y );
    if ( Button  = mbLeft ) and
       ( Cell.X >= FixedCols ) and
       ( Cell.Y >= 0 ) then begin
      if Cell.Y < FixedRows then begin
        TitleClick( Cell.X, Cell.Y )
      end else begin
        CellClick ( Cell.X, Cell.Y );
      end;
    end;
  end;
end;

procedure TIB_BufferGrid.CellClick( ACol, ARow: integer );
begin
  if Assigned( FOnCellClick ) then begin
    FOnCellClick( ACol, ARow );
  end;
end;

procedure TIB_BufferGrid.TitleClick( ACol, ARow: integer );
begin
  if Assigned( FOnTitleClick ) then begin
    FOnTitleClick( ACol, ARow );
  end;
end;

procedure TIB_BufferGrid.WMVScroll( var Message: TWMVScroll );
var
  SI: TScrollInfo;
begin
  if not AcquireFocus then Exit;
  if FIB_Datalink.Active then begin
    with Message, FIB_DataLink.IB_DataSet, FIB_Datalink do begin
      case ScrollCode of
        SB_LINEUP:   if not BOF then MoveBy( -1 );
        SB_LINEDOWN: if not EOF then MoveBy(  1 );
        SB_PAGEUP:   if not BOF then MoveBy( -VisibleRowCount );
        SB_PAGEDOWN: if not EOF then MoveBy(  VisibleRowCount );
        SB_THUMBTRACK: Exit;
        SB_THUMBPOSITION:
          begin
            SI.cbSize := sizeof( SI );
            SI.fMask := SIF_ALL;
            GetScrollInfo( Self.Handle, SB_VERT, SI );
            if SI.nTrackPos < BufferRowCount then begin
              RowNum := SI.nTrackPos;
            end else begin
              Last
            end;
          end;
        SB_BOTTOM:    Last;
        SB_TOP:       First;
        SB_ENDSCROLL: Exit;
      end;
    end;
  end;
end;

procedure TIB_BufferGrid.InitColumnWidths;
begin
  ColCount  := 1;
  RowCount  := 2;
  FixedRows := 1;
end;

procedure TIB_BufferGrid.UpdateColumnWidths;
var
  ii: integer;
  DataWidth: integer;
  HeaderWidth: integer;
  NewWidth: integer;
begin
  if FIB_DataLink.Prepared then begin
    with DataSource.IB_DataSet do begin
      ColCount := OutputCount;
      for ii := 0 to OutputCount - 1 do with OutputRow[ ii ] do begin
        HeaderWidth := Length( Trim( DisplayName ) );
        if ( SQLType = SQL_DATE ) or
           ( SQLType = SQL_DATE_ ) then begin
          DataWidth := 16;
        end else begin
          DataWidth := SQLLen;
        end;
        if HeaderWidth > DataWidth then begin
          NewWidth := HeaderWidth * 8;
        end else begin
          NewWidth := DataWidth * 8;
        end;
        if NewWidth < 70 then begin
          NewWidth := 70;
        end;
        if NewWidth > 200 then begin
          NewWidth := 200;
        end;
        ColWidths[ ii ] := NewWidth;
        ColWidths[ ii ] := NewWidth;
      end;
    end;
  end;
end;

procedure TIB_BufferGrid.CMEnter( var Message: TCMEnter );
begin
  FIB_DataLink.SetFocus;
  inherited;
end;

procedure TIB_BufferGrid.KeyDown( var Key: Word; Shift: TShiftState );
var
  OldScrollBars: TScrollStyle;
begin
  if FIB_DataLink.Active then with DataSource.IB_DataSet do begin
    if ssCtrl in Shift then case Key of
      VK_HOME: begin First; Key := 0; end;
      VK_DOWN: begin Next;  Key := 0; end;
      VK_UP:   begin Prior; Key := 0; end;
      VK_END:  begin Last;  Key := 0; end;
    end;
  end;
  OldScrollBars := ScrollBars;
  try
    if ScrollBars = ssBoth then begin
      THackCustomGrid( Self ).FScrollBars := ssHorizontal;
    end else begin
      THackCustomGrid( Self ).FScrollBars := ssNone;
    end;
    inherited KeyDown( Key, Shift );
  finally
    THackCustomGrid( Self ).FScrollBars := OldScrollBars;
  end;
end;

procedure TIB_BufferGrid.CMExit( var Message: TMessage );
begin
  try
    if FIB_Datalink.Prepared then
      with FIB_Datalink.IB_Dataset do
        if ( DataSetState = dsInserting ) and
           ( not FIB_Datalink.ControlIsModified ) then Cancel
                                                  else FIB_DataLink.UpdateData;
  except
    SetFocus;
    raise;
  end;
  inherited;
end;

function TIB_BufferGrid.AcquireFocus: Boolean;
begin
  Result := True;
  if FAcquireFocus and
     CanFocus and
     not ( csDesigning in ComponentState ) then begin
    SetFocus;
    Result := Focused or ( InplaceEditor <> nil ) and InplaceEditor.Focused;
  end;
end;

{------------------------------------------------------------------------------}

function TIB_BufferGrid.GetEditMask( ACol, ARow: Longint ): string;
begin
  Result := '';
  if Assigned( FOnGetEditMask ) then begin
    FOnGetEditMask( Self, ACol, ARow, Result );
  end;
end;

function TIB_BufferGrid.GetEditText( ACol, ARow: Longint ): string;
begin
  Result := Cells[ ACol, ARow ];
  if Assigned( FOnGetEditText ) then begin
    FOnGetEditText( Self, ACol, ARow, Result );
  end;
  FEditText := Result;
end;

function TIB_BufferGrid.SelectCell( ACol, ARow: longint ): boolean;
begin
  Result := true;
  FIB_Datalink.UpdateData;
  HideEditor;
  if not FMovingRow then begin
    if Assigned( FOnSelectCell ) then begin
      FOnSelectCell( Self, ACol, ARow, Result );
    end;
    if Result and
      ( DataSource <> nil ) and
      ( DataSource.IB_DataSet <> nil ) then begin
      with DataSource.IB_DataSet do begin
        if Active and
          ( ARow  > FixedRows - 1 ) and
          ( ARow <> RowNum ) then begin
          try
            FAligningRow := true;
            RowNum := ARow;
          finally
            FAligningRow := false;
          end;
        end;
      end;
    end;
  end;
end;

procedure TIB_BufferGrid.SetEditText(       ACol, ARow: Longint;
                                      const Value: string );
begin
  if Assigned( FOnSetEditText ) then begin
    FOnSetEditText( Self, ACol, ARow, Value );
  end;
  if ( FEditText <> Value ) and
     ( FIB_DataLink.Modify ) then begin
    FEditText := Value;
  end;
end;

procedure TIB_BufferGrid.DrawCell( ACol, ARow: Longint;
                                   ARect: TRect;
                                   AState: TGridDrawState );
begin
  if Assigned( FOnDrawCell ) then begin
    FOnDrawCell( Self, ACol, ARow, ARect, AState );
  end else begin
    DefaultDrawCell( ACol, ARow, ARect, AState, Cells[ ACol, ARow ] );
  end;
end;

procedure TIB_BufferGrid.TopLeftChanged;
begin
  inherited TopLeftChanged;
  if Assigned( FOnTopLeftChanged ) then begin
    FOnTopLeftChanged( Self );
  end;
end;

procedure TIB_BufferGrid.DefaultDrawCell( ACol, ARow: Longint;
                                          ARect: TRect;
                                          AState: TGridDrawState;
                                          CellText: string );
begin
  ExtTextOut( Canvas.Handle,
              ARect.Left + 2,
              ARect.Top + 2,
              ETO_CLIPPED or ETO_OPAQUE,
              @ARect,
              PChar( CellText ),
              Length( CellText ),
              nil );
end;

procedure TIB_BufferGrid.KeyPress( var Key: Char );
begin
  if ( Key = #13 ) then begin
    SysUpdateData;
  end;
  inherited KeyPress( Key );
end;

{------------------------------------------------------------------------------}

end.

