
{                                                                              }
{ TIB_DATASET                                                                  }
{                                                                              }

  EIB_DataSetError = class( EIB_Error );

  TIB_DataSetEvent = procedure( IB_DataSet: TIB_DataSet ) of object;
  TIB_FetchCallbackStatus = ( csInit, csRefresh, csFinal );
  TIB_FetchCallbackEvent = procedure( IB_DataSet: TIB_DataSet;
                                      Status: TIB_FetchCallbackStatus;
                                      CursorRowNum: longint ) of object;
  TIB_DataSetState = (dsInactive
                     ,dsPrepared
                     ,dsSearching
                     ,dsActive
                     ,dsEditing
                     ,dsInserting
                     ,dsDeleting
                     );

  TIB_DataSet = class( TIB_Statement )
  private
  { Property Storage Fields }
    FIB_DataLink: TIB_DataLink;
    FIB_DataSourceList: TList;
    FChildDataSetList: TList;
    FDataSetState: TIB_DataSetState;
    FLastRowState: TIB_RowState;
    FFetchCallbackInc: cardinal;
    FFetchCallbackActive: boolean;
    FFetchAborted: boolean;
    FReadOnly: boolean;
    FPreventEditing: boolean;
    FPreventInserting: boolean;
    FPreventDeleting: boolean;
    FPreventSearching: boolean;
    FCursorName: string;
    FCursorEOF: boolean;
    FCursorRowNum: longint;
    FCursorRowCount: longint;
    FCursorRowCountValid: boolean;
    FPessimisticLocking: boolean;
    FLockTransaction: TIB_TransactionSingle;
    FIsRowLocked: boolean;
    FIsPosting: boolean;
    FSQLSelect: TStrings;
    FSQLFrom: TStrings;
    FSQLWhere: TStrings;
    FSQLGroup: TStrings;
    FSQLHaving: TStrings;
    FSQLUnion: TStrings;
    FSQLPlan: TStrings;
    FSQLOrder: TStrings;
    FSQLOrderItemNo: integer;
    FSQLOrderItems: TStrings;
    FSQLForUpdate: TStrings;
    FSQLParentLinks: TStrings;
    FSQLParentSearch: boolean;
    FSQLParentWhere: string;
    FSQLWhereLow: TStrings;
    FSQLWhereMed: TStrings;
    FSQLWhereHigh: TStrings;
    FSQLWhereParams: TStrings;
    FSQLWhereChanged: boolean;
    FSQLOrderChanged: boolean;
    FSQLGetDefaults: boolean;
    FSQLGeneratorLinks: TStrings;
    FDisplayNames: TStrings;
    FDisplayMasks: TStrings;
    FUseColorScheme: boolean;
  { Hints that appear in the TIB_NavigatorBar }
    FHintEdit,
    FHintInsert,
    FHintDelete,
    FHintPost,
    FHintCancel,
    FHintFirst,
    FHintLast,
    FHintNext,
    FHintPrior,
    FHintSearch: string;
  { Fetch callback dialog }
    dlgCancelQuery: TdlgCancelQuery;
  { Optimization/System Flags }
    flag_open_after_load: boolean;
    flag_junk_value: integer;
  { Event Storage Fields }
    FBeforeOpen: TIB_DataSetEvent;
    FAfterOpen: TIB_DataSetEvent;
    FBeforeClose: TIB_DataSetEvent;
    FAfterClose: TIB_DataSetEvent;
    FBeforeSearch: TIB_DataSetEvent;
    FAfterSearch: TIB_DataSetEvent;
    FBeforeEdit: TIB_DataSetEvent;
    FAfterEdit: TIB_DataSetEvent;
    FBeforeInsert: TIB_DataSetEvent;
    FAfterInsert: TIB_DataSetEvent;
    FBeforeDelete: TIB_DataSetEvent;
    FAfterDelete: TIB_DataSetEvent;
    FBeforeCancel: TIB_DataSetEvent;
    FAfterCancel: TIB_DataSetEvent;
    FBeforePost: TIB_DataSetEvent;
    FAfterPost: TIB_DataSetEvent;
    FOnFetchCallback: TIB_FetchCallbackEvent;
  protected
  { Inherited Property Access Methods }
    procedure SetIB_Connection( AValue: TIB_Connection ); override;
    procedure SetIB_Transaction( AValue: TIB_Transaction ); override;
    procedure SetActive( Value: boolean ); override;
  { Buffering Property Access Methods }
    function GetBufferRowCount: longint; virtual;
    function GetBufferHasEOF: boolean; virtual;
    function GetBufferRowNum: longint; virtual;
    procedure SetBufferRowNum( AValue: longint ); virtual;
    function GetBufferBOF: boolean; virtual;
    function GetBufferEOF: boolean; virtual;
    function GetBufferRow: TIB_Row; virtual;
  { Property Access Methods }
    function GetCursorEOF: boolean; virtual;
    function GetCursorBOF: boolean; virtual;
    function GetEOF: boolean; virtual;
    function GetBOF: boolean; virtual;
    function GetCanModify: boolean; virtual;
    function GetCanEdit: boolean; virtual;
    function GetCanInsert: boolean; virtual;
    function GetCanDelete: boolean; virtual;
    function GetCanSearch: boolean; virtual;
    function GetNeedToPost: boolean; virtual;
    function GetDataSourceCount: integer; virtual;
    function GetDataSources( Index: integer ): TIB_DataSource; virtual;
    function GetDataSetState: TIB_DataSetState; virtual;
    procedure SetDataSetState( AValue: TIB_DataSetState ); virtual;
    procedure SetIB_DataSource( AValue: TIB_DataSource ); virtual;
    function GetIB_DataSource: TIB_DataSource; virtual;
    function GetRowNum: longint; virtual;
    procedure SetRowNum( AValue: longint ); virtual;
    function GetCursorRowCount: longint; virtual;
    function GetUnidirectional: boolean; virtual;
    procedure SetReadOnly( AValue: boolean ); virtual;
    function GetReadOnly: boolean; virtual;
    procedure SetPreventEditing( AValue: boolean ); virtual;
    procedure SetPreventInserting( AValue: boolean ); virtual;
    procedure SetPreventDeleting( AValue: boolean ); virtual;
    procedure SetPreventSearching( AValue: boolean ); virtual;
    function GetIsRowLocked: boolean; virtual;
  { General methods }
    procedure AfterTransactionEnd        ( TLink: TIB_TransactionLink;
                                               T: TIB_Transaction ); override;
    procedure AfterTransactionSavePoint  ( TLink: TIB_TransactionLink;
                                               T: TIB_Transaction ); override;
    function GetSQLParentLinkCount: integer; virtual;
    procedure SetSQLParentLinks( Value: TStrings ); virtual;
    function GetSQLOrderItems: TStrings; virtual;
    procedure SetSQLOrderItems( Value: TStrings ); virtual;
    procedure SetSQLOrderItemNo( Value: integer ); virtual;
    function GetSQLGeneratorLinks: TStrings; virtual;
    procedure SetSQLGeneratorLinks( Value: TStrings ); virtual;
    function GetDisplayNames: TStrings; virtual;
    procedure SetDisplayNames( Value: TStrings ); virtual;
    function GetDisplayMasks: TStrings; virtual;
    procedure SetDisplayMasks( Value: TStrings ); virtual;
    function GetSQLParentLink( ParamName: string ): string; virtual;
    procedure SetSQLParentLink( ParamName: string; TableName: string); virtual;
    procedure ExecFieldLinks; virtual;
    function GetParentDataSet: TIB_DataSet; virtual;
    function GetParentRelation( LinkNo: integer ): string; virtual;
    function GetParentColumn( LinkNo: integer ): string; virtual;
    function GetChildRelation( LinkNo: integer ): string; virtual;
    function GetChildColumn( LinkNo: integer ): string; virtual;
    function GetSQLSection( Index: integer ): TStrings;
    procedure SetSQLSection( Index: integer; Value: TStrings );
    procedure SQLSelectChange( Sender: TObject );
    procedure SQLFromChange( Sender: TObject );
    procedure SQLWhereChange( Sender: TObject );
    procedure SQLGroupChange( Sender: TObject );
    procedure SQLHavingChange( Sender: TObject );
    procedure SQLUnionChange( Sender: TObject );
    procedure SQLPlanChange( Sender: TObject );
    procedure SQLOrderChange( Sender: TObject );
    procedure SQLForUpdateChange( Sender: TObject );
  { Inherited Methods }
    procedure Loaded; override;
    procedure SysAfterPrepare; override;
    procedure SysPreparedChanged; override;
    procedure SysAfterUnprepare; override;
    procedure BeforeConnectionDisconnect( CLink: TIB_ConnectionLink;
                                          C: TIB_Connection ); override;
    procedure BeforeTransactionAssignment( TLink: TIB_TransactionLink;
                                             Old: TIB_Transaction ); override;
    procedure SysExecSelect; override;
    procedure SysExecute; override;
    procedure SysClose; override;
    procedure SysBeforeOutputDataChange( Sender: TIB_Row;
                                         AColumn: TIB_Column); override;
    procedure SysBeforeExecuteForOutput; override;
    procedure SysAfterExecuteForOutput; override;
    procedure SysActiveChanged; override;
    procedure SysPrepareSQL; override;
    procedure SysPrepareFailed; override;
    function SysPrepare: boolean; override;
    procedure SysUnprepare; override;
    procedure SysUpdateDescriptors; override;
    procedure SysGetCursorName; virtual;
    function SysGetCursorRowCount: longint; virtual;
    procedure OnSQLChange( Sender: TObject ); override;
  { System Methods }
    procedure SysFetchSingle; virtual;
    function SysOpen: boolean; dynamic;
    function SysCount: longint; dynamic;
    procedure SysOpenAfterLoad; dynamic;
    procedure SysEdit; dynamic;
    procedure SysEditRow; dynamic;
    procedure SysInsert; dynamic;
    procedure SysInsertRow; dynamic;
    procedure SysGetDefaults; dynamic;
    procedure SysDelete; dynamic;
    procedure SysDeleteRow; dynamic;
    procedure SysPost; dynamic;
    procedure SysPostRow; dynamic;
    procedure SysPostEditedRow; dynamic;
    procedure SysPostInsertedRow; dynamic;
    procedure SysPostDeletedRow; dynamic;
    procedure SysCancel; dynamic;
    procedure SysCancelRow; dynamic;
    procedure SysCancelEditedRow; dynamic;
    procedure SysCancelInsertedRow; dynamic;
    procedure SysCancelDeletedRow; dynamic;
    procedure SysClearLock; dynamic;
    procedure SysLast; dynamic;
    procedure SysBeforeLast; dynamic;
    procedure SysLastRow; dynamic;
    procedure SysAfterLast; dynamic;
    procedure SysFirst; dynamic;
    procedure SysBeforeFirst; dynamic;
    procedure SysFirstRow; dynamic;
    procedure SysAfterFirst; dynamic;
    procedure SysMoveBy( JumpRecs: longint ); dynamic;
    procedure SysCursorMoveByRow( JumpRecs: longint ); dynamic;
    procedure SysBeforeMoveBy( JumpRecs: longint ); dynamic;
    procedure SysMoveByRow( JumpRecs: longint ); dynamic;
    procedure SysAfterMoveBy( JumpRecs: longint ); dynamic;
    procedure SysBeforeNavigate; dynamic;
    procedure SysAfternavigate; dynamic;
    procedure SysDataSetStateChanged; dynamic;
    procedure SysExecSQLParentLinks; dynamic;
    procedure SysExecParentLinks; dynamic;
    procedure SysProcessCallback( Status: TIB_FetchCallbackStatus ); virtual;
  { Internal method handlers }
    procedure SysBeforeOpen; virtual;
    procedure SysAfterOpen; virtual;
    procedure SysBeforeClose; virtual;
    procedure SysAfterClose; virtual;
    procedure SysBeforeSearch; dynamic;
    procedure SysAfterSearch; dynamic;
    procedure SysBeforeEdit; dynamic;
    procedure SysAfterEdit; dynamic;
    procedure SysBeforeInsert; dynamic;
    procedure SysAfterInsert; dynamic;
    procedure SysBeforeDelete; dynamic;
    procedure SysAfterDelete; dynamic;
    procedure SysBeforePost; virtual;
    procedure SysAfterPost; virtual;
    procedure SysBeforeCancel; dynamic;
    procedure SysAfterCancel; dynamic;
  { System Cursor handling methods }
    procedure SysFetchFirst; virtual;
    procedure SysFetchAll; dynamic;
    procedure SysEditCursorRow; dynamic;
    procedure SysInsertCursorRow; dynamic;
    procedure SysDeleteCursorRow; dynamic;
    procedure SysAfterFetchCursorEOF; dynamic;
    procedure SysFetchNext; virtual;
    procedure SysAfterFetchCursorRow; virtual;
  { Parent DataSet Linking Methods }
    procedure ParentPrepareSQL( Sender: TIB_DataLink;
                                IB_DataSource: TIB_DataSource ); virtual;
    procedure ParentStateChanged( Sender: TIB_DataLink;
                                  IB_DataSource: TIB_DataSource ); virtual;
    procedure ParentDataChanged( Sender: TIB_DataLink;
                                 IB_DataSource: TIB_DataSource;
                                 IB_Column: TIB_Column ); virtual;
  { API Function Calls }
    procedure API_CreateCursor;
    function API_FetchRow: integer;
    procedure API_CloseCursor; 
  { SQL Generator procedures }
    procedure SQL_CursorUpdate;
    function SQL_CursorLock: boolean;
    procedure SQL_CursorInsert;
    procedure SQL_CursorDelete; 
  { Event Dispatch Methods }
    procedure DoBeforeOpen; dynamic;
    procedure DoAfterOpen; dynamic;
    procedure DoBeforeClose; dynamic;
    procedure DoAfterClose; dynamic;
    procedure DoBeforeSearch; dynamic;
    procedure DoAfterSearch; dynamic;
    procedure DoBeforeEdit; dynamic;
    procedure DoAfterEdit; dynamic;
    procedure DoBeforeInsert; dynamic;
    procedure DoAfterInsert; dynamic;
    procedure DoAfterDelete; dynamic;
    procedure DoBeforeDelete; dynamic;
    procedure DoBeforePost; dynamic;
    procedure DoAfterPost; dynamic;
    procedure DoBeforeCancel; dynamic;
    procedure DoAfterCancel; dynamic;
    procedure DoProcessCallback( Status: TIB_FetchCallbackStatus ); virtual;
  { Link Event Dispatch Methods }
    procedure DoLinkDataSetStateChanged;
  { Properties }
    property DataSourceCount: integer read GetDataSourceCount;
    property DataSources[ Index: integer ]: TIB_DataSource
        read GetDataSources;
    property CursorRowNum: longint read FCursorRowNum;
    property CursorBOF: boolean read GetCursorBOF;
    property CursorEOF: boolean read GetCursorEOF;
    property SQLSelect:    TStrings Index 1 read GetSQLSection
                                            write SetSQLSection
                                            stored false;
    property SQLFrom:      TStrings Index 2 read GetSQLSection
                                            write SetSQLSection
                                            stored false;
    property SQLWhere:     TStrings Index 3 read GetSQLSection
                                            write SetSQLSection
                                            stored false;
    property SQLGroup:     TStrings Index 4 read GetSQLSection
                                            write SetSQLSection
                                            stored false;
    property SQLHaving:    TStrings Index 5 read GetSQLSection
                                            write SetSQLSection
                                            stored false;
    property SQLUnion:     TStrings Index 6 read GetSQLSection
                                            write SetSQLSection
                                            stored false;
    property SQLPlan:      TStrings Index 7 read GetSQLSection
                                            write SetSQLSection
                                            stored false;
    property SQLOrder:     TStrings Index 8 read GetSQLSection
                                            write SetSQLSection
                                            stored false;
    property SQLForUpdate: TStrings Index 9 read GetSQLSection
                                            write SetSQLSection
                                            stored false;
    property IsRowLocked: boolean read GetIsRowLocked;
    property PessimisticLocking: boolean read FPessimisticLocking
                                         write FPessimisticLocking
                                         default false;
  { Events }
    property OnFetchCallback: TIB_FetchCallbackEvent read FOnFetchCallback
                                                     write FOnFetchCallback;
    property BeforeOpen:   TIB_DataSetEvent read FBeforeOpen
                                            write FBeforeOpen;
    property AfterOpen:    TIB_DataSetEvent read FAfterOpen
                                            write FAfterOpen;
    property BeforeClose:  TIB_DataSetEvent read FBeforeClose
                                            write FBeforeClose;
    property AfterClose:   TIB_DataSetEvent read FAfterClose
                                            write FAfterClose;
    property BeforeSearch: TIB_DataSetEvent read FBeforeSearch
                                            write FBeforeSearch;
    property AfterSearch:  TIB_DataSetEvent read FAfterSearch
                                            write FAfterSearch;
    property BeforeEdit:   TIB_DataSetEvent read FBeforeEdit
                                            write FBeforeEdit;
    property AfterEdit:    TIB_DataSetEvent read FAfterEdit
                                            write FAfterEdit;
    property BeforeInsert: TIB_DataSetEvent read FBeforeInsert
                                            write FBeforeInsert;
    property AfterInsert:  TIB_DataSetEvent read FAfterInsert
                                            write FAfterInsert;
    property BeforeDelete: TIB_DataSetEvent read FBeforeDelete
                                            write FBeforeDelete;
    property AfterDelete:  TIB_DataSetEvent read FAfterDelete
                                            write FAfterDelete;
    property BeforePost:   TIB_DataSetEvent read FBeforePost
                                            write FBeforePost;
    property AfterPost:    TIB_DataSetEvent read FAfterPost
                                            write FAfterPost;
    property BeforeCancel: TIB_DataSetEvent read FBeforeCancel
                                            write FBeforeCancel;
    property AfterCancel:  TIB_DataSetEvent read FAfterCancel
                                            write FAfterCancel;
  public
  { Inherited Methods }
    constructor Create( AOwner : TComponent ); override;
    destructor Destroy; override;
    procedure Execute; override;
    procedure Unprepare; override;
    procedure SetFocus; virtual;
    procedure InvalidateSQL; override;
  { Methods }
    procedure FetchSingle; virtual;
    procedure Open; virtual;
    procedure Close; virtual;
    procedure First; virtual;
    procedure Last; virtual;
    procedure Next; virtual;
    procedure Prior; virtual;
    procedure MoveBy( JumpRecs: longint ); virtual;
    procedure Edit; virtual;
    procedure Insert; virtual;
    procedure Delete; virtual;
    procedure Post; virtual;
    procedure Cancel; virtual;
    procedure Abort; virtual;
    function Count: longint; virtual;
    procedure Search; virtual;
    procedure SaveSearch; virtual;
    procedure ClearSearch; virtual;
    procedure RecallSearch; virtual;
    procedure AddSQLWhereClause( WhereClause: string ); virtual;
    function IsLinkedTo( IB_DataSource: TIB_DataSource ): boolean; virtual;
    procedure UpdateData; virtual;
    procedure DataChanged; virtual;
    procedure GetSQLOrderItemsList( const SQLOrderItemsList: TStrings );
    procedure DefaultProcessCallback( Status: TIB_FetchCallbackStatus );
  { Abstract Buffering Properties }
    property BufferRow: TIB_Row read GetBufferRow;
    property BufferRowNum: longint read GetBufferRowNum write SetBufferRowNum;
    property BufferEOF: boolean read GetBufferEOF;
    property BufferBOF: boolean read GetBufferBOF;
    property BufferRowCount: longint read GetBufferRowCount;
    property BufferHasEOF: boolean read GetBufferHasEOF;
  { Properties }
    property DataSetState: TIB_DataSetState read GetDataSetState;
    property Unidirectional: boolean read GetUnidirectional;
    property EOF: boolean read GetEOF;
    property BOF: boolean read GetBOF;
    property RowNum: longint read GetRowNum write SetRowNum;
    property CursorRowCount: longint read GetCursorRowCount;
    property CanModify: boolean read GetCanModify;
    property CanEdit: boolean read GetCanEdit;
    property CanInsert: boolean read GetCanInsert;
    property CanDelete: boolean read GetCanDelete;
    property CanSearch: boolean read GetCanSearch;
    property NeedToPost: boolean read GetNeedToPost;
    property IsPosting: boolean read FIsPosting;
    property ReadOnly: boolean read GetReadOnly write SetReadOnly default false;
    property PreventEditing: boolean read FPreventEditing
                                      write SetPreventEditing
                                      default false;
    property PreventInserting: boolean read FPreventInserting
                                       write SetPreventInserting
                                       default false;
    property PreventDeleting: boolean read FPreventDeleting
                                      write SetPreventDeleting
                                      default false;
    property PreventSearching: boolean read FPreventSearching
                                       write SetPreventSearching
                                       default false;
    property FetchCallbackInc: cardinal read FFetchCallbackInc
                                        write FFetchCallbackInc
                                        default 256;
    property FetchCallbackActive: boolean read FFetchCallbackActive;
    property FetchAborted: boolean read FFetchAborted;
    property ParentDataSet: TIB_DataSet read GetParentDataSet;
    property ParentRelation[ Index: integer ]: string read GetParentRelation;
    property ParentColumn  [ Index: integer ]: string read GetParentColumn;
    property ChildRelation [ Index: integer ]: string read GetChildRelation;
    property ChildColumn   [ Index: integer ]: string read GetChildColumn;
    property SQLParentLinkCount: integer read GetSQLParentLinkCount;
    property SQLParentLink[ Index: string ]: string read GetSQLParentLink
                                                    write SetSQLParentLink;
    property SQLWhereChanged: boolean read FSQLWhereChanged;
    property SQLOrderChanged: boolean read FSQLOrderChanged;
    property SQLWhereLow: TStrings read FSQLWhereLow;
    property SQLWhereMed: TStrings read FSQLWhereMed;
    property SQLWhereHigh: TStrings read FSQLWhereHigh;
    property SQLWhereParams: TStrings read FSQLWhereParams;
  published
    property DisplayMasks: TStrings read GetDisplayMasks write SetDisplayMasks;
    property DisplayNames: TStrings read GetDisplayNames write SetDisplayNames;
    property HintInsert: string read FHintInsert write FHintInsert;
    property HintEdit:   string read FHintEdit   write FHintEdit;
    property HintDelete: string read FHintDelete write FHintDelete;
    property HintPost:   string read FHintPost   write FHintPost;
    property HintCancel: string read FHintCancel write FHintCancel;
    property HintFirst:  string read FHintFirst  write FHintFirst;
    property HintLast:   string read FHintLast   write FHintLast;
    property HintNext:   string read FHintNext   write FHintNext;
    property HintPrior:  string read FHintPrior  write FHintPrior;
    property HintSearch: string read FHintSearch write FHintSearch;
    property IB_DataSource: TIB_DataSource read GetIB_DataSource
                                             write SetIB_DataSource;
    property SQLGetDefaults: boolean read FSQLGetDefaults
                                     write FSQLGetDefaults
                                     default false;
    property SQLGeneratorLinks: TStrings read GetSQLGeneratorLinks
                                         write SetSQLGeneratorLinks;
    property SQLOrderItems: TStrings read GetSQLOrderItems
                                     write SetSQLOrderItems;
    property SQLOrderItemNo: integer read FSQLOrderItemNo
                                     write SetSQLOrderItemNo
                                     default -1;
    property SQLParentLinks: TStrings read FSQLParentLinks
                                      write SetSQLParentLinks;
    property SQLParentSearch: boolean read FSQLParentSearch
                                      write FSQLParentSearch
                                      default false;
    property UseColorScheme: boolean read FUseColorScheme
                                     write FUseColorScheme
                                     default false;
{------------------------------------------------------------------------------}
    property _DataSourceCount: integer read GetDataSourceCount
                                       write flag_junk_value
                                       stored false;
  end;

// IMP_IB_DATASET
