{----------------------------------------------------------------------}
{         TLedAuto component v1.0 - Modified by Gabriel Asensio 1.997  }
{ Based in RoundLed v1.1 Component - by Martyn Dowsett CIS 100676,1560 }
{ Note.: RoundLed can 'blink' - Please read LedAuto.txt file           }
{ e-mail.: gasensio@cec.caan.es                                        }
{----------------------------------------------------------------------}

unit LedAuto;
{$R LEDAUTO.RES}
interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls;

type
 TLedStatus    = (lsOff, lsOn);
 TLedType      = (ltIn, ltOut);
 TLedOnColors  = (lcSilver,lcRed,lcLime,lcYellow,lcBlue,lcFuchsia,lcAqua,lcWhite);
 TLedOffColors = (lcBlack,lcMaroon,lcGreen,lcOlive,lcNavy,lcPurple,lcTeal,lcGray);
 TActualColors = array[0..7] of TColor;

const
 ActualOnColors: TActualColors=(clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,clAqua,clWhite);
 ActualOffColors:TActualColors=(clBlack,clMaroon,clGreen,clOlive,clNavy,clPurple,clTeal,clGray);
type
 TLedAuto = class(TCustomControl)
  private
   { Private declarations }
   FLedPiccy: TBitmap;
   FLedIsOn: boolean;
   FLedRect: TRect;
   FLedBulbRect: TRect;
   FAutoSelectColor: boolean;
   FLedOnColor: TLedOnColors;
   FLedOffColor: TLedOffColors;
   FLedStatus: TLedStatus;
   FLedType: TLedType;
   FDescrip: String;
   FBus: Integer;
   FValue: Integer;
   FGroup: String;
   procedure SetLedOffColor(value: TLedOffColors);
   procedure SetLedOnColor(value: TLedOnColors);
   procedure SetLedStatus(value: TLedStatus);
   procedure SetLedType(value: TLedType);
   protected
 { Protected declarations }
   procedure Paint; override;
   procedure Click; override;
   procedure WMSize(var Message: TWMSize); message WM_SIZE;
  public
   { Public declarations }
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
  published
   {Published declarations }
   property AutoSelectColor: boolean read FAutoSelectColor write FAutoSelectColor default True;
   property LedOnColor: TLedOnColors read FLedOnColor write SetLedOnColor default lcRed;
   property LedOffColor: TLedOffColors read FLedOffColor write SetLedOffColor default lcMaroon;
   property LedStatus: TLedStatus read FLedStatus write SetLedStatus default lsOff;
   property LedType: TLedType read FLedType write SetLedType default ltOut;
   property Descrip: String read FDescrip write FDescrip;
   property Bus: Integer read FBus write FBus;
   property Value: Integer read FValue write FValue;
   property Group: String read FGroup write FGroup;
   property OnClick;
   property ShowHint;
  end;

procedure Register;

implementation
{------------------------------------------------------------------------------}
constructor TLedAuto.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 FLedPiccy := TBitmap.Create;
 FLedPiccy.Handle := LoadBitmap(hInstance, 'GENERICLED');
 with FLedRect do
  begin
   Top := 0;
   Left := 0;
   Right := FLedPiccy.Width;
   Bottom := FLedPiccy.Height;
  end;
  {This TRect stores dimensions for 'filament' of LED}
  with FLedBulbRect do
   begin
    Top := 5;
    Left := 5;
    Right := 10;
    Bottom := 10;
   end;
  {Set defaults}
  FAutoSelectColor := True;
  Width := FLedPiccy.Width;
  FLedOffColor := lcMaroon;
  FLedOnColor := lcRed;
  FLedStatus := lsOff;
  FLedType:= ltOut;
  FLedIsOn := false;
  Height := FLedPiccy.Height;
  FValue:=0;
  FDescrip:='';
  FBus:=128;
  FGroup:='';
end;
{------------------------------------------------------------------------------}
destructor TLedAuto.Destroy;
begin
 FLedPiccy.Free;
 inherited Destroy;
end;
{------------------------------------------------------------------------------}
procedure TLedAuto.Paint;
begin
 Canvas.Brush.Color := (Parent as TWinControl).Brush.Color;
 Canvas.BrushCopy(FLedRect,FLedPiccy, FLedRect, clOlive);
 with Canvas do
 begin
  {Draw colored section of led}
  if FLedIsOn then
     Brush.Color := ActualOnColors[ord(FLedOnColor)]
  else
     Brush.Color := ActualOffColors[ord(FLedOffColor)];
     BrushCopy(FLedBulbRect,FLedPiccy, FLedBulbRect, clMaroon);
 end;
end;
{------------------------------------------------------------------------------}
procedure TLedAuto.WMSize(var Message: TWMSize);
begin
 {Keep control the same size as the LED bitmap}
 inherited;
 Width := FLedPiccy.Width;
 Height := FLedPiccy.Height;
end;
{------------------------------------------------------------------------------}
procedure TLedAuto.SetLedOffColor(value: TLedOffColors);
begin
 FLedOffColor := value;
 if FAutoSelectColor then
    FLedOnColor := TLedOnColors(FLedOffColor);
    Invalidate;
end;
{------------------------------------------------------------------------------}
procedure TLedAuto.SetLedOnColor(value: TLedOnColors);
begin
 FLedOnColor := value;
 if FAutoSelectColor then
    FLedOffColor:= TLedOffColors(FLedOnColor);
    Invalidate;
end;
{------------------------------------------------------------------------------}
procedure TLedAuto.Click;
begin
 inherited Click;
 {internal code for 'click' led }
 if FLedType = ltOut then
   begin
    if FLedStatus = lsOff then FLedStatus:= lsOn else FLedStatus:=lsOff;
    FLedIsOn := not FLedIsOn;
    Invalidate;
   end;
end;
{------------------------------------------------------------------------------}
procedure TLedAuto.SetLedStatus(value: TLedStatus);
begin
  if value <> FLedStatus then
     begin
      FLedStatus := value;
       case FLedStatus of
        lsOff: FLedIsOn := false;
        lsOn:  FLedIsOn := true;
       end;
       Invalidate;
      end;
end;
{------------------------------------------------------------------------------}
procedure TLedAuto.SetLedType(value: TLedType);
begin
  if value <> FLedType then FLedType := value;
  Invalidate;
end;
{------------------------------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('MisComp', [TLedAuto]);
end;

end.
