{ A unit to allow display of bitmaps in TComoboboxes and TListboxes
  Use an ImageList to store the pictures: assign the ImageList to
  the components ImageList property, write something in the Items property and go!

  Freeware.
  Copyright @ 1997 by Peter Thrnqvist
  Contact: NMA96PTH@STUDENT2.LU.SE or
           DAT97PTH@LUDAT.LTH.SE
}

unit ListComb;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls;

type
  TImageComboBox = class(TCustomComboBox)
  private
    { Private declarations }
    FImages:TImagelist;
    FWidth,FHeight:integer;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure ResetItemHeight;
  protected
    { Protected declarations }
    FOnChange: TNotifyEvent;
    procedure CreateWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure MeasureItem(Index: Integer;var Height: Integer);override;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure Change; override;
    procedure SetImages(Value:TImageList);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
    property ImageList:TImageList read FImages write SetImages;
    property Items;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property Tag;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

{ TImageListBox }
 TTextAlign=(ttaTop,ttaCenter,ttaBottom);
 TImageAlign=(tiaLeft,tiaCenter,tiaRight);
TImageListBox = class(TCustomListBox)
  private
    { Private declarations }
    ImLeftOff,ImTopOff: integer;
    TxtFlags: integer;
    FImages:TImagelist;
    FWidth:integer;
    FHeight:integer;
    FImageAlign:TImageAlign;
    FTextAlign:TTextAlign;
//    procedure SetImageAlign(Value:TImageAlign);
//    procedure SetTextAlign(Value:TTextAlign);
//    procedure SetAllAlign(var Rect:TRect);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure ResetItemHeight;
    procedure SetImages(Value:TImageList);
  protected
    { Protected declarations }
    FOnChange: TNotifyEvent;
    procedure CreateWnd; override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure MeasureItem(Index: Integer;var Height: Integer);override;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure Change; dynamic;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Align;
//    property AlignImage:TImageAlign read FImageAlign write SetImageAlign;
//    property AlignText:TTextAlign read FTextAlign write SetTextAlign;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
    property ImageList:TImageList read FImages write SetImages;
    property IntegralHeight;
    property ItemHeight;
    property Items;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
    property Sorted;
    property Tag;
  end;


procedure Register;

implementation

function GetItemHeight(Font: TFont): Integer;
var DC: HDC; SaveFont: HFont; Metrics: TTextMetric;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  Result := Metrics.tmHeight;
end;

{ TImageComboBox }

constructor TImageComboBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWidth := 0;
  FHeight := 0;
  Style := csOwnerDrawFixed;
  Color := clTeal;
  ResetItemHeight;
end;


procedure TImageComboBox.SetImages(Value:TImageList);
begin
	if FImages <> Value then
 begin
   FImages := Value;
   FWidth := FImages.Width;
   FHeight := FImages.Height;
   ResetItemHeight;
   RecreateWnd;
 end;
end;


destructor TImageComboBox.Destroy;
begin
  inherited Destroy;
end;



procedure TImageComboBox.CreateWnd;
begin
  inherited CreateWnd;
end;

procedure TImageComboBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var Offset:integer;
begin
  with Canvas do
  begin
    FillRect(Rect);
    Offset := FWidth + 8;
    if Assigned(FImages) and (FImages.Count >= Index) then
    begin
      FImages.DrawingStyle := dsTransparent;
      FImages.Draw(Canvas,Rect.Left + 2,Rect.Top,Index);
      Rect.Left := Rect.Left + Offset;
    end;
    DrawText(Canvas.Handle, PChar(Items[Index]), -1, Rect,
             DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
  end;
end;

procedure TImageComboBox.MeasureItem(Index: Integer;var Height: Integer);
begin
  Height:= ItemHeight;
end;


procedure TImageComboBox.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ResetItemHeight;
  RecreateWnd;
end;

procedure TImageComboBox.ResetItemHeight;
var
  nuHeight: Integer;
begin
  nuHeight :=  GetItemHeight(Font) + 4;
  if nuHeight < (FHeight + 4) then nuHeight := FHeight + 4;
  ItemHeight := nuHeight;
end;


procedure TImageComboBox.Change;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;


procedure TImageComboBox.CNCommand(var Message: TWMCommand);
begin
inherited;
   case Message.NotifyCode of
{    CBN_EDITCHANGE:
      Change;}
    CBN_SELCHANGE:
        Change;
  end;
end;


{ TImageListBox }

constructor TImageListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  Color := clTeal;
  Height := 300;
  Width := 120;
  FWidth := 0;
  FHeight := 0;
//  SetImageAlign(tiaCenter);
//  SetTextAlign(ttaBottom);
  Style := lbOwnerDrawFixed;
  IntegralHeight := True;
  ResetItemHeight;
end;

destructor TImageListBox.Destroy;
begin
  inherited Destroy;
end;

procedure TImageListBox.SetImages(Value:TImageList);
begin
 if FImages <> Value then
 begin
 	FImages := Value;
   FWidth := FImages.Width;
   FHeight := FImages.Height;
   ResetItemHeight;
   RecreateWnd;
 end;
end;


procedure TImageListBox.CreateWnd;
begin
  inherited CreateWnd;
  SetBkMode(Canvas.Handle,TRANSPARENT);

end;

procedure TImageListBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var Tmp:integer;
begin
  with Canvas do
  begin
    FillRect(Rect);
    if Assigned(FImages) and (FImages.Count > 0) then
    begin
//      SetAllAlign(Rect);
      tmp := (Rect.Right - Rect.Left) div 2 - FWidth div 2;
      FImages.Draw(Canvas,Rect.Left + {ImLeftOff} tmp,Rect.Top + {ImTopOff} 4,Index);
      InflateRect(Rect,1,-8);
    end;
    DrawText(Canvas.Handle, PChar(Items[Index]), -1, Rect,
             DT_SINGLELINE or DT_NOPREFIX or {TxtFlags} DT_CENTER or DT_BOTTOM);
  end;
end;

procedure TImageListBox.MeasureItem(Index: Integer;var Height: Integer);
begin
  Height:= ItemHeight;
end;

procedure TImageListBox.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ResetItemHeight;
  RecreateWnd;
end;

procedure TImageListBox.ResetItemHeight;
var FFont : integer;
begin
  case FTextAlign of
    ttaTop,ttaBottom: ItemHeight := GetItemHeight(Font)  + FHeight + 8;
    ttaCenter:        ItemHeight := FHeight + 8;
  end;
  FFont := GetItemHeight(Font);
  if ItemHeight < FFont then ItemHeight := FFont;
end;


procedure TImageListBox.Change;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;


procedure TImageListBox.CNCommand(var Message: TWMCommand);
begin

inherited;
   case Message.NotifyCode of
{    CBN_EDITCHANGE:
      Change;}
    CBN_SELCHANGE:
        Change;
  end;
end;
{
procedure TImageListBox.SetImageAlign(Value:TImageAlign);
begin
	if FImageAlign <> Value then
 begin
   FImageAlign := Value;
   ResetItemHeight;
   RecreateWnd;
 end;
end;
}
{
procedure TImageLIstBox.SetTextAlign(Value:TTextAlign);
begin
	if FTextAlign <> Value then
 begin
   FTextAlign := Value;
   ResetItemHeight;
   RecreateWnd;
 end;
end;
}
{
procedure TImageListBox.SetAllAlign(var Rect:TRect);
begin
  case FImageAlign of
    tiaLeft:
    begin
      ImLeftOff := 4;
      TxtFlags := DT_LEFT;
      InflateRect(Rect,-(FWidth + 4) * 2,1);
    end;
    tiaCenter:
    begin
      ImLeftOff := ((Rect.Right - Rect.Left) div 2) - (FWidth div 2);
      TxtFlags := DT_CENTER;
    end;
    tiaRight:
    begin
      ImLeftOff := ((Rect.Right - Rect.Left) - FWidth) - 8;
      InflateRect(Rect,-4,1);
      TxtFlags := DT_LEFT;
    end;
   end;


  case FTextAlign of
    ttaTop:
    begin
      ImTopOff := GetItemHeight(Font) + 4;
//      InflateRect(Rect,1,-4);
      TxtFlags := TxtFlags or DT_TOP;
    end;
    ttaCenter:
    begin
      ImTopOff := 4;
      TxtFlags := TxtFlags or DT_CENTER;
    end;
    ttaBottom:
    begin
      ImTopOff := 4;
//      InflateRect(Rect,1,-4);
      TxtFlags := TxtFlags or DT_BOTTOM;
    end;
  end;

//   ResetItemHeight;
//   RecreateWnd;
end;
}
procedure Register;
begin
  RegisterComponents('Samples', [TImageComboBox,TImageListBox]);
end;

end.
