{**************************************************************************

  LongList

  This unit defines two classes TLongListbox, TLongOutline that
  show the whole string of an entry, when the mouse is moved over,
  like the Win95 TreeView component.

  Just install it to your Component-Library.

  Author: Hannes Danzl (e9026733@stud3.tuwien.ac.at)

  FREEWARE

  If you change it, please send me a copy!!

************************************************************************** }

unit longlist;

interface

uses
  Winprocs, wintypes,Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, outline, grids;

type
  TLongEdit= class (TEdit)
  private
    procedure wmSetFocus(var msg:TWmSetFocus); message wm_setfocus;
    procedure wmLMouseDown(var msg:TWMLButtonDown);message wm_LButtonDown;
    procedure wmLButtonDblClk(var msg:TWMLButtonDblClk);message wm_LButtonDblClk;
    procedure wmLButtonUp(var msg:TWMLButtonUp);message wm_LButtonUp;
  public
    TheList:TComponent;
  end;

  TLongListbox = class(TListBox)
  private
    { Private-Deklarationen }
    LongLabel:TLongEdit;
    OldIndex:integer;
    fBackColor:TColor;
    fTextColor:TColor;
    fOnlyWhenFocused:Boolean;
    OldFormMouseMove:TMouseMoveEvent;
  protected
    { Protected-Deklarationen }
    procedure Loaded;override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer);override;
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent);override;
  published
    { Published-Deklarationen }
    property OnlyWhenFocused:Boolean read FOnlyWhenFocused write fOnlyWhenFocused;
    property BackColor:TColor read FBackColor write fBackColor;
    property TextColor:TColor read FTextColor write fTextColor;
  end;

  TLongOutline = class(TOutline)
  private
    { Private-Deklarationen }
    OldIndex:integer;
    fBackColor:TColor;
    fTextColor:TColor;
    fOnlyWhenFocused:Boolean;
    OldFormMouseMove:TMouseMoveEvent;
    procedure wmKillFocus(var msg:TWmKillFocus); message wm_Killfocus;
  protected
    { Protected-Deklarationen }
    procedure Loaded;override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer);override;
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  public
    { Public-Deklarationen }
    LongLabel:TLongEdit;
    constructor Create(AOwner: TComponent);override;
  published
    { Published-Deklarationen }
    property OnlyWhenFocused:Boolean read FOnlyWhenFocused write fOnlyWhenFocused;
    property BackColor:TColor read FBackColor write fBackColor;
    property TextColor:TColor read FTextColor write fTextColor;
  end;

procedure Register;

implementation

procedure TLongEdit.wmSetFocus(var msg:TWmSetFocus);
begin
end;

procedure TLongEdit.wmLMouseDown(var msg:TWMLButtonDown);
var apoint:TPoint;
begin
  if TheList is TLongListbox then
  begin
    apoint:=point(msg.xpos,msg.ypos);
    apoint:=clienttoscreen(apoint);
    apoint:=TLongListbox(TheList).ScreentoClient(apoint);
    sendmessage(TLongListbox(TheList).handle,msg.msg,msg.keys,makelong(apoint.x,apoint.y));
  end;
  if TheList is TLongOutline then
  begin
    apoint:=point(msg.xpos,msg.ypos);
    apoint:=clienttoscreen(apoint);
    apoint:=TLongOutline(TheList).ScreentoClient(apoint);
    sendmessage(TLongOutline(TheList).handle,msg.msg,msg.keys,makelong(apoint.x,apoint.y));
  end;
end;

procedure TLongEdit.wmLButtonUp(var msg:TWMLButtonUp);
var apoint:TPoint;
begin
  if TheList is TLongListbox then
  begin
    apoint:=point(msg.xpos,msg.ypos);
    apoint:=clienttoscreen(apoint);
    apoint:=TLongListbox(TheList).ScreentoClient(apoint);
    sendmessage(TLongListbox(TheList).handle,msg.msg,msg.keys,makelong(apoint.x,apoint.y));
  end;
  if TheList is TLongOutline then
  begin
    apoint:=point(msg.xpos,msg.ypos);
    apoint:=clienttoscreen(apoint);
    apoint:=TLongOutline(TheList).ScreentoClient(apoint);
    postmessage(TLongOutline(TheList).handle,msg.msg,msg.keys,makelong(apoint.x,apoint.y));
  end;
end;

procedure TLongEdit.wmLButtonDblClk(var msg:TWMLButtonDblClk);
var apoint:TPoint;
begin
  if TheList is TLongListbox then
  begin
    apoint:=point(msg.xpos,msg.ypos);
    apoint:=clienttoscreen(apoint);
    apoint:=TLongListbox(TheList).ScreentoClient(apoint);
    sendmessage(TLongListbox(TheList).handle,msg.msg,msg.keys,makelong(apoint.x,apoint.y));
  end;
  if TheList is TLongOutline then
  begin
    apoint:=point(msg.xpos,msg.ypos);
    apoint:=clienttoscreen(apoint);
    apoint:=TLongOutline(TheList).ScreentoClient(apoint);
    postmessage(TLongOutline(TheList).handle,msg.msg,msg.keys,makelong(apoint.x,apoint.y));
  end;
end;

constructor TLongListbox.Create(AOwner: TComponent);
begin
  inherited Create(aOwner);
  LongLabel:=nil;
  OldIndex:=-1;
  fOnlyWhenFocused:=false;
  FBackColor:=$00EAFFFF;
  FTextColor:=clBlack;
end;

procedure TLongListbox.Loaded;
var aform:TForm;
    amousemove:TMouseMoveEvent;
begin
  inherited Loaded;
  aform:=TForm(GetParentForm(self));
  if aform<>nil then
  begin
    aMousemove:=aform.OnMouseMove;
    if assigned(aMousemove) then
      if @aMousemove<>@OldFormMouseMove then
        OldFormMouseMove:=aform.OnMouseMove
    else
      OldFormMouseMove:=nil;
    aform.OnMouseMove:=FormMouseMove;
  end;
end;

procedure TLongListbox.MouseMove(Shift: TShiftState; X, Y: Integer);
var apoint:tpoint;
    anitemindex:longint;
    theitemrect:Trect;
    ahandle:HWND;
begin
  inherited MouseMove(shift,x,y);
  ahandle:=getfocus;
  if (fOnlyWhenFocused) and (ahandle<>handle) then
    exit;
  apoint:=point(x,y);
  anitemindex:=itematpos(apoint,true);
  if oldindex<>anitemindex then
  begin
    oldindex:=anitemindex;
    if anitemindex>-1 then
    begin
      if LongLabel<>nil then
      begin
        LongLabel.Free;
        LongLabel:=nil;
      end;
      if canvas.textwidth(items[anitemindex])>width then
      begin
        LongLabel:=TLongEdit.Create(self);
        LongLabel.TheList:=self;
        theitemrect:=itemrect(anitemindex);
        LongLabel.left:=left+theitemrect.left+1;
        LongLabel.top:=top+theitemrect.top;
        LongLabel.Parent:=Getparentform(self);
        LongLabel.Borderstyle:=bsSingle;
        LongLabel.Color:=fBackColor;
        LongLabel.Font.Color:=fTextColor;
        LongLabel.readonly:=true;
        LongLabel.BringToFront;
        LongLabel.Font.assign(Font);
        Canvas.Font.Assign(Font);
        LongLabel.Width:=Canvas.TextWidth(items[anitemindex])+8;
        Longlabel.Height:=Canvas.TextHeight(items[anitemindex])+4;
        LongLabel.Text:=items[anitemindex];
        LongLabel.Show;
      end;
    end;
  end;
end;

procedure TLongListbox.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if assigned(OldFormMouseMove) then
    OldFormMouseMove(sender,shift,x,y);
  if LongLabel<>nil then
  begin
    LongLabel.Free;
    LongLabel:=nil;
    oldindex:=-1;
  end;
end;

constructor TLongOutline.Create(AOwner: TComponent);
begin
  inherited Create(aOwner);
  LongLabel:=nil;
  OldIndex:=-1;
  fOnlyWhenFocused:=false;
  FBackColor:=$00EAFFFF;
  FTextColor:=clBlack;
end;

procedure TLongOutline.MouseMove(Shift: TShiftState; X, Y: Integer);
var apoint:tpoint;
    anitemindex:longint;
    theitemrect:Trect;


function ItemRect(anindex,y:longint):Trect;
  var theindex:longint;
      a:longint;
  begin
    theindex:=anindex;
    for a:=1 to anindex-1 do
      if items[a].isvisible=false then
        dec(theindex);
    result:=cellrect(0,theindex-1);
    case OutlineStyle of
      osText: result.Left:=result.Left+DefaultRowHeight * (items[anindex].level-1);
      osPlusMinusText: result.Left:=result.Left+DefaultRowHeight * (items[anindex].level);
      osPlusMinusPictureText: result.Left:=result.Left+DefaultRowHeight * (items[anindex].level+1);
      osPictureText: result.Left:=result.Left+DefaultRowHeight * (items[anindex].level);
      osTreeText: result.Left:=result.Left+DefaultRowHeight * (items[anindex].level-1);
      osTreePictureText: result.Left:=result.Left+DefaultRowHeight * (items[anindex].level);
    end;
    if ooDrawTreeRoot in Options then Inc(Result.Left, DefaultRowHeight);
  end;

begin
  inherited MouseMove(shift,x,y);
  if (fOnlyWhenFocused) and (not getfocus=handle) then
    exit;
  apoint:=point(x,y);
  anitemindex:=getitem(x,y);
  if oldindex<>anitemindex then
  begin
    oldindex:=anitemindex;
    if anitemindex>0 then
    begin
      if LongLabel<>nil then
      begin
        LongLabel.Free;
        LongLabel:=nil;
      end;
      theitemrect:=itemrect(anitemindex, y);
      if (canvas.textwidth(items[anitemindex].text)+theitemrect.left)>width-4 then
      begin
        LongLabel:=TLongEdit.Create(self);
        LongLabel.TheList:=self;
        LongLabel.left:=left+theitemrect.left+1;
        LongLabel.top:=top+theitemrect.top;
        LongLabel.Parent:=Getparentform(self);
        LongLabel.Borderstyle:=bsSingle;
        LongLabel.Color:=fBackColor;
        LongLabel.Font.Color:=fTextColor;
        LongLabel.readonly:=true;
        LongLabel.BringToFront;
        LongLabel.Font.assign(Font);
        Canvas.Font.Assign(Font);
        LongLabel.Width:=Canvas.TextWidth(items[anitemindex].text)+8;
        Longlabel.Height:=Canvas.TextHeight(items[anitemindex].text)+4;
        LongLabel.Text:=items[anitemindex].text;
        LongLabel.Show;
      end;
    end;
  end;
end;

procedure TLongOutline.wmKillFocus(var msg:TWmKillFocus);
begin
  if LongLabel<>nil then
  begin
    LongLabel.Free;
    LongLabel:=nil;
    oldindex:=-1;
  end;
end;

procedure TLongOutline.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if assigned(OldFormMouseMove) then
    OldFormMouseMove(sender,shift,x,y);
  if LongLabel<>nil then
  begin
    LongLabel.Free;
    LongLabel:=nil;
    oldindex:=-1;
  end;
end;

procedure TLongOutline.Loaded;
var aform:TForm;
    amousemove:TMouseMoveEvent;
begin
  inherited Loaded;
  aform:=TForm(GetParentForm(self));
  if aform<>nil then
  begin
    aMousemove:=aform.OnMouseMove;
    if assigned(aMousemove) then
      if @aMousemove<>@OldFormMouseMove then
        OldFormMouseMove:=aform.OnMouseMove
    else
      OldFormMouseMove:=nil;
    aform.OnMouseMove:=FormMouseMove;
  end;
end;

procedure Register;
begin
  RegisterComponents('Beispiele', [TLongListbox, TLongOutline]);
end;

end.
