unit MVGrid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids;

type
  PCellData = ^TCellData;

  TCellData = record
    Marked: boolean;
    Col: Longint;
    Row: Longint;
    Next: PCellData;
  end;

  PValueCelula = ^TValueCelula;

  TValueCelula = record
    Value: variant;
    Col: longint;
    Row: longint;
    Next: PValueCelula;
  end;

  TMVStringGrid = class(TStringGrid)
  private
    { Private declarations }
    FMarkColor: TColor;
    FListStart: PCellData;
    FListPointer: PCellData;
    FFirstValue: PValueCelula;
    FValuePointer: PValueCelula;
    FRightClickSelect: Boolean;
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect;
      AState: TGridDrawState); override;
    procedure GetMarkColor(Color: Tcolor);
    function GetValue(ACol, ARow: longint): variant;
    procedure SetValue(ACol, ARow: longint; const Value: variant);
    procedure SetMark(ACol, ARow: Longint; Value :Boolean);
    function GetMark(ACol, ARow: Longint): Boolean;
    procedure WMRButtonDown(var Info: TWMRButtonDown); message WM_RBUTTONDOWN;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Value[ACol, ARow: longint]: variant read GetValue write SetValue;
    property Mark[ACol, ARow: longint]: Boolean read GetMark write SetMark;
  published
    { Published declarations }
    property MarkColor: TColor read FMarkColor write GetMarkColor default clRed;
    property RightClickSelect: Boolean read FRightClickSelect write FRightClickSelect default false;
  end;

procedure Register;

implementation

constructor TMVStringGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  New(FListStart);
  FListStart^.Next := nil;
  New(FFirstValue);
  FFirstValue^.Next := nil;
end;

destructor TMVStringGrid.Destroy;
begin
  FListPointer := FListStart^.Next;
  while FListPointer <> nil do
  begin
    FListStart^.Next := FListPointer^.Next;
    Dispose(FListPointer);
    FListPointer := FListStart^.Next;
  end;
  Dispose(FListStart);

  FValuePointer := FFirstValue^.Next;
  while FListPointer <> nil do
  begin
    FFirstValue^.Next := FValuePointer^.Next;
    Dispose(FValuePointer);
    FValuePointer := FFirstValue^.Next;
  end;
  Dispose(FFirstValue);

  inherited Destroy;

end;

function TMVStringGrid.GetValue(ACol, ARow: longint): variant;
begin
  Result := null;
  FValuePointer := FFirstValue^.Next;
  while (FValuePointer <> nil) and (Result = null) do
    begin
      if (FValuePointer^.Col = ACol) and (FValuePointer^.Row = ARow) then
        Result := FValuePointer^.Value
      else
        FValuePointer := FValuePointer^.Next;
    end;
end;

procedure TMVStringGrid.SetValue(ACol, ARow: longint; const Value: variant);
var
  Done: Boolean;
  P   : PValueCelula;
begin
 if Value = null then
  begin
    if GetValue(ACol,ARow) <> null then
    begin
      FValuePointer := FFirstValue^.Next;
      P := FFirstValue;
      while FValuePointer <> nil do
      begin
        if (FValuePointer^.Col = ACol) and (FValuePointer^.Row = ARow) then
          begin
            P^.Next := FValuePointer^.Next;
            Dispose(FValuePointer);
            FValuePointer := nil;
          end
        else
          begin
            P := FValuePointer;
            FValuePointer := FValuePointer^.Next;
          end;
      end;
    end;
  end
 else
  begin
    Done := False;
    FValuePointer := FFirstValue^.Next;
    while (FValuePointer <> nil) and (not Done) do
    begin
      if (FValuePointer^.Col = ACol) and (FValuePointer^.Row = ARow) then
      begin
        FValuePointer^.Value := Value;
        Done := True;
      end;
      FValuePointer := FValuePointer^.Next;
    end;
    if not Done then
    begin
      New(FValuePointer);
      FValuePointer^.Next := FFirstValue^.Next;
      FFirstValue^.Next := FValuePointer;
      FValuePointer^.Col := ACol;
      FValuePointer^.Row := ARow;
      FValuePointer^.Value := Value;
    end;
  end;
end;

procedure TMVStringGrid.SetMark(ACol, ARow: Longint; Value :Boolean);
var
  Done: Boolean;
begin
  if GetMark(ACol,ARow) <> Value then
  begin
    Done := False;
    FListPointer := FListStart^.Next;
    while (FListPointer <> nil) and (not Done) do
    begin
      if (FListPointer^.Col = ACol) and (FListPointer^.Row = ARow) then
      begin
        FListPointer^.Marked := Value;
        Done := True;
      end;
      FListPointer := FListPointer^.Next;
    end;
    if not Done then
    begin
      New(FListPointer);
      FListPointer^.Next := FListStart^.Next;
      FListStart^.Next := FListPointer;
      FListPointer^.Col := ACol;
      FListPointer^.Row := ARow;
      FListPointer^.Marked := Value;
    end;
    Invalidate; //Paint grid as soon as possible;
  end;
end;

function TMVStringGrid.GetMark(ACol, ARow: Longint): Boolean;
begin
  Result := False;
  FListPointer := FListStart^.Next;
  while (FListPointer <> nil) and (Result = False) do
    begin
      if (FListPointer^.Col = ACol) and (FListPointer^.Row = ARow)
        and (FListPointer^.Marked = True) then
        Result := true
      else
        FListPointer := FListPointer^.Next;
    end;
end;

procedure TMVStringGrid.DrawCell(ACol, ARow: Longint; ARect: TRect;
      AState: TGridDrawState);
begin
  if GetMark(ACol,ARow) then
  begin
    Canvas.Brush.Color := FMarkColor;
    Canvas.FillRect(ARect);
  end;
  inherited;
end;

procedure TMVStringGrid.GetMarkColor(Color: Tcolor);
begin
  FMarkColor := Color;
  Invalidate; //Paint grid ASAP;
end;

procedure TMVStringGrid.WMRButtonDown(var Info: TWMRButtonDown);
var
  ACol, ARow: Longint;
begin
  MouseToCell(Info.XPos, Info.YPos, ACol, ARow);
  if (FRightClickSelect) and (ARow >= FixedRows) and (ACol >= FixedCols) then
  begin
    Col := ACol;
    Row := ARow;
  end;
end;

procedure Register;
begin
  RegisterComponents('Mattar', [TMVStringGrid]);
end;

end.
