//--------------------------------------------------------------------------//
// PackView - shows Package required libraries and contained units.         //
//            Generates package source code                                 //
//                                                                          //
//                       Alex G. Fedorov, 1997                              //
//--------------------------------------------------------------------------//
unit PackUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    OpenDialog1: TOpenDialog;
    Memo1: TMemo;
    Panel1: TPanel;
    SpeedButton4: TSpeedButton;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    procedure OpenPackage;
    procedure SpeedButton1Click(Sender: TObject);
    procedure MakeSource;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure SpeedButton4Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
  private
    { Private declarations }
   PkgFlagDesc : String;
  public
    { Public declarations }
  end;
  procedure PackageInfoProc
   (const Name: string; NameType: TNameType; Flags : Byte; Param: Pointer);
  function IsPackage  (Module : HModule) : Bool;
  function GetPkgFlags(Module : HModule) : Integer;
  function GetPkgDesc (Module : HModule) : String;

var
  Form1: TForm1;

implementation

{$R *.DFM}
const
 sPrgName  : String = 'Delphi 3.0 Package Viewer, ver. 0.9a';
 sAuthor   : String = '      by Alex G. Fedorov, 1997     ';
 sPrompt   : String = 'Save source in file?';
 sMsgTitle : String = 'Saving package source';
 sCont     : String = '@';                        // for CONTAINS
 sReq      : String = '#';                        // for REQURES
 NotPkg    : String = 'Not a Delphi 3.0 package'; // Error message
// Hints for interface elements
 OpenHint  : String = 'Open Delphi Package';
 SaveHint  : String = 'Save package source';
 ExitHint  : String = 'Exit Package Viewer';
 HelpHint  : String = 'Get short help';
var
 Module   : THandle;                          // Handle to loaded package
 List     : TStringList;                      // Global storage
 FileName : String;

function IsPackage;
//------------------------------------------------------------------//
// Checks if DLL supplied by handle contains 'PACKAGEINFO' Resource //
//------------------------------------------------------------------//
var
 ResInfo : HRsrc;
begin
 ResInfo := FindResource(Module, 'PACKAGEINFO', RT_RCDATA);
 If ResInfo <> 0 Then Result := True Else Result := False;
end;

//-------------------------------------------------//
// Get Package Flags from PACKAGEINFO resource     //
//-------------------------------------------------//
function GetPkgFlags;
type
 PPkgInfo = ^TPkgInfo;
 TPkgInfo = packed record
  Flags : Integer;
  // to be added later
 end;
var
 ResInfo : HRsrc;
 Data    : THandle;
 PkgInfo : PPkgInfo;

begin
 Result := 0;
 ResInfo := FindResource(Module, 'PACKAGEINFO', RT_RCDATA);
 If ResInfo <> 0 Then
 try
  Data := LoadResource(Module, ResInfo);
  If Data <> 0 Then
   try
    begin
     PkgInfo := LockResource(Data);
     Result := PkgInfo^.Flags;
    end;
   finally
    UnlockResource(Data);
   end;
  finally
   FreeResource(ResInfo);
 end;
end;

function GetPkgDesc(Module : HModule) : String;
begin
 If IsPackage(Module) Then
// At this point we know, that Module is a 32-bit NE-format file,
// so we can skip some additional checkings and dig straight info
// NE header table
  Begin
  End
 Else Result := 'Undefined';
end;

//------------------------------------------//
// Callback procedure for GetPackageInfo    //
//------------------------------------------//
procedure PackageInfoProc;
type
 PStrings = ^TStringList;
var
 Info : String;
begin
  If NameType = ntContainsUnit Then
   Info := sCont Else Info := sReq;
//  If IsUnit Then Info := sCont Else Info := sReq;
  PStrings(Param)^.Add(Info+Name);
//  PStrings(Param)^.Add('Flags='+Format('%d', [Flags]));
end;

//------------------------------------------//
//                                          //
//------------------------------------------//
procedure TForm1.OpenPackage;
var
 Flags    : Integer;
 I        : Byte;
 S        : String;
begin
 FileName := OpenDialog1.FileName;

// Upper case file name
 For I := 1 to Length(FileName) do
  FileName[I] := Upcase(FileName[I]);

 Module := LoadLibrary(PChar(FileName));
 If Module > 32 Then
  Begin
   If IsPackage(Module) Then
   Begin
    List.Clear;
    GetPackageInfo(Module, @List, Flags, PackageInfoProc);
{!! Check FLAGS !!}
    Flags := GetPkgFlags(Module);
    S := '';
    If Flags <> 0 Then
     Begin
      If Flags AND $01  <> 0 Then S := S + '{$IMPLICITBUILD OFF}'^M^J;
      If Flags AND $02  <> 0 Then S := S + '{$DESIGNONLY ON}'^M^J;
      If Flags AND $04  <> 0 Then S := S + '{$RUNONLY ON}'^M^J;
      PkgFlagDesc := S;
     End;
{!!}
    MakeSource;                        //Fill Memo with Package Source
   End
    Else
      ShowMessage(NotPkg);
    FreeLibrary(Module);
  End;
end;

//------------------------------------
// Open package with standard dialog
//------------------------------------
procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
 If OpenDialog1.Execute Then
  Begin
   OpenPackage;
  End;
end;

//---------------------
// Show package source
// TODO:
// - add description
// - add directive
//---------------------
procedure TForm1.MakeSource;
var
 I        : Byte;
 S        : String;
 Line     : String;
begin
 With Memo1.Lines do
  Begin
   Clear;
   Add('//');
   Add('//Package source for '+ ExtractFileName(FileName));
   Add('//');
//
// Add description
// Need some hack to find description in DLL binary file
//
   Add('{$DESCRIPTION '+GetPkgDesc(Module)+'}');
// Add compiler directive
   If PkgFlagDesc <> '' Then Add(PkgFlagDesc);
   Add('PACKAGE '+ChangeFileExt(ExtractFileName(FileName), '')+';');
// Fill in Requires section
   Line := '';
   For I := 0 to List.Count -1 do
    Begin
     If Pos(sReq, List.Strings[I]) > 0 Then
      Begin
        S := List.Strings[I];
        System.Delete(S, 1, Length(sReq));
        Line := Line + S + ', ';
      End;
    End;
   System.Delete(Line, Length(Line)-1, 2);
   If Line <> '' Then
    Begin
     Add('REQUIRES');
     Add('  '+Line+';');
    End;
// Fill in Contains section
   Add('CONTAINS');
   Line := '';
   For I := 0 to List.Count-1 do
    Begin
     If Pos(sCont, List.Strings[I]) > 0 Then
      Begin
        S := List.Strings[I];
        System.Delete(S, 1, Length(sCont));
        Line := Line + S + ', ';
      End;
    End;
   System.Delete(Line, Length(Line)-1, 2);
   Add('  '+Line+';');
   Add('END.');
// Flags..
{   For I := 0 to List.Count-1 do
    begin
     If Pos('Flags=', List.Strings[I]) > 0 Then
      Begin
       Add(List.Strings[I]);
      End;
    end;
}
  End;
  SpeedButton2.Visible := True;
End;

procedure TForm1.FormCreate(Sender: TObject);
begin
 List := TStringList.Create;
 Caption := sPrgName;
 SpeedButton2.Visible := False;
 Application.HintColor := $f0fff0;
// Assign hint text
 SpeedButton1.Hint := OpenHint;
 SpeedButton2.Hint := SaveHint;
 SpeedButton3.Hint := ExitHint;
 SpeedButton4.Hint := HelpHint;

end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 List.Free;
end;

//--------------------------
// Write source to file
//--------------------------
procedure TForm1.SpeedButton2Click(Sender: TObject);
begin
 If MessageBox(Handle, PChar(sPrompt),PChar(sMsgTitle), mb_OkCancel) = IDOK
  Then
   Memo1.Lines.SaveToFile(ChangeFileExt(ExtractFileName(FileName), '.SRC'));
end;

//--------------------------
// Exit from program
//--------------------------
procedure TForm1.SpeedButton3Click(Sender: TObject);
begin
 Close;
end;

//--------------------------
// Show some stuff
//--------------------------
procedure TForm1.SpeedButton4Click(Sender: TObject);
begin
 ShowMessage(sPrgName+#13#10+sAuthor);
end;

end.
