unit PageControl98;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CommCtrl, ComCtrls, CommCtrlEx;

type
  TTabPosition = (tpTop, tpBottom, tpLeft, tpRight);
  TTabType = (ttDefault, ttOpposite);
  TTabStyle = (tsDefault, tsBoutons);
  TTabJustification = (tjLeftText, tjCenterText);
  TTabDrawStyle = (tdDefault, tdOwnerDrawFixed);

  TDrawTabEvent = procedure(Page: TPageControl; Tab: TTabSheet;
                            const Rect: TRect) of object;

  TPageControl98 = class(TPageControl)
  private
    FImages: TImageList;
    FImageChangeLink: TChangeLink;
    FTabType: TTabType;
    FTabPosition: TTabPosition;
    FTabStyle: TTabStyle;
    FTabJustification: TTabJustification;
    FHotTrack: Boolean;
    FMultiSelect: Boolean;
    FDrawStyle: TTabDrawStyle;
    FCanvas: TCanvas;
    FOnDrawTab: TDrawTabEvent;
    FDefaultDrawing: Boolean;
    procedure ImageListChange (Sender: TObject);
    procedure SetImages (Value: TImageList);
    procedure SetImage (Index: Integer; imIndex: Integer);
    function GetImage (Index: Integer): Integer;
    procedure SetTabPosition (Value: TTabPosition);
    procedure SetTabType (Value: TTabType);
    procedure SetTabStyle (Value: TTabStyle);
    procedure SetTabJustification (Value: TTabJustification);
    procedure SetHotTrack (Value: Boolean);
    procedure SetDrawStyle (Value: TTabDrawStyle);
    procedure SetMultiSelect (Value: Boolean);
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure CNNotify(var Message: TWMNotify); message CN_NOTIFY;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DrawTab(Tab: TTabSheet; const Rect: TRect); dynamic;
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;

    property Image[Index: Integer]: Integer read GetImage write SetImage; default;
  published
    property DefaultDrawing: Boolean read FDefaultDrawing write FDefaultDrawing;
    property DrawStyle: TTabDrawStyle read FDrawStyle write SetDrawStyle;
    property Images: TImageList read FImages write SetImages;
    property MultiSelect: Boolean read FMultiSelect write SetMultiSelect;
    property TabPosition: TTabPosition read FTabPosition write SetTabPosition;
    property TabType: TTabType read FTabType write SetTabType;
    property TabStyle: TTabStyle read FTabStyle write SetTabStyle;
    property TabJustification: TTabJustification read FTabJustification write SetTabJustification;
    property HotTrack: Boolean read FHotTrack write SetHotTrack;
    property OnDrawTab: TDrawTabEvent read FOnDrawTab write FOnDrawTab;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Exemples', [TPageControl98]);
end;

procedure TPageControl98.CNNotify(var Message: TWMNotify);
begin
  with Message.NMHdr^ do
    case code of
      TCN_SELCHANGE:
        Change;
      TCN_SELCHANGING:
        begin
          Message.Result := 1;
          if CanChange then Message.Result := 0;
        end;
      NM_HOVER :
        begin
          MessageBeep(MB_Ok);
        end;
    end;
end;

procedure TPageControl98.DrawTab(Tab: TTabSheet; const Rect: TRect);
Var LogRec: TLOGFONT;
    OldFont,
    NewFont: HFONT;
begin
  with FCanvas do begin
    if DefaultDrawing then begin
      FCanvas.FillRect(Rect);
      if ( TabPosition in [tpLeft, tpRight] ) then begin
        GetObject(Font.Handle, SizeOf(LogRec), @LogRec);
        if ( TabPosition = tpLeft ) then
          LogRec.lfEscapement := 900
        else
          LogRec.lfEscapement := -900;
        LogRec.lfOutPrecision := OUT_TT_ONLY_PRECIS;
        NewFont:= CreateFontIndirect(LogRec);
        OldFont:= SelectObject(FCanvas.Handle,NewFont);
      end;
      if ( TabPosition in [tpLeft, tpRight] ) then begin
        FCanvas.TextOut(Rect.Left + (Rect.Right - Rect.Left - FCanvas.TextHeight(Tab.Caption)) Div 2,
                        Rect.Top + 1 + (Rect.Bottom - Rect.Top - FCanvas.TextWidth(Tab.Caption)) Div 2, Tab.Caption);
        NewFont:= SelectObject(FCanvas.Handle,OldFont);
        DeleteObject(NewFont);
      end
      else
        FCanvas.TextOut(Rect.Left + (Rect.Right - Rect.Left - FCanvas.TextWidth(Tab.Caption)) Div 2,
                        Rect.Top + 1 + (Rect.Bottom - Rect.Top - FCanvas.TextHeight(Tab.Caption)) Div 2, Tab.Caption);
    end;
  end;
  if Assigned(FOnDrawTab) then
    FOnDrawTab(Self, Tab, Rect);
end;

procedure TPageControl98.CNDrawItem(var Message: TWMDrawItem);
var
  SaveIndex: Integer;
begin
  with Message.DrawItemStruct^ do
  begin
    SaveIndex := SaveDC(hDC);
    FCanvas.Handle := hDC;
    FCanvas.Font := Font;
    FCanvas.Brush.Color:= clBtnFace;
    FCanvas.Brush.Style := bsSolid;
    DrawTab(Pages[itemID], rcItem);
    FCanvas.Handle := 0;
    RestoreDC(hDC, SaveIndex);
  end;
  Message.Result := 1;
end;

procedure TPageControl98.SetMultiSelect (Value: Boolean);
begin
  if ( FMultiSelect <> Value ) then begin
    FMultiSelect:= Value;
    RecreateWnd;
  end;
end;

procedure TPageControl98.SetTabPosition (Value: TTabPosition);
begin
  if ( FTabPosition <> Value ) then begin
    FTabPosition:= Value;
    RecreateWnd;
  end;
end;

procedure TPageControl98.SetTabType (Value: TTabType);
begin
  if ( FTabType <> Value ) then begin
    FTabType:= Value;
    RecreateWnd;
  end;
end;

procedure TPageControl98.SetTabStyle (Value: TTabStyle);
begin
  if ( FTabStyle <> Value ) then begin
    FTabStyle:= Value;
    RecreateWnd;
  end;
end;

procedure TPageControl98.SetTabJustification (Value: TTabJustification);
begin
  if ( FTabJustification <> Value ) then begin
    FTabJustification:= Value;
    RecreateWnd;
  end;
end;

procedure TPageControl98.SetHotTrack (Value: Boolean);
begin
  if ( FHotTrack <> Value ) then begin
    FHotTrack:= Value;
    RecreateWnd;
  end;
end;

procedure TPageControl98.SetDrawStyle (Value: TTabDrawStyle);
begin
  if ( FDrawStyle <> Value ) then begin
    FDrawStyle:= Value;
    RecreateWnd;
  end;
end;

procedure TPageControl98.CreateParams(var Params: TCreateParams);
begin
  InitTabCommonControlEx;
  inherited CreateParams(Params);
  with Params do begin
    if ( FMultiSelect ) then
      Style:= Style or TCS_MULTISELECT
    else
      Style:= Style and Not( TCS_MULTISELECT );
    if ( FDrawStyle = tdOwnerDrawFixed ) then
      Style:= Style or TCS_OWNERDRAWFIXED
    else
      Style:= Style and Not( TCS_OWNERDRAWFIXED );
    if ( FHotTrack ) then
      Style:= Style or TCS_HOTTRACK
    else
      Style:= Style and Not( TCS_HOTTRACK );
    if ( FTabStyle = tsDefault ) then
      Style:= Style and Not(TCS_BUTTONS)
    else
      Style:= Style or TCS_BUTTONS;
    if ( FTabType = ttOpposite ) then
      Style:= Style or TCS_SCROLLOPPOSITE
    else
      Style:= Style and Not TCS_SCROLLOPPOSITE;
    case FTabJustification of
      tjLeftText   : Style:= Style or TCS_FORCELABELLEFT and Not ( TCS_RIGHTJUSTIFY );
      tjCenterText : Style:= Style and Not ( TCS_FORCELABELLEFT or TCS_RIGHTJUSTIFY );
    end;
    case FTabPosition of
      tpTop    : Style:= Style and Not ( TCS_VERTICAL or TCS_RIGHT or TCS_BOTTOM );
      tpBottom : Style:= Style and Not ( TCS_VERTICAL or TCS_RIGHT) or TCS_BOTTOM;
      tpLeft   : Style:= Style and Not ( TCS_RIGHT or TCS_BOTTOM ) or TCS_VERTICAL or TCS_MULTILINE;
      tpRight  : Style:= Style and Not ( TCS_BOTTOM ) or TCS_RIGHT or TCS_VERTICAL or TCS_MULTILINE;
    end;
  end;
end;

constructor TPageControl98.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FDefaultDrawing:= TRUE;
  FCanvas := TControlCanvas.Create;
  TControlCanvas(FCanvas).Control := Self;
  FImageChangeLink := TChangeLink.Create;
  FImageChangeLink.OnChange := ImageListChange;
end;

destructor TPageControl98.Destroy;
begin
  FImageChangeLink.Free;
  FCanvas.Free;
  inherited Destroy;
end;

procedure TPageControl98.ImageListChange(Sender: TObject);
begin
  if HandleAllocated then
    SendMessage(Handle, TCM_SETIMAGELIST, 0, Longint(TImageList(Sender).Handle));
end;

procedure TPageControl98.SetImages (Value: TImageList);

Var i: Integer;

begin
  if Assigned (FImages) then
    Images.UnRegisterChanges(FImageChangeLink);

  FImages := Value;

  if Assigned (FImages) then
    begin
      Images.RegisterChanges (FImageChangeLink);
      SendMessage(Handle, TCM_SETIMAGELIST, 0, Longint(Images.Handle));
    end
  else
    SendMessage(Handle, TCM_SETIMAGELIST, 0, Longint(0));
  for i:= 0 to PageCount - 1 do begin
    if ( FImages.Count > i ) then
      SetImage( i, i );
  end;
end;

procedure TPageControl98.SetImage (Index: Integer; imIndex: Integer);
var
  imItem: TTCItem;
begin
  imItem.iImage := imIndex;
  imItem.mask := TCIF_IMAGE;
  SendMessage (Handle, TCM_SETITEM, Index, Longint(@imItem));
  Invalidate;
end;

function TPageControl98.GetImage (Index: Integer): Integer;
var
  imItem: TTCItem;
begin
  if Assigned (FImages) then
    begin
      imItem.mask := TCIF_IMAGE;
      SendMessage (Handle, TCM_GETITEM, Index, Longint(@imItem));
      Result := imItem.iImage;
    end
  else
    Result := -1
end;

end.