unit TB97;
{
  Toolbar97 version 1.36
  by Jordan Russell
  email:  jordanr7@aol.com
  www:    http://members.aol.com/jordanr7

  Please do not release modified versions of this source code unless it has
  major differences. If you've made any changes that you think should have
  been there in the first place, feel free to submit them to me so I can
  incorporate them.

  Credits: About 95% of the toolbar code was written by me. I did get a few
  ideas from the freeware XToolBar component's source code (a good component,
  but a little too buggy IMHO).

  Misc. Notes:
  - 'TB97SMOOTH' is where I attempted to add smooth dragging support.
    Didn't finish it though... Try $DEFINEing it and see what happens.
    (Requires Windows NT or Microsoft Plus! with "Show window contents while
    dragging" enabled)
  - 'dropdown' stuff is where I started to do dropdown support, but decided
    it wasn't good enough. I'll finish it soon.

  Summary of changes to source code in this version:

  NOTE: Search for "[MODIFIED]" to see some of the modifications in this
  version I logged. I may have forgot to put that on some.

  1.36 - Fixed the very rare problem of captions being the inactive color when
         it should be the active color. (Modified TToolbar97.RedrawCaption)
       - Fixed problem of floating toolbars sometimes being one pixel too
         tall. (Modified TToolbar97.ArrangeControls)
       - ArrangeControls is now disabled while component is loading, which
         makes it load a lot faster. (Modified TToolbar97.ArrangeControls and
         Loaded)
       - Changed a lot of ClientToScreen/ScreenToClient pairs to
         MapWindowPoints, which is a little faster and more optimized.
       - A couple of other minor fixes.
}

{$ALIGN ON}
{$BOOLEVAL OFF}
{$LONGSTRINGS ON}
{$WRITEABLECONST ON}

interface

uses
  Windows, Messages, Classes, Controls, Forms, Graphics, Buttons, StdCtrls
  {$IFDEF TB97SMOOTH} , ExtCtrls {$ENDIF} ;

const
  Toolbar97Version = '1.36';

type
  { TDock97 }

  TDockBoundLines = set of (blTop, blBottom, blLeft, blRight);
  TDockPosition = (dpTop, dpBottom, dpLeft, dpRight);
  TDockType = (dtNotDocked, dtTopBottom, dtLeftRight);

  TToolbar97 = class;

  TInsertRemoveEvent = procedure (Sender: TObject; Inserting: Boolean;
    Bar: TToolbar97) of object;

  TDock97 = class(TCustomControl)
  private
    { Property values }
    FPosition: TDockPosition;
    FBoundLines: TDockBoundLines;
    FOnInsertRemoveBar: TInsertRemoveEvent;

    { Internal }
    DisableArrangeToolbars: Integer; { Increment to disable ArrangeToolbars }
    DockList: TList;         { List of the visible toolbars docked. Items are casted in TToolbar97's }
    RowInfo: TList;          { List of info on eaxch rows. Items are pointers to TRowInfo's }

    { Property access methods }
    procedure SetBoundLines (Value: TDockBoundLines);
    procedure SetPosition (Value: TDockPosition);

    { Internal }
    procedure FreeRowInfo;
    function GetRowOf (const Y: Integer; var Before: Boolean): Integer;
    function GetDesignModeRowOf (const Y: Integer): Integer;
    function GetHighestRow: Integer;
    procedure RemoveBlankRows;
    procedure InsertRowBefore (const BeforeRow: Integer);
    procedure ChangeDockList (const Insert: Boolean; const Bar: TToolbar97;
      const IsVisible: Boolean);
    procedure ChangeWidthHeight (const NewWidth, NewHeight: Integer);
    procedure BuildRowInfo;
    procedure ArrangeToolbars;
  protected
    procedure AlignControls (AControl: TControl; var Rect: TRect); override;
    procedure Loaded; override;
    procedure SetParent (AParent: TWinControl); override;
    procedure Paint; override;
    procedure WMSize (var Msg: TWMSize); message WM_SIZE;
  public
    constructor Create (AOwner: TComponent); override;
    procedure CreateParams (var Params: TCreateParams); override;
    destructor Destroy; override;
  published
    property BoundLines: TDockBoundLines read FBoundLines write SetBoundLines default [];
    property Color default clBtnFace;
    property PopupMenu;
    property Position: TDockPosition read FPosition write SetPosition default dpTop;

    property OnInsertRemoveBar: TInsertRemoveEvent read FOnInsertRemoveBar write FOnInsertRemoveBar;
  end;

  { TToolbar97 }

  TToolbar97 = class(TCustomControl)
  private
    { Property variables }
    FBarHeight, FBarWidth, FDockedTotalBarHeight, FDockedTotalBarWidth, FDockPos, FDockRow: Integer;
    FDefaultDock: TDock97;
    FOnRecreating, FOnRecreated, FOnDockChanging, FOnDockChanged, FOnClose: TNotifyEvent;
    FCloseButton: Boolean;
    FCanDockLeftRight: Boolean;
    FFloatingRect: TRect;
    FFloatingRightX: Integer;

    { Dragging variables }
    DragRect: TRect;
    OldTopLeft: TPoint;
    ScreenDC: HDC;

    { Moving variables }
    Moving: Boolean;         { True if currently moving the toolbar }
    MoveDock: TDock97;       { The potential Dock97 the mouse is currently over }
    MoveNewSize: array[TDockType] of TPoint;

    { Sizing variables }
    Sizing: Boolean;         { True if currently resizing the toolbar }
    SizeHitTest: Integer;    { While sizing: The side of the toolbar that user clicked. HTLEFT, HTRIGHT, HTTOP, or HTBOTTOM }
    SizeCurX: Integer;       { While sizing: Current width }
    SizeNewSizes: TList;     { While sizing: List of valid new sizes. Items are casted into TSmallPoints }

    { Lists }
    SlaveInfo,               { List of slave controls. Items are pointers to TSlaveInfo's }
    GroupInfo,               { List of the control "groups". List items are pointers to TGroupInfo's }
    LineSeps: TList;         { List of the Y locations of line separators. Items are casted in Integers }

    { Misc. }
    UpdatingBounds: Integer; { Increment while internally changing the bounds. This allows it to move the toolbar freely }
    DisableArrangeControls: Integer; { Increment to disable ArrangeControls }
    Hidden: Integer;         { Incremented while the toolbar is temporarily hidden }
    AdjustWidthAndHeight: Boolean; { When True, the next call to ArrangeControls will always adjust the width/height of the toolbar }

    { When floating }
    FloatParent: TWinControl; { The actual Parent of the toolbar when it is floating }
    CaptionIsActive: Boolean; { True if caption is currently of the active color }
    CloseButtonPushed: Boolean; { True if Close button is currently depressed }
    CloseButtonMouseOver: Boolean; { True if mouse is over Close Button while it's pressed }

    OldFormWindowProc: Pointer; { The previous form window procedure }

    { Smooth dragging stuff }
    {$IFDEF TB97SMOOTH}
    PrevPos: TPoint;
    {$ENDIF}

    { Property access methods }
    procedure SetCloseButton (Value: Boolean);
    function  GetDockedTo: TDock97;
    procedure SetDockedTo (Value: TDock97);
    procedure SetDockPos (Value: Integer);
    procedure SetDockRow (Value: Integer);

    { Internal }
    procedure FreeGroupInfo (const List: TList);
    procedure BuildGroupInfo (const List: TList; const TranslateSlave: Boolean;
      const OldDockType, NewDockType: TDockType);
    procedure MoveOnScreen (const OnlyIfFullyOffscreen: Boolean);
    procedure ShouldBeVisible (const Control: TControl; const DockType: TDockType;
      const SetIt: Boolean; var AVisible: Boolean);
    procedure AutoArrangeControls;
    procedure ArrangeControls (const CanMove, CanResize: Boolean;
      const OldDockType, NewDockType: TDockType; RightX: Integer;
      const NewClientSize: PPoint);
    procedure DrawDraggingRect (Rect: TRect; const Dock: TDock97);
    procedure NewFormWindowProc (var Message: TMessage);
    function  NewMainWindowHook (var Message: TMessage): Boolean;
    procedure CancelModes;
    procedure Dropped;
    procedure MouseMover (Shift: TShiftState; X, Y: Integer);
    procedure RedrawCaption (const OnlyCloseButton, AutoActive: Boolean;
      IsActive: Boolean);

    { Messages }
    procedure CMVisibleChanged (var Message: TMessage); message CM_VISIBLECHANGED;
    procedure WMCancelMode (var Message: TWMCancelMode); message WM_CANCELMODE;
    procedure WMActivate (var Message: TWMActivate); message WM_ACTIVATE;
    procedure WMMouseActivate (var Message: TWMMouseActivate); message WM_MOUSEACTIVATE;
    procedure WMGetMinMaxInfo (var Msg: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
    procedure WMNCHitTest (var Message: TWMNCHitTest); message WM_NCHITTEST;
    procedure WMNCLButtonDown (var Message: TWMNCLButtonDown); message WM_NCLBUTTONDOWN;
    procedure WMNCPaint (var Message: TWMNCPaint); message WM_NCPAINT;
    procedure WMNCCalcSize (var Message: TWMNCCalcSize); message WM_NCCALCSIZE;
  protected
    { Protected declarations }
    procedure CreateParams (var Params: TCreateParams); override;
    procedure Loaded; override;
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove (Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure AlignControls (AControl: TControl; var Rect: TRect); override;
    procedure Paint; override;
    procedure SetParent (AParent: TWinControl); override;
    procedure SetBounds (ALeft, ATop, AWidth, AHeight: Integer); override;
  public
    { Public declarations }
    property FloatingRect: TRect read FFloatingRect write FFloatingRect;
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadPosition (const BaseRegistryKey: String);
    procedure SavePosition (const BaseRegistryKey: String);
    procedure SetSlaveControl (const ATopBottom, ALeftRight: TControl);
  published
    { Published declarations }
    property CanDockLeftRight: Boolean read FCanDockLeftRight write FCanDockLeftRight default True;
    property Caption;
    property Color default clBtnFace;
    property CloseButton: Boolean read FCloseButton write SetCloseButton default True;
    property DefaultDock: TDock97 read FDefaultDock write FDefaultDock;
    property DockedTo: TDock97 read GetDockedTo write SetDockedTo;
    property DockRow: Integer read FDockRow write SetDockRow default 0;
    property DockPos: Integer read FDockPos write SetDockPos default -1;
    property PopupMenu;

    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnRecreated: TNotifyEvent read FOnRecreated write FOnRecreated;
    property OnRecreating: TNotifyEvent read FOnRecreating write FOnRecreating;
    property OnDockChanged: TNotifyEvent read FOnDockChanged write FOnDockChanged;
    property OnDockChanging: TNotifyEvent read FOnDockChanging write FOnDockChanging;
  end;

  TToolbarSep97 = class(TCustomControl)
  protected
    Invisible: Boolean;
    procedure SetParent (AParent: TWinControl); override;
    procedure Paint; override;
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  end;

  { TToolbarButton97.
    Same as Delphi 3's TSpeedButton except:
    - Doesn't have a Flat property. It's always Flat.
    - Has a new Opaque property for nonblinky redraws.
    - A lot of miscellaneous fixes.
  }

  {TButtonDropDirection = (ddDown, ddRight);}

  TToolbarButton97 = class(TGraphicControl)
  private
    FGroupIndex: Integer;
    FGlyph: Pointer;
    FOpaque: Boolean;
    FDown: Boolean;
    FMoving: Boolean;
    {FDropdownMenu: TPopupMenu;
    FDropDirection: TButtonDropDirection;}
    FAllowAllUp: Boolean;
    FLayout: TButtonLayout;
    FSpacing: Integer;
    FMargin: Integer;
    FMouseInControl: Boolean;
    procedure GlyphChanged(Sender: TObject);
    procedure UpdateExclusive;
    function GetGlyph: TBitmap;
    procedure SetGlyph(Value: TBitmap);
    function GetNumGlyphs: TNumGlyphs;
    procedure SetNumGlyphs(Value: TNumGlyphs);
    procedure SetDown(Value: Boolean);
    procedure SetAllowAllUp(Value: Boolean);
    procedure SetGroupIndex(Value: Integer);
    procedure SetLayout(Value: TButtonLayout);
    procedure SetOpaque(Value: Boolean);
    procedure SetSpacing(Value: Integer);
    procedure SetMargin(Value: Integer);
    procedure UpdateTracking;
    procedure InvalidateIt;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMButtonPressed(var Message: TMessage); message CM_BUTTONPRESSED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMSysColorChange(var Message: TMessage); message CM_SYSCOLORCHANGE;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    FState: TButtonState;
    function GetPalette: HPALETTE; override;
    procedure Loaded; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    function GlyphDraw(Canvas: TCanvas; const Client: TRect; const Offset: TPoint;
      const Caption: string; Layout: TButtonLayout; Margin, Spacing: Integer;
      State: TButtonState; Transparent: Boolean): TRect;
  published
    property AllowAllUp: Boolean read FAllowAllUp write SetAllowAllUp default False;
    property GroupIndex: Integer read FGroupIndex write SetGroupIndex default 0;
    property Down: Boolean read FDown write SetDown default False;
    {property DropdownMenu: TPopupMenu read FDropdownMenu write FDropdownMenu default nil;
    property DropDirection: TButtonDropDirection read FDropDirection write FDropDirection default ddDown;}
    property Caption;
    property Enabled;
    property Font;
    property Glyph: TBitmap read GetGlyph write SetGlyph;
    property Layout: TButtonLayout read FLayout write SetLayout default blGlyphLeft;
    property Margin: Integer read FMargin write SetMargin default -1;
    property NumGlyphs: TNumGlyphs read GetNumGlyphs write SetNumGlyphs default 1;
    property Opaque: Boolean read FOpaque write SetOpaque default True;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Spacing: Integer read FSpacing write SetSpacing default 4;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  { TEdit97 }

  TEdit97 = class(TCustomEdit)
  private
    MouseInControl: Boolean;
    procedure RedrawBorder;
    procedure NewAdjustHeight;
    procedure CMEnabledChanged (var Msg: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged (var Msg: TMessage); message CM_FONTCHANGED;
    procedure WMSetFocus (var Msg: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus (var Msg: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMPaint (var Msg: TWMPaint); message WM_PAINT;
    procedure WMNCCalcSize (var Msg: TWMNCCalcSize); message WM_NCCALCSIZE;
    procedure WMNCPaint (var Msg: TWMNCPaint); message WM_NCPAINT;
  protected
    procedure MouseMove (Shift: TShiftState; X, Y: Integer); override;
    procedure Loaded; override;
  public
    constructor Create (AOwner: TComponent); override;
  published
    property CharCase;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    {$IFDEF VER100}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

uses
  SysUtils, CommCtrl, Registry, Consts;

const
  { All spacing & margin values are here. It's recommended that you don't
    change any of this! }
  HorzSpacing = 6;
  VertSpacing = 4;
  LineSpacing = 6;
  { TopMargin is really a left margin when docked to left or right side.
    Likewise, LeftMargin is really a top margin. }
  TopMargin: array[TDockType] of Integer = (2, 2, 2);
  BottomMargin: array[TDockType] of Integer = (1, 2, 2);
  LeftMargin: array[TDockType] of Integer = (4, 11, 11);
  RightMargin: array[TDockType] of Integer = (4, 2, 2);
  BoundLineSize = 1;

  ForceDockAtTopRow = 0;
  ForceDockAtLeftPos = -8;

  DefaultBarWidthHeight = 8;

  PositionLeftOrRight = [dpLeft, dpRight];

  { Names of registry values }
  rvVisible = 'Visible';
  rvDockedTo = 'DockedTo';
  rvDockRow = 'DockRow';
  rvDockPos = 'DockPos';
  rvFloatLeft = 'FloatLeft';
  rvFloatTop = 'FloatTop';
  rvFloatRight = 'FloatRight';
  rvFloatBottom = 'FloatBottom';
  rvFloatRightX = 'FloatRightX';

type
  { Used in GroupInfo lists }
  PGroupInfo = ^TGroupInfo;
  TGroupInfo = record
    GroupWidth,           { Width in pixels of the group, if all controls were
                            lined up left-to-right }
    GroupHeight: Integer; { Heights in pixels of the group, if all controls were
                            lined up top-to-bottom }
    Members: TList;
  end;

  { Used in SlaveInfo lists }
  PSlaveInfo = ^TSlaveInfo;
  TSlaveInfo = record
    LeftRight,
    TopBottom: TControl;
  end;

  { Used in RowInfo lists }
  PRowInfo = ^TRowInfo;
  TRowInfo = record
    BarHeight, BarWidth, DockedTotalBarHeight, DockedTotalBarWidth: Integer;
  end;

  TFloatParent = class(TWinControl)
    procedure CMVisibleChanged (var Message: TMessage); message CM_VISIBLECHANGED;
  end;

function InstallNewWindowProc (const AForm: TForm; const NewProc: TWndMethod;
  const NewHook: TWindowHook): Pointer; forward;
procedure UninstallNewWindowProc (const AForm: TForm); forward;
type
  PUsedFormInfo = ^TUsedFormInfo;
  TUsedFormInfo = record
    Form: TForm;
    Old, New: Pointer;
    Hook: TWindowHook;
    RefCount: Integer;
  end;

var
  UsedForms: TList;
  {$IFDEF TB97SMOOTH}
  SmoothDrag: Boolean;
  SmoothDragging: TToolbar97;
  IdleHook: HHOOK;
  {$ENDIF}

procedure Register;
begin
  RegisterComponents ('Toolbar97',
    [TToolbar97, TDock97, TToolbarButton97, TToolbarSep97, TEdit97]);
end;

//////////////////////////////////////////////////////////////////////////////
// Misc. functions
//////////////////////////////////////////////////////////////////////////////

function MetricsCaptionHeight: Integer;
{ Returns height of the caption of a small window }
begin
  if NewStyleControls{if Win 4.0 or later} then
    Result := GetSystemMetrics(SM_CYSMCAPTION)
  else
    { Win 3.x doesn't support small captions, so use a predefined value }
    Result := MulDiv(16, Screen.PixelsPerInch, 96);
end;
function MetricsBorderWidth: Integer;
{ Returns width of a thick border }
begin
  Result := GetSystemMetrics(SM_CXBORDER)+GetSystemMetrics(SM_CXFRAME)-1;
end;
function MetricsBorderHeight: Integer;
{ Returns height of a thick border }
begin
  Result := GetSystemMetrics(SM_CYBORDER)+GetSystemMetrics(SM_CYFRAME)-1;
end;

procedure AddNCAreaToRect (var R: TRect);
var
  CaptionHeight, BorderWidth, BorderHeight: Integer;
begin
  CaptionHeight := MetricsCaptionHeight;
  BorderWidth := MetricsBorderWidth;
  BorderHeight := MetricsBorderHeight;

  Dec (R.Left, BorderWidth);
  Inc (R.Right, BorderWidth);
  Inc (R.Bottom, CaptionHeight + BorderHeight*2);
end;
procedure RemoveNCAreaFromRect (var R: TRect);
var
  CaptionHeight, BorderWidth, BorderHeight: Integer;
begin
  CaptionHeight := MetricsCaptionHeight;
  BorderWidth := MetricsBorderWidth;
  BorderHeight := MetricsBorderHeight;

  Inc (R.Left, BorderWidth);
  Dec (R.Right, BorderWidth);
  Dec (R.Bottom, CaptionHeight + BorderHeight*2);
end;

{$IFDEF TB97SMOOTH}
function GetDragFullWindows: Boolean;
var
  S: BOOL;
begin
  Result := False;
  if SystemParametersInfo(SPI_GETDRAGFULLWINDOWS, 0, @S, 0) then
    Result := S;
end;
{$ENDIF}

function ControlInToolbar97 (Control: TControl): Boolean;
begin
  Result := False;
  while Control <> nil do begin
    if Control is TToolbar97 then begin
      Result := True;
      Break;
    end;
    Control := Control.Parent;
  end;
end;

function ControlIsChildOf (Control, IsChildOf: TControl): Boolean;
begin
  Result := False;
  while Control <> nil do begin
    if Control = IsChildOf then begin
      Result := True;
      Break;
    end;
    Control := Control.Parent;
  end;
end;

{$IFDEF TB97SMOOTH}
function ForegroundIdle (code: Integer; wParam: WPARAM; lParam: LPARAM): DWORD; stdcall;
var
  P: TPoint;
begin
  if (code = HC_ACTION) and Assigned(SmoothDragging) then
    with SmoothDragging do begin
      GetCursorPos (P);
      if (P.X <> PrevPos.X) or (P.Y <> PrevPos.Y) then begin
        PrevPos := P;
        P := ScreenToClient(P);
        MouseMover ([ssLeft], P.X, P.Y);
      end;
    end;
  Result := CallNextHookEx(IdleHook, code, wParam, lParam);
end;
{$ENDIF}

function InstallNewWindowProc (const AForm: TForm; const NewProc: TWndMethod;
  const NewHook: TWindowHook): Pointer;
{ Installs a new window procedure on the specified form that overrides the
  existing one.
  Toolbar97 uses this set up the enhanced window procedure for the owner
  form.
  Returns address of old window procedure, which the new window procedure
  should call. }
var
  I: Integer;
  Info: PUsedFormInfo;
begin
  Result := nil;
  for I := 0 to UsedForms.Count-1 do
    with PUsedFormInfo(UsedForms[I])^ do
      { If AForm already exists in list, only increment the reference count }
      if Form = AForm then begin
        Inc (RefCount);
        Exit;
      end;
  New (Info);
  try
    with Info^ do begin
      Form := AForm;
      New := MakeObjectInstance(NewProc);
      Old := Pointer(SetWindowLong(AForm.Handle, GWL_WNDPROC, LongInt(New)));
      Hook := NewHook;
      Application.HookMainWindow (Hook);
      RefCount := 1;
      Result := Old;
    end;
    UsedForms.Add (Info);
  except
    Dispose (Info);
    raise;
  end;
end;

procedure UninstallNewWindowProc (const AForm: TForm);
{ Removes the new window procedure installing using InstallNewWindowProc from
  the specified form. }
var
  I: Integer;
begin
  for I := 0 to UsedForms.Count-1 do
    with PUsedFormInfo(UsedForms[I])^ do
      if Form = AForm then begin
        Dec (RefCount);
        if RefCount = 0 then begin
          if Form.HandleAllocated then
            SetWindowLong (Form.Handle, GWL_WNDPROC, LongInt(Old));
          FreeObjectInstance (New);
          Application.UnhookMainWindow (Hook);
          Dispose (UsedForms[I]);
          UsedForms.Remove (UsedForms[I]);
        end;
      end;
end;

function GetDockTypeOf (const Control: TDock97): TDockType;
begin
  if Control = nil then
    Result := dtNotDocked
  else begin
    if not(Control.Position in PositionLeftOrRight) then
      Result := dtTopBottom
    else
      Result := dtLeftRight;
  end;
end;


//////////////////////////////////////////////////////////////////////////////
// TDock97 - internal
//////////////////////////////////////////////////////////////////////////////

constructor TDock97.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);

  Inc (DisableArrangeToolbars);
  try
    ControlStyle := ControlStyle +
      [csAcceptsControls] -
      [csCaptureMouse, csClickEvents, csNoStdEvents,
       csOpaque{or else non-opaque buttons look wrong}];
    Position := dpTop;
    DockList := TList.Create;
    RowInfo := TList.Create;
    Color := clBtnFace;
  finally
    Dec (DisableArrangeToolbars);
  end;
  { Rearranging was disabled, so manually rearrange it now }
  ArrangeToolbars;
end;

procedure TDock97.CreateParams (var Params: TCreateParams);
begin
  inherited CreateParams (Params);
  with Params.WindowClass do
    { Disable complete redraws when size changes. CS_H/VREDRAW cause flicker
      and are not necessary for this control at run time }
    if not(csDesigning in ComponentState) then
      Style := Style and not(CS_HREDRAW or CS_VREDRAW);
end;

destructor TDock97.Destroy;
begin
  FreeRowInfo;
  if Assigned(RowInfo) then
    RowInfo.Free;
  if Assigned(DockList) then
    DockList.Free;
  inherited Destroy;
end;

procedure TDock97.SetParent (AParent: TWinControl);
begin
  if (AParent is TToolbar97) or (AParent is TDock97) then
    raise Exception.Create('A TDock97 control cannot be placed inside a ' +
      'TToolbar97 or another TDock97');

  inherited SetParent (AParent);
end;

procedure TDock97.FreeRowInfo;
var
  I: Integer;
begin
  if RowInfo = nil then Exit;
  for I := RowInfo.Count-1 downto 0 do begin
    FreeMem (RowInfo.Items[I]);
    RowInfo.Remove (RowInfo.Items[I]);
  end;
end;

procedure TDock97.BuildRowInfo;
var
  R, I: Integer;
  HighestBarHeight, HighestBarWidth: Integer;
  NewRowInfo: PRowInfo;
begin
  FreeRowInfo;
  for R := 0 to GetHighestRow do begin
    HighestBarHeight := DefaultBarWidthHeight;
    HighestBarWidth := DefaultBarWidthHeight;
    for I := 0 to DockList.Count-1 do begin
      with TToolbar97(DockList[I]) do begin
        if FDockRow <> R then Continue;
        if FBarHeight > HighestBarHeight then
          HighestBarHeight := FBarHeight;
        if FBarWidth > HighestBarWidth then
          HighestBarWidth := FBarWidth;
      end;
    end;
    GetMem (NewRowInfo, SizeOf(TRowInfo));
    try
      with NewRowInfo^ do begin
        BarHeight := HighestBarHeight;
        BarWidth := HighestBarWidth;
        DockedTotalBarHeight := TopMargin[dtTopBottom] + HighestBarHeight + BottomMargin[dtTopBottom];
        DockedTotalBarWidth := TopMargin[dtLeftRight] + HighestBarWidth + BottomMargin[dtLeftRight];
      end;
      RowInfo.Add (NewRowInfo);
    except
      FreeMem (NewRowInfo);
      raise;
    end;
  end;
end;

function GetRowInfo (const Row: Integer; const Dock: TDock97;
  const DefaultToolbar: TToolbar97): TRowInfo;
begin
  if Row <= Dock.RowInfo.Count-1 then
    Result := PRowInfo(Dock.RowInfo[Row])^
  else begin
    if DefaultToolbar = nil then
      FillChar (Result, SizeOf(Result), 0)
    else
    with Result do begin
      BarHeight := DefaultToolbar.FBarHeight;
      BarWidth := DefaultToolbar.FBarWidth;
      DockedTotalBarHeight := DefaultToolbar.FDockedTotalBarHeight;
      DockedTotalBarWidth := DefaultToolbar.FDockedTotalBarWidth;
    end;
  end;
end;

function TDock97.GetRowOf (const Y: Integer; var Before: Boolean): Integer;
{ Returns row number of the specified Y. Before is set to True if it was
  close to being in between two rows. }
var
  HighestRow, R, CurY, NextY: Integer;
  CurRowInfo: TRowInfo;
begin
  Result := 0;  Before := False;
  HighestRow := GetHighestRow;
  CurY := BoundLineSize * Ord(blTop in BoundLines);
  for R := 0 to HighestRow+1 do begin
    if R <= HighestRow then begin
      CurRowInfo := GetRowInfo(R, Self, nil);
      NextY := CurY + CurRowInfo.DockedTotalBarHeight;
    end
    else
      NextY := High(NextY); 
    if Y <= CurY+5 then begin
      Result := R;
      Before := True;
      Break;
    end;
    if (Y >= CurY+5) and (Y <= NextY-5) then begin
      Result := R;
      Break;
    end;
    CurY := NextY;
  end;
end;

function TDock97.GetDesignModeRowOf (const Y: Integer): Integer;
{ Similar to GetRowOf, but is a little different to accomidate design mode
  better }
var
  HighestRowPlus1, R, CurY, NextY: Integer;
  CurRowInfo: TRowInfo;
begin
  Result := 0;
  HighestRowPlus1 := GetHighestRow+1;
  CurY := BoundLineSize * Ord(blTop in BoundLines);
  for R := 0 to HighestRowPlus1 do begin
    Result := R;
    if R = HighestRowPlus1 then Break;
    CurRowInfo := GetRowInfo(R, Self, nil);
    NextY := CurY + CurRowInfo.DockedTotalBarHeight;
    if Y <= NextY-1 then
      Break;
    CurY := NextY;
  end;
end;

function TDock97.GetHighestRow: Integer;
{ Returns highest used row number, or -1 if no rows are used }
var
  I: Integer;
begin
  Result := -1;
  for I := 0 to DockList.Count-1 do
    with TToolbar97(DockList[I]) do
      if FDockRow > Result then
        Result := FDockRow;
end;

procedure TDock97.RemoveBlankRows;
{ Deletes any blank row numbers, adjusting the docked toolbars' FDockRow as
  needed }
var
  HighestRow, R, I: Integer;
  RowIsEmpty: Boolean;
begin
  HighestRow := GetHighestRow;
  R := 0;
  while R <= HighestRow do begin
    RowIsEmpty := True;
    for I := 0 to DockList.Count-1 do
      if TToolbar97(DockList[I]).FDockRow = R then begin
        RowIsEmpty := False;
        Break;
      end;
    if RowIsEmpty then begin
      { Shift all ones higher than R back one }
      for I := 0 to DockList.Count-1 do
        with TToolbar97(DockList[I]) do
          if FDockRow > R then
            Dec (FDockRow);
      Dec (HighestRow);
    end;
    Inc (R);
  end;
end;

procedure TDock97.InsertRowBefore (const BeforeRow: Integer);
{ Inserts a blank row before BeforeRow, adjusting all the docked toolbars'
  FDockRow as needed }
var
  I: Integer;
begin
  for I := 0 to DockList.Count-1 do
    with TToolbar97(DockList[I]) do
      if FDockRow >= BeforeRow then
        Inc (FDockRow);
end;

function CompareDockRowPos (Item1, Item2: Pointer): Integer;
begin
  if TToolbar97(Item1).FDockRow <> TToolbar97(Item2).FDockRow then
    Result := TToolbar97(Item1).FDockRow - TToolbar97(Item2).FDockRow
  else
    Result := TToolbar97(Item1).FDockPos - TToolbar97(Item2).FDockPos;
end;

procedure TDock97.ChangeWidthHeight (const NewWidth, NewHeight: Integer);
{ Same as setting Width/Height directly or using SetBounds, but does not
  lose Align position }
begin
  case Align of
    alTop, alLeft: SetBounds (Left, Top, NewWidth, NewHeight);
    alBottom: SetBounds (Left, Top-NewHeight+Height, NewWidth, NewHeight);
    alRight: SetBounds (Left-NewWidth+Width, Top, NewWidth, NewHeight);
  end;
end;

procedure TDock97.AlignControls (AControl: TControl; var Rect: TRect);
begin
  ArrangeToolbars;
end;

procedure TDock97.ArrangeToolbars;
{ The main procedure to arrange all the toolbars docked to it }
var
  LeftRight: Boolean;
  HighestRow, R, PosOffset, CurDockPos, CurRowPixel, I, J: Integer;
  HighestTotalBarHeight, HighestTotalBarWidth,
  CurTotalBarHeight, CurTotalBarWidth: Integer;
begin
  if (DisableArrangeToolbars > 0) or (csLoading in ComponentState) then
    Exit;

  LeftRight := Position in PositionLeftOrRight;

  if DockList.Count = 0 then begin
    if not LeftRight then
      ChangeWidthHeight (Width, 9 * Ord(csDesigning in ComponentState))
    else
      ChangeWidthHeight (9 * Ord(csDesigning in ComponentState), Height);
    Exit;
  end;

  { Ensure list is in correct ordering according to DockRow/DockPos }
  DockList.Sort (CompareDockRowPos);

  { Remove any blank rows }
  RemoveBlankRows;
  { Rebuild the RowInfo, since rows numbers were probably shifted after
    RemoveBlankRows }
  BuildRowInfo;

  { Find highest row number }
  HighestRow := GetHighestRow;

  { Make up for BoundLines area }
  if not LeftRight then
    PosOffset := BoundLineSize * Ord(blLeft in BoundLines)
  else
    PosOffset := BoundLineSize * Ord(blTop in BoundLines);

  { Arrange, first without actually moving the toolbars onscreen }
  for R := 0 to HighestRow do begin
    CurDockPos := 0;
    for I := 0 to DockList.Count-1 do begin
      with TToolbar97(DockList[I]) do begin
        if FDockRow <> R then Continue;
        if FDockPos <= CurDockPos then
          FDockPos := CurDockPos
        else
          CurDockPos := FDockPos;
        if not LeftRight then
          Inc (CurDockPos, Width)
        else
          Inc (CurDockPos, Height);
      end;
    end;
  end;
  { Try to move all the toolbars that are offscreen to a fully visible position }
  for R := 0 to HighestRow do begin
    for I := 0 to DockList.Count-1 do begin
      if TToolbar97(DockList[I]).FDockRow <> R then Continue;
      for J := DockList.Count-1 downto I do begin
        with TToolbar97(DockList[J]) do begin
          if FDockRow <> R then Continue;
          if not LeftRight then begin
            if FDockPos+Width > Self.ClientWidth-PosOffset*2 then begin
              TToolbar97(DockList[I]).FDockPos :=
                TToolbar97(DockList[I]).FDockPos - ((FDockPos+Width) - (Self.ClientWidth-PosOffset*2));
              Break;
            end;
          end
          else begin
            if FDockPos+Height > Self.ClientHeight-PosOffset*2 then begin
              TToolbar97(DockList[I]).FDockPos :=
                TToolbar97(DockList[I]).FDockPos - ((FDockPos+Height) - Self.ClientHeight);
              Break;
            end;
          end;
        end;
      end;
    end;
  end;
  { Arrange again, this time actually moving the toolbars }
  if not LeftRight then
    CurRowPixel := BoundLineSize * Ord(blTop in BoundLines)
  else
    CurRowPixel := BoundLineSize * Ord(blLeft in BoundLines);
  for R := 0 to HighestRow do begin
    CurDockPos := 0;
    HighestTotalBarHeight := DefaultBarWidthHeight;
    HighestTotalBarWidth := DefaultBarWidthHeight;
    for I := 0 to DockList.Count-1 do begin
      with TToolbar97(DockList[I]) do begin
        if FDockRow <> R then Continue;
        CurTotalBarHeight := GetRowInfo(FDockRow, DockedTo, TToolbar97(DockList[I])).DockedTotalBarHeight;
        CurTotalBarWidth := GetRowInfo(FDockRow, DockedTo, TToolbar97(DockList[I])).DockedTotalBarWidth;
        if CurTotalBarHeight > HighestTotalBarHeight then
          HighestTotalBarHeight := CurTotalBarHeight;
        if CurTotalBarWidth > HighestTotalBarWidth then
          HighestTotalBarWidth := CurTotalBarWidth;
        if FDockPos <= CurDockPos then
          FDockPos := CurDockPos
        else
          CurDockPos := FDockPos;
        Inc (UpdatingBounds);
        try
          if not LeftRight then
            SetBounds (CurDockPos+PosOffset, CurRowPixel, Width, CurTotalBarHeight)
          else
            SetBounds (CurRowPixel, CurDockPos+PosOffset, CurTotalBarWidth, Height);
        finally
          Dec (UpdatingBounds);
        end;
        if not LeftRight then
          Inc (CurDockPos, Width)
        else
          Inc (CurDockPos, Height);
      end;
    end;
    if not LeftRight then
      Inc (CurRowPixel, HighestTotalBarHeight)
    else
      Inc (CurRowPixel, HighestTotalBarWidth);
  end;

  { Set the size of the dock }
  if not LeftRight then
    ChangeWidthHeight (Width, CurRowPixel + (BoundLineSize * Ord(blBottom in BoundLines)))
  else
    ChangeWidthHeight (CurRowPixel + (BoundLineSize * Ord(blRight in BoundLines)), Height);
end;

procedure TDock97.ChangeDockList (const Insert: Boolean;
  const Bar: TToolbar97; const IsVisible: Boolean);
var
  Modified: Boolean;
begin
  Modified := False;

  if Insert then begin
    { Delete if already exists }
    if DockList.IndexOf(Bar) <> -1 then
      DockList.Remove (Bar);
    { Only add to dock list if visible }
    if IsVisible then begin
      DockList.Add (Bar);
      Modified := True;
    end;
  end
  else begin
    if DockList.IndexOf(Bar) <> -1 then begin
      DockList.Remove (Bar);
      Modified := True;
    end;
  end;

  ArrangeToolbars;

  if (Modified) and (Assigned(FOnInsertRemoveBar)) then
    FOnInsertRemoveBar (Self, Insert, Bar);
end;

procedure TDock97.Loaded;
begin
  inherited Loaded;
  { Rearranging is disabled while the component is loading, so now that it's
    loaded, rearrange it. }
  ArrangeToolbars;
end;

procedure TDock97.WMSize (var Msg: TWMSize);
begin
  inherited;
  if (blBottom in BoundLines) or (blRight in BoundLines) then
    { Need this so garbage doesn't get left behind }
    Invalidate;
end;

procedure TDock97.Paint;
var
  R: TRect;
begin
  inherited Paint;
  with Canvas do begin
    { Draw dotted border in design mode }
    if csDesigning in ComponentState then begin
      Pen.Style := psDot;
      Pen.Color := clBtnShadow;
      Brush.Style := bsClear;
      R := ClientRect;
      Rectangle (R.Left, R.Top, R.Right, R.Bottom);
      Pen.Style := psSolid;
    end;

    { Draw BoundLines }
    if blTop in BoundLines then begin
      Pen.Color := clBtnShadow;
      MoveTo (0, 0);  LineTo (Width, 0);
    end;
    if blBottom in BoundLines then begin
      Pen.Color := clBtnHighlight;
      MoveTo (0, Height-1);  LineTo (Width, Height-1);
    end;
    if blLeft in BoundLines then begin
      Pen.Color := clBtnShadow;
      MoveTo (0, 0);  LineTo (0, Height);
    end;
    if blRight in BoundLines then begin
      Pen.Color := clBtnHighlight;
      MoveTo (Width-1, 0);  LineTo (Width-1, Height);
    end;
  end;
end;

//////////////////////////////////////////////////////////////////////////////
// TDock97 - property access methods
//////////////////////////////////////////////////////////////////////////////

procedure TDock97.SetBoundLines (Value: TDockBoundLines);
begin
  if FBoundLines <> Value then begin
    FBoundLines := Value;

    ArrangeToolbars;
    { Invalidate so that bound lines are redrawn. This is needed since
      H/VREDRAW are disabled at run time }
    Invalidate;
  end;
end;

procedure TDock97.SetPosition (Value: TDockPosition);
begin
  if ControlCount <> 0 then
    raise Exception.Create('Cannot change Position of a TDock97 ' +
      'if it already contains controls');
  FPosition := Value;
  case Position of
    dpTop: Align := alTop;
    dpBottom: Align := alBottom;
    dpLeft: Align := alLeft;
    dpRight: Align := alRight;
  end;
end;


//////////////////////////////////////////////////////////////////////////////
// TFloatParent - Internal
//////////////////////////////////////////////////////////////////////////////

procedure TFloatParent.CMVisibleChanged (var Message: TMessage);
{ This updates all of FloatParent's childrens' state to reflect the change }
const
  HideShow: array[Boolean] of UINT = (SWP_HIDEWINDOW, SWP_SHOWWINDOW);
var
  I: Integer;
begin
  inherited;
  for I := 0 to ControlCount-1 do
    if Controls[I] is TWinControl then
      SetWindowPos (TWinControl(Controls[I]).Handle, 0, 0, 0, 0, 0,
        HideShow[Visible] or SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE or
        SWP_NOZORDER);
end;


//////////////////////////////////////////////////////////////////////////////
// TToolbar97 - Internal
//////////////////////////////////////////////////////////////////////////////

constructor TToolbar97.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);

  if not(AOwner is TForm) then
    raise Exception.Create('TToolbar97 must be owned by a form');

  GroupInfo := TList.Create;
  SlaveInfo := TList.Create;
  LineSeps := TList.Create;
  SizeNewSizes := TList.Create;

  Inc (DisableArrangeControls);
  try
    ControlStyle := ControlStyle +
      [csAcceptsControls, csClickEvents, csDoubleClicks, csSetCaption] -
      [csCaptureMouse{capturing is done manually},
       csOpaque{or else non-opaque buttons look wrong}];

    if not(csDesigning in ComponentState) then begin
      FloatParent := TFloatParent.Create(AOwner);
      FloatParent.Parent := TForm(AOwner);
      { Set up the new window procedure for the owner form of the toolbar }
      OldFormWindowProc := InstallNewWindowProc(TForm(AOwner), NewFormWindowProc,
        NewMainWindowHook);
    end
    else
      FloatParent := TForm(AOwner);

    FCanDockLeftRight := True;
    FCloseButton := True;
    FDockPos := -1;
    FBarHeight := DefaultBarWidthHeight;
    FBarWidth := DefaultBarWidthHeight;
    Color := clBtnFace;
    DockedTo := nil;

    Inc (UpdatingBounds);
    try
      SetBounds (Left, Top, 129, 49);
    finally
      Dec (UpdatingBounds);
    end;
  finally
    Dec (DisableArrangeControls);
  end;
  AutoArrangeControls;
end;

procedure TToolbar97.WMGetMinMaxInfo (var Msg: TWMGetMinMaxInfo);
begin
  inherited;
  { This removes the minimum size limit of the window, so it can resize it
    to however small is necessary. This has no effect on Win95/NT 4.0 when it
    uses the WS_EX_TOOLWINDOW style, but is required for NT 3.x. }
  Msg.MinMaxInfo.ptMinTrackSize :=
    Point(MetricsBorderWidth*2, MetricsBorderHeight*2);
end;

procedure TToolbar97.CreateParams (var Params: TCreateParams);
begin
  inherited CreateParams (Params);
  with Params do begin
    if Parent = FloatParent then begin
      if csDesigning in ComponentState then
        Style := Style or WS_BORDER or WS_THICKFRAME
      else
        Style := (Style or WS_BORDER or WS_THICKFRAME or WS_POPUP)
          and not(WS_CHILD or WS_GROUP or WS_TABSTOP);
      { Win 3.x doesn't support WS_EX_TOOLWINDOW }
      if NewStyleControls{if Win 4.0 or later} then
        ExStyle := ExStyle or WS_EX_TOOLWINDOW;
    end;
  end;
end;

destructor TToolbar97.Destroy;
var
  I: Integer;
begin
  if Assigned(SizeNewSizes) then
    SizeNewSizes.Free;
  if Assigned(LineSeps) then
    LineSeps.Free;
  if Assigned(SlaveInfo) then begin
    for I := SlaveInfo.Count-1 downto 0 do begin
      FreeMem (SlaveInfo.Items[I]);
      SlaveInfo.Remove (SlaveInfo.Items[I]);
    end;
    SlaveInfo.Free;
  end;
  FreeGroupInfo (GroupInfo);
  if Assigned(GroupInfo) then
    GroupInfo.Free;
  if not(csDesigning in ComponentState) then
    UninstallNewWindowProc (TForm(Owner));

  inherited Destroy;
end;

procedure TToolbar97.MoveOnScreen (const OnlyIfFullyOffscreen: Boolean);
{ Moves the (floating) toolbar so that it is fully (or at least mostly) in
  view on the screen }
var
  R, S, Test: TRect;
begin
  if DockedTo = nil then begin
    R := BoundsRect;
    S := Rect(0, 0, Screen.Width, Screen.Height);

    if (OnlyIfFullyOffscreen) and (IntersectRect(Test, R, S)) then
      Exit;

    if R.Right > S.Right then
      OffsetRect (R, S.Right - R.Right, 0);
    if R.Bottom > S.Bottom then
      OffsetRect (R, 0, S.Bottom - R.Bottom);
    if R.Left < 0 then
      OffsetRect (R, -R.Left, 0);
    if R.Top < 0 then
      OffsetRect (R, 0, -R.Top);
    BoundsRect := R;
  end;
end;

procedure TToolbar97.Loaded;
var
  R: TRect;
begin
  inherited Loaded;
  { Adjust coordinates if it was initially floating }
  if (not(csDesigning in ComponentState)) and (DockedTo = nil) then begin
    R := BoundsRect;
    MapWindowPoints (TForm(Owner).Handle, 0, R, 2);
    BoundsRect := R;
    MoveOnScreen (False);
  end;
  { Arranging of controls is disabled while component was loading, so rearrange
    it now }
  { [MODIFIED] Added following line }
  AutoArrangeControls;
end;

procedure TToolbar97.SetSlaveControl (const ATopBottom, ALeftRight: TControl);
var
  NewVersion: PSlaveInfo;
begin
  GetMem (NewVersion, SizeOf(TSlaveInfo));
  with NewVersion^ do begin
    TopBottom := ATopBottom;
    LeftRight := ALeftRight;
  end;
  SlaveInfo.Add (NewVersion);
  AutoArrangeControls;
end;

procedure TToolbar97.LoadPosition (const BaseRegistryKey: String);
var
  Reg: TRegIniFile;

  function FindDock (AName: String): TDock97;
  var
    I: Integer;
  begin
    Result := nil;
    with TForm(Owner) do
      for I := 0 to ControlCount-1 do
        if (Controls[I] is TDock97) and (Controls[I].Name = AName) then begin
          Result := TDock97(Controls[I]);
          Break;
        end;
  end;
  procedure ReadValues;
  begin
    FDockRow := Reg.ReadInteger(Name, rvDockRow, FDockRow);
    FDockPos := Reg.ReadInteger(Name, rvDockPos, FDockPos);
    FFloatingRect.Left := Reg.ReadInteger(Name, rvFloatLeft, 0);
    FFloatingRect.Top := Reg.ReadInteger(Name, rvFloatTop, 0);
    FFloatingRect.Right := Reg.ReadInteger(Name, rvFloatRight, 0);
    FFloatingRect.Bottom := Reg.ReadInteger(Name, rvFloatBottom, 0);
    FFloatingRightX := Reg.ReadInteger(Name, rvFloatRightX, 0);
  end;
var
  Dock: TDock97;
  DockedToName: String;
begin
  Reg := TRegIniFile.Create(BaseRegistryKey);
  try
    Visible := Reg.ReadBool(Name, rvVisible, Visible);
    DockedToName := Reg.ReadString(Name, rvDockedTo, '');
    if Length(DockedToName) <> 0 then begin
      if DockedToName <> 'nil' then begin
        Dock := FindDock(DockedToName);
        if Dock <> nil then begin
          ReadValues;
          if not IsRectEmpty(FFloatingRect) then
            AddNCAreaToRect (FFloatingRect);
          DockedTo := Dock;
        end;
      end
      else begin
        ReadValues;
        AddNCAreaToRect (FFloatingRect);
        DockedTo := nil;          
        MoveOnScreen (False);
      end;
    end;
  finally
    Reg.Free;
  end;
end;

procedure TToolbar97.SavePosition (const BaseRegistryKey: String);
var
  Reg: TRegIniFile;
  R: TRect;

  procedure WriteValues (DockedToName: String);
  begin
    Reg.WriteString  (Name, rvDockedTo, DockedToName);
    Reg.WriteInteger (Name, rvDockRow, FDockRow);
    Reg.WriteInteger (Name, rvDockPos, FDockPos);
    Reg.WriteInteger (Name, rvFloatLeft, R.Left);
    Reg.WriteInteger (Name, rvFloatTop, R.Top);
    Reg.WriteInteger (Name, rvFloatRight, R.Right);
    Reg.WriteInteger (Name, rvFloatBottom, R.Bottom);
    Reg.WriteInteger (Name, rvFloatRightX, FFloatingRightX);
  end;
begin
  if (DockedTo <> nil) and (Length(DockedTo.Name) = 0) then
    raise Exception.Create ('DockTo''s Name property not set');
  Reg := TRegIniFile.Create(BaseRegistryKey);
  try
    Reg.WriteBool (Name, rvVisible, Visible);
    if DockedTo <> nil then begin
      R := FFloatingRect;
      if not IsRectEmpty(FFloatingRect) then
        RemoveNCAreaFromRect (R);
      WriteValues (DockedTo.Name);
    end
    else begin
      R := BoundsRect;
      RemoveNCAreaFromRect (R);
      WriteValues ('nil');
    end;
  finally
    Reg.Free;
  end;
end;

procedure TToolbar97.FreeGroupInfo (const List: TList);
var
  I: Integer;
  L: PGroupInfo;
begin
  if List = nil then Exit;
  for I := List.Count-1 downto 0 do begin
    L := List.Items[I];
    if Assigned(L) then begin
      if Assigned(L.Members) then
        L.Members.Free;
      FreeMem (L);
    end;
    List.Remove (L);
  end;
end;

type
  TListSortExCompare = function (const Item1, Item2, ExtraData: Pointer): Integer;
procedure ListSortEx (const List: TList; const Compare: TListSortExCompare;
  const ExtraData: Pointer);
{ A new sorting procedure. Similar to TList.Sort, but lets you pass an
  ExtraData pointer }
  procedure QuickSortEx (L: Integer; const R: Integer);
  var
    I, J: Integer;
    P: Pointer;
  begin
    repeat
      I := L;
      J := R;
      P := List[(L + R) shr 1];
      repeat
        while Compare(List[I], P, ExtraData) < 0 do Inc(I);
        while Compare(List[J], P, ExtraData) > 0 do Dec(J);
        if I <= J then
        begin
          List.Exchange (I, J);
          Inc (I);
          Dec (J);
        end;
      until I > J;
      if L < J then QuickSortEx (L, J);
      L := I;
    until I >= R;
  end;
begin
  if List.Count <> 0 then
    QuickSortEx (0, List.Count-1);
end;

type
  PCompareExtra = ^TCompareExtra;
  TCompareExtra = record
    CurDockType: TDockType;
    Toolbar: TToolbar97;
  end;

function CompareControls (const Item1, Item2, ExtraData: Pointer): Integer;
  function GetLineOf (Y: Integer): Integer;
  var
    NewY, I: Integer;
  begin
    with PCompareExtra(ExtraData)^ do begin
      NewY := Y - TopMargin[CurDockType];
      for I := 0 to Toolbar.LineSeps.Count-1 do
        if Integer(Toolbar.LineSeps[I]) <= Y then
          Dec (NewY, LineSpacing);
      Result := NewY div Toolbar.FBarHeight;
    end;
  end;
var
  Item1Row, Item2Row: Integer;
begin
  Result := TControl(Item1).Left - TControl(Item2).Left;
  case PCompareExtra(ExtraData).CurDockType of
    dtNotDocked: begin
        Item1Row := GetLineOf(TControl(Item1).Top);
        Item2Row := GetLineOf(TControl(Item2).Top);
        if Item1Row <> Item2Row then
          Result := Item1Row - Item2Row;
      end;
    dtLeftRight:
      Result := TControl(Item1).Top - TControl(Item2).Top;
  end;
end;

procedure TToolbar97.BuildGroupInfo (const List: TList;
  const TranslateSlave: Boolean; const OldDockType, NewDockType: TDockType);
var
  I, J: Integer;
  IsVisible: Boolean;
  GI: PGroupInfo;
  Children: TList; {items casted into TControls}
  NewGroup: Boolean;
  Extra: TCompareExtra;
begin
  FreeGroupInfo (List);
  if ControlCount <> 0 then begin
    Children := TList.Create;
    for I := 0 to ControlCount-1 do begin
      IsVisible := Controls[I].Visible;
      if TranslateSlave then begin
        for J := 0 to SlaveInfo.Count-1 do
          with PSlaveInfo(SlaveInfo[J])^ do begin
            if TopBottom = Controls[I] then begin
              IsVisible := NewDockType <> dtLeftRight;
              Break;
            end;
            if LeftRight = Controls[I] then begin
              IsVisible := NewDockType = dtLeftRight;
              Break;
            end;
          end;
      end;
      if IsVisible then
        Children.Add (Controls[I]);
    end;

    Extra.CurDockType := OldDockType;
    Extra.Toolbar := Self;
    ListSortEx (Children, CompareControls, @Extra);

    GI := nil;
    NewGroup := True;
    for I := 0 to Children.Count-1 do begin
      if NewGroup then begin
        NewGroup := False;
        List.Add (AllocMem(SizeOf(TGroupInfo)));
        GI := List[List.Count-1];
        FillChar (GI^, SizeOf(TGroupInfo), 0);
        GI.Members := TList.Create;
      end;
      GI.Members.Add (Children[I]);
      if TControl(Children[I]) is TToolbarSep97 then
        NewGroup := True
      else begin
        with TControl(Children[I]) do begin
          Inc (GI.GroupWidth, Width);
          Inc (GI.GroupHeight, Height);
        end;
      end;
    end;

    Children.Free;
  end;
end;

procedure TToolbar97.AutoArrangeControls;
var
  DT: TDockType;
begin
  DT := GetDockTypeOf(DockedTo);
  ArrangeControls (True, True, DT, DT, FFloatingRightX, nil);
end;

procedure TToolbar97.ShouldBeVisible (const Control: TControl;
  const DockType: TDockType; const SetIt: Boolean; var AVisible: Boolean);
{ Sets AVisible only if it is a master or slave control. AVisible is left as is
  otherwise }
var
  J: Integer;
begin
  for J := 0 to SlaveInfo.Count-1 do
    with PSlaveInfo(SlaveInfo[J])^ do
      if TopBottom = Control then begin
        AVisible := DockType <> dtLeftRight;
        if SetIt then begin
          TopBottom.Visible := AVisible;
          LeftRight.Visible := not AVisible;
          if AVisible then
            with LeftRight do
              SetBounds (TopBottom.Left, TopBottom.Top, Width, Height);
        end;
      end
      else
      if LeftRight = Control then begin
        AVisible := DockType = dtLeftRight;
        if SetIt then begin
          TopBottom.Visible := not AVisible;
          LeftRight.Visible := AVisible;
          if AVisible then
            with TopBottom do
              SetBounds (LeftRight.Left, LeftRight.Top, Width, Height);
        end;
      end;
end;

procedure TToolbar97.ArrangeControls (const CanMove, CanResize: Boolean;
  const OldDockType, NewDockType: TDockType; RightX: Integer;
  const NewClientSize: PPoint);
{ This arranges the controls on the toolbar }
var
  OldBarHeight, OldBarWidth, OldDockedTotalBarHeight, OldDockedTotalBarWidth,
  I, NewBarHeight, NewBarWidth: Integer;
  AllowWrap, V: Boolean;
  ClientPosSize, PosSpacing, BarPosSize,
  CurPosPixel, CurLinePixel, G: Integer;
  GInfo: TList;
  GI: PGroupInfo;
  Member: TControl;
  MemberIsSep: Boolean;
  GroupPosSize, MemberPosSize: Integer;
  NewLine, Prec1Line: Boolean;
  MinPosPixels, MinRowPixels, CW, CH: Integer;
  DocksBarHeight, DocksBarWidth, DocksTotalBarHeight, DocksTotalBarWidth: Integer;
  JustDidSep: TToolbarSep97;  PrevMinPosPixels: Integer;
label 1;
begin
  { [MODIFIED] Added (csLoading in ComponentState) below. Now it doesn't
    arrange the controls while component is loading, which makes it faster }
  if (DisableArrangeControls > 0) or (csLoading in ComponentState) then Exit;

  if (NewDockType <> dtNotDocked) or (RightX = 0) then
    RightX := High(RightX);

  Inc (DisableArrangeControls);
  try
    OldBarWidth := FBarWidth;
    OldBarHeight := FBarHeight;
    OldDockedTotalBarHeight := FDockedTotalBarHeight;
    OldDockedTotalBarWidth := FDockedTotalBarWidth;

    try
      NewBarHeight := DefaultBarWidthHeight;
      NewBarWidth := DefaultBarWidthHeight;
      for I := 0 to ControlCount-1 do begin
        if Controls[I] is TToolbarSep97 then Continue;
        with Controls[I] do begin
          V := Visible;
          ShouldBeVisible (Controls[I], NewDockType, CanMove, V);
          if not V then Continue;
          if Height > NewBarHeight then
            NewBarHeight := Height;
          if Width > NewBarWidth then
            NewBarWidth := Width;
        end;
      end;
      FBarHeight := NewBarHeight;
      FBarWidth := NewBarWidth;
      { the following line is mirrored in BuildRowInfo }
      FDockedTotalBarHeight := TopMargin[dtTopBottom] + FBarHeight + BottomMargin[dtTopBottom];
      FDockedTotalBarWidth := TopMargin[dtLeftRight] + FBarWidth + BottomMargin[dtLeftRight];

      DocksBarHeight := FBarHeight;
      DocksBarWidth := FBarWidth;
      DocksTotalBarHeight := FDockedTotalBarHeight;
      DocksTotalBarWidth := FDockedTotalBarWidth;
      if CanMove and Assigned(DockedTo) then begin
        with DockedTo do begin
          BuildRowInfo;
          with GetRowInfo(FDockRow, DockedTo, Self) do begin
            DocksBarHeight := BarHeight;
            DocksBarWidth := BarWidth;
            DocksTotalBarHeight := DockedTotalBarHeight;
            DocksTotalBarWidth := DockedTotalBarWidth;
          end;
        end;
      end;

      if CanMove then
        GInfo := GroupInfo
      else
        GInfo := TList.Create;
      try
        BuildGroupInfo (GInfo, not CanMove, OldDockType, NewDockType);

        if CanMove then
          LineSeps.Clear;

        AllowWrap := NewDockType = dtNotDocked;
        if GInfo.Count <> 0 then begin
          if NewDockType <> dtLeftRight then begin
            ClientPosSize := ClientWidth;
            PosSpacing := HorzSpacing;
            BarPosSize := FBarHeight;
          end
          else begin
            ClientPosSize := ClientHeight;
            PosSpacing := VertSpacing;
            BarPosSize := FBarWidth;
          end;
          MinPosPixels := 0;
          CurPosPixel := 0;
          CurLinePixel := TopMargin[NewDockType];
          Prec1Line := True;  NewLine := True;
          JustDidSep := nil;  PrevMinPosPixels := 0;
          for G := 0 to GInfo.Count-1 do begin
            GI := PGroupInfo(GInfo[G]);

            if NewDockType <> dtLeftRight then
              GroupPosSize := GI.GroupWidth
            else
              GroupPosSize := GI.GroupHeight;
            if (not AllowWrap) or
               (Prec1Line) then begin
              if NewLine then begin
                NewLine := False;
                Inc (CurPosPixel, LeftMargin[NewDockType])
              end;
              if CurPosPixel+GroupPosSize+RightMargin[NewDockType] > RightX then
                goto 1;
              if CurPosPixel > MinPosPixels then MinPosPixels := CurPosPixel;
            end
            else begin
            1:CurPosPixel := LeftMargin[NewDockType];
              if CurPosPixel > MinPosPixels then MinPosPixels := CurPosPixel;
              if (G <> 0) and (PGroupInfo(GInfo[G-1]).Members.Count <> 0) then begin
                Inc (CurLinePixel, BarPosSize+LineSpacing);
                if Assigned(JustDidSep) then
                  MinPosPixels := PrevMinPosPixels;
                if CanMove then begin
                  if Assigned(JustDidSep) then
                    JustDidSep.Invisible := True;

                  LineSeps.Add (Pointer(CurLinePixel));
                end;
              end;
            end;
            Prec1Line := True;
            for I := 0 to GI.Members.Count-1 do begin
              Member := TControl(GI.Members[I]);
              MemberIsSep := Member is TToolbarSep97;
              with Member do begin
                if not MemberIsSep then begin
                  if NewDockType <> dtLeftRight then
                    MemberPosSize := Width
                  else
                    MemberPosSize := Height;
                end
                else begin
                  if NewDockType <> dtLeftRight then
                    MemberPosSize := HorzSpacing
                  else
                    MemberPosSize := VertSpacing;
                end;
                { If past right (or bottom) side of screen, proceed to next line }
                if not MemberIsSep and
                   (CurPosPixel+MemberPosSize+RightMargin[NewDockType] > RightX) then begin
                  CurPosPixel := LeftMargin[NewDockType];
                  if CurPosPixel > MinPosPixels then MinPosPixels := CurPosPixel;
                  Inc (CurLinePixel, FBarHeight);
                  Prec1Line := False;
                end;
                if NewDockType <> dtLeftRight then begin
                  if not MemberIsSep then begin
                    if CanMove then
                      SetBounds (CurPosPixel, CurLinePixel+((DocksBarHeight-Height) div 2), Width, Height);
                    Inc (CurPosPixel, Width);
                  end
                  else begin
                    if CanMove then
                      SetBounds (CurPosPixel, CurLinePixel, HorzSpacing, DocksBarHeight);
                    Inc (CurPosPixel, HorzSpacing);
                  end;
                end
                else begin
                  if not MemberIsSep then begin
                    if CanMove then
                      SetBounds (CurLinePixel+((DocksBarWidth-Width) div 2), CurPosPixel, Width, Height);
                    Inc (CurPosPixel, Height);
                  end
                  else begin
                    if CanMove then
                      SetBounds (CurLinePixel, CurPosPixel, DocksBarWidth, VertSpacing);
                    Inc (CurPosPixel, VertSpacing);
                  end;
                end;
                PrevMinPosPixels := MinPosPixels;
                if not MemberIsSep then
                  JustDidSep := nil
                else begin
                  JustDidSep := TToolbarSep97(Member);
                  JustDidSep.Invisible := False;
                end;
                if CurPosPixel > MinPosPixels then MinPosPixels := CurPosPixel;
              end;
            end;
          end;
        end
        else begin
          if DockedTo <> nil then begin
            MinPosPixels := LeftMargin[NewDockType];
            CurLinePixel := TopMargin[NewDockType];
            if not(DockedTo.Position in PositionLeftOrRight) then begin
              Inc (MinPosPixels, DefaultBarWidthHeight);
              BarPosSize := GetRowInfo(FDockRow, DockedTo, Self).BarHeight;
            end
            else begin
              Inc (MinPosPixels, DefaultBarWidthHeight);
              BarPosSize := GetRowInfo(FDockRow, DockedTo, Self).BarWidth;
            end;
          end
          else begin
            MinPosPixels := LeftMargin[NewDockType] + DefaultBarWidthHeight;
            CurLinePixel := TopMargin[NewDockType];
            BarPosSize := DefaultBarWidthHeight;
          end;
        end;

        if csDesigning in ComponentState then
          Invalidate;
      finally
        if not CanMove then begin
          FreeGroupInfo (GInfo);
          GInfo.Free;
        end;
      end;

      if not(csDesigning in ComponentState) then
        AdjustWidthAndHeight := True;

      if CanMove then begin
        CW := ClientWidth;
        CH := ClientHeight;
      end
      else begin
        CW := 0;
        CH := 0;
      end;

      Inc (MinPosPixels, RightMargin[NewDockType]);
      if NewDockType <> dtLeftRight then begin
        if (AdjustWidthAndHeight) or (CW < MinPosPixels) then
          CW := MinPosPixels;
      end
      else begin
        if (AdjustWidthAndHeight) or (CH < MinPosPixels) then
          CH := MinPosPixels;
      end;

      MinRowPixels := CurLinePixel + BarPosSize + BottomMargin[NewDockType];
      if NewDockType <> dtLeftRight then begin
        if (AdjustWidthAndHeight) or (CH < MinRowPixels) then
          CH := MinRowPixels;
      end
      else begin
        if (AdjustWidthAndHeight) or (CW < MinRowPixels) then
          CW := MinRowPixels;
      end;

      { [MODIFIED] Added 'if DockedTo <> nil then' around the following block,
        which should have been there in the first place.
        This fixes problem of floating toolbars being one pixel too tall. }
      if DockedTo <> nil then begin
        if NewDockType <> dtLeftRight then begin
          if CH < DocksTotalBarHeight then
            CH := DocksTotalBarHeight;
        end
        else begin
          if CW < DocksTotalBarWidth then
            CW := DocksTotalBarWidth;
        end;
      end;

      AdjustWidthAndHeight := False;

      if CanResize then begin
        Inc (UpdatingBounds);
      try
          if ClientWidth <> CW then
            ClientWidth := CW;
          if ClientHeight <> CH then
            ClientHeight := CH;
        finally
          Dec (UpdatingBounds);
        end;
      end;
      if Assigned(NewClientSize) then begin
        NewClientSize.X := CW;
        NewClientSize.Y := CH;
      end;
    finally
      if not CanMove then begin
        FBarWidth := OldBarWidth;
        FBarHeight := OldBarHeight;
        FDockedTotalBarHeight := OldDockedTotalBarHeight;
        FDockedTotalBarWidth := OldDockedTotalBarWidth;
      end;
    end;
  finally
    Dec (DisableArrangeControls);
  end;
end;

procedure TToolbar97.AlignControls (AControl: TControl; var Rect: TRect);
{ VCL calls this whenever any child controls in the toolbar are moved, sized,
  inserted, etc. It doesn't make use of the AControl and Rect parameters,
  since it doesn't need to. }
begin
  AutoArrangeControls;
end;

procedure TToolbar97.SetBounds (ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds (ALeft, ATop, AWidth, AHeight);
  { This allows you to drag the toolbar at design time }
  if (csDesigning in ComponentState) and not(csLoading in ComponentState) and
     (DockedTo <> nil) and (UpdatingBounds = 0) then begin
    if not(DockedTo.Position in PositionLeftOrRight) then begin
      FDockRow := DockedTo.GetDesignModeRowOf(ATop+(AHeight div 2));
      FDockPos := ALeft;
    end
    else begin
      FDockRow := DockedTo.GetDesignModeRowOf(ALeft+(AWidth div 2));
      FDockPos := ATop;
    end;
    DockedTo.ArrangeToolbars;
  end;
end;

procedure TToolbar97.SetParent (AParent: TWinControl);
begin
  if not(csDesigning in ComponentState) and (AParent = TForm(Owner)) then
    AParent := FloatParent;

  if (AParent <> nil) and not(AParent is TDock97) and
     not(AParent = Owner) and not(AParent is TFloatParent) then
    raise Exception.Create('TToolbar97 can only be placed on a TDock97 or directly on the form');

  if Assigned(FOnRecreating) then
    FOnRecreating (Self);

  if Parent is TDock97 then
    TDock97(Parent).ChangeDockList (False, Self, Visible or (Hidden <> 0));

  inherited SetParent (AParent);

  if Parent is TDock97 then
    TDock97(Parent).ChangeDockList (True, Self, Visible or (Hidden <> 0));

  if Assigned(FOnRecreated) then
    FOnRecreated (Self);
end;

function GetCaptionRect (const Control: TWinControl;
  const AdjustForBorder, MinusCloseButton: Boolean): TRect;
begin
  Result := Rect(0, 0, Control.ClientWidth, MetricsCaptionHeight-1);
  if MinusCloseButton then
    Dec (Result.Right, MetricsCaptionHeight-1);
  if AdjustForBorder then
    OffsetRect (Result, MetricsBorderWidth, MetricsBorderHeight);
end;

function GetCloseButtonRect (const Control: TWinControl;
  const AdjustForBorder: Boolean): TRect;
begin
  Result := Rect(0, 0, Control.ClientWidth, MetricsCaptionHeight-1);
  if AdjustForBorder then
    OffsetRect (Result, MetricsBorderWidth, MetricsBorderHeight);
  Result.Left := Result.Right - (MetricsCaptionHeight-1);
end;

procedure TToolbar97.WMNCCalcSize (var Message: TWMNCCalcSize);
begin
  inherited;
  if DockedTo = nil then
    Inc (Message.CalcSize_Params.rgrc[0].Top, MetricsCaptionHeight);
end;

procedure TToolbar97.RedrawCaption (const OnlyCloseButton, AutoActive: Boolean;
  IsActive: Boolean);
{ Redraws the caption (title bar) of the toolbar when it is floating.
  At design time, the caption is always drawn with the activated color. }
const
  ActiveFlags: array[Boolean] of UINT = (0, DC_ACTIVE);
  ActiveBkColors: array[Boolean] of Integer = (COLOR_INACTIVECAPTION, COLOR_ACTIVECAPTION);
  ActiveTextColors: array[Boolean] of Integer = (COLOR_INACTIVECAPTIONTEXT, COLOR_CAPTIONTEXT);
  CloseButtonState: array[Boolean] of UINT = (0, DFCS_PUSHED);
  LineBelowCaptionColors: array[Boolean] of Integer = (COLOR_WINDOWFRAME, COLOR_BTNFACE);

  procedure Win3DrawCaption (const DC: HDC; const R: TRect; const ActiveColor: Boolean);
  { Emulates DrawCaption, which isn't supported in NT 3.x }
  var
    R2: TRect;
    SaveTextColor, SaveBkColor: COLORREF;
    NewFont, SaveFont: HFONT;
    NewBrush: HBRUSH;
    Cap: String;

    function CaptionTextWidth: Integer;
    var
      Size: TSize;
    begin
      GetTextExtentPoint32 (DC, @Cap[1], Length(Cap), Size);
      Result := Size.cx;
    end;
  begin
    R2 := R;

    { Fill the rectangle }
    NewBrush := CreateSolidBrush(GetSysColor(ActiveBkColors[ActiveColor]));
    FillRect (DC, R2, NewBrush);
    DeleteObject (NewBrush);

    Inc (R2.Left);
    Dec (R2.Right);

    NewFont := CreateFont(11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 'MS Sans Serif');
    SaveFont := SelectObject(DC, NewFont);

    { Add ellipsis to caption if necessary }
    Cap := Caption;
    if CaptionTextWidth > R2.Right-R2.Left then begin
      Cap := Cap + '...';
      while CaptionTextWidth > R2.Right-R2.Left do begin
        if Length(Cap) <= 4 then Break;
        Delete (Cap, Length(Cap)-3, 1)
      end;
    end;

    { Draw the text }
    SaveBkColor := SetBkColor(DC, GetSysColor(ActiveBkColors[ActiveColor]));
    SaveTextColor := SetTextColor(DC, GetSysColor(ActiveTextColors[ActiveColor]));
    DrawText (DC, @Cap[1], Length(Cap), R2, DT_SINGLELINE or DT_NOPREFIX or DT_VCENTER);
    SetTextColor (DC, SaveTextColor);
    SetBkColor (DC, SaveBkColor);

    SelectObject (DC, SaveFont);
    DeleteObject (NewFont);
  end;
var
  DC: HDC;
  R: TRect;
  NewDrawCaption: function(p1: HWND; p2: HDC; const p3: TRect; p4: UINT): BOOL; stdcall;
  NewPen, SavePen: HPen;
  NewBrush: HBrush;
begin
  if DockedTo <> nil then Exit;

  if AutoActive then
    IsActive := ControlIsChildOf(FindControl(GetActiveWindow), TForm(Owner));
  CaptionIsActive := IsActive;
  if csDesigning in ComponentState then
    IsActive := True;

  DC := GetWindowDC(Handle);
  try
    if not OnlyCloseButton then begin
      { Caption }
      R := GetCaptionRect(Self, True, FCloseButton);
      if NewStyleControls then begin
        { Use a dynamic import of DrawCaption since it's only in Win95/NT 4.0.
          Also note that Delphi's Win32 help for DrawCaption is totally wrong!
          I got updated info from www.microsoft.com/msdn/sdk/ }
        NewDrawCaption := GetProcAddress(GetModuleHandle(user32), 'DrawCaption');
        NewDrawCaption (Handle, DC, R,
          ActiveFlags[IsActive] or DC_TEXT or DC_SMALLCAP);
      end
      else
        Win3DrawCaption (DC, R, IsActive);

      { Line below caption }
      R := GetCaptionRect(Self, True, False);
      NewPen := CreatePen(PS_SOLID, 1, GetSysColor(LineBelowCaptionColors[NewStyleControls]));
      SavePen := SelectObject(DC, NewPen);
      MoveToEx (DC, R.Left, R.Bottom, nil);
      LineTo (DC, R.Right, R.Bottom);
      SelectObject (DC, SavePen);
      DeleteObject (NewPen);
    end;

    { Close button }
    if FCloseButton then begin
      R := GetCloseButtonRect(Self, True);
      InflateRect (R, -1, -1);
      DrawFrameControl (DC, R, DFC_CAPTION, DFCS_CAPTIONCLOSE or
        CloseButtonState[CloseButtonPushed and CloseButtonMouseOver]);
      if not OnlyCloseButton then begin
        { Caption-colored frame around close button }
        R := GetCloseButtonRect(Self, True);
        NewBrush := CreateSolidBrush(GetSysColor(ActiveBkColors[IsActive]));
        FrameRect (DC, R, NewBrush);
        DeleteObject (NewBrush);
      end;
    end;

  finally
    ReleaseDC (Handle, DC);
  end;
end;

procedure TToolbar97.WMNCPaint (var Message: TWMNCPaint);
begin
  inherited;
  RedrawCaption (False, True, False);
end;

procedure TToolbar97.DrawDraggingRect (Rect: TRect; const Dock: TDock97);
var
  Border: Integer;
begin
  {$IFDEF TB97SMOOTH}
  if SmoothDrag then Exit;
  {$ENDIF}
  if Dock <> nil then
    DrawFocusRect (ScreenDC, Rect)
  else begin
    for Border := 1 to MetricsBorderWidth do begin
      DrawFocusRect (ScreenDC, Rect);
      InflateRect (Rect, -1, -1);
    end;
  end;
end;

procedure TToolbar97.Paint;
var
  DockType: TDockType;
  R: TRect;
  X, Y, S: Integer;
begin
  inherited Paint;

  if DockedTo = nil then
    DockType := dtNotDocked
  else begin
    if not(DockedTo.Position in PositionLeftOrRight) then
      DockType := dtTopBottom
    else
      DockType := dtLeftRight;
  end;

  if DockType <> dtNotDocked then begin
    { Border }
    R := ClientRect;  DrawEdge (Canvas.Handle, R, BDR_RAISEDINNER, BF_RECT);

    { The two things at the left, or top }
    if DockType <> dtLeftRight then begin
      Y := ClientHeight-2;
      R := Rect(4, 2, 7, Y);  DrawEdge (Canvas.Handle, R, BDR_RAISEDINNER, BF_RECT);
      Canvas.Pixels[4, Y-1] := clBtnHighlight;
      R := Rect(7, 2, 10, Y);  DrawEdge (Canvas.Handle, R, BDR_RAISEDINNER, BF_RECT);
      Canvas.Pixels[7, Y-1] := clBtnHighlight;
    end
    else begin
      X := ClientWidth-2;
      R := Rect(2, 4, X, 7);  DrawEdge (Canvas.Handle, R, BDR_RAISEDINNER, BF_RECT);
      Canvas.Pixels[X-1, 4] := clBtnHighlight;
      R := Rect(2, 7, X, 10);  DrawEdge (Canvas.Handle, R, BDR_RAISEDINNER, BF_RECT);
      Canvas.Pixels[X-1, 7] := clBtnHighlight;
    end;
  end;

  { Long separators when not docked }
  if DockedTo = nil then
    for S := 0 to LineSeps.Count-1 do begin
      Y := Integer(LineSeps[S]) + 1;
      Canvas.Pen.Color := clBtnShadow;
      Canvas.MoveTo (0, Y-5);  Canvas.LineTo (ClientWidth, Y-5);
      Canvas.Pen.Color := clBtnHighlight;
      Canvas.MoveTo (0, Y-4);  Canvas.LineTo (ClientWidth, Y-4);
    end;
end;

procedure TToolbar97.CMVisibleChanged (var Message: TMessage);
begin
  if Hidden = 0 then begin
    if Visible then begin
      if DockedTo <> nil then
        DockedTo.ChangeDockList (True, Self, Visible);
    end
    else begin
      if DockedTo <> nil then
        DockedTo.ChangeDockList (False, Self, Visible);
    end;
  end;
  inherited;
end;

procedure TToolbar97.CancelModes;
begin
  if Moving then begin
    Moving := False;

    DrawDraggingRect (DragRect, MoveDock);
    {$IFDEF TB97SMOOTH}
    if not SmoothDrag then
    {$ENDIF}
      ReleaseDC (0, ScreenDC);

    ReleaseCapture;
    {$IFDEF TB97SMOOTH}
    if SmoothDrag then begin
      if IdleHook <> 0 then
        UnhookWindowsHookEx (IdleHook);
      SmoothDragging := nil;
    end;
    {$ENDIF}
  end;

  if Sizing then begin
    Sizing := False;

    DrawDraggingRect (DragRect, nil);
    ReleaseDC (0, ScreenDC);

    ReleaseCapture;
  end;

  if CloseButtonPushed then begin
    CloseButtonPushed := False;
    if CloseButtonMouseOver then
      RedrawCaption (True, True, False);
    ReleaseCapture;
  end;
end;

procedure TToolbar97.WMCancelMode (var Message: TWMCancelMode);
{ This message is sent when a dialog box or menu comes up while the user is
  moving, sizing, or pressing the close button. This cancels whatever it's
  doing }
begin
  CancelModes;
  Message.Result := 0;
  inherited;
end;

procedure TToolbar97.WMActivate (var Message: TWMActivate);
begin
  if Message.Active <> WA_INACTIVE then
    TForm(Owner).Perform (WM_NCACTIVATE, 1, 0);
  inherited;
end;

procedure TToolbar97.WMMouseActivate (var Message: TWMMouseActivate);
begin
  inherited;
  if not(csDesigning in ComponentState) then begin
    { When floating, prevent the toolbar from activating when clicked.
      This is so it doesn't take the focus away from the window that had it }
    if DockedTo = nil then
      Message.Result := MA_NOACTIVATE;

    { Similar to calling BringWindowToTop, but doesn't activate it }
    SetWindowPos (Handle, HWND_TOP, 0, 0, 0, 0,
      SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE);
  end;
end;

procedure TToolbar97.MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  function ControlExistsAtPos (const P: TPoint): Boolean;
  var
    I: Integer;
  begin
    Result := False;
    for I := 0 to ControlCount-1 do
      if not(Controls[I] is TToolbarSep97) and Controls[I].Visible and
         PtInRect(Controls[I].BoundsRect, P) then begin
        Result := True;
        Break;
      end;
  end;
var
  DT: TDockType;
begin
  if Button <> mbLeft then CancelModes;
  inherited MouseDown(Button, Shift, X, Y);
  if Button <> mbLeft then Exit;

  { Ignore message if user clicked on a control that was probably disabled }
  if ControlExistsAtPos(Point(X, Y)) then Exit;

  if ssDouble in Shift then begin
    if DockedTo <> nil then
      DockedTo := nil
    else begin
      FDockRow := ForceDockAtTopRow;
      FDockPos := ForceDockAtLeftPos;
      DockedTo := DefaultDock;
    end;
  end
  else
  if not Moving then begin
    Moving := True;
    MoveDock := nil;
    OldTopLeft := Point(X, Y);
    {$IFDEF TB97SMOOTH}
    SmoothDrag := GetDragFullWindows;
    if not SmoothDrag then
    {$ENDIF}
      ScreenDC := GetDC(0);
    SetRectEmpty (DragRect);
    SetCapture (Handle);
    { In SmoothDrag mode, set up a timer to process mouse messages. This is
      needed because the mouse is not captured when the cursor is over a
      different thread's window. This allows all mouse moves to be directed
      to the toolbar. }
    {$IFDEF TB97SMOOTH}
    if SmoothDrag then begin
      PrevPos := Point(-1, -1);
      SmoothDragging := Self;
      IdleHook := SetWindowsHookEx(WH_FOREGROUNDIDLE, ForegroundIdle, 0, GetCurrentThreadId);
    end;
    {$ENDIF}

    { Set up sizes }
    DT := GetDockTypeOf(DockedTo);
    ArrangeControls (False, False, DT, dtNotDocked, FFloatingRightX, @MoveNewSize[dtNotDocked]);
    ArrangeControls (False, False, DT, dtTopBottom, FFloatingRightX, @MoveNewSize[dtTopBottom]);
    ArrangeControls (False, False, DT, dtLeftRight, FFloatingRightX, @MoveNewSize[dtLeftRight]);

    { Relay message to MouseMove }
    {$IFDEF TB97SMOOTH}
    if not SmoothDrag then
    {$ENDIF}
      MouseMove (Shift, X, Y);
  end;
end;

procedure TToolbar97.MouseMover (Shift: TShiftState; X, Y: Integer);
var
  OldMoveDock: TDock97;
  OldDragRect: TRect;
  I: Integer;

  function CheckIfCanDockTo (Control: TDock97): Boolean;
  var
    DT: TDockType;
    Test1, Test2, CheckIntersect: TRect;
  begin
    with Control do begin
      if not(Control.Position in PositionLeftOrRight) then
        DT := dtTopBottom
      else
        DT := dtLeftRight;
      Test1.TopLeft := Parent.ScreenToClient(DragRect.TopLeft);
      Test1.Right := Test1.Left + MoveNewSize[DT].X;
      Test1.Bottom := Test1.Top + MoveNewSize[DT].Y;
      if ((DockedTo = nil) and (Control.Position in PositionLeftOrRight)) or
         ((DockedTo <> nil) and
          ((DockedTo.Position in PositionLeftOrRight) <> (Control.Position in PositionLeftOrRight))) then
        OffsetRect (Test1, OldTopLeft.X-(MoveNewSize[dtLeftRight].X div 2), 0);

      Test2 := BoundsRect;
      { Inflate if it is zero in width or height }
      if (Test2.Top = Test2.Bottom) or (Test2.Left = Test2.Right) then
        case Control.Position of
          dpTop: Dec (Test2.Top, 1);
          dpBottom: Inc (Test2.Bottom, 1);
          dpLeft: Dec (Test2.Left, 1);
          dpRight: Inc (Test2.Right, 1);
        end;

      Result := IntersectRect(CheckIntersect, Test1, Test2);
    end;
  end;
begin
  if Moving then begin
    if not(ssLeft in Shift) then begin
      CancelModes;
      Exit;
    end;

    OldMoveDock := MoveDock;
    OldDragRect := DragRect;

    if DockedTo = nil then begin
      DragRect := BoundsRect;
      RemoveNCAreaFromRect (DragRect);
      OffsetRect (DragRect, X-OldTopLeft.X, Y-OldTopLeft.Y);
    end
    else begin
      DragRect := ClientRect;
      MapWindowPoints (Handle, 0, DragRect, 2);
      OffsetRect (DragRect, X-OldTopLeft.X, Y-OldTopLeft.Y);
    end;

    { Check if it can dock. Don't try to dock if user is holding down Ctrl }
    MoveDock := nil;
    if not(ssCtrl in Shift) then begin
      { Search through the form's controls and see if it can find a
        Dock97. If it finds one, assign it to MoveDock. }
      with TForm(Owner) do begin
        { Try top/bottom first }
        for I := 0 to ControlCount-1 do
          if (Controls[I] is TDock97) and
             not(TDock97(Controls[I]).Position in PositionLeftOrRight) and
             CheckIfCanDockTo(TDock97(Controls[I])) then begin
            MoveDock := TDock97(Controls[I]);
            Break;
          end;
        { Then left/right }
        if (MoveDock = nil) and (FCanDockLeftRight) then
          for I := 0 to ControlCount-1 do
            if (Controls[I] is TDock97) and
               (TDock97(Controls[I]).Position in PositionLeftOrRight) and
               CheckIfCanDockTo(TDock97(Controls[I])) then begin
              MoveDock := TDock97(Controls[I]);
              Break;
            end;
      end;
    end;

    if Assigned(MoveDock) then begin
      if ((DockedTo = nil) and (MoveDock.Position in PositionLeftOrRight)) or
         ((DockedTo <> nil) and
          ((DockedTo.Position in PositionLeftOrRight) <> (MoveDock.Position in PositionLeftOrRight))) then
        OffsetRect (DragRect, OldTopLeft.X-(MoveNewSize[dtLeftRight].X div 2), 0);
    end;

    DragRect.Right := DragRect.Left + MoveNewSize[GetDockTypeOf(MoveDock)].X;
    DragRect.Bottom := DragRect.Top + MoveNewSize[GetDockTypeOf(MoveDock)].Y;

    { Make up for margin differences if not docked and wanting to dock, or
      vice versa }
    if GetDockTypeOf(MoveDock) <> dtLeftRight then begin
      OffsetRect (DragRect, (MoveNewSize[GetDockTypeOf(MoveDock)].X-MoveNewSize[GetDockTypeOf(DockedTo)].X) div -2, 0);
      if (GetDockTypeOf(DockedTo) = dtLeftRight) and (GetDockTypeOf(MoveDock) = dtTopBottom) then
          OffsetRect (DragRect, -(OldTopLeft.X - (ClientWidth div 2)), 0);
    end;

    { Add non-client area to DragRect if not docking }
    if MoveDock = nil then
      AddNCAreaToRect (DragRect);

    { Draw the new FocusRect }
    DrawDraggingRect (DragRect, MoveDock);

    { Hide the old FocusRect }
    if not IsRectEmpty(OldDragRect) then
      DrawDraggingRect (OldDragRect, OldMoveDock);

    {$IFDEF TB97SMOOTH}
    if SmoothDrag then begin
      Dropped;
      SetCapture (Handle);
    end;
    {$ENDIF}
  end
  else
  if Sizing then begin
    OldDragRect := DragRect;

    RemoveNCAreaFromRect (DragRect);

    if SizeHitTest in [HTLEFT, HTRIGHT] then begin
      if SizeHitTest = HTLEFT then
        X := ClientWidth - X;
      for I := SizeNewSizes.Count-1 downto 0 do
        if (I = SizeNewSizes.Count-1) or (X >= TSmallPoint(SizeNewSizes[I]).X) then
          with TSmallPoint(SizeNewSizes[I]) do begin
            SizeCurX := X;
            if SizeHitTest = HTRIGHT then
              DragRect.Right := DragRect.Left + X
            else
              DragRect.Left := DragRect.Right - X;
            DragRect.Bottom := DragRect.Top + Y;
          end;
    end
    else begin
      if SizeHitTest = HTTOP then
        Y := ClientHeight - Y;
      for I := 0 to SizeNewSizes.Count-1 do
        if (I = 0) or (Y >= TSmallPoint(SizeNewSizes[I]).Y) then
          with TSmallPoint(SizeNewSizes[I]) do begin
            SizeCurX := X;
            if SizeHitTest = HTBOTTOM then
              DragRect.Bottom := DragRect.Top + Y
            else
              DragRect.Top := DragRect.Bottom - Y;
            DragRect.Right := DragRect.Left + X;
          end;
    end;

    AddNCAreaToRect (DragRect);

    if not EqualRect(DragRect, OldDragRect) then begin
      { Draw the new FocusRect }
      DrawDraggingRect (DragRect, nil);

      { Hide the old FocusRect }
      if not IsRectEmpty(OldDragRect) then
        DrawDraggingRect (OldDragRect, nil);
    end;
  end;
end;

procedure TToolbar97.MouseMove (Shift: TShiftState; X, Y: Integer);
var
  NewMouseOver: Boolean;
  P: TPoint;
begin
  inherited MouseMove (Shift, X, Y);

  {$IFDEF TB97SMOOTH}
  if not SmoothDrag then
  {$ENDIF}
    MouseMover (Shift, X, Y);

  if CloseButtonPushed then begin
    P := ClientToScreen(Point(X, Y));
    Dec (P.X, Left);  Dec (P.Y, Top);

    NewMouseOver := PtInRect(GetCloseButtonRect(Self, True), P);
    if CloseButtonMouseOver <> NewMouseOver then begin
      CloseButtonMouseOver := NewMouseOver;
      RedrawCaption (True, True, False);
    end;
  end;
end;

procedure TToolbar97.Dropped;
var
  NewDockRow: Integer;
  Before: Boolean;
  DragRectClient: TRect;
  C: Integer;
begin
  if MoveDock <> nil then begin
    DragRectClient := DragRect;
    MapWindowPoints (0, MoveDock.Handle, DragRectClient, 2);
    OffsetRect (DragRectClient,
      (-BoundLineSize) * Ord(blLeft in MoveDock.BoundLines),
      (-BoundLineSize) * Ord(blTop  in MoveDock.BoundLines));
    if not(MoveDock.Position in PositionLeftOrRight) then
      C := (DragRectClient.Top+DragRectClient.Bottom) div 2
    else
      C := (DragRectClient.Left+DragRectClient.Right) div 2;
    NewDockRow := TDock97(MoveDock).GetRowOf(C, Before);
    if Before then
      TDock97(MoveDock).InsertRowBefore (NewDockRow);
    FDockRow := NewDockRow;
    if not(MoveDock.Position in PositionLeftOrRight) then
      FDockPos := DragRectClient.Left
    else
      FDockPos := DragRectClient.Top;
    DockedTo := MoveDock;
  end
  else begin
    FFloatingRect := DragRect;
    DockedTo := nil;
  end;
end;

procedure TToolbar97.MouseUp (Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button <> mbLeft then CancelModes;
  inherited MouseUp (Button, Shift, X, Y);
  if Button <> mbLeft then Exit;

  if Moving then begin
    CancelModes;

    {$IFDEF TB97SMOOTH}
    if not SmoothDrag then
    {$ENDIF}
      Dropped;

    { Make sure it doesn't go completely off the screen }
    MoveOnScreen (True);
  end
  else
  if Sizing then begin
    CancelModes;

    FFloatingRightX := SizeCurX;
    Dropped;

    { Make sure it doesn't go completely off the screen }
    MoveOnScreen (True);
  end
  else
  if CloseButtonPushed then begin
    CancelModes;
    if CloseButtonMouseOver and FCloseButton then begin
      { Hide the window after close button is pushed }
      Hide;
      if Assigned(FOnClose) then
        FOnClose (Self);
    end;
  end;
end;

procedure TToolbar97.WMNCHitTest (var Message: TWMNCHitTest);
var
  P: TPoint;
begin
  inherited;
  if DockedTo <> nil then Exit;

  with Message do begin
    P := SmallPointToPoint(Pos);
    Dec (P.X, Left);  Dec (P.Y, Top);
    case Result of
      HTNOWHERE: begin
          if PtInRect(GetCaptionRect(Self, True, False), P) then begin
            Result := HTCLIENT;
            if FCloseButton and PtInRect(GetCloseButtonRect(Self, True), P) then
              Result := HTCLOSE;
          end;
        end;
      HTSIZEFIRST..HTSIZELAST: begin
          if P.Y <= MetricsBorderHeight then Result := HTTOP else
          if P.Y >= Height-MetricsBorderHeight-1 then Result := HTBOTTOM else
          if P.X <= MetricsBorderWidth then Result := HTLEFT else
          if P.X >= Width-MetricsBorderWidth-1 then Result := HTRIGHT else
            Result := HTNOWHERE;
        end;
    end;
  end;
end;

procedure TToolbar97.WMNCLButtonDown (var Message: TWMNCLButtonDown);
  procedure BuildSizeNewSizes;
  var
    DT: TDockType;
    Max, X, LastY, SkipTo: Integer;
    S: TPoint;
  begin
    DT := GetDockTypeOf(DockedTo);
    ArrangeControls (False, False, DT, dtNotDocked, 0, @S);
    { NOTE: MouseMover expects SizeNewSize to be sorted from highest X to
      lowest X. This adds them to the list in that order }
    SizeNewSizes.Clear;
    SizeNewSizes.Add (Pointer(PointToSmallPoint(S)));
    LastY := S.Y;
    Max := S.X;
    SkipTo := High(SkipTo);
    for X := Max-1 downto LeftMargin[dtNotDocked]+FBarWidth+RightMargin[dtNotDocked]+1 do begin
      if X > SkipTo then Continue;
      ArrangeControls (False, False, DT, dtNotDocked, X, @S);
      if X = S.X then begin
        if S.Y = LastY then
          SizeNewSizes.Delete (SizeNewSizes.Count-1);
        if SizeNewSizes.IndexOf(Pointer(PointToSmallPoint(S))) = -1 then
          SizeNewSizes.Add (Pointer(PointToSmallPoint(S)));
        LastY := S.Y;
      end
      else
        SkipTo := S.X;
    end;
  end;
var
  P: TPoint;
begin
  if DockedTo <> nil then begin
    inherited;
    Exit;
  end;

  case Message.HitTest of
    HTLEFT, HTRIGHT, HTTOP, HTBOTTOM:
      if not Sizing then begin
        BuildSizeNewSizes;
        SizeHitTest := Message.HitTest;
        Sizing := True;
        ScreenDC := GetDC(0);
        DragRect := BoundsRect;
        DrawDraggingRect (DragRect, nil);
        SetCapture (Handle);
        P := ScreenToClient(Point(Message.XCursor, Message.YCursor));
        MouseMove ([], P.X, P.Y);
      end;
    HTCLOSE: begin
        if not CloseButtonPushed and FCloseButton then begin
          CloseButtonPushed := True;
          CloseButtonMouseOver := True;
          RedrawCaption (True, True, False);
          SetCapture (Handle);
          { MouseUp handles letting up on close button }
        end;
      end;
  else
    inherited;
  end;
end;

procedure ShowHideFloatParents (const Control: TWinControl; const ShowIt: Boolean);
var
  I: Integer;
begin
  for I := 0 to Control.ControlCount-1 do begin
    if not(Control.Controls[I] is TWinControl) then
      Continue;
    ShowHideFloatParents (TWinControl(Control.Controls[I]), ShowIt);
    if Control.Controls[I] is TFloatParent then
      TFloatParent(Control.Controls[I]).Visible := ShowIt;
  end;
end;

procedure TToolbar97.NewFormWindowProc (var Message: TMessage);
var
  CallOld: Boolean;
  I: Integer;
  NewActive: Boolean;
begin
  CallOld := True;
  case Message.Msg of
    WM_ACTIVATE: with TWMActivate(Message) do begin
        { Show inactive title on toolbars when a different form is activated }
        NewActive := Active <> WA_INACTIVE;
        if (not NewActive) and
           (ControlInToolbar97(FindControl(ActiveWindow))) then
          NewActive := True;
        with TForm(Owner) do
          for I := 0 to ComponentCount-1 do
            if Components[I] is TToolbar97 then
              with TToolbar97(Components[I]) do
                if NewActive <> CaptionIsActive then
                  RedrawCaption (False, False, NewActive);

        if (TForm(Owner).FormStyle <> fsMDIForm) and
           ControlInToolbar97(TForm(Owner).ActiveControl) then
          CallOld := False;

        ShowHideFloatParents (TForm(Owner),
          Application.Active and not(IsIconic(TForm(Owner).Handle)));
      end;
    WM_SETFOCUS: begin
        { Prevent re-focus of toolbar }
        if (TForm(Owner).FormStyle <> fsMDIForm) and
           ControlInToolbar97(TForm(Owner).ActiveControl) then
          CallOld := False;
      end;
    WM_WINDOWPOSCHANGED: begin
        ShowHideFloatParents (TForm(Owner),
          Application.Active and not(IsIconic(TForm(Owner).Handle)));
      end;
  end;
  if CallOld then
    with Message do
      Result := CallWindowProc(OldFormWindowProc, TForm(Owner).Handle,
        Msg, wParam, lParam);
end;

function TToolbar97.NewMainWindowHook (var Message: TMessage): Boolean;
begin
  Result := False;
  case Message.Msg of
    CM_ACTIVATE, CM_DEACTIVATE: begin
        ShowHideFloatParents (TForm(Owner),
          (Message.Msg = CM_ACTIVATE) and not(IsIconic(TForm(Owner).Handle)));
        { Correct the color of the form's caption if necessary }
        TForm(Owner).Perform (WM_NCACTIVATE, Ord(Message.Msg = CM_ACTIVATE), 0);
      end;
  end;
end;

//////////////////////////////////////////////////////////////////////////////
// TToolbar97 - property access methods
//////////////////////////////////////////////////////////////////////////////

procedure TToolbar97.SetCloseButton (Value: Boolean);
begin
  if FCloseButton <> Value then begin
    FCloseButton := Value;

    { Update the close button's visibility }
    RedrawCaption (False, True, False);
  end;
end;

function TToolbar97.GetDockedTo: TDock97;
var
  Test: TWinControl;
begin
  Test := Parent;
  if Test = FloatParent then
    Result := nil
  else
    Result := TDock97(Test);
end;
procedure TToolbar97.SetDockedTo (Value: TDock97);
var
  OldDockedTo: TDock97;
  HiddenInced: Boolean;
begin
  OldDockedTo := DockedTo;

  if Assigned(FOnDockChanging) then
    FOnDockChanging (Self);

  if Assigned(Value) then
    Inc (Value.DisableArrangeToolbars);
  try
    { Before changing between docked and floating state (and vice-versa)
      or between docks, hide the toolbar. This prevents any flashing while
      it's being moved }
    HiddenInced := False;
    if not(csDesigning in ComponentState) and (Value <> OldDockedTo) and (Visible) then begin
      Inc (Hidden);
      HiddenInced := True;
      if Assigned(OldDockedTo) then
        { Need to disable arranging of current dock so it doesn't lose it's
          FDockRow/FDockPos it's going to set later }
        Inc (OldDockedTo.DisableArrangeToolbars);
      try
        Hide; {must Hide AFTER incing Hidden}
      finally
        if Assigned(OldDockedTo) then
          Dec (OldDockedTo.DisableArrangeToolbars);
      end;
    end;
    try
      if Value <> nil then begin
        { Must pre-arrange controls in new dock orientation before changing
          the Parent }
        if Parent <> nil then
          ArrangeControls (True, False, GetDockTypeOf(OldDockedTo),
            GetDockTypeOf(Value), FFloatingRightX, nil);
        Inc (DisableArrangeControls);
        try
          Parent := Value;
        finally
          Dec (DisableArrangeControls);
        end;
        AdjustWidthAndHeight := True;
        AutoArrangeControls;
      end
      else begin
        if IsRectEmpty(FFloatingRect) then begin
          FFloatingRect := BoundsRect;
          AddNCAreaToRect (FFloatingRect);
          OffsetRect (FFloatingRect, -FFloatingRect.Left, -FFloatingRect.Top);
        end;
        AdjustWidthAndHeight := True;
        { Must pre-arrange controls in new dock orientation before changing
          the Parent }
        if Parent <> nil then
          ArrangeControls (True, False, GetDockTypeOf(OldDockedTo),
            GetDockTypeOf(Value), FFloatingRightX, nil);
        Inc (DisableArrangeControls);
        try
          Parent := FloatParent;
          BoundsRect := FFloatingRect;
        finally
          Dec (DisableArrangeControls);
        end;
        AdjustWidthAndHeight := True;
        AutoArrangeControls;
      end;
    finally
      if HiddenInced then begin
        Dec (Hidden);
        Show;
      end;
    end;
  finally
    if Assigned(Value) then
      Dec (Value.DisableArrangeToolbars);
  end;
  if Assigned(Value) then
    Value.ArrangeToolbars;

  if Assigned(FOnDockChanged) then
    FOnDockChanged (Self);
end;

procedure TToolbar97.SetDockPos (Value: Integer);
begin
  FDockPos := Value;
  if Assigned(DockedTo) then
    DockedTo.ArrangeToolbars;
end;

procedure TToolbar97.SetDockRow (Value: Integer);
begin
  FDockRow := Value;
  if Assigned(DockedTo) then
    DockedTo.ArrangeToolbars;
end;


//////////////////////////////////////////////////////////////////////////////
// TToolbarSep97 - internal
//////////////////////////////////////////////////////////////////////////////

procedure TToolbarSep97.SetParent (AParent: TWinControl);
begin
  if (AParent <> nil) and not(AParent is TToolbar97) then
    raise Exception.Create('TToolbarSep97 can only be placed on a TToolbar97');
  inherited;
end;

procedure TToolbarSep97.Paint;
var
  R: TRect;
  Z: Integer;
begin
  inherited;
  if not(Parent is TToolbar97) or Invisible then Exit;

  with Canvas do begin
    { Draw dotted border in design mode }
    if csDesigning in ComponentState then begin
      Pen.Style := psDot;
      Pen.Color := clBtnShadow;
      Brush.Style := bsClear;
      R := ClientRect;
      Rectangle (R.Left, R.Top, R.Right, R.Bottom);
      Pen.Style := psSolid;
    end;

    if GetDockTypeOf(TToolbar97(Parent).DockedTo) <> dtLeftRight then begin
      Z := Width div 2;
      if not Invisible then
        Pen.Color := clBtnShadow
      else
        Pen.Color := Color;
      MoveTo (Z-1, 0);  LineTo (Z-1, Height);
      if not Invisible then
        Pen.Color := clBtnHighlight
      else
        Pen.Color := Color;
      MoveTo (Z, 0);  LineTo (Z, Height);
    end
    else begin
      Z := Height div 2;
      if not Invisible then
        Pen.Color := clBtnShadow
      else
        Pen.Color := Color;
      MoveTo (0, Z-1);  LineTo (Width, Z-1);
      if not Invisible then
        Pen.Color := clBtnHighlight
      else
        Pen.Color := Color;
      MoveTo (0, Z);  LineTo (Width, Z);
    end;
  end;
end;

procedure TToolbarSep97.MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  P: TPoint;
begin
  inherited;
  if not (Parent is TToolbar97) then Exit;

  { Relay the message to the parent }
  P := Parent.ScreenToClient(ClientToScreen(Point(X, Y)));
  TToolbar97(Parent).MouseDown (Button, Shift, P.X, P.Y);
end;


//////////////////////////////////////////////////////////////////////////////
// TToolbarButton97 - internal
//////////////////////////////////////////////////////////////////////////////

type
  TGlyphList = class(TImageList)
  private
    Used: TBits;
    FCount: Integer;
    function AllocateIndex: Integer;
  public
    constructor CreateSize(AWidth, AHeight: Integer);
    destructor Destroy; override;
    function AddMasked(Image: TBitmap; MaskColor: TColor): Integer;
    procedure Delete(Index: Integer);
    property Count: Integer read FCount;
  end;

  TGlyphCache = class
  private
    GlyphLists: TList;
  public
    constructor Create;
    destructor Destroy; override;
    function GetList(AWidth, AHeight: Integer): TGlyphList;
    procedure ReturnList(List: TGlyphList);
    function Empty: Boolean;
  end;

  TButtonGlyph = class
  private
    FOriginal: TBitmap;
    FGlyphList: TGlyphList;
    FIndexs: array[TButtonState] of Integer;
    FTransparentColor: TColor;
    FNumGlyphs: TNumGlyphs;
    FOnChange: TNotifyEvent;
    procedure GlyphChanged(Sender: TObject);
    procedure SetGlyph(Value: TBitmap);
    procedure SetNumGlyphs(Value: TNumGlyphs);
    procedure Invalidate;
    function CreateButtonGlyph(State: TButtonState): Integer;
    procedure DrawButtonGlyph(Canvas: TCanvas; const GlyphPos: TPoint;
      State: TButtonState; Transparent: Boolean);
    procedure DrawButtonText(Canvas: TCanvas; const Caption: string;
      TextBounds: TRect; State: TButtonState);
    procedure CalcButtonLayout(Canvas: TCanvas; const Client: TRect;
      const Offset: TPoint; const Caption: string; Layout: TButtonLayout;
      Margin, Spacing: Integer; var GlyphPos: TPoint; var TextBounds: TRect);
  public
    constructor Create;
    destructor Destroy; override;
    { return the text rectangle }
    function Draw(Canvas: TCanvas; const Client: TRect; const Offset: TPoint;
      const Caption: string; Layout: TButtonLayout; Margin, Spacing: Integer;
      State: TButtonState; Transparent: Boolean): TRect;
    property Glyph: TBitmap read FOriginal write SetGlyph;
    property NumGlyphs: TNumGlyphs read FNumGlyphs write SetNumGlyphs;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

{ TGlyphList }

constructor TGlyphList.CreateSize(AWidth, AHeight: Integer);
begin
  inherited CreateSize(AWidth, AHeight);
  Used := TBits.Create;
end;

destructor TGlyphList.Destroy;
begin
  Used.Free;
  inherited Destroy;
end;

function TGlyphList.AllocateIndex: Integer;
begin
  Result := Used.OpenBit;
  if Result >= Used.Size then
  begin
    Result := inherited Add(nil, nil);
    Used.Size := Result + 1;
  end;
  Used[Result] := True;
end;

function TGlyphList.AddMasked(Image: TBitmap; MaskColor: TColor): Integer;
  {$IFNDEF VER100}
  procedure NewReplaceMasked (Index: Integer; NewImage: TBitmap; MaskColor: TColor);
    procedure CheckImage(Image: TGraphic);
    begin
      if Image = nil then Exit;
      if (Image.Height < Height) or (Image.Width < Width) then
        raise EInvalidOperation.Create(LoadStr(SInvalidImageSize));
    end;
  var
    TempIndex: Integer;
    Image, Mask: TBitmap;
  begin
    if HandleAllocated then
    begin
      CheckImage(NewImage);
      TempIndex := inherited AddMasked(NewImage, MaskColor);
      if TempIndex <> -1 then
      try
        Image := TBitmap.Create;
        Mask := TBitmap.Create;
        try
          Image.Height := Height;
          Image.Width := Width;
          Mask.Height := Height;
          Mask.Width := Width;
          ImageList_Draw(Handle, TempIndex, Image.Canvas.Handle, 0, 0, ILD_NORMAL);
          ImageList_Draw(Handle, TempIndex, Mask.Canvas.Handle, 0, 0, ILD_MASK);
          if not ImageList_Replace(Handle, Index, Image.Handle, Mask.Handle) then
            raise EInvalidOperation.Create(LoadStr(SReplaceImage));
        finally
          Image.Free;
          Mask.Free;
        end;
      finally
        inherited Delete(TempIndex);
      end
      else
        raise EInvalidOperation.Create(LoadStr(SReplaceImage));
    end;
    Change;
  end;
  {$ENDIF}
begin
  Result := AllocateIndex;
  {$IFDEF VER100}
  ReplaceMasked(Result, Image, MaskColor);
  {$ELSE}
  { Delphi 2's ReplaceMasked doesn't work like Delphi 3's. I don't know what
    the problem could be so I copied the code from Delphi 3's ReplaceMasked. }
  NewReplaceMasked (Result, Image, MaskColor);
  {$ENDIF}
  Inc(FCount);
end;

procedure TGlyphList.Delete(Index: Integer);
begin
  if Used[Index] then
  begin
    Dec(FCount);
    Used[Index] := False;
  end;
end;

{ TGlyphCache }

constructor TGlyphCache.Create;
begin
  inherited Create;
  GlyphLists := TList.Create;
end;

destructor TGlyphCache.Destroy;
begin
  GlyphLists.Free;
  inherited Destroy;
end;

function TGlyphCache.GetList(AWidth, AHeight: Integer): TGlyphList;
var
  I: Integer;
begin
  for I := GlyphLists.Count - 1 downto 0 do
  begin
    Result := GlyphLists[I];
    with Result do
      if (AWidth = Width) and (AHeight = Height) then Exit;
  end;
  Result := TGlyphList.CreateSize(AWidth, AHeight);
  GlyphLists.Add(Result);
end;

procedure TGlyphCache.ReturnList(List: TGlyphList);
begin
  if List = nil then Exit;
  if List.Count = 0 then
  begin
    GlyphLists.Remove(List);
    List.Free;
  end;
end;

function TGlyphCache.Empty: Boolean;
begin
  Result := GlyphLists.Count = 0;
end;

var
  GlyphCache: TGlyphCache = nil;
  Pattern: TBitmap = nil;
  ButtonCount: Integer = 0;

procedure CreateBrushPattern;
var
  X, Y: Integer;
begin
  Pattern := TBitmap.Create;
  Pattern.Width := 8;
  Pattern.Height := 8;
  with Pattern.Canvas do
  begin
    Brush.Style := bsSolid;
    Brush.Color := clBtnFace;
    FillRect(Rect(0, 0, Pattern.Width, Pattern.Height));
    for Y := 0 to 7 do
      for X := 0 to 7 do
        if (Y mod 2) = (X mod 2) then  { toggles between even/odd pixles }
          Pixels[X, Y] := clBtnHighlight;     { on even/odd rows }
  end;
end;


{ TButtonGlyph }

constructor TButtonGlyph.Create;
var
  I: TButtonState;
begin
  inherited Create;
  FOriginal := TBitmap.Create;
  FOriginal.OnChange := GlyphChanged;
  FTransparentColor := clOlive;
  FNumGlyphs := 1;
  for I := Low(I) to High(I) do
    FIndexs[I] := -1;
  if GlyphCache = nil then GlyphCache := TGlyphCache.Create;
end;

destructor TButtonGlyph.Destroy;
begin
  FOriginal.Free;
  Invalidate;
  if Assigned(GlyphCache) and GlyphCache.Empty then
  begin
    GlyphCache.Free;
    GlyphCache := nil;
  end;
  inherited Destroy;
end;

procedure TButtonGlyph.Invalidate;
var
  I: TButtonState;
begin
  for I := Low(I) to High(I) do
  begin
    if FIndexs[I] <> -1 then FGlyphList.Delete(FIndexs[I]);
    FIndexs[I] := -1;
  end;
  GlyphCache.ReturnList(FGlyphList);
  FGlyphList := nil;
end;

procedure TButtonGlyph.GlyphChanged(Sender: TObject);
begin
  if Sender = FOriginal then
  begin
    FTransparentColor := FOriginal.TransparentColor;
    Invalidate;
    if Assigned(FOnChange) then FOnChange(Self);
  end;
end;

procedure TButtonGlyph.SetGlyph(Value: TBitmap);
var
  Glyphs: Integer;
begin
  Invalidate;
  FOriginal.Assign(Value);
  if (Value <> nil) and (Value.Height > 0) then
  begin
    FTransparentColor := Value.TransparentColor;
    if Value.Width mod Value.Height = 0 then
    begin
      Glyphs := Value.Width div Value.Height;
      if Glyphs > 4 then Glyphs := 1;
      SetNumGlyphs(Glyphs);
    end;
  end;
end;

procedure TButtonGlyph.SetNumGlyphs(Value: TNumGlyphs);
begin
  if (Value <> FNumGlyphs) and (Value > 0) then
  begin
    Invalidate;
    FNumGlyphs := Value;
    GlyphChanged(Glyph);
  end;
end;

{$IFNDEF VER100}
type
  PMaxLogPalette = ^TMaxLogPalette;
  TMaxLogPalette = packed record
    palVersion: Word;
    palNumEntries: Word;
    palPalEntry: array [Byte] of TPaletteEntry;
  end;
function CopyPalette(Palette: HPALETTE): HPALETTE;
var
  PaletteSize: Integer;
  LogPal: TMaxLogPalette;
begin
  Result := 0;
  if Palette = 0 then Exit;
  PaletteSize := 0;
  if GetObject(Palette, SizeOf(PaletteSize), @PaletteSize) = 0 then Exit;
  if PaletteSize = 0 then Exit;
  with LogPal do
  begin
    palVersion := $0300;
    palNumEntries := PaletteSize;
    GetPaletteEntries(Palette, 0, PaletteSize, palPalEntry);
  end;
  Result := CreatePalette(PLogPalette(@LogPal)^);
end;
{$ENDIF}

function TButtonGlyph.CreateButtonGlyph(State: TButtonState): Integer;
const
  ROP_DSPDxax = $00E20746;
var
  TmpImage, DDB, MonoBmp: TBitmap;
  IWidth, IHeight: Integer;
  IRect, ORect: TRect;
  I: TButtonState;
  DestDC: HDC;
  {$IFNDEF VER100}
  {Tmp2: TBitmap;}
  {$ENDIF}
begin
  if (State = bsDown) and (NumGlyphs < 3) then State := bsUp;
  Result := FIndexs[State];
  if Result <> -1 then Exit;
  if (FOriginal.Width or FOriginal.Height) = 0 then Exit;
  IWidth := FOriginal.Width div FNumGlyphs;
  IHeight := FOriginal.Height;
  if FGlyphList = nil then
  begin
    if GlyphCache = nil then GlyphCache := TGlyphCache.Create;
    FGlyphList := GlyphCache.GetList(IWidth, IHeight);
  end;
  TmpImage := TBitmap.Create;
  try
    TmpImage.Width := IWidth;
    TmpImage.Height := IHeight;
    IRect := Rect(0, 0, IWidth, IHeight);
    TmpImage.Canvas.Brush.Color := clBtnFace;
    TmpImage.Palette := CopyPalette(FOriginal.Palette);
    I := State;
    if Ord(I) >= NumGlyphs then I := bsUp;
    ORect := Rect(Ord(I) * IWidth, 0, (Ord(I) + 1) * IWidth, IHeight);
    case State of
      bsUp, bsDown,
      bsExclusive:
        begin
          TmpImage.Canvas.CopyRect(IRect, FOriginal.Canvas, ORect);
          { Delphi 2 doesn't have a TransparentMode like Delphi 3 }
          {$IFDEF VER100}
          if FOriginal.TransparentMode = tmFixed then
          {$ENDIF}
            FIndexs[State] := FGlyphList.AddMasked(TmpImage, FTransparentColor)
          {$IFDEF VER100}
          else
            FIndexs[State] := FGlyphList.AddMasked(TmpImage, clDefault);
          {$ENDIF}
        end;
      bsDisabled:
        begin
          MonoBmp := nil;
          DDB := nil;
          try
            MonoBmp := TBitmap.Create;
            DDB := TBitmap.Create;
            DDB.Assign(FOriginal);
            {$IFDEF VER100}
            DDB.HandleType := bmDDB;
            {$ENDIF}
            if NumGlyphs > 1 then
            with TmpImage.Canvas do
            begin    { Change white & gray to clBtnHighlight and clBtnShadow }
              CopyRect(IRect, DDB.Canvas, ORect);
              MonoBmp.Monochrome := True;
              MonoBmp.Width := IWidth;
              MonoBmp.Height := IHeight;

              { Convert white to clBtnHighlight }
              DDB.Canvas.Brush.Color := clWhite;
              MonoBmp.Canvas.CopyRect(IRect, DDB.Canvas, ORect);
              Brush.Color := clBtnHighlight;
              DestDC := Handle;
              SetTextColor(DestDC, clBlack);
              SetBkColor(DestDC, clWhite);
              BitBlt(DestDC, 0, 0, IWidth, IHeight,
                     MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);

              { Convert gray to clBtnShadow }
              DDB.Canvas.Brush.Color := clGray;
              MonoBmp.Canvas.CopyRect(IRect, DDB.Canvas, ORect);
              Brush.Color := clBtnShadow;
              DestDC := Handle;
              SetTextColor(DestDC, clBlack);
              SetBkColor(DestDC, clWhite);
              BitBlt(DestDC, 0, 0, IWidth, IHeight,
                     MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);

              { Convert transparent color to clBtnFace }
              DDB.Canvas.Brush.Color := ColorToRGB(FTransparentColor);
              MonoBmp.Canvas.CopyRect(IRect, DDB.Canvas, ORect);
              Brush.Color := clBtnFace;
              DestDC := Handle;
              SetTextColor(DestDC, clBlack);
              SetBkColor(DestDC, clWhite);
              BitBlt(DestDC, 0, 0, IWidth, IHeight,
                     MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
            end
            else
            begin
              { Create a disabled version }
              with MonoBmp do
              begin
                Assign(FOriginal);
                {$IFDEF VER100}
                HandleType := bmDDB;
                {$ENDIF}
                Canvas.Brush.Color := clBlack;
                Width := IWidth;
                if Monochrome then
                begin
                  Canvas.Font.Color := clWhite;
                  Monochrome := False;
                  Canvas.Brush.Color := clWhite;
                end;
                Monochrome := True;
              end;
              with TmpImage.Canvas do
              begin
                Brush.Color := clBtnFace;
                FillRect(IRect);
                Brush.Color := clBtnHighlight;
                SetTextColor(Handle, clBlack);
                SetBkColor(Handle, clWhite);
                BitBlt(Handle, 1, 1, IWidth, IHeight,
                  MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
                Brush.Color := clBtnShadow;
                SetTextColor(Handle, clBlack);
                SetBkColor(Handle, clWhite);
                BitBlt(Handle, 0, 0, IWidth, IHeight,
                  MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
              end;
            end;
          finally
            DDB.Free;
            MonoBmp.Free;
          end;
          FIndexs[State] := FGlyphList.AddMasked(TmpImage, clDefault);
        end;
    end;
  finally
    TmpImage.Free;
  end;
  Result := FIndexs[State];
  FOriginal.Dormant;
end;

procedure TButtonGlyph.DrawButtonGlyph(Canvas: TCanvas; const GlyphPos: TPoint;
  State: TButtonState; Transparent: Boolean);
var
  Index: Integer;
begin
  if FOriginal = nil then Exit;
  if (FOriginal.Width = 0) or (FOriginal.Height = 0) then Exit;
  Index := CreateButtonGlyph(State);
  with GlyphPos do
    if Transparent or (State = bsExclusive) then
      ImageList_DrawEx(FGlyphList.Handle, Index, Canvas.Handle, X, Y, 0, 0,
        clNone, clNone, ILD_Transparent)
    else
      ImageList_DrawEx(FGlyphList.Handle, Index, Canvas.Handle, X, Y, 0, 0,
        ColorToRGB(clBtnFace), clNone, ILD_Normal);
end;

procedure TButtonGlyph.DrawButtonText(Canvas: TCanvas; const Caption: string;
  TextBounds: TRect; State: TButtonState);
begin
  with Canvas do
  begin
    Brush.Style := bsClear;
    if State = bsDisabled then
    begin
      OffsetRect(TextBounds, 1, 1);
      Font.Color := clBtnHighlight;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, 0);
      OffsetRect(TextBounds, -1, -1);
      Font.Color := clBtnShadow;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, 0);
    end else
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
        DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  end;
end;

procedure TButtonGlyph.CalcButtonLayout(Canvas: TCanvas; const Client: TRect;
  const Offset: TPoint; const Caption: string; Layout: TButtonLayout; Margin,
  Spacing: Integer; var GlyphPos: TPoint; var TextBounds: TRect);
var
  TextPos: TPoint;
  ClientSize, GlyphSize, TextSize: TPoint;
  TotalSize: TPoint;
begin
  { calculate the item sizes }
  ClientSize := Point(Client.Right - Client.Left, Client.Bottom -
    Client.Top);

  if FOriginal <> nil then
    GlyphSize := Point(FOriginal.Width div FNumGlyphs, FOriginal.Height) else
    GlyphSize := Point(0, 0);

  if Length(Caption) > 0 then
  begin
    TextBounds := Rect(0, 0, Client.Right - Client.Left, 0);
    DrawText(Canvas.Handle, PChar(Caption), Length(Caption), TextBounds, DT_CALCRECT);
    TextSize := Point(TextBounds.Right - TextBounds.Left, TextBounds.Bottom -
      TextBounds.Top);
  end
  else
  begin
    TextBounds := Rect(0, 0, 0, 0);
    TextSize := Point(0,0);
  end;

  { If the layout has the glyph on the right or the left, then both the
    text and the glyph are centered vertically.  If the glyph is on the top
    or the bottom, then both the text and the glyph are centered horizontally.}
  if Layout in [blGlyphLeft, blGlyphRight] then
  begin
    GlyphPos.Y := (ClientSize.Y - GlyphSize.Y + 1) div 2;
    TextPos.Y := (ClientSize.Y - TextSize.Y + 1) div 2;
  end
  else
  begin
    GlyphPos.X := (ClientSize.X - GlyphSize.X + 1) div 2;
    TextPos.X := (ClientSize.X - TextSize.X + 1) div 2;
  end;

  { if there is no text or no bitmap, then Spacing is irrelevant }
  if (TextSize.X = 0) or (GlyphSize.X = 0) then
    Spacing := 0;

  { adjust Margin and Spacing }
  if Margin = -1 then
  begin
    if Spacing = -1 then
    begin
      TotalSize := Point(GlyphSize.X + TextSize.X, GlyphSize.Y + TextSize.Y);
      if Layout in [blGlyphLeft, blGlyphRight] then
        Margin := (ClientSize.X - TotalSize.X) div 3
      else
        Margin := (ClientSize.Y - TotalSize.Y) div 3;
      Spacing := Margin;
    end
    else
    begin
      TotalSize := Point(GlyphSize.X + Spacing + TextSize.X, GlyphSize.Y +
        Spacing + TextSize.Y);
      if Layout in [blGlyphLeft, blGlyphRight] then
        Margin := (ClientSize.X - TotalSize.X + 1) div 2
      else
        Margin := (ClientSize.Y - TotalSize.Y + 1) div 2;
    end;
  end
  else
  begin
    if Spacing = -1 then
    begin
      TotalSize := Point(ClientSize.X - (Margin + GlyphSize.X), ClientSize.Y -
        (Margin + GlyphSize.Y));
      if Layout in [blGlyphLeft, blGlyphRight] then
        Spacing := (TotalSize.X - TextSize.X) div 2
      else
        Spacing := (TotalSize.Y - TextSize.Y) div 2;
    end;
  end;

  case Layout of
    blGlyphLeft:
      begin
        GlyphPos.X := Margin;
        TextPos.X := GlyphPos.X + GlyphSize.X + Spacing;
      end;
    blGlyphRight:
      begin
        GlyphPos.X := ClientSize.X - Margin - GlyphSize.X;
        TextPos.X := GlyphPos.X - Spacing - TextSize.X;
      end;
    blGlyphTop:
      begin
        GlyphPos.Y := Margin;
        TextPos.Y := GlyphPos.Y + GlyphSize.Y + Spacing;
      end;
    blGlyphBottom:
      begin
        GlyphPos.Y := ClientSize.Y - Margin - GlyphSize.Y;
        TextPos.Y := GlyphPos.Y - Spacing - TextSize.Y;
      end;
  end;

  { fixup the result variables }
  with GlyphPos do
  begin
    Inc(X, Client.Left + Offset.X);
    Inc(Y, Client.Top + Offset.Y);
  end;
  OffsetRect(TextBounds, TextPos.X + Client.Left + Offset.X,
    TextPos.Y + Client.Top + Offset.X);
end;

function TButtonGlyph.Draw(Canvas: TCanvas; const Client: TRect;
  const Offset: TPoint; const Caption: string; Layout: TButtonLayout;
  Margin, Spacing: Integer; State: TButtonState; Transparent: Boolean): TRect;
var
  GlyphPos: TPoint;
begin
  CalcButtonLayout(Canvas, Client, Offset, Caption, Layout, Margin, Spacing,
    GlyphPos, Result);
  DrawButtonGlyph(Canvas, GlyphPos, State, Transparent);
  DrawButtonText(Canvas, Caption, Result, State);
end;

{ TToolbarButton97 }

constructor TToolbarButton97.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetBounds (0, 0, 23, 22);
  ControlStyle := [csCaptureMouse, csDoubleClicks, csOpaque];
  FGlyph := TButtonGlyph.Create;
  TButtonGlyph(FGlyph).OnChange := GlyphChanged;
  ParentFont := True;
  FOpaque := True;
  FSpacing := 4;
  FMargin := -1;
  FLayout := blGlyphLeft;
  Inc(ButtonCount);
end;

destructor TToolbarButton97.Destroy;
begin
  TButtonGlyph(FGlyph).Free;
  Dec(ButtonCount);
  if ButtonCount = 0 then
  begin
    Pattern.Free;
    Pattern := nil;
  end;
  inherited Destroy;
end;

procedure TToolbarButton97.Paint;
const
  DownStyles: array[Boolean] of Integer = (BDR_RAISEDINNER, BDR_SUNKENOUTER);
var
  Bmp: TBitmap;
  DrawCanvas: TCanvas;
  PaintRect: TRect;
  Offset: TPoint;
begin
  if FOpaque then
    Bmp := TBitmap.Create
  else
    Bmp := nil;
  try
    if FOpaque then begin
      Bmp.Width := Width;
      Bmp.Height := Height;
      DrawCanvas := Bmp.Canvas;
      with DrawCanvas do begin
        Brush.Color := Self.Color;
        FillRect (ClientRect);
      end;
    end
    else
      DrawCanvas := Canvas;
    if not Enabled then
    begin
      FState := bsDisabled;
      FMoving := False;
    end
    else if FState = bsDisabled then
      if FDown and (GroupIndex <> 0) then
        FState := bsExclusive
      else
        FState := bsUp;
    DrawCanvas.Font := Self.Font;
    PaintRect := Rect(0, 0, Width, Height);

    if (FState in [bsDown, bsExclusive]) or
       (FMouseInControl and (FState <> bsDisabled)) or
       (csDesigning in ComponentState) then
      DrawEdge(DrawCanvas.Handle, PaintRect, DownStyles[FState in [bsDown, bsExclusive]],
       BF_RECT);
    InflateRect(PaintRect, -1, -1);

    if FState in [bsDown, bsExclusive] then
    begin
      if (FState = bsExclusive) and (not FMouseInControl) then
      begin
        if Pattern = nil then CreateBrushPattern;
        DrawCanvas.Brush.Bitmap := Pattern;
        DrawCanvas.FillRect(PaintRect);
      end;
      Offset.X := 1;
      Offset.Y := 1;
    end
    else
    begin
      Offset.X := 0;
      Offset.Y := 0;
    end;
    TButtonGlyph(FGlyph).Draw(DrawCanvas, PaintRect, Offset, Caption, FLayout, FMargin,
      FSpacing, FState, True);

    if FOpaque then
      Canvas.Draw (0, 0, Bmp);
  finally
    if FOpaque then
      Bmp.Free;
  end;
end;

procedure TToolbarButton97.UpdateTracking;
var
  P: TPoint;
begin
  if Enabled then
  begin
    GetCursorPos(P);
    FMouseInControl := not (FindDragTarget(P, True) = Self);
    if FMouseInControl then
      Perform(CM_MOUSELEAVE, 0, 0)
    else
      Perform(CM_MOUSEENTER, 0, 0);
  end;
end;

procedure TToolbarButton97.Loaded;
var
  State: TButtonState;
begin
  inherited Loaded;
  if Enabled then
    State := bsUp
  else
    State := bsDisabled;
  TButtonGlyph(FGlyph).CreateButtonGlyph(State);
end;

procedure TToolbarButton97.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
{var
  PopupPoint: TPoint;}
begin
  inherited MouseDown(Button, Shift, X, Y);
  if Enabled then begin
    { added following line to fix some rare problems }
    FMouseInControl := True;
    if Button = mbLeft then begin
      if not FDown then
      begin
        FState := bsDown;
        InvalidateIt;
      end
      else
        if FAllowAllUp then
          InvalidateIt;
      FMoving := True;
      {if Assigned(DropdownMenu) then begin
        PopupPoint := ClientToScreen(ClientRect.TopLeft);
        if FDropDirection = ddDown then
          Inc (PopupPoint.Y, Height)
        else
          Inc (PopupPoint.X, Width);
        DropdownMenu.Popup (PopupPoint.X, PopupPoint.Y);
        MouseUp (Button, Shift, X, Y);
      end;}
    end;
  end;
end;

procedure TToolbarButton97.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  NewState: TButtonState;
begin
  inherited MouseMove(Shift, X, Y);
  if FMoving then
  begin
    if not FDown then NewState := bsUp
    else NewState := bsExclusive;
    if (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y < ClientHeight) then
      if FDown then NewState := bsExclusive else NewState := bsDown;
    if NewState <> FState then
    begin
      FState := NewState;
      InvalidateIt;
    end;
  end;
end;

procedure TToolbarButton97.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DoClick: Boolean;
begin
  { added following 2 lines to remove active border when right button is clicked }
  if (Button <> mbLeft) and Enabled then
    Perform (CM_MOUSELEAVE, 0, 0);
  inherited MouseUp(Button, Shift, X, Y);
  if FMoving then
  begin
    FMoving := False;
    DoClick := (X >= 0) and (X < ClientWidth) and (Y >= 0) and (Y < ClientHeight);
    if FGroupIndex <> 0 then begin
      if DoClick then
      begin
        SetDown(not FDown);
        if FDown then Repaint;
      end
      else
      begin
        if FDown then FState := bsExclusive;
        InvalidateIt;
      end;
    end;
    try
      if DoClick then Click;
    finally
      { Modified: This method has been modified a lot to make it leave the button
        down until user OnClick handler returns }
      if FState = bsDown then
        FState := bsUp;
      UpdateTracking;
    end;
  end;
end;

procedure TToolbarButton97.Click;
begin
  {if not Assigned(DropDownMenu) then}
    inherited Click;
end;

function TToolbarButton97.GlyphDraw(Canvas: TCanvas; const Client: TRect;
  const Offset: TPoint; const Caption: string; Layout: TButtonLayout;
  Margin, Spacing: Integer; State: TButtonState; Transparent: Boolean): TRect;
begin
  Result := TButtonGlyph(FGlyph).Draw(Canvas, Client, Offset, Caption,
    Layout, Margin, Spacing, State, Transparent);
end;

function TToolbarButton97.GetPalette: HPALETTE;
begin
  Result := Glyph.Palette;
end;

function TToolbarButton97.GetGlyph: TBitmap;
begin
  Result := TButtonGlyph(FGlyph).Glyph;
end;

procedure TToolbarButton97.SetGlyph(Value: TBitmap);
begin
  TButtonGlyph(FGlyph).Glyph := Value;
  InvalidateIt;
end;

function TToolbarButton97.GetNumGlyphs: TNumGlyphs;
begin
  Result := TButtonGlyph(FGlyph).NumGlyphs;
end;

procedure TToolbarButton97.SetNumGlyphs(Value: TNumGlyphs);
begin
  if Value < 0 then Value := 1
  else if Value > 4 then Value := 4;
  if Value <> TButtonGlyph(FGlyph).NumGlyphs then
  begin
    TButtonGlyph(FGlyph).NumGlyphs := Value;
    InvalidateIt;
  end;
end;

procedure TToolbarButton97.GlyphChanged(Sender: TObject);
begin
  InvalidateIt;
end;

procedure TToolbarButton97.UpdateExclusive;
var
  Msg: TMessage;
begin
  if (FGroupIndex <> 0) and (Parent <> nil) then
  begin
    Msg.Msg := CM_BUTTONPRESSED;
    Msg.WParam := FGroupIndex;
    Msg.LParam := Longint(Self);
    Msg.Result := 0;
    Parent.Broadcast(Msg);
  end;
end;

procedure TToolbarButton97.SetDown(Value: Boolean);
var
  NeedInvalidate: Boolean;
begin
  if FGroupIndex = 0 then Value := False;
  if Value <> FDown then
  begin
    if FDown and (not FAllowAllUp) then Exit;
    FDown := Value;
    if Value then
    begin
      { Was this, but wasn't working right with Opaque buttons }
      {if FState = bsUp then InvalidateIt;
      FState := bsExclusive;}
      NeedInvalidate := FState <> bsExclusive;
      FState := bsExclusive;
      if NeedInvalidate then InvalidateIt;
    end
    else
    begin
      FState := bsUp;
      Repaint;
    end;
    if Value then UpdateExclusive;
  end;
end;

procedure TToolbarButton97.SetGroupIndex(Value: Integer);
begin
  if FGroupIndex <> Value then
  begin
    FGroupIndex := Value;
    UpdateExclusive;
  end;
end;

procedure TToolbarButton97.SetLayout(Value: TButtonLayout);
begin
  if FLayout <> Value then
  begin
    FLayout := Value;
    InvalidateIt;
  end;
end;

procedure TToolbarButton97.SetMargin(Value: Integer);
begin
  if (Value <> FMargin) and (Value >= -1) then
  begin
    FMargin := Value;
    InvalidateIt;
  end;
end;

procedure TToolbarButton97.SetOpaque (Value: Boolean);
begin
  if Value <> FOpaque then begin
    FOpaque := Value;
    if Value then
      ControlStyle := ControlStyle + [csOpaque]
    else
      ControlStyle := ControlStyle - [csOpaque];
  end;
end;

procedure TToolbarButton97.InvalidateIt;
begin
  if FOpaque then
    Paint
  else begin
    Invalidate;
    {NEW: Call Update to reduce the "blinking"}
    Update;
  end;
end;

procedure TToolbarButton97.SetSpacing(Value: Integer);
begin
  if Value <> FSpacing then
  begin
    FSpacing := Value;
    InvalidateIt;
  end;
end;

procedure TToolbarButton97.SetAllowAllUp(Value: Boolean);
begin
  if FAllowAllUp <> Value then
  begin
    FAllowAllUp := Value;
    UpdateExclusive;
  end;
end;

procedure TToolbarButton97.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  inherited;
  if FDown then DblClick;
end;

procedure TToolbarButton97.CMEnabledChanged(var Message: TMessage);
const
  NewState: array[Boolean] of TButtonState = (bsDisabled, bsUp);
begin
  TButtonGlyph(FGlyph).CreateButtonGlyph(NewState[Enabled]);
  UpdateTracking;
  Repaint;
end;

procedure TToolbarButton97.CMButtonPressed(var Message: TMessage);
var
  Sender: TToolbarButton97;
begin
  if Message.WParam = FGroupIndex then
  begin
    Sender := TToolbarButton97(Message.LParam);
    if Sender <> Self then
    begin
      if Sender.Down and FDown then
      begin
        FDown := False;
        FState := bsUp;
        InvalidateIt;
      end;
      FAllowAllUp := Sender.AllowAllUp;
    end;
  end;
end;

procedure TToolbarButton97.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure TToolbarButton97.CMFontChanged(var Message: TMessage);
begin
  InvalidateIt;
end;

procedure TToolbarButton97.CMTextChanged(var Message: TMessage);
begin
  InvalidateIt;
end;

procedure TToolbarButton97.CMSysColorChange(var Message: TMessage);
begin
  with TButtonGlyph(FGlyph) do
  begin
    InvalidateIt;
    CreateButtonGlyph(FState);
  end;
end;

procedure TToolbarButton97.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled then
  begin
    FMouseInControl := True;
    {NEW: Like office 97, only draw the active border when application is active}
    if Application.Active then
      Repaint;
  end;
end;

procedure TToolbarButton97.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled and not FMoving then
  begin
    FMouseInControl := False;
    InvalidateIt;
  end;
end;


//////////////////////////////////////////////////////////////////////////////
// TEdit97 - internal
//////////////////////////////////////////////////////////////////////////////

constructor TEdit97.Create (AOwner: TComponent);
begin
  inherited;
  AutoSize := False;
  Ctl3D := False;
  BorderStyle := bsNone;
  ControlStyle := ControlStyle - [csFramed]; {fixes a VCL bug with NT 3.x}
  Height := 19;
end;

procedure TEdit97.MouseMove (Shift: TShiftState; X, Y: Integer);
var
  BR: TRect;
begin
  inherited;
  if (csDesigning in ComponentState) or (ssLeft in Shift) then
    Exit;

  BR := ClientRect;
  InflateRect (BR, 3, 3);
  if PtInRect(BR, Point(X, Y)) and not(Screen.ActiveControl is TEdit97) then begin
    if GetCapture <> Handle then begin
      MouseInControl := True;
      SetCapture (Handle);
      RedrawBorder;
    end;
  end
  else begin
    if GetCapture = Handle then begin
      MouseInControl := False;
      ReleaseCapture;
      RedrawBorder;
    end;
  end;
end;

procedure TEdit97.NewAdjustHeight;
var
  DC: HDC;
  SaveFont: HFONT;
  Metrics: TTextMetric;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics (DC, Metrics);
  SelectObject (DC, SaveFont);
  ReleaseDC (0, DC);

  Height := Metrics.tmHeight + 6;
end;

procedure TEdit97.Loaded;
begin
  inherited;
  if not(csDesigning in ComponentState) then
    NewAdjustHeight;
end;

procedure TEdit97.CMEnabledChanged (var Msg: TMessage);
const
  EnableColors: array[Boolean] of TColor = (clBtnFace, clWindow);
begin
  inherited;
  Color := EnableColors[Enabled];
end;

procedure TEdit97.CMFontChanged (var Msg: TMessage);
begin
  inherited;
  if not((csDesigning in ComponentState) and (csLoading in ComponentState)) then
    NewAdjustHeight;
end;

procedure TEdit97.WMSetFocus (var Msg: TWMSetFocus);
begin
  inherited;
  if not(csDesigning in ComponentState) then
    RedrawBorder;
end;

procedure TEdit97.WMKillFocus (var Msg: TWMKillFocus);
begin
  inherited;
  if not(csDesigning in ComponentState) then
    RedrawBorder;
end;

procedure TEdit97.WMPaint (var Msg: TWMPaint);
begin
  inherited;
  RedrawBorder;
end;

procedure TEdit97.WMNCCalcSize (var Msg: TWMNCCalcSize);
begin
  inherited;
  InflateRect (Msg.CalcSize_Params.rgrc[0], -3, -3);
end;

procedure TEdit97.WMNCPaint (var Msg: TWMNCPaint);
begin
  inherited;
  RedrawBorder;
end;

(*
 For some reason, it doesn't receive WM_NCMOUSEMOVE messages. Why not?? Can
 someone help?
procedure TEdit97.WMNCMouseMove (var Msg: TWMNCMouseMove);
var
  P: TPoint;
begin
  inherited;
  P := ScreenToClient(Point(Msg.XCursor, Msg.YCursor));
  MouseInControl := True;
  SetCapture (Handle);
  RedrawBorder;
end;
*)

procedure TEdit97.RedrawBorder;
var
  DC: HDC;
  R: TRect;
  B: HBrush;
begin
  DC := GetWindowDC(Handle);
  try
    R := Rect(0, 0, Width, Height);
    if ((csDesigning in ComponentState) and Enabled) or
       (not(csDesigning in ComponentState) and ((GetCapture = Handle) or Focused)) then begin
      DrawEdge (DC, R, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
      B := CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
      FrameRect (DC, R, B);
      DeleteObject (B);
      B := CreateSolidBrush(GetSysColor(COLOR_WINDOW));
      InflateRect (R, -1, -1);
      FrameRect (DC, R, B);
      DeleteObject (B);
    end
    else begin
      B := CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
      FrameRect (DC, R, B);
      InflateRect (R, -1, -1);
      FrameRect (DC, R, B);
      DeleteObject (B);
      B := CreateSolidBrush(GetSysColor(COLOR_WINDOW));
      InflateRect (R, -1, -1);
      FrameRect (DC, R, B);
      DeleteObject (B);
    end;
  finally
    ReleaseDC (Handle, DC);
  end;
end;


const
  Sig: PChar = '- Toolbar97 version ' + Toolbar97Version + ' by Jordan Russell -';
var
  SigX: Pointer;

initialization
  UsedForms := TList.Create;

finalization
  UsedForms.Free;

  {$HINTS OFF}
  SigX := @Sig;
end.
