unit tcpip;
(*@/// interface *)
interface

uses
  sysutils,
  classes,
  controls,
  forms,
  winsock,
(*$ifdef ver80 *)
  winprocs,
  wintypes,
(*$else *)
  windows,
(*$endif *)
  messages,
  ip_misc;

type
  t_socket_state=(invalid,valid,connected,state_unknown);
  t_timemode=(tzUTC,tzLocal);
  t_ftp_mode=(tftp_download,tftp_upload);
  ETcpIpError=class(Exception);
(*@///   ESocketError=class(ETcpIpError) *)
ESocketError=class(ETcpIpError)
  errornumber: word;
  constructor Create(number:word);
  end;
(*@\\\0000000301*)
(*@///   EProtocolError=class(ETcpIpError) *)
EProtocolError=class(ETcpIpError)
  errornumber: word;
  protocoll: string;
  constructor Create(const proto,Msg:String; number:word);
  end;
(*@\\\0000000201*)

  t_filetype=(ft_none, ft_dir, ft_file, ft_link);
(*@///   t_filedata=record ... end; *)
t_filedata=packed record
  filetype: t_filetype;
  size: integer;
  name: string;
  datetime: TDateTime;
  end;
(*@\\\*)
  t_lpr_types=(lp_plain, lp_ascii, lp_dvi, lp_plot, lp_ditroff, lp_ps,
               lp_pr, lp_fortran, lp_troff, lp_raster, lp_cif);

  TTraceProc = procedure (const s:string; level:byte) of object;

(*@///   TTcpIp = class(TComponent) *)
TTcpIp = class(TComponent)
protected
  f_handle: THandle;
  f_Socket: tsocket;
  f_hostname: string;
  f_tracer: TTraceProc;
  f_socket_number: smallint;
  ip_address: longint;  (* Network order! *)
  f_eof: boolean;
  f_stream: TStream;
  procedure WndProc(var Msg : TMessage); virtual;
  procedure open_socket_out(var socket:TSocket; Socket_number:smallint;ip_address:longint);
  procedure open_socket_in(var socket:TSocket; Socket_number:smallint;ip_address:longint);
  procedure close_socket(var socket:TSocket);
  procedure close_socket_linger(var socket:TSocket);
  function accept_socket_in(socket:TSocket; var SockInfo:TSockAddr):TSocket;
  function socket_state(socket:TSocket):T_Socket_State;
  function Socket_by_name(const service:string):smallint;
  function read_line:string;
  procedure read_var(var buf; size:integer; var _ok:integer);
  procedure write_buf(const buf; size:integer);
  procedure write_s(const s:string);
  procedure SetStream(value:TStream); (* for the property write of f_stream *)
  procedure action; VIRTUAL;
public
  procedure Login; virtual;
  procedure Logout; virtual;
  property OnTrace:TTraceProc read f_tracer write f_tracer;
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  function eof:boolean; VIRTUAL;
  end;
(*@\\\0000001A01*)
(*@///   TFinger = class(TTcpIp) *)
TFinger = class(TTcpIp)
protected
  f_user: string;
public
  constructor Create(Aowner:TComponent); override;
  property stream: TStream read f_stream;
  procedure action; override;
published
  property Hostname: string read f_hostname write f_hostname;
  property User: string read f_user write f_user;
  end;
(*@\\\0000000612*)
(*@///   TFingerd = class(TTcpIp) *)
type
(*@///   TFingerInfo=record *)
TFingerInfo=record
  hostname: string;
  address: longint;
  request: string;
  end;
(*@\\\0000000203*)
  TFingerRequest=procedure (Sender:TObject; FingerInfo:TFingerInfo) of object;

TFingerd = class(TTcpIp)
protected
  f_fingerrequest: TFingerRequest;
  f_answer: TStringList;
  procedure WndProc(var Msg : TMessage); override;
  procedure SetAnswer(Value: TStringList);
  procedure do_action; virtual;                    (* The real action *)
public
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  procedure action; override;                      (* Only to set up *)
published
  property OnFingerRequest:TFingerRequest read f_fingerrequest write f_fingerrequest;
  property Answer: TStringList read f_answer write SetAnswer;
  end;
(*@\\\0000000E01*)
(*@///   TTime = class(TTcpIp) *)
TTime = class(TTcpIp)
protected
  f_time: TDateTime;
  f_timemode: T_TimeMode;
public
  constructor Create(Aowner:TComponent); override;
  procedure action; override;
  property time: TDateTime read f_time;
published
  property Hostname: string read f_hostname write f_hostname;
  property TimeMode: T_TimeMode read f_timemode write f_timemode default tzUTC;
  end;
(*@\\\0000000B01*)
(*@///   THTTP = class(TTcpIp) *)
THTTP = class(TTcpIp)
protected
  f_url: string;
  f_path: string;   (* The real request string, calculated internally *)
  f_proxy: string;
  f_sender: string;
  f_reference: string;
  f_agent: string;
  f_nocache: boolean;
  f_status_nr: integer;
  f_status_txt: string;
  f_size: integer;
  f_type: string;
  procedure GetHead;
  procedure GetBody;
public
  property stream: TStream read f_stream write SetStream;
  property content_size: integer read f_size;
  property content_type: string read f_type;
  property status_number: integer read f_status_nr;
  property status_text: string read f_status_txt;
  procedure action; override;                          (* the GET method *)
  procedure Post;                                      (* the POST method, untested!  *)
published
  property URL: string read f_url write f_url;
  property Proxy: string read f_proxy write f_proxy;
  property Sender: string read f_sender write f_sender;
  property Agent: string read f_agent write f_agent;
  property Reference: string read f_reference write f_reference;
  property NoCache: boolean read f_nocache write f_nocache;
  end;
(*@\\\0000001701*)
(*@///   TSMTP = class(TTcpIp) *)
TSMTP = class(TTcpIp)
protected
  f_user, f_host: string;
  f_status_nr: integer;
  f_status_txt: string;
  f_receipts, f_body: tstringlist;
  procedure SetBody(Value: TStringList);
  procedure SetRecipients(Value: TStringList);
  procedure response;
public
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  procedure action; override;
  property Recipients: TStringlist read f_receipts write SetRecipients;
  property Message: TStringList read f_body write SetBody;
  property Sender: string read f_user write f_user;
published
  property Hostname: string read f_hostname write f_hostname;
  end;
(*@\\\0000000901*)
(*@///   TPop3 = class(TTcpIp) *)
TPop3 = class(TTcpIp)
protected
  f_user: string;
  f_pass: string;
  f_list: TList;
  f_mail: TStringList;
  procedure response;
public
  property Mail: TStringlist read f_mail;
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  procedure action; override;     (* retrieval of first message *)
  procedure Login; override;
  procedure GetHeaders;
  procedure Logout; override;
  procedure GetMail(index: integer);
  procedure DeleteMail(index:integer);
published
  property Hostname: string read f_hostname write f_hostname;
  property UserName: string read f_user write f_user;
  property Password: string read f_pass write f_pass;
end;
(*@\\\000000020A*)
(*@///   TRExec = class(TTcpIp) *)
TRExec = class(TTcpIp)
protected
  f_user: string;
  f_pass: string;
  f_command: string;
  procedure login; override;
public
  constructor Create(Aowner:TComponent); override;
  procedure action; override;
  property stream: TStream read f_stream;
published
  property Hostname: string read f_hostname write f_hostname;
  property UserName: string read f_user write f_user;
  property Password: string read f_pass write f_pass;
  property Command: string read f_command write f_command;
  end;
(*@\\\0000000A15*)
(*@///   TFTP = class(TTcpIp) *)
TFTP = class(TTcpIp)
protected
  f_url: string;
  f_status_nr: integer;
  f_status_txt: string;
  f_user: string;
  f_password: string;
  f_comm_socket: tsocket;
  f_passive: boolean;
  f_port: word;
  f_mode: t_ftp_mode;
  f_cur_dir: TStringList;
  f_cur_dir_index: integer;
  procedure response;
  function read_line_comm:string;
  procedure write_s_comm(const s:string);
  procedure get_datasocket;
public   (* will become public once tested *)
  procedure changedir(const f_dir:string);
  procedure removefile(const filename:string);
  procedure removedir(const dirname:string);
  procedure makedir(const dirname:string);
  procedure renamefile(const prior,after:string);
  procedure getdir(const dirname:string);
  function getdirentry:t_filedata;
public
  property stream: TStream read f_stream write SetStream;
  property status_number: integer read f_status_nr;
  property status_text: string read f_status_txt;
  procedure login; override;
  procedure logout; override;
  procedure download;
  procedure upload;
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  procedure action; override;
published
  property Hostname: string read f_hostname write f_hostname;
  property URI: string read f_url write f_url;
  property Password:string read f_password write f_password;
  property Username:string read f_user write f_user;
  property Passive:boolean read f_passive write f_passive default true;
  property Port:word read f_port write f_port default 21;
  property Mode:t_ftp_mode read f_mode write f_mode default tftp_download;
end;
(*@\\\0000001B01*)
(*@///   TNNTP = class(TTcpIp) *)
TNNTP = class(TTcpIp)
protected
  f_news: TStringList;
  f_newsgroups: TStringList;
  f_status_nr: integer;
  f_status_txt: string;
  procedure response;
  procedure action; override;     (* ??? *)
  procedure SetNews(value:TStringlist);
  procedure GetArticleInternally;
public
  property News: TStringlist read f_news write SetNews;
  property NewsGroups: TStringlist read f_newsgroups;
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  procedure Login; override;
  procedure Logout; override;
    (* To get an article from a URL like nntp://hostname/number *)
  procedure GetArticle(const group:string; index:integer);
    (* To get an article from a URL like news:msgid *)
  procedure GetArticleID(const msgid:string);
  procedure PostArticle;
    (* Methods more for a Newsreader *)
  procedure GetAllNewsgroups;
  procedure GetNewNewsgroups(since:TDateTime);
  procedure SetGroup(const group:string; var low,high,count: integer);
  procedure GetArticleNr(index:integer);
  procedure SetCurrentArticle(index:integer);
  procedure GetCurrentArticle;
  procedure GetNextArticle;
  procedure GetPreviousArticle;
published
  property Hostname: string read f_hostname write f_hostname;
end;
(*@\\\0000002001*)
(*@///   Tlpr = class(TTcpIp) *)
Tlpr = class(TTcpIp)
protected
  f_printtype: t_lpr_types;
  f_banner: boolean;
  f_count: integer;
  f_user: string;
  f_queue: string;
  f_user_mail: string;
  f_jobname: string;
  f_title: string;
  procedure response;
public
  constructor Create(Aowner:TComponent); override;
  property stream: TStream read f_stream write SetStream;
  procedure action; override;
  procedure SendPrintData;
  procedure GetQueueStatus(detailed:boolean);
published
  property Hostname: string read f_hostname write f_hostname;
  property User: string read f_user write f_user;
  property PrintQueue: string read f_queue write f_queue;
  property MailTo: string read f_user_mail write f_user_mail;
  property JobName: string read f_jobname write f_jobname;
  property PrintType:t_lpr_types read f_printtype write f_printtype default lp_ascii;
  property CopyCount:integer read f_count write f_count default 1;
  property PrintBanner:boolean read f_banner write f_banner default false;
  property PrintTitle:string read f_title write f_title;
  end;
(*@\\\000000112C*)

(*@///   TMailNews = class(TComponent) *)
TMailNews = class(TComponent)
protected
  f_from, f_sender, f_subject: string;
  f_body: TStringlist;
  f_add_header: TStringlist;
  f_message: TStringlist;
  f_references: string;
  f_replyto: string;
  procedure SetBody(Value: TStringList);
  procedure SetHeader(Value: TStringList);
public
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  procedure action; VIRTUAL;
published
  property Sender: string read f_sender write f_sender;
  property From: string read f_from write f_from;
  property Body: TStringList read f_body write SetBody;
  property Header: TStringList read f_add_header write SetHeader;
  property Subject:string read f_subject write f_subject;
  property References:string read f_references write f_references;
  property ReplyTo:string read f_replyto write f_replyto;
end;
(*@\\\0000001601*)
(*@///   TMail = class(TMailNews) *)
TMail = class(TMailNews)
protected
  f_smtp: TSMTP;
  f_receipts: TStringlist;
  f_disclose_receipts: boolean;
  procedure SetRecipients(Value: TStringList);
public
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  procedure action; override;
published
  property SMTP: TSMTP read f_smtp write f_smtp;
  property Disclose_Receipts: boolean read f_disclose_receipts write f_disclose_receipts default false;
  property Recipients: TStringlist read f_receipts write SetRecipients;
end;
(*@\\\0000000E01*)
(*@///   TNews = class(TMailNews) *)
TNews = class(TMailNews)
protected
  f_nntp: TNNTP;
  f_newsgroups: TStringlist;
  f_organization: string;
  procedure SetNewsgroups(Value: TStringList);
public
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  procedure action; override;
published
  property NNTP: TNNTP read f_nntp write f_nntp;
  property Newsgroups: TStringlist read f_newsgroups write SetNewsgroups;
  property Organization: string read f_organization write f_organization;
end;
(*@\\\0000000E0B*)
(*@///   TMime = class(TComponent) *)
TMime = class(TComponent)
protected
  f_mail: TMailNews;
  f_attach_files:TStringlist;
  boundary: string;
  procedure SetAttachFiles(value:TStringlist);
public
  constructor Create(Aowner:TComponent); override;
  destructor Destroy; override;
  procedure AttachFile(const filename:string);
  procedure action;
published
  property Mail: TMailNews read f_mail write f_mail;
  property AttachedFiles:TStringList read f_attach_files write SetAttachFiles;
end;
(*@\\\*)

const
  uwm_socketevent = wm_user+$100; (* my magic message number *)

const
  lpr_count: integer = 0;         (* the current job number *)

procedure Register;
(*@\\\0000002B01*)
(*@/// implementation *)
implementation

(*@/// Parse a FTP directory line into a filedata record (UNIX and DOS style only) *)
const month_string: array[0..11] of string =
  ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');

(*@/// function getmonth(const s:string):integer;         Month -> Integer *)
function getmonth(const s:string):integer;
var
  i: integer;
begin
  result:=0;
  for i:=0 to 11 do
    if s=month_string[i] then begin
      result:=i+1;
      EXIT;
      end;
  end;
(*@\\\000000030E*)

const
  empty_filedata:t_filedata=
    (filetype:ft_none; size:0; name:''; datetime:0);

(*@/// function parse_line_unix(const s: string):t_filedata; *)
function parse_line_unix(const v: string):t_filedata;
(* known problems: filename with spaces (most unix's don't allow the anyway) *)
(*                 links aren't parsed at all                                *)
var
  t,date: string;
  y,m,d,h,n,s: word;
begin
  try
    case v[1] of
      'd': result.filetype:=ft_dir;
      '-': result.filetype:=ft_file;
      'l': result.filetype:=ft_link;
      end;
    result.name:=copy(v,posn(' ',v,-1)+1,length(v));
    t:=copy(v,12,length(v)-length(result.name)-12);
    date:=copy(t,length(t)-11,12);
    decodedate(now,y,m,d);
    h:=0; n:=0; s:=0;
    if pos(':',date)>0 then begin
      h:=strtoint(copy(date,8,2));
      n:=strtoint(copy(date,11,2));
      end
    else
      y:=strtoint(copy(date,9,4));
    d:=strtoint(trim(copy(date,5,2)));
    m:=getmonth(copy(date,1,3));
    t:=copy(t,1,length(t)-13);
    result.size:=strtoint(copy(t,posn(' ',t,-1)+1,length(t)));
    result.datetime:=encodedate(y,m,d)+encodetime(h,n,s,0);
  except
    result:=empty_filedata;
    end;
  end;
(*@\\\0000000201*)
(*@/// function parse_line_dos(const s: string):t_filedata; *)
function parse_line_dos(const v: string):t_filedata;
(* known problems: filename with spaces (why do something like that?) *)
var
  t: string;
  sd,st: string;
  ds: char;
begin
  ds:=DateSeparator;
  sd:=ShortdateFormat;
  st:=Shorttimeformat;
  try
    if pos('<DIR>',v)=0 then
      result.filetype:=ft_file
    else
      result.filetype:=ft_dir;
    result.name:=copy(v,posn(' ',v,-1)+1,length(v));
    t:=copy(v,1,length(v)-length(result.name)-1);
    result.size:=strtoint('0'+copy(t,posn(' ',t,-1)+1,length(t)));
    DateSeparator:='-';
    ShortDateFormat:='mm/dd/yy';
    Shorttimeformat:='hh:nnAM/PM';
    result.datetime:=strtodatetime(copy(t,1,17));
  except
    result:=empty_filedata;
    end;
  DateSeparator:=ds;
  ShortdateFormat:=sd;
  Shorttimeformat:=st;
  end;
(*@\\\0000000201*)

(*@/// function parse_ftp_line(const s:string):t_filedata; *)
function parse_ftp_line(const s:string):t_filedata;
begin
  if copy(s,1,5)='total' then     (* first line for some UNIX ftp server *)
    result:=empty_filedata
  else if s[1] in ['d','l','-','s'] then
    result:=parse_line_unix(s)
  else if s[1] in ['0'..'9'] then
    result:=parse_line_dos(s);
  end;
(*@\\\0000000301*)
(*@\\\0000000301*)

(*@/// procedure stream_write_s(h:TMemoryStream; const s:string);  // string -> stream *)
procedure stream_write_s(h:TMemoryStream; const s:string);
var
  buf: pointer;
begin
  buf:=@s[1];
  h.write(buf^,length(s));
  end;
(*@\\\*)

const
  back_log=2;  (* possible values 1..5 *)
  fingerd_timeout=5;

(*@/// class EProtocolError(ETcpIpError) *)
constructor EProtocolError.Create(const proto,Msg:String; number:word);
begin
  Inherited Create(Msg);
  protocoll:=proto;
  errornumber:=number;
  end;
(*@\\\0000000301*)
(*@/// class ESocketError(ETcpIpError) *)
constructor ESocketError.Create(number:word);
begin
  inherited create('Error creating socket');
  errornumber:=number;
  end;
(*@\\\*)


(*@/// procedure parse_url(const url:string; var proto,user,pass,host,port,path:string); *)
procedure parse_url(const url:string; var proto,user,pass,host,port,path:string);
var
  p,q: integer;
  s: string;
begin
  proto:='';
  user:='';
  pass:='';
  host:='';
  port:='';
  path:='';

  p:=pos('://',url);
  if p=0 then begin
    if lowercase(copy(url,1,7))='mailto:' then begin   (* mailto:// not common *)
      proto:='mailto';
      p:=pos(':',url);
      end;
    end
  else begin
    proto:=copy(url,1,p-1);
    inc(p,2);
    end;
  s:=copy(url,p+1,length(url));

  p:=pos('/',s);
  if p=0 then  p:=length(s)+1;
  path:=copy(s,p,length(s));
  s:=copy(s,1,p-1);

  p:=poscn(':',s,-1);
  q:=poscn('@',s,-1);
  if (p=0) and (q=0) then begin   (* no user, password or port *)
    host:=s;
    EXIT;
    end
  else if q<p then begin  (* a port given *)
    port:=copy(s,p+1,length(s));
    host:=copy(s,q+1,p-q-1);
    if q=0 then EXIT; (* no user, password *)
    s:=copy(s,1,q-1);
    end
  else begin
    host:=copy(s,q+1,length(s));
    s:=copy(s,1,q-1);
    end;
  p:=pos(':',s);
  if p=0 then
    user:=s
  else begin
    user:=copy(s,1,p-1);
    pass:=copy(s,p+1,length(s));
    end;
  end;
(*@\\\*)

{ The base component }
(*@/// class ttcpip(TComponent) *)
(*@/// constructor ttcpip.Create(Aowner:TComponent); *)
constructor ttcpip.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_stream:=TMemorystream.Create;
  f_Socket:=INVALID_SOCKET;
  ip_address:=INVALID_IP_ADDRESS;
    (* A windows dummy handle to get messages *)
  f_handle:=AllocateHwnd(self.WndProc);
  end;
(*@\\\000000040D*)
(*@/// destructor ttcpip.Destroy; *)
destructor ttcpip.Destroy;
begin
  f_stream.free;
  if f_socket<>invalid_socket then
    logout;
  DeallocateHwnd(f_Handle);
  inherited destroy;
  end;
(*@\\\0000000601*)

(*@/// procedure ttcpip.WndProc(var Msg : TMessage); *)
procedure ttcpip.WndProc(var Msg : TMessage);
begin
  end;
(*@\\\0000000301*)

(*@/// procedure ttcpip.open_socket_out(var socket:TSocket; Socket_number:smallint;ip_address:longint); *)
procedure ttcpip.open_socket_out(var socket:TSocket; Socket_number:smallint;ip_address:longint);
var
  RemoteAddress : TSockAddr;
begin
  close_socket(socket);
(*@///   open the socket and connect it *)
f_Socket:=Winsock.Socket(PF_INET,SOCK_STREAM,IPPROTO_IP);
with RemoteAddress do begin
  Sin_Family:=PF_INET;
  Sin_Port:=Winsock.htons(Socket_number);
  Sin_addr:=TInAddr(ip_address);
  end;
if Winsock.Connect(Socket,RemoteAddress,
   SizeOf(RemoteAddress))=SOCKET_ERROR then begin
  Close_Socket(socket);
  if assigned(f_tracer) then
    f_tracer('Failed to open output socket '+inttostr(Socket_number)+' to host '+
             ip2string(ip_address),1);
  end
else
  if assigned(f_tracer) then
    f_tracer('Opened output socket '+inttostr(Socket_number)+' to host '+
             ip2string(ip_address)+' successfully; ID '+inttostr(socket),1);
(*@\\\*)
  end;
(*@\\\0000000601*)
(*@/// procedure ttcpip.open_socket_in(var socket:TSocket; Socket_number:smallint;ip_address:longint); *)
procedure ttcpip.open_socket_in(var socket:TSocket; Socket_number:smallint;ip_address:longint);
var
  LocalAddress : TSockAddr;
begin
  close_socket(socket);
(*@///   open the socket and let it listen *)
f_Socket:=Winsock.Socket(PF_INET,SOCK_STREAM,IPPROTO_IP);
with LocalAddress do begin
  Sin_Family:=PF_INET;
  Sin_Port:=Winsock.htons(Socket_number);
  Sin_addr:=TInAddr(ip_address);
  end;
if Winsock.bind(socket,LocalAddress,
   SizeOf(LocalAddress))=SOCKET_ERROR then begin
  if assigned(f_tracer) then
    f_tracer('Failed to bind socket '+inttostr(Socket_number)+' for local ip '+
             ip2string(ip_address),1);
  Close_Socket(socket);
  EXIT;
  end
else
  if assigned(f_tracer) then
    f_tracer('Bound to socket '+inttostr(Socket_number)+' for local ip '+
             ip2string(ip_address),1);
if Winsock.Listen(Socket,back_log)=SOCKET_ERROR then begin
  Close_Socket(socket);
  if assigned(f_tracer) then
    f_tracer('Failed to set input socket '+inttostr(Socket_number)+
             ' to listening mode',1);
  end
else
  if assigned(f_tracer) then
    f_tracer('Set input socket '+inttostr(Socket_number)+
             ' to listening mode sucessfully; ID '+inttostr(socket),1);
(*@\\\0000001701*)
  end;
(*@\\\0000000601*)
(*@/// function ttcpip.accept_socket_in(socket:TSocket; var SockInfo:TSockAddr):TSocket; *)
function ttcpip.accept_socket_in(socket:TSocket; var SockInfo:TSockAddr):TSocket;
var
  x: u_int;
  LocalAddress : TSockAddr;
  temp_socket: TSocket;
begin
  x:=SizeOf(LocalAddress);
(*$ifndef ver1000 *)
  temp_socket:=Winsock.Accept(Socket,LocalAddress,x);
(*$else *)       { Delphi 3 ARGH! }
  temp_socket:=Winsock.Accept(Socket,@LocalAddress,@x);
(*$endif *)
  if temp_socket=SOCKET_ERROR then begin
    (* no incoming call available *)
    temp_socket:=INVALID_SOCKET;
    if assigned(f_tracer) then
      f_tracer('No incoming connection found on socket ID '+inttostr(Socket),1);
    end
  else
    if assigned(f_tracer) then
      f_tracer('Incoming connection found on socket ID '+inttostr(Socket)+
               '; generated socket ID '+inttostr(temp_socket),1);
  accept_socket_in:=temp_socket;
  sockinfo:=LocalAddress;
  end;
(*@\\\000000030B*)
(*@/// function ttcpip.socket_state(socket:TSocket):T_Socket_State; *)
function ttcpip.socket_state(socket:TSocket):T_Socket_State;
var
  peer_adr: TSockAddr;
  x: u_int;
begin
  if socket=INVALID_SOCKET then
    socket_state:=invalid
  else begin
    x:=sizeof(TSockAddr);
    if winsock.getpeername(socket,peer_adr,x)=0 then
      socket_state:=connected
    else begin
      if winsock.WSAGetLastError<>WSAENOTCONN then
        socket_state:=state_unknown
      else
        socket_state:=valid
      end;
    end;
  end;
(*@\\\000000040B*)
(*@/// procedure ttcpip.close_socket(var socket:TSocket); *)
procedure ttcpip.close_socket(var socket:TSocket);
begin
  if socket<>INVALID_SOCKET then begin
    Winsock.CloseSocket(socket);
    if assigned(f_tracer) then
      f_tracer('Closed socket ID '+inttostr(socket),1);
    socket:=INVALID_SOCKET;
    end;
  end;
(*@\\\0000000601*)
(*@/// procedure ttcpip.close_socket_linger(var socket:TSocket); *)
procedure ttcpip.close_socket_linger(var socket:TSocket);
var
  linger: TLinger;
begin
  linger.l_onoff:=1;
  linger.l_linger:=fingerd_timeout;
  winsock.setsockopt(f_socket,sol_socket,SO_LINGER,pchar(@linger),sizeof(linger));
  winsock.shutdown(f_socket,1);
  close_socket(f_socket);
  end;
(*@\\\003E000801000801000801000801000801000801*)
(*@/// function ttcpip.Socket_by_name(const service:string):smallint; *)
function ttcpip.Socket_by_name(const service:string):smallint;
var
  service_entry : PServEnt;
  s: string;
begin
  s:=service+#0;
(*$ifdef ver80 *)
  service_entry:=Winsock.GetServByName(pchar(@s[1]),'tcp');
(*$else *)
  service_entry:=Winsock.GetServByName(pchar(s),'tcp');
(*$endif *)
  if service_entry=nil then
    result:=0
  else
    result:=winsock.htons(service_entry^.s_port);
  end;
(*@\\\0000000833*)

(*@/// procedure ttcpip.Login; *)
procedure ttcpip.Login;
begin
  ip_address:=lockup_hostname(f_hostname);
  if ip_address=INVALID_IP_ADDRESS then
    raise ETcpIpError.Create('Couldn''t resolve hostname '+f_hostname);
  open_socket_out(f_socket,f_Socket_number,ip_address);
  if f_socket=INVALID_SOCKET then
    raise ESocketError.Create(WSAGetLastError);
  f_eof:=false;
  end;
(*@\\\000000050A*)
(*@/// procedure ttcpip.LogOut; *)
procedure ttcpip.LogOut;
begin
  close_socket(f_socket);
  f_socket:=invalid_socket;
  end;
(*@\\\0000000301*)


(*@/// function ttcpip.eof:boolean;            !!! *)
function ttcpip.eof:boolean;
begin
  eof:=f_eof or (socket_state(f_socket)<>connected);
  end;
(*@\\\0000000301*)
(*@/// procedure ttcpip.read_var(var buf; size:integer; var _ok:integer); *)
procedure ttcpip.read_var(var buf; size:integer; var _ok:integer);
var
  temp_buf: pointer;
  error: integer;
begin
  temp_buf:=NIL;
  try
    if @buf=NIL then
      getmem(temp_buf,size)  (* alloc for the -> /dev/null *)
    else
      temp_buf:=@buf;
    repeat
      _ok:=Winsock.recv(F_Socket,temp_Buf^,Size,0);
      if _ok<=0 then begin
        error:=Winsock.WSAGetLastError;
        (* listening socket is always non-blocking, but this causes
           problems with the recv command *)
        f_eof:=error<>wsaewouldblock;
        end
      else
        if assigned(f_tracer) then
          f_tracer('Received '+inttostr(_ok)+' bytes on socket ID '+inttostr(f_socket),1);
    until f_eof or (_ok>0);
  finally
    if @buf=NIL then
      freemem(temp_buf,size)
    end;
  end;
(*@\\\0000000F10*)
(*@/// function ttcpip.read_line:string; *)
function ttcpip.read_line:string;
var
  x,y: char;
  ok: integer;
  s: string;
begin
  s:='';
  y:=#0;
  repeat
    read_var(x,1,ok);
    if x=#13 then
      y:=x
    else if x=#10 then begin
      if y=#13 then begin
        read_line:=s;
        EXIT;
        end;
      end
    else begin
      s:=s+x;
      y:=#0;
      end;
  until eof;
  end;
(*@\\\0000000801*)
(*@/// procedure ttcpip.write_buf(const buf; size:integer); *)
procedure ttcpip.write_buf(const buf; size:integer);
begin
  if Winsock.Send(F_Socket,pointer(@buf)^,size,0)=SOCKET_ERROR then
     EXIT  (* Error writing *)
  else
  if assigned(f_tracer) then
    f_tracer('Sent '+inttostr(size)+' bytes on socket ID '+inttostr(f_socket),1);
  end;
(*@\\\0000000701*)
(*@/// procedure ttcpip.write_s(const s:string); *)
procedure ttcpip.write_s(const s:string);
begin
(*$ifdef ver80 *)
  write_buf(pchar(@s[1])^,length(s));
(*$else *)
  write_buf(pchar(s)^,length(s));
(*$endif *)
  end;
(*@\\\000000041A*)

(*@/// procedure ttcpip.SetStream(value:TStream); *)
procedure ttcpip.SetStream(value:TStream);
begin
  TMemoryStream(f_stream).LoadFromStream(value);
  end;
(*@\\\0000000301*)

(*@/// procedure ttcpip.action; *)
procedure ttcpip.action;
const
  buf_size=$1000;
var
  buf,p: pointer;
  ok,ok2:integer;
begin
  login;
  buf:=NIL;
  try
    TMemorystream(f_stream).clear;
    getmem(buf,buf_size);
    while not eof do begin
      read_var(buf^,buf_size,ok);
      p:=buf;
      while ok>0 do begin   (* just to be sure everything goes into the stream *)
        ok2:=f_stream.write(p^,ok);
        dec(ok,ok2);
        p:=pointer(longint(p)+ok2);
        end;
    end;
    f_stream.seek(0,0);
  finally
    freemem(buf,buf_size);
    end;
  end;
(*@\\\0000000B05*)
(*@\\\0000000E01*)

{ Finger client and demon }
(*@/// class tfinger(ttcpip) *)
(*@/// constructor tfinger.Create(Aowner:TComponent); *)
constructor tfinger.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_Socket_number:=IPPORT_FINGER;  (* 79 *)
  end;
(*@\\\0000000403*)

(*@/// procedure tfinger.action; *)
procedure tfinger.action;
const
  buf_size=$1000;
var
  buf,p: pointer;
  ok,ok2:integer;
  s: string;
begin
  login;
  buf:=NIL;
  s:=f_user+#13#10;
  write_s(s);
  try
    TMemorystream(f_stream).clear;
    getmem(buf,buf_size);
    while not eof do begin
      read_var(buf^,buf_size,ok);
      p:=buf;
      while ok>0 do begin   (* just to be sure everything goes into the stream *)
        ok2:=f_stream.write(p^,ok);
        dec(ok,ok2);
        p:=pointer(longint(p)+ok2);
        end;
    end;
    f_stream.seek(0,0);  (* set the stream back to start *)
  finally
    freemem(buf,buf_size);
    end;
  logout;
  end;
(*@\\\0000001107*)
(*@\\\0000000301*)
(*@/// class tfingerd(ttcpip) *)
(*@/// constructor tfingerd.Create(Aowner:TComponent); *)
constructor tfingerd.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_Socket_number:=IPPORT_FINGER;  (* 79 *)
  f_answer:=TStringList.Create;
  end;
(*@\\\0000000501*)
(*@/// destructor tfingerd.Destroy; *)
destructor tfingerd.Destroy;
begin
  f_answer.Free;
  inherited destroy;
  end;
(*@\\\0000000301*)
(*@/// procedure tfingerd.do_action; *)
procedure tfingerd.do_action;
var
  i: integer;
  temp_socket: TSocket;
  finger_info:TFingerInfo;
  sockinfo: TSockAddr;
  s: string;
begin
  temp_socket:=f_socket;
  self.f_socket:=accept_socket_in(f_socket,sockinfo);
  f_eof:=false;
  finger_info.address:=longint(sockinfo.Sin_addr);
  s:=self.read_line;
  finger_info.request:=s;
  finger_info.hostname:='';   (* NYI !!! *)
  if assigned(f_fingerrequest) then
    f_fingerrequest(self,finger_info);
  for i:=0 to f_answer.count-1 do begin
    self.write_s(f_answer.strings[i]+#13#10);
    end;
  close_socket_linger(f_socket);
  f_socket:=temp_socket;
  end;
(*@\\\0000000701*)
(*@/// procedure tfingerd.SetAnswer(Value: TStringList); *)
procedure tfingerd.SetAnswer(Value: TStringList);
begin
  f_answer.assign(value);
  end;
(*@\\\0000000301*)
(*@/// procedure tfingerd.WndProc(var Msg : TMessage); *)
procedure tfingerd.WndProc(var Msg : TMessage);
begin
  if msg.msg<>uwm_socketevent then EXIT;
  if msg.lparamhi=socket_error then
  else begin
    case msg.lparamlo of
      fd_accept: begin
        do_action;
        end;
      end;
    end;
  end;
(*@\\\000000030F*)
(*@/// procedure tfingerd.action; *)
procedure tfingerd.action;
begin
  open_socket_in(f_socket,f_Socket_number,my_ip_address);
  if f_socket=INVALID_SOCKET then
    raise ESocketError.Create(WSAGetLastError);
  winsock.WSAAsyncSelect(f_socket,f_handle,uwm_socketevent,fd_accept);
  end;
(*@\\\0000000301*)
(*@\\\0000000301*)

{ HTTP and FTP - the file transfer protocols }
(*@/// class thttp(ttcpip) *)
(*@/// procedure thttp.action; *)
procedure thttp.action;
var
  proto,user,pass,host,port,path: string;
begin
(*@///   parse url and proxy to f_hostname, f_path and f_socket_number *)
if f_proxy<>'' then begin
  parse_url(f_proxy,proto,user,pass,host,port,path);
  f_path:=f_url;
  if port='' then port:='8080';
  end
else begin
  parse_url(f_url,proto,user,pass,host,port,f_path);
  if port='' then port:='80';
  end;
if proto='' then  proto:='http';
if f_path='' then f_path:='/';

f_hostname:=host;
f_Socket_number:=strtoint(port);
(*@\\\0000000B15*)

  login;
  gethead;
  logout;

  login;
  getbody;

  logout;
  end;
(*@\\\0000000501*)
(*@/// procedure thttp.GetHead; *)
procedure thttp.GetHead;
var
  s: string;
  proto,user,pass,port: string;
  field,data: string;
begin
(*@///   Request the header data *)
self.write_s('HEAD '+f_path+' HTTP/1.0'+#13#10);   (* send the query *)
if assigned(f_tracer) then
  f_tracer('Requested HEAD data using HTTP/1.0',2);
if f_sender<>'' then
  self.write_s('From: '+f_sender+#13#10);
if f_reference<>'' then
  self.write_s('Referer: '+f_reference+#13#10);
if f_agent<>'' then
  self.write_s('User-Agent: '+f_agent+#13#10);
if f_nocache then
  self.write_s('Pragma: no-cache'+#13#10);
self.write_s(#13#10);                          (* finalize the request *)
(*@\\\0000000401*)
(*@///   read in the response *)
repeat
  s:=self.read_line;
  if false then
(*@///   else if left(s,8)='HTTP/1.0' then    http-status-reply *)
else if copy(s,1,8)='HTTP/1.0' then begin
  f_status_nr:=strtoint(copy(s,10,3));
  f_status_txt:=copy(s,14,length(s));
  if assigned(f_tracer) then
    f_tracer('Received HTTP status '+copy(s,10,length(s)),2);
  if f_status_nr>=400 then EXIT;   (* HTTP error returned *)
  end
(*@\\\*)
(*@///   else if pos(':',s)>0         then    parse the response string *)
else if pos(':',s)>0 then begin
  field:=lowercase(copy(s,1,pos(':',s)-1));
  data:=copy(s,pos(':',s)+2,length(s));
  if assigned(f_tracer) then
    f_tracer('Received HTTP header line: '+s,2);
  if false then
{   else if field='date' then }
{   else if field='mime-version' then }
{   else if field='pragma' then }
{   else if field='allow' then }
(*@///   else if field='location' then   change the uri !!! *)
else if field='location' then begin
  if proxy<>'' then
    f_path:=data            (* it goes via a proxy, so just change the uri *)
  else begin
    parse_url(data,proto,user,pass,f_hostname,port,f_path);
    if port<>'' then  f_Socket_number:=strtoint(port);
    end;
  end
(*@\\\0000000701*)
{   else if field='server' then }
{   else if field='www-authenticate' then }
{   else if field='content-encoding' then }
(*@///   else if field='content-length' then *)
else if field='content-length' then
  f_size:=strtoint(data)
(*@\\\000000011C*)
(*@///   else if field='content-type' then *)
else if field='content-type' then
  f_type:=data;
(*@\\\*)
{   else if field='expires' then }
{   else if field='last-modified' then }
  end
(*@\\\0000000B01*)
(*@///   else                                 some very strange response, ignore it *)
else;
(*@\\\*)
until s='';
(*@\\\0000000501*)
  if f_status_nr>=400 then
    raise EProtocolError.Create('HTTP',f_status_txt,f_status_nr);
  end;
(*@\\\0000000A01*)
(*@/// procedure thttp.GetBody; *)
procedure thttp.GetBody;
const
  buf_size=$1000;
var
  buf,p: pointer;
  ok,ok2:integer;
  s: string;
  field,data: string;
begin
(*@///   Request the data itself *)
self.write_s('GET '+f_path+' HTTP/1.0'+#13#10);   (* send the query *)
if assigned(f_tracer) then
  f_tracer('Requested data using HTTP/1.0',2);
if f_sender<>'' then
  self.write_s('From: '+f_sender+#13#10);
if f_reference<>'' then
  self.write_s('Referer: '+f_reference+#13#10);
if f_agent<>'' then
  self.write_s('User-Agent: '+f_agent+#13#10);
if f_nocache then
  self.write_s('Pragma: no-cache'+#13#10);
self.write_s(#13#10);                          (* finalize the request *)
(*@\\\*)
(*@///   read in the response *)
repeat
  s:=self.read_line;
  if false then
(*@///   else if copy(s,1,8)='HTTP/1.0' then    http-status-reply *)
else if copy(s,1,8)='HTTP/1.0' then begin
  f_status_nr:=strtoint(copy(s,10,3));
  f_status_txt:=copy(s,14,length(s));
  if assigned(f_tracer) then
    f_tracer('Received HTTP status '+copy(s,10,length(s)),2);
  if f_status_nr>=300 then
{     raise_par_string(te_tcpip_http_error, }
{       e_tcpip_http_error,status_nr,status_txt) }
  end
(*@\\\*)
(*@///   else if pos(':',s)>0           then    parse the response string *)
else if pos(':',s)>0 then begin
  field:=lowercase(copy(s,1,pos(':',s)-1));
  data:=copy(s,pos(':',s)+2,length(s));
  if assigned(f_tracer) then
    f_tracer('Received HTTP header line: '+s,2);
  if false then
{   else if field='date' then }
{   else if field='mime-version' then }
{   else if field='pragma' then }
{   else if field='allow' then }
{   else if field='location' then }
{   else if field='server' then }
{   else if field='www-authenticate' then }
{   else if field='content-encoding' then }
(*@///   else if field='content-length' then *)
else if field='content-length' then
  f_size:=strtoint(data)
(*@\\\*)
(*@///   else if field='content-type' then *)
else if field='content-type' then
  f_type:=data;
(*@\\\*)
{   else if field='expires' then }
{   else if field='last-modified' then }
  end
(*@\\\*)
(*@///   else                                   some very strange response, ignore it *)
else;
(*@\\\*)
until s='';
(*@\\\*)
  if f_status_nr>=300 then
    raise EProtocolError.Create('HTTP',f_status_txt,f_status_nr);
  buf:=NIL;
  try
    getmem(buf,buf_size);
    TMemorystream(f_stream).clear;
    while not eof do begin
      read_var(buf^,buf_size,ok);
      p:=buf;
      while ok>0 do begin   (* just to be sure everything goes into the stream *)
        ok2:=f_stream.write(p^,ok);
        dec(ok,ok2);
        p:=pointer(longint(p)+ok2);
        end;
    end;
    f_stream.seek(0,0);  (* set the stream back to start *)
  finally
    freemem(buf,buf_size);
    end;
  end;
(*@\\\0000001101*)
(*@/// procedure thttp.Post; *)
procedure thttp.Post;
const
  buf_size=$1000;
var
  buf,p: pointer;
  ok,ok2:integer;
  proto,user,pass,host,port,path: string;
  s: string;
  field,data: string;
begin
  if f_stream.size=0 then  EXIT;

(*@///   parse url and proxy to f_hostname, f_path and f_socket_number *)
if f_proxy<>'' then begin
  parse_url(f_proxy,proto,user,pass,host,port,path);
  f_path:=f_url;
  if port='' then port:='8080';
  end
else begin
  parse_url(f_url,proto,user,f_path,host,port,path);
  if port='' then port:='80';
  end;
if proto='' then  proto:='http';
if path='' then path:='/';

f_hostname:=host;
f_Socket_number:=strtoint(port);
(*@\\\0000000234*)

  login;
(*@///   Send the header *)
self.write_s('GET '+f_path+' HTTP/1.0'+#13#10);   (* send the query *)
if assigned(f_tracer) then
  f_tracer('Requested data using HTTP/1.0',2);
if f_sender<>'' then
  self.write_s('From: '+f_sender+#13#10);
if f_reference<>'' then
  self.write_s('Referer: '+f_reference+#13#10);
if f_agent<>'' then
  self.write_s('User-Agent: '+f_agent+#13#10);
if f_nocache then
  self.write_s('Pragma: no-cache'+#13#10);
self.write_s('Content-Length: '+inttostr(stream.size)+#13#10);
self.write_s(#13#10);                          (* finalize the request *)
(*@\\\0000000C01*)
(*@///   Send the data *)
buf:=NIL;
try
  getmem(buf,buf_size);
  ok:=1;
  while ok>0 do begin
    ok:=f_stream.read(buf^,buf_size);
    write_buf(buf^,ok);
  end;
finally
  if buf<>NIL then
    freemem(buf,buf_size);
  end;
(*@\\\*)
  (* ??? Will there be an answer? I think so *)
(*@///   read in the response header *)
repeat
  s:=self.read_line;
  if false then
(*@///   else if copy(s,1,8)='HTTP/1.0' then    http-status-reply *)
else if copy(s,1,8)='HTTP/1.0' then begin
  f_status_nr:=strtoint(copy(s,10,3));
  f_status_txt:=copy(s,14,length(s));
  if assigned(f_tracer) then
    f_tracer('Received HTTP status '+copy(s,10,length(s)),2);
  if f_status_nr>=300 then
{     raise_par_string(te_tcpip_http_error, }
{       e_tcpip_http_error,status_nr,status_txt) }
  end
(*@\\\*)
(*@///   else if pos(':',s)>0           then    parse the response string *)
else if pos(':',s)>0 then begin
  field:=lowercase(copy(s,1,pos(':',s)-1));
  data:=copy(s,pos(':',s)+2,length(s));
  if assigned(f_tracer) then
    f_tracer('Received HTTP header line: '+s,2);
  if false then
{   else if field='date' then }
{   else if field='mime-version' then }
{   else if field='pragma' then }
{   else if field='allow' then }
{   else if field='location' then }
{   else if field='server' then }
{   else if field='www-authenticate' then }
{   else if field='content-encoding' then }
(*@///   else if field='content-length' then *)
else if field='content-length' then
  f_size:=strtoint(data)
(*@\\\*)
(*@///   else if field='content-type' then *)
else if field='content-type' then
  f_type:=data;
(*@\\\*)
{   else if field='expires' then }
{   else if field='last-modified' then }
  end
(*@\\\*)
(*@///   else                                   some very strange response, ignore it *)
else;
(*@\\\*)
until s='';
if f_status_nr>=300 then
  raise EProtocolError.Create('HTTP',f_status_txt,f_status_nr);
(*@\\\*)
(*@///   read in the response body *)
buf:=NIL;
try
  getmem(buf,buf_size);
  TMemorystream(f_stream).clear;
  while not eof do begin
    read_var(buf^,buf_size,ok);
    p:=buf;
    while ok>0 do begin   (* just to be sure everything goes into the stream *)
      ok2:=f_stream.write(p^,ok);
      dec(ok,ok2);
      p:=pointer(longint(p)+ok2);
      end;
  end;
  f_stream.seek(0,0);  (* set the stream back to start *)
finally
  freemem(buf,buf_size);
  end;
(*@\\\*)
  logout;
  end;
(*@\\\0000000721*)
(*@\\\0000000201*)
(*@/// class tftp(ttcpip) *)
(*@/// constructor tftp.Create(Aowner:TComponent); *)
constructor tftp.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_port:=21;
  f_socket_number:=21;
  f_user:='ftp';
  f_password:='nobody@nowhere'; (* only to make it running without setting user/password *)
  f_passive:=true;
  f_mode:=tftp_download;
  f_cur_dir:=TStringlist.Create;
  f_comm_socket:=INVALID_SOCKET;
  end;
(*@\\\0000000B01*)
(*@/// destructor tftp.Destroy; *)
destructor tftp.Destroy;
begin
  if f_socket<>invalid_socket then
    closesocket(f_socket);
  f_socket:=f_comm_socket;
  f_comm_socket:=INVALID_SOCKET;
  f_cur_dir.free;
  inherited destroy;
  end;
(*@\\\0000000601*)

(*@/// procedure tftp.action; *)
procedure tftp.action;
begin
  login;
  TMemorystream(f_stream).clear;
  case f_mode of
    tftp_download: download;
    tftp_upload:   upload;
    end;
  logout;
  end;
(*@\\\0000000403*)
(*@/// procedure tftp.response; *)
procedure tftp.response;
var
  s: string;
begin
  s:=self.read_line_comm;
  try
    f_status_nr:=strtoint(copy(s,1,3));
  except
    f_status_nr:=999;
  end;
  f_status_txt:=copy(s,5,length(s));
  if f_status_nr>=400 then
    raise EProtocolError.Create('FTP',f_status_txt,f_status_nr);
  (* if the answer consists of several lines read and discard all the following *)
  while (pos('-',s)=4) or (pos(' ',s)=1) do
    s:=self.read_line_comm;
  end;
(*@\\\0000001001*)
(*@/// procedure tftp.login; *)
procedure tftp.login;
begin
  inherited login;
  f_comm_socket:=f_socket;
  self.response;   (* Read the welcome message *)
  self.write_s_comm('USER '+f_user+#13#10);
  self.response;
  self.write_s_comm('PASS '+f_password+#13#10);
  self.response;
  self.write_s_comm('TYPE I'+#13#10);  (* always use binary *)
  self.response;
  end;
(*@\\\0000000403*)
(*@/// procedure tftp.logout; *)
procedure tftp.logout;
begin
  if f_comm_socket<>INVALID_SOCKET then begin
    self.write_s_comm('QUIT'+#13#10);
    self.response;
    end;
  inherited logout;
  end;
(*@\\\0000000401*)
(*@/// procedure tftp.download; *)
procedure tftp.download;
const
  buf_size=$1000;
var
  buf,p: pointer;
  ok,ok2:integer;
begin
  if f_url<>'' then begin
    get_datasocket;
    self.write_s_comm('RETR '+f_url+#13#10);  (* can I use the path here? *)
    self.response;
    buf:=NIL;
    try
      getmem(buf,buf_size);
      TMemorystream(f_stream).clear;
      while not eof do begin
        read_var(buf^,buf_size,ok);
        p:=buf;
        while ok>0 do begin   (* just to be sure everything goes into the stream *)
          ok2:=f_stream.write(p^,ok);
          dec(ok,ok2);
          p:=pointer(longint(p)+ok2);
          end;
      end;
      f_eof:=false;
      self.response;
      shutdown(f_socket,1);
      closesocket(f_socket);
      f_stream.seek(0,0);  (* set the stream back to start *)
    finally
      freemem(buf,buf_size);
      end;
    end;
  end;
(*@\\\0000001D01*)
(*@/// procedure tftp.upload; *)
procedure tftp.upload;
const
  buf_size=$1000;
var
  buf: pointer;
  ok:integer;
begin
  if f_url<>'' then begin
    get_datasocket;
    self.write_s_comm('STOR '+f_url+#13#10);  (* can I use the path here? *)
    self.response;
    buf:=NIL;
    try
      getmem(buf,buf_size);
      ok:=1;
      while ok>0 do begin
        ok:=f_stream.read(buf^,buf_size);
        write_buf(buf^,ok);
      end;
      f_eof:=false;
      self.response;
      shutdown(f_socket,1);
      closesocket(f_socket);
    finally
      freemem(buf,buf_size);
      end;
    end;
  end;
(*@\\\0000000801*)
(*@/// procedure tftp.get_datasocket; *)
procedure tftp.get_datasocket;
var
  po: integer;
  ip: longint;
  s,t: string;
  temp_socket: TSocket;
  SockInfo:TSockAddr;
begin
  f_socket:=INVALID_SOCKET;
(*@///   if self.passive then  ask for the port and open the socket active *)
if self.passive then begin
  self.write_s_comm('PASV'+#13#10);
  self.response;
  if f_status_nr<>227 then
    raise EProtocolError.Create('FTP',f_status_txt,f_status_nr)
  else begin
    s:=copy(f_status_txt,pos('(',f_status_txt)+1,length(f_status_txt));
    s:=copy(s,1,pos(')',s)-1);

    po:=posn(',',s,4);
    t:=copy(s,1,po-1);
    while pos(',',t)<>0 do
      t[pos(',',t)]:='.';

(*@///     ip_address:=Winsock.Inet_Addr(PChar(t));  { try a xxx.xxx.xxx.xx first } *)
(*$ifdef ver80 *)
  hostname:=hostname+#0;
  ip_address:=Winsock.Inet_Addr(@t[1]);  (* try a xxx.xxx.xxx.xx first *)
(*$else *)
  ip_address:=Winsock.Inet_Addr(PChar(t));  (* try a xxx.xxx.xxx.xx first *)
(*$endif *)
(*@\\\000000010E*)
    s:=copy(s,po+1,length(s));
    try
      f_socket_number:=strtoint(copy(s,1,pos(',',s)-1))*256
                     +strtoint(copy(s,pos(',',s)+1,length(s)));
      self.open_socket_out(f_socket, f_socket_number, ip_address);
    except
      f_socket:=INVALID_SOCKET;
      end;
    end;
  end
(*@\\\0000000F01*)
(*@///   else                  send the port and open the socket passive *)
else begin
  ip:=my_ip_address;
  self.write_s_comm('PORT '+inttostr(ip and $000000ff       )+','+
                            inttostr(ip and $0000ff00 shr 8)+','+
                            inttostr(ip and $00ff0000 shr 16)+','+
                            inttostr(ip and $ff000000 shr 24)+','+
                            inttostr(f_port and $ff00 shr 8   )+','+
                            inttostr(f_port and $00ff         )+#13#10);
  self.response;
  open_socket_in(f_socket,f_port,ip);
  (* take the first out of the queue and close the listening socket *)
  temp_socket:=accept_socket_in(f_socket,SockInfo);
  if temp_socket=INVALID_SOCKET then
    {do nothing}
  else begin
    close_socket(f_socket); (* no more listening necessary *)
    f_socket:=temp_socket;
    end;
  end;
(*@\\\*)
  end;
(*@\\\*)

(*@/// procedure tftp.changedir(const f_dir:string); *)
procedure tftp.changedir(const f_dir:string);
begin
  self.write_s_comm('CWD '+f_dir+#13#10);
  self.response;
  end;
(*@\\\0000000319*)
(*@/// procedure tftp.removefile(const filename:string); *)
procedure tftp.removefile(const filename:string);
begin
  self.write_s_comm('DELE '+filename+#13#10);
  self.response;
  end;
(*@\\\*)
(*@/// procedure tftp.removedir(const dirname:string); *)
procedure tftp.removedir(const dirname:string);
begin
  self.write_s_comm('RMD '+dirname+#13#10);
  self.response;
  end;
(*@\\\*)
(*@/// procedure tftp.makedir(const dirname:string); *)
procedure tftp.makedir(const dirname:string);
begin
  self.write_s_comm('MKD '+dirname+#13#10);
  self.response;
  end;
(*@\\\*)
(*@/// procedure tftp.renamefile(const prior,after:string); *)
procedure tftp.renamefile(const prior,after:string);
begin
  self.write_s_comm('RNFR '+prior+#13#10);
  self.response;
  self.write_s_comm('RNTO '+after+#13#10);
  self.response;
  end;
(*@\\\*)
(*@/// procedure tftp.getdir(const dirname:string); *)
procedure tftp.getdir(const dirname:string);
const
  buf_size=$1000;
var
  buf,p: pointer;
  ok,ok2:integer;
begin
  get_datasocket;
  self.write_s_comm('TYPE A'+#13#10);
  self.response;
  self.write_s_comm('LIST '+dirname+#13#10);
  self.response;
  buf:=NIL;
  try
    getmem(buf,buf_size);
    TMemorystream(f_stream).clear;
    while not eof do begin
      read_var(buf^,buf_size,ok);
      p:=buf;
      while ok>0 do begin   (* just to be sure everything goes into the stream *)
        ok2:=f_stream.write(p^,ok);
        dec(ok,ok2);
        p:=pointer(longint(p)+ok2);
        end;
    end;
    f_eof:=false;
    self.response;
    shutdown(f_socket,1);
    closesocket(f_socket);
    self.write_s_comm('TYPE I'+#13#10);  (* always use binary *)
    self.response;
    f_stream.seek(0,0);  (* set the stream back to start *)
    f_cur_dir.clear;
    f_cur_dir.LoadFromStream(f_stream);
    f_cur_dir_index:=0;
  finally
    freemem(buf,buf_size);
    end;
  end;
(*@\\\*)
(*@/// function tftp.getdirentry:t_filedata; *)
function tftp.getdirentry:t_filedata;
begin
  result:=empty_filedata;
  while (f_cur_dir_index<f_cur_dir.count) and ((result.filetype=ft_none)
      or (result.name='.') or (result.name='..')) do begin
    result:=parse_ftp_line(f_cur_dir[f_cur_dir_index]);
    inc(f_cur_dir_index);
    end;
  end;
(*@\\\*)

(*@/// function tftp.read_line_comm:string; *)
function tftp.read_line_comm:string;
var
  temp_socket: TSocket;
begin
  temp_socket:=f_socket;
  f_socket:=f_comm_socket;
  result:=read_line;
  f_socket:=temp_socket;
  end;
(*@\\\000000010E*)
(*@/// procedure tftp.write_s_comm(const s:string); *)
procedure tftp.write_s_comm(const s:string);
var
  temp_socket: TSocket;
begin
  temp_socket:=f_socket;
  f_socket:=f_comm_socket;
  write_s(s);
  f_socket:=temp_socket;
  end;
(*@\\\000000010F*)
(*@\\\000000152D*)

{ Time, RExec, LPR - the most useful UNIX services }
(*@/// class ttime(ttcpip) *)
(*@/// constructor ttime.Create(Aowner:TComponent); *)
constructor ttime.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_Socket_number:=37;
  f_time:=0;
  f_timemode:=tzUTC;
  end;
(*@\\\0000000601*)

(*@/// procedure ttime.action; *)
procedure ttime.action;
var
  ok:integer;
  b: byte;
  bias: integer;
begin
  login;
  f_time:=0;
  while not eof do begin
    read_var(b,1,ok);
    if ok=1 then
      f_time:=f_time*256+b;
    end;
  f_time:=f_time/86400+encodedate(1900,1,1);
  if f_timemode<>tzUTC then begin
(* Alternative: use SystemTimeToTzSpecificLocalTime, but only works in NT *)
    bias:=TimeZoneBias;
    f_time:=f_time-bias/1440;  (* bias is in minutes *)
    end;
  end;
(*@\\\0000000E18*)
(*@\\\0000000310*)
(*@/// class trexec(ttcpip) *)
(*@/// constructor trexec.Create(Aowner:TComponent); *)
constructor trexec.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_Socket_number:=512;   (* exec *)
  end;
(*@\\\0000000501*)

(*@/// procedure trexec.action; *)
procedure trexec.action;
const
  buf_size=$1000;
var
  buf,p: pointer;
  ok,ok2:integer;
begin
  login;
  buf:=NIL;
  try
    getmem(buf,buf_size);
    while not eof do begin
      read_var(buf^,buf_size,ok);
      p:=buf;
      while ok>0 do begin   (* just to be sure everything goes into the stream *)
        ok2:=f_stream.write(p^,ok);
        dec(ok,ok2);
        p:=pointer(longint(p)+ok2);
        end;
    end;
    f_stream.seek(0,0);  (* set the stream back to start *)
  finally
    freemem(buf,buf_size);
    end;
  logout;
  end;
(*@\\\0000000201*)
(*@/// procedure trexec.login; *)
procedure trexec.login;
begin
  inherited login;
  self.write_s(f_user+#0);
  self.write_s(f_pass+#0);
  self.write_s(f_command+#0);
  end;
(*@\\\*)
(*@\\\0000000301*)
(*@/// class Tlpr(TTcpIp) *)
(*@/// constructor Tlpr.Create(Aowner:TComponent); *)
constructor Tlpr.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_Socket_number:=515;
  f_printtype:=lp_ascii;
  f_count:=1;
  end;
(*@\\\000000060E*)

(*@/// procedure Tlpr.action; *)
procedure Tlpr.action;
begin
  login;
  SendPrintData;
  logout;
  end;
(*@\\\0000000501*)
(*@/// procedure Tlpr.SendPrintData; *)
procedure Tlpr.SendPrintData;
const
  buf_size=$1000;
var
  buf: pointer;
  ok:integer;
  i: integer;
  s: string;
  job_name: string;
  config_stream: TMemoryStream;
begin
  (* sanity checks *)
  if (f_queue='') or (f_stream.size=0) or (f_count=0) or (f_user='') then EXIT;
  s:=#02+f_queue+#10;
  write_s(s);
  self.response;
  job_name:=inttostr(lpr_count+1000);
  job_name:=copy(job_name,length(job_name)-2,3)+my_hostname;
(*@///   send the data to print *)
s:=#03+inttostr(stream.size)+' dfA'+job_name+#10;
write_s(s);
self.response;
f_stream.seek(0,0);  (* set the stream back to start *)
buf:=NIL;
try
  getmem(buf,buf_size);
  ok:=1;
  while ok>0 do begin
    ok:=f_stream.read(buf^,buf_size);
    write_buf(buf^,ok);
    end;
  write_s(#0);   (* finish the plot *)
finally
  freemem(buf,buf_size);
  end;
(*@\\\0000000201*)
(*@///   collect and send the description data *)
config_stream:=NIL;
try
  config_stream:=TMemorystream.Create;
(*@///   H originating host *)
s:='H'+ip2string(my_ip_address)+#10;
stream_write_s(config_stream,s);
(*@\\\0000000201*)
(*@///   P responsible user *)
s:='P'+copy(f_user,1,31)+#10;
stream_write_s(config_stream,s);
(*@\\\*)
(*@///   M address to send the mail to *)
if f_user_mail<>'' then begin
  s:='M'+f_user_mail+#10;
  stream_write_s(config_stream,s);
  end;
(*@\\\0000000303*)
(*@///   J jobname (for banner) *)
if f_jobname<>'' then begin
  s:='M'+copy(f_jobname,1,99)+#10;
  stream_write_s(config_stream,s);
  end;
(*@\\\0000000401*)
(*@///   C class name = host name of sender (for banner) *)
s:='C'+copy(my_hostname,1,99)+#10;
stream_write_s(config_stream,s);
(*@\\\*)
(*@///   L banner page *)
if f_banner then begin
  s:='L'+f_user+#10;
  stream_write_s(config_stream,s);
  end;
(*@\\\0000000303*)
(*@///   T title (for lp_pr only) *)
if f_title<>'' then begin
  s:='T'+copy(f_title,1,79)+#10;
  stream_write_s(config_stream,s);
  end;
(*@\\\0000000303*)
(*@///   the print command itself *)
case f_printtype of
  lp_plain:   s:='l';
  lp_ascii:   s:='f';
  lp_dvi:     s:='d';
  lp_plot:    s:='g';
  lp_ditroff: s:='n';
  lp_ps:      s:='o';
  lp_pr:      s:='p';
  lp_fortran: s:='r';
  lp_troff:   s:='t';
  lp_raster:  s:='v';
  lp_cif:     s:='c';
  end;
s:=s+job_name+#10;
for i:=1 to f_count do
  stream_write_s(config_stream,s);
(*@\\\*)
(*@///   U unlink the file after the printing *)
s:='U'+jobname+#10;
stream_write_s(config_stream,s);
(*@\\\*)
(*@///   send the data *)
config_stream.seek(0,0);  (* set the stream back to start *)
s:=#02+inttostr(config_stream.size)+' cfA'+job_name+#10;
write_s(s);
self.response;
buf:=NIL;
try
  getmem(buf,buf_size);
  ok:=1;
  while ok>0 do begin
    ok:=config_stream.read(buf^,buf_size);
    write_buf(buf^,ok);
    end;
  write_s(#0);   (* finish the config data *)
finally
  freemem(buf,buf_size);
  end;
(*@\\\0000000207*)
finally
  config_stream.free;
  end;
(*@\\\0000000D01*)
  inc(lpr_count);
  end;
(*@\\\0000001401*)
(*@/// procedure Tlpr.GetQueueStatus(detailed:boolean); *)
procedure Tlpr.GetQueueStatus(detailed:boolean);
const
  buf_size=$1000;
var
  buf,p: pointer;
  ok,ok2:integer;
  s: string;
begin
  if (f_queue='') then EXIT;
  if detailed then
    s:=#04+f_queue+#10
  else
    s:=#03+f_queue+#10;
  write_s(s);
  buf:=NIL;
  try
    getmem(buf,buf_size);
    while not eof do begin
      read_var(buf^,buf_size,ok);
      p:=buf;
      while ok>0 do begin   (* just to be sure everything goes into the stream *)
        ok2:=f_stream.write(p^,ok);
        dec(ok,ok2);
        p:=pointer(longint(p)+ok2);
        end;
    end;
    f_stream.seek(0,0);  (* set the stream back to start *)
  finally
    freemem(buf,buf_size);
    end;
  end;
(*@\\\0000000D01*)
(*@/// procedure Tlpr.response; *)
procedure Tlpr.response;
var
  b: byte;
  ok: integer;
begin
  read_var(b,1,ok);
  if (ok<>1) or (b<>0) then
    raise EProtocolError.Create('LPR','',999);
  end;
(*@\\\0000000305*)

{ remove jobs }
{ get status }
(*@\\\000000051E*)

{ The Mail and News protocols }
(*@/// class tsmtp(ttcpip) *)
(*@/// constructor tsmtp.Create(Aowner:TComponent); *)
constructor tsmtp.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_Socket_number:=25;
  f_host:=my_hostname;
  f_receipts:=TStringList.Create;
  f_body:=TStringList.Create;
  end;
(*@\\\000000050D*)
(*@/// destructor tsmtp.Destroy; *)
destructor tsmtp.Destroy;
begin
  f_receipts.Free;
  f_body.Free;
  inherited destroy;
  end;
(*@\\\*)

(*@/// procedure tsmtp.action;                           Send only !!! *)
procedure tsmtp.action;
var
  i: integer;
begin
  if (f_receipts=NIL) or (f_receipts.count=0)
    or (f_body=NIL) or (f_body.count=0) or (f_user='') then  EXIT;
    (* not all necessary data filled in *)
  login;
(*@///   Open Connection and submit header data *)
self.response;   (* Read the welcome message *)

self.write_s('HELO '+f_host+#13#10);   (* open connection *)
self.response;
if f_status_nr>=300 then
  raise EProtocolError.Create('SMTP',f_status_txt,f_status_nr);


self.write_s('MAIL FROM: '+f_user+#13#10);   (* send header data *)
self.response;
if f_status_nr>=300 then
  raise EProtocolError.Create('SMTP',f_status_txt,f_status_nr);


for i:=0 to f_receipts.count-1 do begin
  self.write_s('RCPT TO: '+f_receipts.strings[i]+#13#10);   (* submit the receipts *)
  self.response;
  (* Log error users for later check ? *)
  end;

self.write_s('DATA'+#13#10);   (* ready to send the mail *)
self.response;
if f_status_nr=354 then begin
  for i:=0 to f_body.count-1 do begin
    if f_body.strings[i]='.' then f_body.strings[i]:=',';
    self.write_s(f_body.strings[i]+#13#10);
    end;
  self.write_s('.'+#13#10);
  end;
self.response;
if f_status_nr>=300 then
  raise EProtocolError.Create('SMTP',f_status_txt,f_status_nr);
(*@\\\0000001801*)
  end;
(*@\\\0000000501*)
(*@/// procedure tsmtp.response; *)
procedure tsmtp.response;
var
  s: string;
begin
  s:=self.read_line;
  f_status_nr:=strtoint(copy(s,1,3));
  f_status_txt:=copy(s,5,length(s));
  (* if the answer consists of several lines read and discard all the following *)
  while pos('-',s)=4 do
    s:=self.read_line;
  end;
(*@\\\0000000108*)

(*@/// procedure tsmtp.SetBody(Value: TStringList); *)
procedure tsmtp.SetBody(Value: TStringList);
begin
  f_body.assign(value);
  end;
(*@\\\0000000401*)
(*@/// procedure tsmtp.SetRecipients(Value: TStringList); *)
procedure tsmtp.SetRecipients(Value: TStringList);
begin
  f_receipts.assign(value);
  end;
(*@\\\0000000401*)
(*@\\\0000000501*)
(*@/// class tpop3(ttcpip) *)
(*@/// constructor tpop3.Create(Aowner:TComponent); *)
constructor tpop3.Create(Aowner:TComponent);
begin
  inherited create(Aowner);
  f_list:=NIL;
  f_mail:=TStringlist.Create;
  f_list:=TList.Create;
  f_socket_number:=110;
  end;
(*@\\\0000000501*)
(*@/// destructor tpop3.Destroy; *)
destructor tpop3.Destroy;
begin
  f_mail.free;
  if f_list<>NIL then
    while f_list.count>0 do begin
      TObject(f_list.items[0]).Free;
      f_list.delete(0);
      end;
  f_list.free;
  inherited destroy;
  end;
(*@\\\0000000401*)

type
(*@///   t_reply=class(TObject) *)
t_reply=class(TObject)
public
  index: integer;
  length: integer;
  from: string;
  subject: string;
  end;
(*@\\\0000000601*)

(*@/// procedure tpop3.action; *)
procedure tpop3.action;
begin
  login;
  if f_list.count<>0 then
    getmail(1);
  logout;
  end;
(*@\\\0000000501*)
(*@/// procedure tpop3.Login; *)
procedure tpop3.Login;
var
  s: string;
  h: t_reply;
begin
  inherited login;
  self.response;
  self.write_s('USER '+f_user+#13#10);   (* open connection *)
  self.response;
  self.write_s('PASS '+f_pass+#13#10);   (* open connection *)
  self.response;
  self.write_s('LIST'+#13#10);   (* open connection *)
  self.response;
  while true do begin
    s:=self.read_line;
    if s='.' then BREAK;
    h:=t_reply.Create;
    h.index:=strtoint(copy(s,1,pos(' ',s)-1));
    h.length:=strtoint(copy(s,pos(' ',s)+1,length(s)));
    h.from:='';
    h.subject:='';
    f_list.add(h);
    end;
  end;
(*@\\\000000140C*)
(*@/// procedure tpop3.GetHeaders; *)
procedure tpop3.GetHeaders;
var
  i: integer;
  h: t_reply;
  s: string;
begin
  f_mail.clear;
  for i:=f_list.count-1 downto 0 do begin
    h:=t_reply(f_list.items[i]);
    self.write_s('TOP '+inttostr(h.index)+' 1'+#13#10);
    try
      self.response;   (* this may give a EProtocolError on older POP server *)
      while true do begin
        s:=self.read_line;
        if s='.' then BREAK;
        if pos('From:',s)=1 then
          h.from:=copy(s,7,length(s));
        if pos('Subject:',s)=1 then
          h.subject:=copy(s,10,length(s));
        end;
      if h.subject<>'' then
        f_mail.insert(0,h.from+#7+h.subject)
      else
        f_mail.insert(0,h.from)
    except
      on EProtocolError do
        f_mail.insert(0,inttostr(h.index));
      (* ignore errors due to unimplemented TOP *)
      end;
    end;
  end;
(*@\\\0000000C01*)
(*@/// procedure tpop3.Logout; *)
procedure tpop3.Logout;
begin
  self.write_s('QUIT'+#13#10);
  self.response;
  inherited logout;
  if f_list<>NIL then
    while f_list.count>0 do begin
      TObject(f_list.items[0]).Free;
      f_list.delete(0);
      end;
  end;
(*@\\\0000000601*)
(*@/// procedure tpop3.GetMail(index: integer); *)
procedure tpop3.GetMail(index: integer);
var
  s: string;
begin
  self.write_s('RETR '+inttostr(index)+#13#10);
  self.response;
  f_mail.clear;
  while true do begin
    s:=self.read_line;
    if s='.' then BREAK;
    f_mail.add(s);
    end;
  end;
(*@\\\0000000701*)
(*@/// procedure tpop3.DeleteMail(index:integer); *)
procedure tpop3.DeleteMail(index:integer);
begin
  self.write_s('DELE '+inttostr(index)+#13#10);
  self.response;
  end;
(*@\\\*)
(*@/// procedure tpop3.response;                       // single line response *)
procedure tpop3.response;
var
  s: string;
begin
  s:=self.read_line;
  if copy(s,1,3)='+OK' then
    { everything OK }
  else if copy(s,1,4)='-ERR' then
    raise EProtocolError.Create('POP3',s,500)
  else
    raise EProtocolError.Create('POP3',s,999);
  end;
(*@\\\0000000B01*)
(*@\\\0000000D01*)
(*@/// class tnntp(ttcpip) *)
(*@/// function nntpdate(date:TDateTime):string; *)
function nntpdate(date:TDateTime):string;
begin
  result:=formatdatetime('yymmdd hhnnss',date);
  end;
(*@\\\0000000330*)

(*@/// constructor tnntp.Create(Aowner:TComponent); *)
constructor tnntp.Create(Aowner:TComponent);
begin
  inherited create(Aowner);
  f_news:=TStringlist.Create;
  f_newsgroups:=TStringlist.Create;
  f_socket_number:=119;
  end;
(*@\\\0000000401*)
(*@/// destructor tnntp.Destroy; *)
destructor tnntp.Destroy;
begin
  f_news.free;
  f_newsgroups.free;
  inherited destroy;
  end;
(*@\\\0000000501*)
(*@/// procedure tnntp.SetNews(value:TStringlist); *)
procedure tnntp.SetNews(value:TStringlist);
begin
  f_news.assign(value);
  end;
(*@\\\*)

(*@/// procedure tnntp.action; *)
procedure tnntp.action;
begin
  login;
  (* ??? *)
  logout;
  end;
(*@\\\0000000401*)
(*@/// procedure tnntp.Login; *)
procedure tnntp.Login;
begin
  inherited login;
  self.response;
  self.write_s('MODE READER'#13#10);   (* some NNTP servers need this *)
  try
    self.response;
  except
    (* ignore if the server doesn't understand this *)
    end;
  end;
(*@\\\0000000201*)
(*@/// procedure tnntp.Logout; *)
procedure tnntp.Logout;
begin
  self.write_s('QUIT'+#13#10);
  self.response;
  inherited logout;
  end;
(*@\\\0000000401*)
(*@/// procedure tnntp.response; *)
procedure tnntp.response;
var
  s: string;
begin
  s:=self.read_line;
  try
    f_status_nr:=strtoint(copy(s,1,3));
  except
    f_status_nr:=999;
  end;
  f_status_txt:=copy(s,5,length(s));
  if f_status_nr>=400 then
    raise EProtocolError.Create('NNTP',f_status_txt,f_status_nr);
  end;
(*@\\\0000000B03*)

(*@/// procedure tnntp.GetArticle(const group:string; index:integer); *)
procedure tnntp.GetArticle(const group:string; index:integer);
begin
  self.write_s('GROUP '+group+#13#10);
  self.response;
  GetArticleNr(index);
  end;
(*@\\\0000000401*)
(*@/// procedure tnntp.GetArticleID(const msgid:string); *)
procedure tnntp.GetArticleID(const msgid:string);
begin
  if copy(msgid,1,1)<>'<' then
    self.write_s('ARTICLE <'+msgid+'>'+#13#10)
  else
    self.write_s('ARTICLE '+msgid+#13#10);
  self.response;
  f_news.clear;
  GetArticleInternally;
  end;
(*@\\\0000000201*)
(*@/// procedure tnntp.PostArticle; *)
procedure tnntp.PostArticle;
var
  i:integer;
begin
  self.write_s('POST'+#13#10);
  self.response;
  for i:=0 to f_news.count-1 do begin
    if f_news.strings[i]='.' then
      write_s('..'+#13#10)
    else
      write_s(f_news.strings[i]+#13#10);
    end;
  write_s('.'+#13#10);
  self.response;
  end;
(*@\\\0000000A01*)
(*@/// procedure tnntp.GetAllNewsgroups; *)
procedure tnntp.GetAllNewsgroups;
var
  s: string;
begin
  f_newsgroups.clear;
  self.write_s('LIST'+#13#10);
  self.response;
  while true do begin
    s:=read_line;
    if s<>'.' then
      f_newsgroups.add(copy(s,1,pos(' ',s)-1))
    else
      BREAK;
    end;
  end;
(*@\\\*)
(*@/// procedure tnntp.GetNewNewsgroups(since:TDateTime); *)
procedure tnntp.GetNewNewsgroups(since:TDateTime);
var
  s: string;
begin
  f_newsgroups.clear;
  self.write_s('NEWGROUPS '+nntpdate(since)+#13#10);
  self.response;
  while true do begin
    s:=read_line;
    if s<>'.' then
      f_newsgroups.add(copy(s,1,pos(' ',s)-1))
    else
      BREAK;
    end;
  end;
(*@\\\0000000501*)
(*@/// procedure tnntp.SetGroup(const group:string; var low,high,count: integer); *)
procedure tnntp.SetGroup(const group:string; var low,high,count: integer);
var
  s1,s2,s3: integer;
begin
  self.write_s('GROUP '+group+#13#10);
  self.response;
  s1:=pos(' ',f_status_txt);
  s2:=posn(' ',f_status_txt,2);
  s3:=posn(' ',f_status_txt,3);
  count:=strtoint(copy(f_status_txt,1,s1-1));
  low:=strtoint(copy(f_status_txt,s1+1,s2-s1-1));
  high:=strtoint(copy(f_status_txt,s2+1,s3-s2-1));
  end;
(*@\\\0000000C01*)
(*@/// procedure tnntp.GetArticleNr(index:integer); *)
procedure tnntp.GetArticleNr(index:integer);
begin
  self.write_s('ARTICLE '+inttostr(index)+#13#10);
  self.response;
  f_news.clear;
  GetArticleInternally;
  end;
(*@\\\0000000201*)
(*@/// procedure tnntp.SetCurrentArticle(index:integer); *)
procedure tnntp.SetCurrentArticle(index:integer);
begin
  self.write_s('STAT '+inttostr(index)+#13#10);
  self.response;
  end;
(*@\\\*)
(*@/// procedure tnntp.GetCurrentArticle; *)
procedure tnntp.GetCurrentArticle;
begin
  f_news.clear;
  self.write_s('HEAD'+#13#10);
  GetArticleInternally;
  f_news.add('');
  self.write_s('BODY'+#13#10);
  GetArticleInternally;
  end;
(*@\\\*)
(*@/// procedure tnntp.GetNextArticle; *)
procedure tnntp.GetNextArticle;
begin
  self.write_s('NEXT'+#13#10);
  self.response;
  GetCurrentArticle;
  end;
(*@\\\0000000501*)
(*@/// procedure tnntp.GetPreviousArticle; *)
procedure tnntp.GetPreviousArticle;
begin
  self.write_s('LAST'+#13#10);
  self.response;
  GetCurrentArticle;
  end;
(*@\\\0000000601*)
(*@/// procedure tnntp.GetArticleInternally; *)
procedure tnntp.GetArticleInternally;
var
  s: string;
begin
  while true do begin
    s:=read_line;
    if s<>'.' then
      f_news.add(s)
    else
      BREAK;
    end;
  end;
(*@\\\*)
(*@\\\0000001101*)

{ Mail and News text components }
(*@/// class tmailnews(tcomponent) *)
(*@/// constructor TMailNews.Create(Aowner:TComponent); *)
constructor TMailNews.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_body:=TStringList.Create;
  f_add_header:=TStringList.Create;
  f_message:=TStringList.Create;
  end;
(*@\\\0000000603*)
(*@/// destructor TMailNews.Destroy; *)
destructor TMailNews.Destroy;
begin
  f_body.Free;
  f_add_header.Free;
  f_message.Free;
  inherited destroy;
  end;
(*@\\\0000000503*)

(*@/// procedure TMailNews.SetBody(Value: TStringList); *)
procedure TMailNews.SetBody(Value: TStringList);
begin
  f_body.assign(value);
  end;
(*@\\\*)
(*@/// procedure TMailNews.SetHeader(Value: TStringList); *)
procedure TMailNews.SetHeader(Value: TStringList);
begin
  f_add_header.assign(value);
  end;
(*@\\\*)

(*@/// procedure TMailNews.action; *)
procedure TMailNews.action;
begin
  f_message.insert(0,'From: '+f_from);           (* normally the first *)
  if f_sender<>'' then  f_message.add('Sender: '+f_sender);
  f_message.insert(1,'Message-ID: <'+inttostr(round((now-encodedate(1970,1,1))*86400))+'@'
     +my_hostname+'>');
  f_message.insert(2,'Date: '+internet_date(now));
  f_message.insert(3,'Subject: '+f_subject);
  if f_references<>'' then  f_message.add('References: '+f_references);
  if f_replyto<>'' then  f_message.add('Reply-To: '+f_replyto);
  f_message.addstrings(f_add_header);
  f_message.add('');  (* An empty line to separate header from body *)
  f_message.addstrings(f_body);
  end;
(*@\\\0000000B01*)
(*@\\\0000000701*)
(*@/// class tmail(tmailnews) *)
(*@/// constructor TMail.Create(Aowner:TComponent); *)
constructor TMail.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_smtp:=NIL;
  f_receipts:=TStringList.Create;
  end;
(*@\\\0000000601*)
(*@/// destructor TMail.Destroy; *)
destructor TMail.Destroy;
begin
  f_receipts.Free;
  inherited destroy;
  end;
(*@\\\0000000401*)

(*@/// procedure TMail.SetRecipients(Value: TStringList); *)
procedure TMail.SetRecipients(Value: TStringList);
begin
  f_receipts.assign(value);
  end;
(*@\\\0000000401*)

(*@/// procedure TMail.action; *)
procedure TMail.action;
var
  i: integer;
begin
  if (f_smtp=NIL) or (f_receipts=NIL) or (f_receipts.count=0)
    or (f_from='') then  EXIT;
  f_smtp.Recipients:=f_receipts;
  if f_smtp.Sender='' then begin
    if f_sender<>'' then
      f_smtp.sender:=f_sender
    else
      f_smtp.sender:=f_from;
    end;
  if f_disclose_receipts then
    f_message.add('To: disclosed recipients list')
  else
    for i:=0 to f_receipts.count-1 do begin
      f_message.add('To: '+f_receipts.strings[i]);
      end;
  inherited action;
  f_smtp.message:=f_message;
  f_smtp.action;
  f_message.clear;
  end;
(*@\\\0000000701*)
(*@\\\0000000618*)
(*@/// class tnews(tmailnews) *)
(*@/// constructor TNews.Create(Aowner:TComponent); *)
constructor TNews.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_nntp:=NIL;
  f_newsgroups:=TStringList.Create;
  end;
(*@\\\0000000501*)
(*@/// destructor TNews.Destroy; *)
destructor TNews.Destroy;
begin
  f_newsgroups.Free;
  inherited destroy;
  end;
(*@\\\000000030F*)

(*@/// procedure TNews.SetNewsgroups(Value: TStringList); *)
procedure TNews.SetNewsgroups(Value: TStringList);
begin
  f_newsgroups.assign(value);
  end;
(*@\\\000000030F*)

(*@/// procedure TNews.action; *)
procedure TNews.action;
var
  s:string;
  i:integer;
begin
  if (f_nntp=NIL) or (f_newsgroups=NIL) or (f_newsgroups.count=0)
    or (f_newsgroups.count>10)  (* no spamming, please *)
    or (f_from='') then  EXIT;
  s:='Newsgroups: ';
  i:=0;
  while true do begin
    s:=s+f_newsgroups.strings[i];
    inc(i);
    if i<f_newsgroups.count then
      s:=s+',';
    f_message.add(s);
    if i>=f_newsgroups.count then  BREAK;
    s:='  ';
    end;
  if f_organization<>'' then  f_message.add(f_organization);
  inherited action;
  f_nntp.login;
  f_nntp.news:=f_message;
  f_nntp.PostArticle;
  f_nntp.logout;
  f_message.clear;
  end;
(*@\\\*)
(*@\\\*)
(*@/// class tmime(tcomponent) *)
(*@/// constructor TMime.Create(Aowner:TComponent); *)
constructor TMime.Create(Aowner:TComponent);
begin
  inherited create(AOwner);
  f_mail:=NIL;
  f_attach_files:=TStringlist.Create;
  boundary:=inttostr(round((now-encodedate(1970,1,1))*86400))+inttohex(my_ip_address,8)+'==';
  end;
(*@\\\0000000603*)
(*@/// destructor TMime.Destroy; *)
destructor TMime.Destroy;
begin
  f_attach_files.free;
  inherited destroy;
  end;
(*@\\\0000000501*)
(*@/// procedure TMime.AttachFile(const filename:string); *)
procedure TMime.AttachFile(const filename:string);
begin
  f_attach_files.add(filename);
  end;
(*@\\\0000000303*)
(*@/// procedure TMime.SetAttachFiles(value:TStringlist); *)
procedure TMime.SetAttachFiles(value:TStringlist);
begin
  f_attach_files.Assign(value);
  end;
(*@\\\*)

(*@/// procedure TMime.action; *)
procedure TMime.action;
var
  data: TStringList;
  stream: TFileStream;
  i: integer;
  s: string;
begin
  if f_mail=NIL then EXIT;

  f_mail.Header.add('MIME-Version: 1.0');
  f_mail.Header.add('Content-Type: multipart/mixed; boundary="'+boundary+'"');
  f_mail.Header.add('Content-Transfer-Encoding: 7bit');
  f_mail.Body.Insert(0,'--'+boundary);
  f_mail.Body.Insert(1,'Content-Type: text/plain');
  for i:=0 to f_attach_files.count-1 do begin
    stream:=NIL;
    data:=NIL;
    try
      try
        stream:=TFileStream.Create(f_attach_files.strings[i],fmOpenRead);
        s:=filename_of(f_attach_files.strings[i]);
        data:=encode_base64(stream);
        f_mail.Body.Add('');
        f_mail.Body.Add('--'+boundary);
{ Change this ASAP - the mailer can't do anything with this type }
        f_mail.Body.Add('Content-Type: application/octet-stream; name="'+s+'"');
        f_mail.Body.Add('Content-Disposition: attachment; filename="'+s+'"');
        f_mail.Body.Add('Content-Transfer-Encoding: base64');
        f_mail.Body.Add('');
        f_mail.Body.addstrings(data);
      finally
        data.free;
        stream.free;
        end;
      except
        (* ignore errors *)
        end;
    end;
  f_mail.Body.Add('');
  f_mail.Body.Add('--'+boundary+'--');
  f_mail.action;
  end;
(*@\\\0000002A01*)
(*@\\\0000000601*)

(*@/// procedure Register; *)
procedure Register;
begin
  RegisterComponents('Internet', [TFinger]);
  RegisterComponents('Internet', [TFingerD]);
  RegisterComponents('Internet', [THTTP]);
  RegisterComponents('Internet', [TFTP]);
  RegisterComponents('Internet', [Tlpr]);
  RegisterComponents('Internet', [TSMTP]);
  RegisterComponents('Internet', [TMail]);
  RegisterComponents('Internet', [TNNTP]);
  RegisterComponents('Internet', [TNews]);
  RegisterComponents('Internet', [TTime]);
  RegisterComponents('Internet', [TRExec]);
  RegisterComponents('Internet', [TPOP3]);
  RegisterComponents('Internet', [TMIME]);
  end;
(*@\\\0000000701*)
(*@\\\0000001F01*)
end.
(*@\\\0001000011*)
