{ a unit to replace the TDriveComboBox from Borland.
  Uses the system Iconlist to display driveicons.
  Freeware.
  Copyright @ 1997 by Peter Thrnqvist
  Contact: NMA96PTH@STUDENT2.LU.SE or
           DAT97PTH@LUDAT.LTH.SE
}

unit Win32Cmb;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,ShellApi;

type
  TImageSize=(isSmall,isLarge);
  TDriveCombo = class(TCustomComboBox)
  private
    { Private declarations }
    FDrives:TStrings;
    FImages:TImagelist;
    FImageWidth:integer;
    FImageSize:TImageSize;
    FItemIndex:integer;
    FOffset:integer;
    FDrive:char;
    FSmall,FLarge:integer;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure ResetItemHeight;
    procedure SetImageSize(Value:TImageSize);
    procedure SetOffset(Value:integer);
  protected
    { Protected declarations }
    FOnChange: TNotifyEvent;
    procedure CreateWnd; override;
    procedure SetDrive(Value:char);
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure MeasureItem(Index: Integer;var Height: Integer);override;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure BuildList; virtual;
    procedure Change; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Drive: char read FDrive write SetDrive;
  published
    { Published declarations }
    property Offset:integer read FOffset write SetOffset;
    property ImageSize: TImageSize read FImageSize write SetImageSize;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

{ TDriveList }

TDriveList = class(TCustomListBox)
  private
    { Private declarations }
    FDrives:TStrings;
    FImages:TImagelist;
    FImageWidth:integer;
    FImageSize:TImageSize;
    FItemIndex:integer;
    FOffset:integer;
    FDrive:char;
    FSmall,FLarge:integer;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure ResetItemHeight;
    procedure SetImageSize(Value:TImageSize);
    procedure SetOffset(Value:integer);
  protected
    { Protected declarations }
    FOnChange: TNotifyEvent;
    procedure CreateWnd; override;
    procedure SetDrive(Value:char);
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure MeasureItem(Index: Integer;var Height: Integer);override;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure BuildList; virtual;
    procedure Change; dynamic;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Drive: char read FDrive write SetDrive;
  published
    { Published declarations }
    property ImageSize: TImageSize read FImageSize write SetImageSize;
    property Align;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property Font;
    property IntegralHeight;
    property ItemHeight;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
    property Sorted;
    property Tag;
  end;


procedure Register;

implementation

function GetItemHeight(Font: TFont): Integer;
var DC: HDC; SaveFont: HFont; Metrics: TTextMetric;
begin
  DC := GetDC(0);
  SaveFont := SelectObject(DC, Font.Handle);
  GetTextMetrics(DC, Metrics);
  SelectObject(DC, SaveFont);
  ReleaseDC(0, DC);
  Result := Metrics.tmHeight;
end;

{ TDriveCombo }
constructor TDriveCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FLarge := GetSystemMetrics(SM_CXICON);
  FSmall := GetSystemMetrics(SM_CXSMICON);

  FDrives := TStringList.Create;

  if FImageSize = isSmall then
    FImages := TImagelist.CreateSize(FSmall,FSmall)
  else
    FImages := TImagelist.CreateSize(FLarge,FLarge);

  FImages.DrawingStyle := dsTransparent;
  FImageWidth := FImages.Width;
  FImages.ShareImages := True;

  FItemIndex := 0;
  FOffset := 4;
  Color := clTeal;
  Style := csOwnerDrawFixed;
  ResetItemHeight;
end;

destructor TDriveCombo.Destroy;
begin
  FDrives.Free;
  FImages.Free;
  inherited Destroy;
end;

procedure TDriveCombo.BuildList;
var Info:TSHFileInfo; ch:char; S:string; i,Options:integer;
begin
  if Items.Count > 0 then
  begin
   if  ItemIndex <> -1 then
     FItemIndex := ItemIndex;
   Items.Clear;
  end;

  Options := SHGFI_SYSICONINDEX;
  if FImageSize = isSmall then
  	Options := Options or SHGFI_SMALLICON
  else
   Options := Options or SHGFI_LARGEICON;

  FImages.Handle := SHGetFileInfo('',0,Info,SizeOf(TShFileInfo),Options);

  for ch := 'A' to 'Z' do
  begin
   S := ch + ':\';
 	i := GetDriveType(PChar(S));
   if i in [0,1] then Continue; // Invalid
   SHGetFileInfo(PChar(S),0,Info,SizeOf(TShFileInfo),SHGFI_DISPLAYNAME or Options);
   Items.AddObject(Info.szDisplayName,TObject(Info.iIcon));
   FDrives.Add(ch);
  end;
  SetDrive(FDrives[FItemIndex][1]);
  Update;
end;


procedure TDriveCombo.CreateWnd;
begin
  inherited CreateWnd;
  BuildList;
  if (csDesigning in ComponentState) then
  	SetDrive(GetCurrentDir[1]);
end;

procedure TDriveCombo.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Offset,i: Integer;
begin
  with Canvas do
  begin
    FillRect(Rect);
    Offset := FImageWidth + FOffset + FOffset;
    if FImages.Count > 0 then
    begin
      i := integer(Items.Objects[Index]);
      FImages.Draw(Canvas,Rect.Left + FOffset,Rect.Top,i);
      Rect.Left := Rect.Left + Offset;
    end;
    DrawText(Canvas.Handle, PChar(Items[Index]), -1, Rect,
             DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
  end;
end;

procedure TDriveCombo.MeasureItem(Index: Integer;var Height: Integer);
begin
  Height:= ItemHeight;
end;


procedure TDriveCombo.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ResetItemHeight;
  RecreateWnd;
end;

procedure TDriveCombo.ResetItemHeight;
var
  nuHeight: Integer;
begin
  nuHeight :=  GetItemHeight(Font);
  if nuHeight < (FImageWidth + 2) then nuHeight := FImageWidth + 2;
  ItemHeight := nuHeight;
end;

procedure TDriveCombo.SetDrive(Value:char);
var i,j:integer;
begin
 j := 0;
 if FItemIndex <> -1 then
  j := FItemIndex;

 Value := UpCase(Value);
	if FDrive <> Value then
 begin
   for i := 0 to Items.Count - 1 do
   	if FDrives[i][1] = Value then
     begin
      	FDrive := Value;
       FItemIndex := i;
       ItemIndex := i;
       Exit;
     end;
  end
  else
    ItemIndex := j;
end;

procedure TDriveCombo.SetImageSize(Value:TImageSize);
var i:integer;
begin
	if FImageSize <> Value then
 begin
   FIMageSize := Value;
   i := 0;
   if FItemIndex <> -1 then
    i := FItemIndex;

   if Items.Count > 0 then Items.Clear;
   if Assigned(FImages) then FImages.Free;

   if Value = isSmall then
     FImages := TImagelist.CreateSize(FSmall,FSmall)
   else
     FImages := TImagelist.CreateSize(FLarge,FLarge);

   FImages.DrawingStyle := dsTransparent;
   FImages.ShareImages := True;
   FImageWidth := FImages.Width;
   ResetItemHeight;
   RecreateWnd;
   BuildList;
   Change;
 end;
end;

procedure TDriveCombo.SetOffset(Value:integer);
begin
	if FOffset <> Value then
 begin
   FOffset := Value;
   Refresh;
 end;
end;

procedure TDriveCombo.Change;
begin
 if ItemIndex <> -1 then
 	FItemIndex := ItemIndex;
 SetDrive(FDrives[FItemIndex][1]);
 if Assigned(FOnChange) then FOnChange(Self);
end;


procedure TDriveCombo.CNCommand(var Message: TWMCommand);
begin

inherited;
   case Message.NotifyCode of
{    CBN_EDITCHANGE:
      Change;}
    CBN_SELCHANGE:
        Change;
  end;
end;


{ TDrivelist }

constructor TDriveList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FLarge := GetSystemMetrics(SM_CXICON);
  FSmall := GetSystemMetrics(SM_CXSMICON);

  FDrives := TStringList.Create;

  if FImageSize = isSmall then
    FImages := TImagelist.CreateSize(FSmall,FSmall)
  else
    FImages := TImagelist.CreateSize(FLarge,FLarge);

  FImages.DrawingStyle := dsTransparent;
  FImageWidth := FImages.Width;
  FImages.ShareImages := True;

  FItemIndex := 0;
  Color := clTeal;
  Height := 300;
  Width := FImageWidth * 6 + 16;
  FOffset := 0;
  Style := lbOwnerDrawFixed;
  ResetItemHeight;
end;

destructor TDriveList.Destroy;
begin
  FDrives.Free;
  FImages.Free;
  inherited Destroy;
end;

procedure TDriveList.BuildList;
var Info:TSHFileInfo; ch:char; S:string; i,Options:integer;
begin
  if Items.Count > 0 then
  begin
   if  ItemIndex <> -1 then
     FItemIndex := ItemIndex;
   Items.Clear;
  end;

  Options := SHGFI_SYSICONINDEX;
  if FImageSize = isSmall then
  	Options := Options or SHGFI_SMALLICON
  else
   Options := Options or SHGFI_LARGEICON;

  FImages.Handle := SHGetFileInfo('',0,Info,SizeOf(TShFileInfo),Options);

  for ch := 'A' to 'Z' do
  begin
   S := ch + ':\';
 	i := GetDriveType(PChar(S));
   if i in [0,1] then Continue; // Invalid
   SHGetFileInfo(PChar(S),0,Info,SizeOf(TShFileInfo),SHGFI_DISPLAYNAME or Options);
   Items.AddObject(Info.szDisplayName,TObject(Info.iIcon));
   FDrives.Add(ch);
  end;
  SetDrive(FDrives[FItemIndex][1]);
  Update;
end;


procedure TDriveList.CreateWnd;
begin
  inherited CreateWnd;
  BuildList;
  if (csDesigning in ComponentState) then
  	SetDrive(GetCurrentDir[1]);
end;

procedure TDriveList.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  HOffset,i: Integer;
begin
  with Canvas do
  begin
    FillRect(Rect);

    HOffset := (Rect.Right-Rect.Left) div 2 - FImageWidth div 2;
    if FImages.Count > 0 then
    begin
      i := integer(Items.Objects[Index]);
      FImages.Draw(Canvas,HOffset,Rect.Top,i);
    end;
    InflateRect(Rect,1,-6);
    SetBkMode(Canvas.Handle,TRANSPARENT);
    DrawText(Canvas.Handle, PChar(Items[Index]), -1, Rect,
             DT_SINGLELINE or DT_BOTTOM or DT_CENTER or DT_NOPREFIX);
  end;
end;

procedure TDriveList.MeasureItem(Index: Integer;var Height: Integer);
begin
  Height:= ItemHeight;
end;


procedure TDriveList.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ResetItemHeight;
  RecreateWnd;
end;

procedure TDriveList.ResetItemHeight;
begin
  ItemHeight := GetItemHeight(Font)  + FImageWidth + 8;
end;

procedure TDriveList.SetDrive(Value:char);
var i,j:integer;
begin
 j := 0;
 if FItemIndex <> -1 then
  j := FItemIndex;

 Value := UpCase(Value);
	if FDrive <> Value then
 begin
   for i := 0 to Items.Count - 1 do
   	if FDrives[i][1] = Value then
     begin
      	FDrive := Value;
       FItemIndex := i;
       ItemIndex := i;
       Exit;
     end;
  end
  else
    ItemIndex := j;
end;

procedure TDriveList.SetImageSize(Value:TImageSize);
var i:integer;
begin
	if FImageSize <> Value then
 begin
   FIMageSize := Value;
   i := 0;
   if FItemIndex <> -1 then
    i := FItemIndex;

   if Items.Count > 0 then Items.Clear;
   if Assigned(FImages) then FImages.Free;

   if Value = isSmall then
     FImages := TImagelist.CreateSize(FSmall,FSmall)
   else
     FImages := TImagelist.CreateSize(FLarge,FLarge);

   FImages.DrawingStyle := dsTransparent;
   FImages.ShareImages := True;
   FImageWidth := FImages.Width;
   ResetItemHeight;
   RecreateWnd;
   BuildList;
   Change;
 end;
end;

procedure TDriveList.SetOffset(Value:integer);
begin
	if FOffset <> Value then
 begin
   FOffset := Value;
   Refresh;
 end;
end;

procedure TDriveList.Change;
begin
 if ItemIndex <> -1 then
 	FItemIndex := ItemIndex;
 SetDrive(FDrives[FItemIndex][1]);
 if Assigned(FOnChange) then FOnChange(Self);
end;


procedure TDriveList.CNCommand(var Message: TWMCommand);
begin

inherited;
   case Message.NotifyCode of
{    CBN_EDITCHANGE:
      Change;}
    CBN_SELCHANGE:
        Change;
  end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TDriveCombo,TDriveList]);
end;

end.
