{*******************************************************}
{                                                       }
{  TEmail component for Borland Delphi 2.04.00          }
{                                                       }
{     Please see the online documentation for           }
{     contributors, copyright, and release history      }
{                                                       }
{*******************************************************}
unit Email;

{$F+} { force far calls }

{$DEFINE UseGenericThunks}

{.$DEFINE DEBUG}
{$IFNDEF DEBUG}
  {$D-} {$L-} {$Q-} {$R-} {$S-}
{$ENDIF}

{$I+} { I/O exceptions needed for CopyAttachment }

interface

uses
  WinTypes, WinProcs,
  Classes,
  SysUtils,

  SMapi; { this is our generic Simple MAPI wrapper }

type
{$IFDEF WIN32}
  SString   = ShortString;
{$ELSE}
  SString   = string;
{$ENDIF}

const
{$IFNDEF Win32}
                          { implementation limits }
  RECIP_MAX       = 65000 div SizeOf(TMapiRecipDesc);
  ATTACH_MAX      = 65000 div SizeOf(TMapiFileDesc);
{$ELSE}
  RECIP_MAX       = 100000;       { arbitrary }
  ATTACH_MAX      = 100000;
{$ENDIF}

type
  TRecipAccessArray     = array [0 .. (RECIP_MAX - 1)] of TMapiRecipDesc;
  TlpRecipArray   = ^TRecipAccessArray;

  TAttachAccessArray    = array [0 .. (ATTACH_MAX - 1)] of TMapiFileDesc;
  TlpAttachArray   = ^TAttachAccessArray;

  TszRecipName    = array [0 .. 256] of Char;
  TlpszRecipName  = ^TszRecipName;

  TszPathName     = array[0 .. 256] of Char;
  TlpszPathname   = ^TszPathname;


const
  EMAIL_OK = SUCCESS_SUCCESS;

{--------------------------}
{ Define TEmail component. }
{--------------------------}

type
  EMAPIunavailable = class(Exception);

  TEmail = class(TComponent)

  private
    hSession        : ULONG;                  { MAPI Session number.  }

    FShowDialog     : Boolean;                { launch dialog after SendMail ?  }
    FAcknowledge    : Boolean;                { acknowledge receipt ?           }
    FAttachment     : TStrings;               { list of attachments             }
    FAttPathNames   : TStrings;               { Attachments temp file names     }
    FCC             : TStrings;               { list of CC's                    }
    FBcc            : Tstrings;               { list of Blind CC's              }
    FDateRecvd      : SString;                { Message date                    }
    FDownLoadFirst  : Boolean;                { Download messages at logon      }
    FHeaderOnly     : Boolean;                { ReadMail with MAPI_ENVELOPE_ONLY}
    FLeaveUnread    : Boolean;                { ReadMail with MAPI_PEEK         }
    FMessageId      : SString;                { next message Id from GetNext... }
    FNoAttachments  : Boolean;                { ReadMail w. MAPI_SUPPRESS_ATTACH}
    FOriginator     : SString;                { Message originator              }
    FOrigAddress    : SString;                { Message Originator's address    }
    FPassword       : SString;                { password for MAPI logon         }
    FpLongText      : PChar;                  { (long)  message body pointer    }
    FProfile        : SString;                { profile string for MAPI logon   }
    FRecip          : TStrings;               { list of recipients              }
    FSubject        : SString;                { message subject text            }
    FText           : SString;                { (short) message body            }
    FUnread         : Boolean;                { Message read/unread status      }
    FUnreadOnly     : Boolean;                { GetNextMessageID returns unread }
    FUseDefProfile  : Boolean;                { use default profile fr. registry}


    function  GetMAPIavail: boolean;         { MAPI dll has been loaded  ? }

    procedure SetRecip(const Recip: TStrings);
    procedure SetCC   (const CC   : TStrings);
    procedure SetBcc  (const Bcc  : TStrings);
    procedure SetAttachment(const Attachment : TStrings);
    function  ListToRecipArray(FArray          : TStrings;
                               AulRecipClass   : ULONG;
                               lpRecipArray    : TlpRecipArray;
                               var nRecipients : integer) : Integer;

  protected
    procedure CheckMAPI; virtual;

  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    function Address                                       : Integer;
    function CheckAttachment(const AnAttachment : SString) : SString;
    function CheckRecipient (const ARecipient   : SString) : SString;
    function CopyAttachment (Index              : Integer;
                             const DestPathName : SString;
                             DeleteAfterCopy    : Boolean) : Integer;
    function CountUnread                                   : ULONG;
    function DeleteMail                                    : Integer;
    procedure DownLoad;
    function GetLongText                                   : PChar;
    function GetNextMessageId                              : SString;
    function Logoff                                        : Integer;
    function Logon                                         : Integer;
    function ReadMail                                      : Integer;
    function SendMail                                      : Integer;
    function SetLongText(pLongText : Pchar)                : Integer;

    property AttPathNames : TStrings read FAttPathNames;
    property DateRecvd    : SString read FDateRecvd;
    property HeaderOnly   : Boolean read FHeaderOnly write FHeaderOnly
                                    Default False;
    property LeaveUnread  : Boolean read FLeaveUnread write FleaveUnread
                                    Default True;
    property MessageId    : SString read FMessageId write FMessageId;
    property Originator   : SString read FOriginator;
    property OrigAddress  : SString read FOrigAddress;
    property NoAttachments: Boolean read FNoAttachments write FNoAttachments
                                    Default False;
    property Unread       : Boolean read Funread;

  published
    property Acknowledge  : Boolean     read Facknowledge write Facknowledge
                                        default false;
    property Attachment   : TStrings    read FAttachment write SetAttachment;
    property Bcc          : TStrings    read FBcc write SetBcc;
    property CC           : TStrings    read FCC write SetCC;
    property MAPIavail    : boolean     read GetMAPIavail;
    property DownLoadFirst: Boolean     read FDownLoadFirst write FDownLoadFirst;
    property UnreadOnly   : boolean     read FUnreadOnly write FUnreadOnly;
    property Password     : SString     read FPassword write FPassword;
    property Profile      : SString     read FProfile write FProfile;
    property Recipient    : TStrings    read FRecip write SetRecip;
    property ShowDialog   : Boolean     read FShowDialog write FShowDialog
                                        default False;
    property Subject      : SString     read FSubject write FSubject;
    property Text         : SString     read FText write FText;
    property UseDefProfile: Boolean     read FUseDefProfile write FUseDefProfile
                                        default False;

{   property UserName: SString read FUserName write FUserName;            }
{   property UserPassword: SString read FUserPassword write FUserPassword;}

  end;

(* procedure Register; has been moved to email16.pas and email32.pas *)

Implementation

uses
 {$IFDEF WIN32}
   Registry
 {$ELSE}
   Call32NT { for thunking 16bit -> 32bit }
 {$ENDIF};


{ Create object. }
{----------------}

constructor TEmail.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { create private stringlist objects }
  { we need to create string lists for the TStrings }
  FRecip        := TStringList.Create;
  FCC           := TStringList.Create;
  FBcc          := TStringList.Create;
  FAttachment   := TStringList.Create;
  FAttPathNames := TStringList.Create;

  { initialize private data defaults}
  FLeaveUnread   := True;
{ Delphi will initialize the following members automatically to zero
  hSession       := 0;
  Facknowledge   := False;
  FMessageId     := '';
  FHeaderOnly    := False;
  FNoAttachments := False;
  FDateRecvd     := '';
  FDownLoadFirst := False;
  FOriginator    := '';
  FOrigAddress   := '';
  FProfile       := '';
  FPassword      := '';
  FUseDefProfile := False;
  FShowDialog    := False;
}
end;


{ Destroy object. }
{-----------------}

destructor TEmail.Destroy;
begin
  If hSession <> 0 then
    Logoff;

  FBcc.Free;
  FBcc := Nil;

  FCC.Free;
  FCC := Nil;

  FRecip.Free;
  FRecip := Nil;

  FAttachment.Free;
  Fattachment := nil;

  FAttPathNames.Free;
  FAttPathNames := Nil;

  inherited Destroy;
end;


function TEmail.GetMAPIavail: boolean;
begin
  Result := MAPIavailable; { call function in SMAPI.PAS }
end;

{ Test whether MAPI is available - if not, raise an exception }

procedure TEmail.CheckMAPI;
{$IFNDEF Win32}
  const
{$ELSE}
  {$IFDEF VER90}
    const
  {$ELSE}
    resourcestring
  {$ENDIF}
{$ENDIF}
  SNoMapi = 'MAPI services are not available on this system';
begin
  if not MAPIavailable then { call function in SMAPI.PAS }
    raise EMAPIunavailable.Create(SNoMapi);
end;

{ This is code to allow the component to thunk to the Win32 API }

{$IFNDEF Win32} {$IFDEF UseGenericThunks}
type
  HKEY = longint;
  PHKEY = ^HKEY;

  PDWORD = ^DWORD;

const
  HKEY_CURRENT_USER = $80000001;
  KEY_QUERY_VALUE = $0001;
  REG_SZ = 1;
var
  id_W32RegCloseKey,
  id_W32RegOpenKeyEx,
  id_W32RegQueryValueEx: longint;

  W32RegCloseKey: function(hKey: HKEY; id: longint): longint;

  W32RegOpenKeyEx: function (hKey: HKEY; lpSubKey: PChar;
                       ulOptions: DWORD; samDesired: longint; phkResult: PHKEY; id: longint): Longint;

  W32RegQueryValueEx: function(hKey: HKEY; lpValueName: PChar; lpReserved: Pointer;
                          lpType: PDWORD; lpData: PByte; lpcbData: PDWORD; id: longint): longint;

procedure InitWin32; far;
begin
  @W32RegQueryValueEx := @Call32;
  id_W32RegQueryValueEx := Declare32('RegQueryValueEx', 'advapi32', 'ippppp');
{
function RegQueryValueEx(hKey: HKEY; lpValueName: PChar;
  lpReserved: Pointer; lpType: PDWORD; lpData: PByte; lpcbData: PDWORD): Longint; stdcall;
}

  @W32RegOpenKeyEx := @Call32;
  id_W32RegOpenKeyEx := Declare32('RegOpenKeyEx', 'advapi32', 'ipiip');
{
function RegOpenKeyEx(hKey: HKEY; lpSubKey: PChar;
  ulOptions: DWORD; samDesired: REGSAM; var phkResult: HKEY): Longint; stdcall;
}

  @W32RegCloseKey := @Call32;
  id_W32RegCloseKey := Declare32('RegCloseKey', 'advapi32', 'i');
{
function RegCloseKey(hKey: HKEY): Longint; stdcall;
}
end;

function RegOpenKeyEx32(ahKey: HKEY; lpSubKey: PChar;
         ulOptions: DWORD; samDesired: longint; var phkResult: HKEY): Longint;
var
  aKey : HKEY;
begin
  RegOpenKeyEx32 := W32RegOpenKeyEx(ahKey, lpSubKey, ulOptions, samDesired, @aKey, id_W32RegOpenKeyEx);
  phkResult := aKey;
end;

function RegCloseKey32(ahKey: HKEY): Longint;
begin
  RegCloseKey32 := W32RegCloseKey(ahKey, id_W32RegCloseKey);
end;


function RegQueryValueEx32 (hKey: HKEY; lpValueName: PChar; lpReserved: Pointer;
                            lpType: DWORD; lpData: PByte; var lpcbData: DWORD): longint;
var
  theData: DWORD;
begin
  theData := lpcbData;
  RegQueryValueEx32 := W32RegQueryValueEx(hKey, lpValueName, nil, @lpType,
                               lpData, @lpcbData, id_W32RegQueryValueEx);

end;
{$ENDIF} {$ENDIF}


{ Logon to E-Mail system. }
{-------------------------}

function TEmail.Logon: Integer;
const
  ProfileKey = 'Software\Microsoft\Windows Messaging Subsystem\Profiles';
var
  flFlags      : ULONG;
  MapiResult   : ULONG;
  LogonProfile : PChar;
  LogonPassword: PChar;
{$IFDEF WIN32}
  Reg : TRegistry;
{$ELSE}
  {$IFDEF UseGenericThunks}
    RegHandle: HKEY;
    RegDataSize: longint;
    RegResult : longint;
  {$ENDIF}
{$ENDIF}

begin
  CheckMAPI;

  Result := SUCCESS_SUCCESS;

  { Check if already logged in. }

  If hSession = 0 then begin

    if FUseDefProfile then begin
    {$IFDEF WIN32}
      Reg := TRegistry.Create;
      try
        Reg.Rootkey := HKEY_CURRENT_USER;
        if Reg.OpenKey(ProfileKey, False) then begin
          try
            FProfile := Reg.Readstring('DefaultProfile');
          except
            FProfile := '';
          end;
        end;
      finally
        Reg.Free;
      end;
    {$ELSE}
      {$IFDEF UseGenericThunks}
        if Call32NTError = false then
        begin
          RegResult := RegOpenKeyEx32( HKEY_CURRENT_USER,
                                       ProfileKey,
                                       0,
                                       KEY_QUERY_VALUE,
                                       RegHandle);

          RegDataSize := SizeOf(FProfile)-1;
          RegResult := RegQueryValueEx32( RegHandle,
                                          'DefaultProfile',
                                          nil,
                                          REG_SZ,
                                          pointer(PChar(@FProfile)+1),
                                          RegDataSize);
          FProfile[0] := char(StrLen(PChar(@FProfile)+1));

          RegResult := RegCloseKey32(RegHandle);
        end;
      {$ENDIF}
    {$ENDIF}
    end;

    LogonProfile := nil;
    LogonPassword := nil;

    try
      if (Length(FProfile) > 0) then begin
        LogonProfile := StrPCopy(StrAlloc(Length(FProfile)+1), FProfile);
      end;

      if (Length(FPassword) > 0) then begin
        LogonPassword := StrPCopy(StrAlloc(Length(FPassword)+1), FPassword);
      end;

      if FDownLoadFirst then flFlags := MAPI_FORCE_DOWNLOAD
      else flFlags := 0;

      MapiResult := MAPILogon(0, LogonProfile, LogonPassword, flFlags, 0, @hSession);
      if (MapiResult <> SUCCESS_SUCCESS) then begin
         flFlags := flFlags or MAPI_Logon_UI;
         Result  := MAPILogon(0, NIL, NIL, flFlags, 0, @hSession);
      end else begin
         Result := MapiResult;
      end;
    finally
      StrDispose(LogonProfile);
      StrDispose(LogonPassword);
    end;
  end;
end;


{ Logoff E-Mail system. }
{-----------------------}

function TEmail.Logoff: Integer;
begin
  CheckMAPI;

  Result := SUCCESS_SUCCESS;

  If hSession <> 0 then
  begin
    Result   := MAPILogoff(hSession, 0, 0, 0);
    hSession := 0;
  end;
end;

{ Download e-mail messages now }
{------------------------------}

procedure TEmail.DownLoad;
begin
  DownLoadFirst := True;
  Logoff;
  Logon;   
end;

{ Send Mail message. }
{--------------------}

function TEmail.SendMail: Integer;
var
  MapiMessage   : TMapiMessage;
  MapiRecipDesc : TMapiRecipDesc;
  MapiFileDesc  : TMapiFileDesc;
  lpRecipArray  : TlpRecipArray;
  lpAttachArray : TlpAttachArray;
  lpszPathname  : TlpszPathname;

  szSubject     : PChar;
  szText        : PChar;

  Attachment    : SString;
  flFlags       : ULONG;
  flLogoff      : Boolean;

  i             : Integer;
  nRecipients   : integer;
  nAttachments  : Integer;

begin
  CheckMAPI;

  {make sure the cleanup does not free garbage }
  lpRecipArray  := nil;
  lpAttachArray := nil;

  flLogoff       := False;

  {check our built-in limits - which have effectively been removed }
  nRecipients := Frecip.Count + FCC.Count + FBCC.Count;
  if nRecipients > RECIP_MAX then begin
    Result := MAPI_E_TOO_MANY_RECIPIENTS;
    exit;
  end;

  nAttachments := FAttachment.Count;
  if (nAttachments > ATTACH_MAX) then begin
    Result := MAPI_E_TOO_MANY_FILES;
    exit;
  end;

  {begin the work}
  try

    If hSession = 0 then flLogoff := True
    else                 flLogoff := False;

    { Logon to mail server if not already logged on. }

    If Logon <> SUCCESS_SUCCESS then begin
      Result := MAPI_E_LOGIN_FAILURE;
      exit;
    end;


    { Initialise MAPI structures and local arrays. }

    FillChar(MapiMessage,   Sizeof(TMapiMessage),   0);
    FillChar(MapiRecipDesc, Sizeof(TMapiRecipDesc), 0);
    FillChar(MapiFileDesc,  Sizeof(TMapiFileDesc),  0);

    lpRecipArray  := TlpRecipArray(StrAlloc(nRecipients*SizeOf(TMapiRecipDesc)));
    FillChar(lpRecipArray^, StrBufSize(PChar(lpRecipArray)), 0);

    lpAttachArray := TlpAttachArray(StrAlloc(nAttachments*SizeOf(TMapiFileDesc)));
    FillChar(lpAttachArray^, StrBufSize(PChar(lpAttachArray)), 0);

    { Fill in subject & message text. }

    szSubject := nil;
    szText := nil;

    try
      if length(FSubject) > 0 then begin
        szSubject := StrAlloc(length(FSubject)+1);
        StrPCopy(szSubject, FSubject);
      end;
      MapiMessage.lpszSubject  := szSubject;

      if (Length(Ftext) > 0) then begin
        szText :=  StrAlloc(length(FText)+1);
        StrPCopy(szText, FText);
      end;
      MapiMessage.lpszNoteText := szText;

      if (FpLongText <> Nil) then
         MapiMessage.lpszNoteText := FpLongText;

      { check and fill in recipients if any}
      nRecipients := 0;
      ListToRecipArray(FRecip, MAPI_TO,  lpRecipArray, nRecipients);
      ListToRecipArray(FCC,    MAPI_CC,  lpRecipArray, nRecipients);
      ListToRecipArray(FBcc,   MAPI_BCC, lpRecipArray, nRecipients);


      MapiMessage.nRecipCount := nRecipients;

      If (nRecipients > 0) then
      begin
        if FShowDialog then
          flFlags := MAPI_DIALOG
        else
          flFlags := 0; { Don't display MAPI Dialog if recipient specified. }

        MapiMessage.lpRecips := @lpRecipArray^;
      end else
      begin
        flFlags := MAPI_DIALOG; { we need to show the dialog }
        MapiMessage.lpRecips  := nil ;
      end;

      { Process file attachments. }

      nAttachments := 0;

      for i := 0 to (Fattachment.Count - 1) do
      begin
        Attachment := CheckAttachment(Fattachment.Strings[i]);
        if (Length(Attachment) = 0) then begin
          Result := MAPI_E_ATTACHMENT_NOT_FOUND;
          exit;
        end;
        lpAttachArray^[i].nPosition    := $ffffffff;  {Top of message. }
        lpszPathname                   := new(TlpszPathname);
        lpAttachArray^[i].lpszPathName := StrPcopy(lpszPathname^,Attachment);
        Inc(nAttachments);
      end;

      MapiMessage.nFileCount := nAttachments;
      if (nAttachments > 0) then begin
        MapiMessage.lpFiles := @lpAttachArray^;
      end else begin
        MapiMessage.lpFiles := nil;
      end;

      { receipt requested ? }
      if Facknowledge then
        MapiMessage.flFlags := MapiMessage.flFlags or MAPI_RECEIPT_REQUESTED;

      { finally send the email message }
      Result := MAPISendMail(hSession, 0, @MapiMessage, flFlags, 0);

    finally
      StrDispose(szSubject);
      StrDispose(szText);
    end;


  finally
    { dispose of the recipient & CC name strings }
    if Assigned(lpRecipArray) then
      for i := 0 to (nRecipients - 1) do begin
        if Assigned(lpRecipArray^[i].lpszName) then
          Dispose(lpRecipArray^[i].lpszName);

        if Assigned(lpRecipArray^[i].lpszAddress) then
          Dispose(lpRecipArray^[i].lpszAddress);
      end;
    {dispose of the recipient/CC/BCC array }
    StrDispose(PChar(lpRecipArray));

    { dispose of the attachment file name strings }
    if Assigned(lpAttachArray) then
      for i := 0 to (nAttachments - 1) do begin
        Dispose(lpAttachArray^[i].lpszPathname);
      end;
    { dispose of the attachment array }
    StrDispose(PChar(lpAttachArray));
    { Auto logoff, if no session was active. }
    if flLogoff = True then Logoff;
  end;

end;


{ Check a recipient }
{-------------------}

function Temail.CheckRecipient(const ARecipient: SString) : SString;
const
  RecipSize = 256;
var
  szRecipName   : PChar;
  lpRecip       : lpMapiRecipDesc;
  flLogoff      : Boolean;
  MapiResult    : ULONG;

begin
  CheckMAPI;

  If hSession = 0 then flLogoff := True
  else                 flLogoff := False;

  { Logon to mail server if not already logged on . }
  If Logon <> SUCCESS_SUCCESS then exit;

  szRecipName := nil;
  try
    szRecipName := StrAlloc(RecipSize);
    szRecipName := StrPCopy(szRecipName, ARecipient);

    MapiResult := MAPIResolveName(hSession, 0, szRecipName, 0, 0, @lpRecip);
    if ((MapiResult = MAPI_E_AMBIGUOUS_RECIPIENT)
                       or (MapiResult = MAPI_E_UNKNOWN_RECIPIENT)) then begin
       MapiResult := MAPIResolveName(hSession, 0, szRecipName, MAPI_DIALOG,
                                    0, @lpRecip);
    end;

    { same changes as below...}

    if (MapiResult <> SUCCESS_SUCCESS) then begin
       Result := '';
       exit;
    end else begin

       { the original address looked like a direct address }
       if ((Pos('[', ARecipient) > 0) and (Pos(']', ARecipient) > 0)) and

       { did Windows recognize this?}
          ((StrScan (lpRecip^.lpszName, '[') = nil) and
           (StrRScan(lpRecip^.lpszName, ']') = nil)) then
       begin
         { use original address, and DO NOT remove the brackets...
           we will need them later when the array is filled to determine
           whether this is a direct address or not }
         Result := ARecipient;
       end
       else
       begin
         { use resolved address }
         Result := StrPas(lpRecip^.lpszName);
       end;

       MAPIFreeBuffer(lpRecip);
    end;
  finally
    StrDispose(szRecipName);
    if flLogoff = True then Logoff;
  end;

end;


{ Check if an attachment is a valid file }
{----------------------------------------}

function Temail.CheckAttachment(const AnAttachment: SString): SString;
begin
  if (not FileExists(AnAttachment)) then
    Result := ''
  else
    Result := AnAttachment;
end;


{ Read e-mail message with FMessageId }
{-------------------------------------}

function TEmail.ReadMail: Integer;
var
  lppMapiMessage : lpMapiMessage;
  lpRecipArray   : TlpRecipArray;
  lpAttachArray  : TlpAttachArray;

  szMessageID : PChar;
  flLogoff    : Boolean;
  MapiResult  : ULONG;
  flFLags     : ULONG;
  i           : integer;

begin
  CheckMAPI;

  { Auto Logoff, if no session active. }
  If hSession = 0 then flLogoff := True
  else                 flLogoff := False;

  { Logon to mail server. }

  If Logon <> SUCCESS_SUCCESS then begin
    Result := MAPI_E_LOGIN_FAILURE;
    exit;
  end;

  szMessageID := nil;
  try
    if (FMessageId = '') then begin
      Result := MAPI_E_INVALID_MESSAGE;
      exit;
    end;

    szMessageId := Stralloc(MsgIdSize);
    FillChar(szMessageID^, MsgIdSize,0);

    StrPCopy(szMessageId, FMessageId);

    flFlags := 0;
    if FLeaveUnread   then flFlags := MAPI_PEEK;
    if FNOAttachments then flFlags := flFlags or MAPI_SUPPRESS_ATTACH;
    if FHeaderOnly    then flFlags := flFlags or MAPI_ENVELOPE_ONLY;

    MapiResult := MapiReadMail(hSession, 0, szMessageId, flFlags, 0, @lppMapiMessage);
    if MapiResult <> SUCCESS_SUCCESS then begin
      Result := MapiResult;
      exit;
    end;

    { copy recipients to CC en Recipient stringlists }

    Frecip.Clear;
    FCC.Clear;
    FBcc.Clear;
    if (lppMapiMessage = nil) then begin
      Result := 1;
      exit;
    end;
    lpRecipArray := @lppMapiMessage^.lpRecips^;
    for i := 0 to (lppMapiMessage^.nRecipCount - 1) do begin
      if (lpRecipArray^[i].ulRecipClass = MAPI_TO) then
         Frecip.Add(StrPas(lpRecipArray^[i].lpszName));
      if (lpRecipArray^[i].ulRecipClass = MAPI_CC) then
         FCC.Add(StrPas(lpRecipArray^[i].lpszName));
      if (lpRecipArray^[i].ulRecipClass = MAPI_BCC) then
         FBcc.Add(StrPas(lpRecipArray^[i].lpszName));
    end;

    { copy attachments to attchment stringlist                          }
    { Roman Popov suggests using lpszPathName instead of lpszFileName.  }
    { It seems that lpszFilename is not used by all MAPI implementations}
    { This is undocumented, but if attachments are requested, both the  }
    { temp pathname of the attachment and the filename part of the      }
    { original attachment pathname are returned. I give you the temp    }
    { pathname if the filename part is empty now.                       }
    { As a bonus you always get the pathname in a new read-only runtime }
    { stringlist property if NoAttachments := False.                    }
    { -- dh 03-05-97 -- }

    FAttachment.Clear;
    FAttPathNames.Clear;              { new read-only property }
    lpAttachArray := @lppMapiMessage^.lpFiles^;
    for i := 0 to (lppMapiMessage^.nFileCount - 1) do begin
         if (lpAttachArray^[i].lpszFileName <> nil) then
           FAttachment.Add(StrPas(lpAttachArray^[i].lpszFileName))
         else
           FAttachment.Add(StrPas(lpAttachArray^[i].lpszPathName));
         if FNoAttachments = False then
           FAttPathNames.Add(StrPas(lpAttachArray^[i].lpszPathName));
    end;

    { copy subject Text }

    FSubject := '';
    if (lppMapiMessage^.lpszSubject <> Nil) then
      FSubject := StrPas(lppMapiMessage^.lpszSubject);

    { copy message body }

    if (FpLongText <> nil) then StrDispose(FpLongText);
    FpLongText := nil;
    Ftext := '';
    if (lppMapiMessage^.lpszNoteText <> Nil) then begin
      if Strlen(lppMapiMessage^.lpszNoteText) < 256 then begin
        Ftext := StrPas(lppMapiMessage^.lpszNoteText);
      end else begin
        FpLongText := Strnew(lppMapiMessage^.lpszNoteText);
      end;
    end;

    { set message status }

    if (Boolean(lppMapiMessage^.flFlags and MAPI_UNREAD) = True)
    then Funread := True else FunRead := False;

    { copy message originator }

    Foriginator := StrPas(lppMapiMessage^.lpOriginator^.lpszName);

    { copy originators mail address }

    if (lppMapiMessage^.lpOriginator^.lpszAddress <> Nil) then
      FOrigAddress := StrPas(lppMapiMessage^.lpOriginator^.lpszAddress);

    { copy message date }

    if (lppMapiMessage^.lpszDateReceived <> Nil) then
      FDateRecvd := StrPas(lppMapiMessage^.lpszDateReceived);

    Result := SUCCESS_SUCCESS;
    MAPIFreeBuffer(lppMapiMessage);
  finally
    StrDispose(szMessageId);
    { Auto logoff, if no session was active. }
    if flLogoff = True then Logoff;
  end;
end;

{ CopyAttachment :  }
{-------------------}

function Temail.CopyAttachment(Index              : Integer;
                               const DestPathname : SString;
                               DeleteAfterCopy    : Boolean) : Integer;
const
  BUFSIZE = 32768;

type
  TBuffer = array[0..(BUFSIZE - 1)] of char;
  TBufp   = ^TBuffer;

var
  FromF, ToF : File;
  NumRead, NumWritten : Integer;
  bufp : TBufp;

{$IFNDEF Win32}
  const
{$ELSE}
  {$IFDEF VER90}
    const
  {$ELSE}
    resourcestring
  {$ENDIF}
{$ENDIF}
  SIllegalAttachmentOperation = 'CopyAttachment called when NoAttachments = True !!';
  SAttachmentIndexRangeError = 'CopyAttachment : index for pathnames stringlist is out of bounds';
  SAttachmentNotFound = 'CopyAttachment : attachment not found !!';
  SAttachmentOutOfMemory = 'CopyAttachment : no memory for copy buffer';
  SAttachmentIOError = 'CopyAttachment : Input-Output error on copy operation';
begin
  { check if attachments present }
  if (FNoAttachments = True) then
    raise EMAPIAttachmentError.Create(SIllegalAttachmentOperation);

  { is the index legal ? }
  if (Index > (FAttPathNames.Count - 1)) then
    raise EMAPIAttachmentError.Create(SAttachmentIndexRangeError);

  { is the file there ? }
  if not FileExists(FAttPathNames.Strings[Index]) then
    raise EMAPIAttachmentError.Create(SAttachmentNotFound);

  { allocate file copy buffer }
  try
    Bufp := New(TBufp);
  except
    raise EMAPIAttachmentError.Create(SAttachmentOutOfMemory);
  end;

  try                          { protect buffer allocation }
    try                        { catch all I-O errors }
      AssignFile(FromF, FAttPathNames.Strings[Index]);
      AssignFile(ToF, DestPathName);

      Reset(FromF, 1);
      try                      { make sure we close inputfile }
        Rewrite(ToF, 1);
        try                    { and outputfile too }
          repeat
            BlockRead(FromF, Bufp^, BUFSIZE, NumRead);
            BlockWrite(ToF, Bufp^, NumRead, NumWritten);
          until (NumRead = 0) or (NumWritten <> NumRead);
        finally
          CloseFile(ToF);
        end;
      finally
        CloseFile(FromF);
      end;

      if (DeleteAfterCopy) then
        SysUtils.DeleteFile(FattPathNames.Strings[Index]);

    except
      raise EMAPIAttachmentError.Create(SAttachmentIOError);
    end;

  finally
    Dispose(Bufp);
  end;

  Result := 0;
end;


{ Delete e-mail message with FMessageId }
{---------------------------------------}

function TEmail.DeleteMail: Integer;
var
  szMessageID : PChar;
  flLogoff    : Boolean;
  MapiResult  : ULONG;
  flFLags     : ULONG;

begin
  CheckMAPI;

  { Auto Logoff, if no session active. }
  If hSession = 0 then flLogoff := True
  else                 flLogoff := False;

  { Logon to mail server. }
  If Logon <> SUCCESS_SUCCESS then begin
    Result := MAPI_E_LOGIN_FAILURE;
    exit;
  end;

  szMessageID := nil;
  try
    if (FMessageId = '') then begin
      Result := MAPI_E_INVALID_MESSAGE;
      exit;
    end;
    szMessageId := Stralloc(MsgIdSize);
    StrPcopy(SzMessageId, FMessageId);

    flFlags := 0;
    MapiResult := MapiDeleteMail(hSession, 0, szMessageId, flFlags, 0);
    if MapiResult <> SUCCESS_SUCCESS then begin
      Result := MapiResult;
      exit;
    end;

    FMessageId := '';      { has become invalid }

    Result := SUCCESS_SUCCESS;

  finally
    StrDispose(szMessageID);
    { Auto logoff, if no session was active. }
    if flLogoff = True then Logoff;
  end;
end;


{ make a copy of the message body and store pointer to it in Temail }
{-------------------------------------------------------------------}

function Temail.SetLongText(pLongText : Pchar) : integer;
begin
   if (FpLongText <> nil) then StrDispose(FpLongText);
   FpLongText := StrNew(pLongText);
   Ftext := '';
   Result := Strlen(pLongText);
end;

{ get a pointer to the message body }
function Temail.GetLongText: Pchar;
begin
   Result := FpLongText;
end;


function TEmail.GetNextMessageId: SString;
var
  szMessageID     : PChar;
  szSeedMessageID : PChar;
  flLogoff        : Boolean;
  flFlags         : ULONG;

begin
  CheckMAPI;

  szMessageID     := nil;
  szSeedMessageID := nil;

  { Auto Logoff, if no session active. }
  if hSession = 0 then flLogoff := True
  else                 flLogoff := False;

  { Logon to mail server. }

  if Logon <> SUCCESS_SUCCESS then begin
    Result := '';
    exit;
  end;

  try
    szMessageID := StrAlloc(MsgIdSize); { MAPI 32bit !! }
    szSeedMessageID := StrAlloc(MsgIdSize); { MAPI 32bit !! }
    FillChar(szSeedMessageID^,MsgIdSize,0);

    if (Length(FMessageId) = 0) then begin
      FillChar(szMessageID^,MsgIdSize,0);
    end else begin
      StrPCopy(szMessageId,FMessageId);
    end;

    StrCopy(szSeedMessageId, szMessageId);
    flFlags := 0;

    if FUnreadOnly then
      inc(flFlags, MAPI_UNREAD_ONLY);
                           
    if (MAPIFindNext(hSession,0,nil,szSeedMessageID,
                                         flflags,0,@szMessageID^) = 0) then begin
       FMessageId := StrPas(szMessageId);
    end else begin
       FMessageId := '';
    end;
  finally
    StrDispose(szMessageID);
    StrDispose(szSeedMessageID);

    { Auto logoff, if no session was active. }
    if flLogoff = True then Logoff;
  end;

  Result := FMessageID;

end;

{ set recipient }
{---------------}

procedure Temail.SetRecip(const Recip: TStrings);
begin
  FRecip.Assign(Recip);
end;

{ set CC }
{--------}

procedure Temail.SetCC(const CC: TStrings);
begin
  FCC.Assign(CC);
end;


{ set  BCC }
{----------}

procedure Temail.SetBcc(const Bcc: TStrings);
begin
  FBcc.Assign(Bcc);
end;


{ set attachment }
{----------------}

procedure Temail.SetAttachment(const Attachment: TStrings);
begin
  FAttachment.Assign(Attachment);
end;


{ Count number of unread messages. }
{----------------------------------}

function TEmail.CountUnread: ULONG;
var
  szMessageID     : PChar;
  szSeedMessageID : PChar;
  flFlags         : ULONG;
  Count           : ULONG;
  flLogoff        : Boolean;
begin
  CheckMAPI;

  Count := 0;

  { Auto Logoff, if no session active. }
  If hSession = 0 then flLogoff := True
  else                 flLogoff := False;

  { Logon to mail server. }
  If Logon <> SUCCESS_SUCCESS then begin
    Result := MAPI_E_LOGIN_FAILURE;
    exit;
  end;

  szMessageId := Nil;
  szSeedMessageId := Nil;
  { Start at first message. }
  try
    szMessageID := StrAlloc(MsgIdSize); { MAPI 32bit !! }
    szSeedMessageID := StrAlloc(MsgIdSize); { MAPI 32bit !! }
    FillChar(szMessageID^,MsgIdSize,0);
    FillChar(szSeedMessageID^,MsgIdSize,0);

    flFlags := MAPI_UNREAD_ONLY;

    while MAPIFindNext(hSession,0,nil,
                      szSeedMessageID,flFlags,0,@szMessageID^) = SUCCESS_SUCCESS do begin
       if StrComp(szSeedMessageId, szMessageId) = 0 then
          break
       else
           StrCopy(szSeedMessageId,szMessageId);
       Count := Count + 1;
    end;

  finally
    StrDispose(szMessageID);
    StrDispose(szSeedMessageID);

    { Auto logoff, if no session was active. }
    if flLogoff = True then Logoff;
  end;

  Result := Count;
end;

 { Following code added by Rudi Claasen }

{ get address dialog }
{--------------------}

function TEmail.Address: Integer;
var
  MapiRecipDesc : TMapiRecipDesc;
  lpRecipArray  : TlpRecipArray;
  lpNwRecipArray: TlpRecipArray;
  lppRecipArray : lpMapiRecipDesc;

  flLogoff      : Boolean;
  MapiResult    : ULONG;

  i             : Integer;

  nRecipients   : integer;    { number of initial recipients after address check }
  nNwRecipients : integer;    { number of recipients returned by MAPIAddress }

begin
  CheckMAPI;

  { make sure the cleanup does not free garbage }
  lpRecipArray  := nil;

  flLogoff       := False;

  { check our built-in limits }
  nRecipients := FCC.Count + FBCC.Count + Frecip.Count;
  if nRecipients > RECIP_MAX then begin
    Result := MAPI_E_TOO_MANY_RECIPIENTS;
    exit;
  end;

  { begin the work}
  try
    lpRecipArray  := TlpRecipArray(StrAlloc(nRecipients*SizeOf(TMapiRecipDesc)));
    FillChar(lpRecipArray^, StrBufSize(PChar(lpRecipArray)), 0);

    If hSession = 0 then flLogoff := True
    else                 flLogoff := False;

    { Logon to mail server if not already logged on. }

    If Logon <> SUCCESS_SUCCESS then begin
      Result := MAPI_E_LOGIN_FAILURE;
      exit;
    end;
    { Initialise MAPI structures and local arrays. }

    FillChar(MapiRecipDesc, Sizeof(TMapiRecipDesc), 0);

    nRecipients := 0;
    { check and fill in recipients if any }
    ListToRecipArray(FRecip, MAPI_TO,  lpRecipArray, nRecipients);
    ListToRecipArray(FCC,    MAPI_CC,  lpRecipArray, nRecipients);
    ListToRecipArray(FBcc,   MAPI_BCC, lpRecipArray, nRecipients);

    Result := -1;
    MapiResult := MapiAddress (hSession,
                               0,
                               '',
                               4,
                               '',
                               nRecipients,
                               lpRecipArray^[0],
                               0,
                               0,
                               @nNwRecipients,
                               lppRecipArray);
    if MapiResult <> SUCCESS_SUCCESS then begin
      Result := MapiResult;
      exit;
    end;
    lpNwRecipArray := @lppRecipArray^;
      { Convert names to TStringList }
    FRecip.Clear;
    FCC.Clear;
    FBcc.Clear;
    for i := 0 to (nNwRecipients - 1) do
    begin
      if lpNwRecipArray^[i].ulRecipClass = MAPI_TO then
        FRecip.Add(StrPas(lpNwRecipArray^[i].lpszName));
      if lpNwRecipArray^[i].ulRecipClass = MAPI_CC then
        FCC.Add(StrPas(lpNwRecipArray^[i].lpszName));
      if lpNwRecipArray^[i].ulRecipClass = MAPI_BCC then
        FBcc.Add(StrPas(lpNwRecipArray^[i].lpszName));
    end;
    MapiFreeBuffer(lppRecipArray);      { added 17/02/97 -- dh --}
    Result := 0;
  finally
      { dispose of the recipient & CC & BCC name strings }
      { free only those that passed initial address check }
    if Assigned(lpRecipArray) then
      for i := 0 to (nRecipients - 1) do begin
        Dispose(lpRecipArray^[i].lpszName);
      end;
      {dispose of the recipient/CC/BCC array }
    StrDispose(PChar(lpRecipArray));
    { Auto logoff, if no session was active. }
    if flLogoff = True then Logoff;
  end;
end;


{ list to recipient arrary }
{--------------------------}

Function TEMail.ListToRecipArray(FArray          : TStrings;
                                 AulRecipClass   : ULONG;
                                 lpRecipArray    : TlpRecipArray;
                                 var nRecipients : integer) : Integer;
const
  RecipSize = 256;
var
  lpRecip       : lpMapiRecipDesc;

  szRecipName   : PChar;
  MapiResult    : ULONG;
  i             : integer;

  s: SString;
begin
  { CheckMAPI;   ListToRecipArray is an internal function that can/should
                 only get called by code that has already called CheckMAPI }

  szRecipName := Nil;
  try
    szRecipName := StrAlloc(RecipSize);
    Result := SUCCESS_SUCCESS;

    for i := 0 to (FArray.Count - 1) do begin
      if Length(FArray.Strings[i]) > 0 then begin     { recipient specified }
        StrPCopy(szRecipName, FArray.Strings[i]);     { check recipients name }
        MapiResult := MAPIResolveName(hSession, 0, szRecipName, 0, 0, @lpRecip);
        if ((MapiResult = MAPI_E_AMBIGUOUS_RECIPIENT)
                           or (MapiResult = MAPI_E_UNKNOWN_RECIPIENT)) then begin
           MapiResult := MAPIResolveName(hSession, 0, szRecipName, MAPI_DIALOG,
                                        0, @lpRecip);
        end;

        { Windows will either be happy about something like [FAX:123456] or not;
          if it is happy it will strip  '[FAX:' and ']', leaving just 123456

          Strategy:
          - Find out whether szRecipName originally contained '[', ']'
          - Check whether they are still there
             -> error, NO direct address
          - [] are gone, so simply add the ORIGINAL string, NOT the modified
              MAPI resolved name to .lpszAddress
        }

        if (MapiResult <> SUCCESS_SUCCESS) then begin
           Result := MapiResult;
        end else begin

           { the original address looked like a direct address }
           if ((Pos('[', FArray.Strings[i]) > 0) and (Pos(']', FArray.Strings[i]) > 0)) and

           { did Windows recognize this?}
              ((StrScan (lpRecip^.lpszName, '[') = nil) and
               (StrRScan(lpRecip^.lpszName, ']') = nil)) then
           begin
             { use original address }
             { do NOT re-assign FArray.Strings }
             MAPIFreeBuffer(lpRecip); { free the buffer }

             with lpRecipArray^[nRecipients] do
             begin
                s := FArray.Strings[i];

               { remove [ ] }
               { this assumes that there is only ONE of '[' and ']' each in the string }
               { testing indicates that for a valid direct address - which we DO have
                 here - this is always the case; no documentation could be found though
                 that assures us of this }
               Delete(s, Pos(']', s), 1);
               Delete(s, Pos('[', s), 1);

               ulRecipClass   := AulRecipClass;
               lpszAddress    := StrPCopy( new(TlpszRecipName)^, s);
               lpszName       := StrPCopy( new(TlpszRecipName)^, s);
             end;
           end
           else
           begin
             { use resolved address }
             FArray.Strings[i] := StrPas(lpRecip^.lpszName);
             MAPIFreeBuffer(lpRecip); { free the buffer }

             with lpRecipArray^[nRecipients] do
             begin
               ulRecipClass   := AulRecipClass;
               lpszName       := StrPCopy( new(TlpszRecipName)^, FArray.Strings[i]);
             end;

           end;

           Inc(nRecipients);
        end;

      end;
    end;

  finally
    StrDispose(szRecipName);
  end;

end;

{------------------------------------------------------------------------------}

initialization
  {$IFNDEF Win32} {$IFDEF UseGenericThunks}{ need some code for generic thunking }
     InitWin32;
  {$ENDIF} {$ENDIF}
end.
