{

  Autor: Jos Sebastin Battig - 1997
  E-Mail: sbattig@bigfoot.com

}

unit Splitter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Cus_Bas, Barsize, ExtCtrls;

type
  TSplitterBar = class(TBarreResize)
  private
    FControlAbove : boolean;
    FControl : TControl;
    LastMouseUp : TMouseEvent;
    LastBeforeMouseUp : TMouseEvent;
    LastAlign : TAlign;
    FNeedRefresh : boolean;
    procedure NewMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure NewBeforeMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure SetControl (value : TControl);
    procedure SetControlAbove (value : boolean);
    procedure SetAlign;
    procedure SetAbove;
    procedure SetIsHorizontal (value : boolean);
    function GetIsHorizontal : boolean;
  protected
    procedure Loaded; override;
  public
    constructor Create (AOwner : TComponent); override;
  published
    property IsHorizontal : boolean read GetIsHorizontal write SetIsHorizontal;
    property Control : TControl read FControl write SetControl;
    property ControlNeedsRefresh : boolean read FNeedRefresh write FNeedRefresh default false;
    property ControlIsAboveOrLeft : boolean read FControlAbove write SetControlAbove default true;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('NewComps', [TSplitterBar]);
end;

constructor TSplitterBar.Create;
begin
  inherited Create (AOwner);
  FControlAbove := true;
end;

procedure TSplitterBar.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState)
    then
    begin
      LastMouseUp := OnMouseUp;
      LastBeforeMouseUp := OnBeforeMouseUp;
      OnMouseUp := NewMouseUp;
      OnBeforeMouseUp := NewBeforeMouseUp;
    end;
end;

procedure TSplitterBar.SetAlign;
begin
  if IsHorizontal
    then if FControlAbove
      then FControl.Align := alTop
      else FControl.Align := alBottom
    else if FControlAbove
      then FControl.Align := alLeft
      else FControl.Align := alRight;
  Align := FControl.Align;
end;

procedure TSplitterBar.SetAbove;
begin
  if IsHorizontal
    then FControlAbove := FControl.Top < Top
    else FControlAbove := FControl.Left < Left;
end;

procedure TSplitterBar.SetControl;
begin
  if value <> FControl
    then
    begin
      FControl := value;
      if (csDesigning in ComponentState) and (FControl <> nil) and IsLoaded
        then
        begin
          SetAbove;
          SetAlign;
        end;
    end;
end;

procedure TSplitterBar.SetControlAbove;
begin
  if value <> FControlAbove
    then
    begin
      FControlAbove := value;
      if (csDesigning in ComponentState) and (FControl <> nil) and IsLoaded
        then SetAlign;
    end;
end;

function TSplitterBar.GetIsHorizontal;
begin
  Result := inherited IsHorizontal;
end;

procedure TSplitterBar.SetIsHorizontal;
begin
  if value <> IsHorizontal
    then
    begin
      if (csDesigning in ComponentState) and (FControl <> nil) and IsLoaded
        then Align := alNone;
      inherited IsHorizontal := value;
      if (csDesigning in ComponentState) and (FControl <> nil) and IsLoaded
        then
        begin
          SetAbove;
          SetAlign;
        end;
    end;
end;

procedure TSplitterBar.NewMouseUp;
begin
  if IsHorizontal
    then if FControlAbove
      then Control.Height := Top - FControl.Top
      else FControl.Height := (Owner as TControl).ClientHeight - Top - Height
    else if FControlAbove
      then FControl.Width := Left - 1
      else FControl.Width := (Owner as TControl).ClientWidth - Left - Width;
  Align := LastAlign;
  if FNeedRefresh
    then FControl.Refresh;
  if assigned (LastMouseUp)
    then LastMouseUp (Sender, Button, Shift, X, Y);
end;

procedure TSplitterBar.NewBeforeMouseUp;
begin
  LastAlign := Align;
  Align := alNone;
  if assigned (LastBeforeMouseUp)
    then LastBeforeMouseUp (Sender, Button, Shift, X, Y);
end;

end.
