unit uDemo;

interface

Uses
	SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
	Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Spin;

type
	TForm1 = class(TForm)
		Panel1: TPanel;
    ColorDialog1: TColorDialog;
		GroupBox2: TGroupBox;
    btnFill: TBitBtn;
    btnPen: TBitBtn;
    EditTitle: TEdit;
		GroupBox3: TGroupBox;
    SpinLongitude: TSpinEdit;
    SpinLatitude: TSpinEdit;
    cbxGrid: TCheckBox;
    GroupBox4: TGroupBox;
    cbxLayer0: TCheckBox;
    cbxLayer1: TCheckBox;
    cbxLayer2: TCheckBox;
    cbxLayer3: TCheckBox;
		cbxLayer4: TCheckBox;
    spinDetail: TSpinEdit;
    Label5: TLabel;
		Panel2: TPanel;
    btnDelete: TBitBtn;
    SpinButton1: TSpinButton;
    btnUp: TSpeedButton;
    btnLeft: TSpeedButton;
    btnExtents: TSpeedButton;
    btnRight: TSpeedButton;
    btnDown: TSpeedButton;
    Label2: TLabel;
    Label3: TLabel;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    EditDataFile: TEdit;
    SpeedButton3: TSpeedButton;
    SpeedButton1: TSpeedButton;
    cbxProjection: TCheckBox;
    Globe: TGlobe;
		procedure GlobeMouseDown(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure GlobeMouseMove(Sender: TObject; Shift: TShiftState; X,
			Y: Integer);
		procedure SpeedButton1Click(Sender: TObject);
		procedure cbxLayer0Click(Sender: TObject);
		procedure EditTitleChange(Sender: TObject);
		procedure btnDeleteClick(Sender: TObject);
		procedure SpeedButton3Click(Sender: TObject);
		procedure cbxGridClick(Sender: TObject);
		procedure btnUpClick(Sender: TObject);
		procedure btnDownClick(Sender: TObject);
		procedure btnRightClick(Sender: TObject);
		procedure btnLeftClick(Sender: TObject);
		procedure spinDetailChange(Sender: TObject);
		procedure FormKeyDown(Sender: TObject; var Key: Word;
			Shift: TShiftState);
		procedure SpinLatLongChange(Sender: TObject);
		procedure btnFillClick(Sender: TObject);
		procedure btnPenClick(Sender: TObject);
		procedure btnExtentsClick(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure SpinButton1DownClick(Sender: TObject);
		procedure SpinButton1UpClick(Sender: TObject);
    procedure cbxProjectionClick(Sender: TObject);
	private
		{ Private declarations }
	public
		{ Public declarations }
		procedure DisplayHint(Sender: TObject);
		procedure DisplayObjectData;
	end;

var
	Form1: TForm1;

implementation

{$R *.DFM}

{-------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
	Application.OnHint := DisplayHint;
	Show;
	Application.ProcessMessages;
	EditDataFile.Text := ExtractFilePath( ParamStr(0)) + 'globe.dat';
	Globe.Globefile := EditDataFile.Text;
	DisplayObjectData;
end;

{-------------------------------------------------------------------------}
procedure TForm1.DisplayObjectData;
begin
	with Globe do
	begin
		btnDelete.Enabled := ( SelectedObject <> nil );
		btnFill.Enabled := ( SelectedObject <> nil );
		btnPen.Enabled := ( SelectedObject <> nil );

		if SelectedObject <> nil then
		begin
			GroupBox2.Caption := Format( 'Selected Object: %d Layer %d',
				[ SelectedObject.Index, SelectedObject.Layer ]);
			EditTitle.Text := SelectedObject.Title;
		end
		else
		begin
			GroupBox2.Caption := 'Selected Object: None';
			editTitle.text := '';
		end;
	end;
end;
{-------------------------------------------------------------------------}
procedure TForm1.GlobeMouseDown(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	with Globe do
		if Button = mbLeft then
		begin
			SelectedObject := ObjectAtDeviceXY( X, Y );	{ Set the Globe SelectedObject }
			DisplayObjectData;
		end;
end;

{-------------------------------------------------------------------------}
procedure TForm1.GlobeMouseMove(Sender: TObject; Shift: TShiftState;
	X, Y: Integer);
var
	pt : TPoint;
	sTmp : string;
	MouseObj : TGlobeObject;
begin
	with Globe do
	begin
		MouseObj := ObjectAtDeviceXY( X, Y );
		if MouseObj <> nil then
			sTmp := MouseObj.Title
		else
			sTmp := '';

		pt := DeviceXYToLL( X, Y );
		Caption := Format( '%s %s', [LLPointAsString( pt ), sTmp] );
	end;
end;

{-------------------------------------------------------------------------}
procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
	if not FileExists( EditDataFile.Text ) then
		Globe.SaveGlobeToFile( EditDataFile.Text )
	else
		if MessageDlg( 'File already exists overwrite?', mtWarning, [mbYes, mbNo], 0 ) = mrYes then
			Globe.SaveGlobeToFile( EditDataFile.Text );
end;

{-------------------------------------------------------------------------}
procedure TForm1.cbxLayer0Click(Sender: TObject);
begin
	Globe.LayerEnabled[TCheckBox( Sender ).Tag] := TCheckBox( Sender ).checked;
end;

{-------------------------------------------------------------------------}
procedure TForm1.btnDeleteClick(Sender: TObject);
begin
	with Globe do
		if SelectedObject <> nil then
		begin
			SelectedObject.Delete;
			Invalidate;		{ re-draw the globe }
		end;
end;

{-------------------------------------------------------------------------}
procedure TForm1.SpeedButton3Click(Sender: TObject);
begin
	with Globe do
		Globe.GlobeFile := EditDataFile.Text;
end;

{-------------------------------------------------------------------------}
procedure TForm1.cbxGridClick(Sender: TObject);
begin
	with Globe do
		if cbxGrid.Checked then
			DrawGrid := dgBehind
		else
			DrawGrid := dgInFront;
end;

{-------------------------------------------------------------------------}
procedure TForm1.btnUpClick(Sender: TObject);
begin
	Globe.XRotation := Globe.XRotation - 15;
end;

{-------------------------------------------------------------------------}
procedure TForm1.btnDownClick(Sender: TObject);
begin
	Globe.XRotation := Globe.XRotation + 15;
end;

{-------------------------------------------------------------------------}
procedure TForm1.btnRightClick(Sender: TObject);
begin
	Globe.YRotation := Globe.YRotation + 5;
end;

{-------------------------------------------------------------------------}
procedure TForm1.btnLeftClick(Sender: TObject);
begin
	Globe.YRotation := Globe.YRotation - 5;
end;

{-------------------------------------------------------------------------}
procedure TForm1.spinDetailChange(Sender: TObject);
begin
	Globe.DetailLevel := SpinDetail.Value;
end;

{-------------------------------------------------------------------------}
procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
	Shift: TShiftState);
begin
	if not ( ssCtrl in Shift )then
		Exit;

	case Key of
	VK_UP :    btnUpClick( Sender );
	VK_RIGHT : btnRightClick( Sender );
	VK_DOWN :  btnDownClick( Sender );
	VK_LEFT :  btnLeftClick( Sender );
	else
		Exit;
	end;
	Key := 0;
end;

{-------------------------------------------------------------------------}
procedure TForm1.SpinLatLongChange(Sender: TObject);
begin
	Globe.LatitudeStep := SpinLatitude.Value;
	Globe.LongitudeStep := SpinLongitude.Value;
end;

{-------------------------------------------------------------------------}
procedure TForm1.btnFillClick(Sender: TObject);
begin
	with Globe do
		if ColorDialog1.Execute then
		begin
			with Globe.Attributes[SelectedObject.AttributeIndex] do
				SelectedObject.AttributeIndex := AddAttribute(
					ColorDialog1.Color, BrushStyle, PenColour, PenStyle );

			Invalidate;
		end;
end;

{-------------------------------------------------------------------------}
procedure TForm1.btnPenClick(Sender: TObject);
begin
	with Globe do
		if ColorDialog1.Execute then
		begin
			with Globe.Attributes[SelectedObject.AttributeIndex] do
				SelectedObject.AttributeIndex := AddAttribute(
					BrushColour, BrushStyle, ColorDialog1.Color, PenStyle );

			Invalidate;
		end;
end;

{-------------------------------------------------------------------------}
procedure TForm1.btnExtentsClick(Sender: TObject);
begin
	Globe.ViewRect := Rect( 0,0,0,0 );
end;

{-------------------------------------------------------------------------}
procedure TForm1.DisplayHint(Sender: TObject);
begin
	Panel2.Caption := Application.Hint;
end;

{-------------------------------------------------------------------------}
procedure TForm1.EditTitleChange(Sender: TObject);
begin
	if Globe.selectedobject <> nil then
		Globe.selectedobject.Title := editTitle.text;
end;

{-------------------------------------------------------------------------}
procedure TForm1.SpinButton1DownClick(Sender: TObject);
begin
	with Globe do
	begin
		if SelectedObject <> nil then
			SelectedObject := Objects[SelectedObject.Index - 1];

		if SelectedObject = nil then
			SelectedObject := Objects[ObjectCount - 1];
		LocateObject( SelectedObject );
		DisplayObjectData;
	end;
end;

{-------------------------------------------------------------------------}
procedure TForm1.SpinButton1UpClick(Sender: TObject);
begin
	with Globe do
	begin
		if SelectedObject <> nil then
			SelectedObject := Objects[SelectedObject.Index + 1];

		if SelectedObject = nil then
			SelectedObject := Objects[0];
		with ViewRect do
			LocateObject( SelectedObject );
		DisplayObjectData;
	end;
end;

{-------------------------------------------------------------------------}
procedure TForm1.cbxProjectionClick(Sender: TObject);
begin
	if cbxProjection.Checked then
		Globe.Projection := gpMercator
	else
		Globe.Projection := gpSpherical;
end;

end.
